using Illusion.Game;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

namespace ADV.Commands.Base
{
    public class Choice : CommandBase
    {
        public class ChoiceData
        {
            public Transform transform;

            public string text;

            public string jump;
        }

        private List<ChoiceData> choices = new List<ChoiceData>();

        private int id = -1;

        public override string[] ArgsLabel
        {
            get
            {
                return new string[2]
                {
                    "Visible",
                    "Case"
                };
            }
        }

        public override string[] ArgsDefault
        {
            get
            {
                return new string[2]
                {
                    bool.TrueString,
                    "text,tag"
                };
            }
        }

        public override void Do()
        {
            base.Do();
            scenario.ChoicesInit();
            int count = 0;
            if (!bool.Parse(args[count++]))
            {
                scenario.textController.Clear();
            }
            else if (!scenario.textController.MessageWindow.text.IsNullOrEmpty())
            {
                (scenario as MainScenario).SafeProc(delegate (MainScenario main)
                {
                    main.BackLog.Logs.SafeProc(main.BackLog.Logs.Count - 1, delegate (BackLog.Data log)
                    {
                        if (!log.voiceList.IsNullOrEmpty())
                        {
                            scenario.currentCharaData.CreateVoiceList();
                            scenario.currentCharaData.voiceList.AddRange(log.voiceList);
                        }
                    });
                });
            }
            RectTransform rectTransform = scenario.Choices.GetChild(0) as RectTransform;
            foreach (var item in args.Skip(count).Select((s, index) => new
            {
                s,
                index
            }))
            {
                if (!item.s.IsNullOrEmpty())
                {
                    string[] array = item.s.Split(',');
                    if (array.Length > 1)
                    {
                        GameObject gameObject = UnityEngine.Object.Instantiate(rectTransform.gameObject, scenario.Choices);
                        gameObject.SetActive(true);
                        gameObject.name = "Choice" + (item.index + 1);
                        choices.Add(new ChoiceData
                        {
                            text = scenario.ReplaceText(array[0], true),
                            jump = scenario.ReplaceVars(array[1]),
                            transform = gameObject.transform
                        });
                    }
                }
            }
            scenario.isChoice = true;
            scenario.Choices.gameObject.SetActive(true);
            choices.ForEach(delegate (ChoiceData p)
            {
                Choice choice = this;
                TextMeshProUGUI componentInChildren = p.transform.GetComponentInChildren<TextMeshProUGUI>();
                componentInChildren.text = p.text;
                Button bt = p.transform.GetComponent<Button>();
                bt.onClick.RemoveAllListeners();
                bt.onClick.AddListener(delegate
                {
                    choice.id = ButtonProc(bt.name);
                });
                p.transform.gameObject.SetActive(true);
            });
            scenario.isSkip = scenario.isSkip && Manager.Config.TextData.ChoicesSkip;
            scenario.isAuto = scenario.isAuto && Manager.Config.TextData.ChoicesAuto;
        }

        public override bool Process()
        {
            base.Process();
            return id >= 0;
        }

        public override void Result(bool processEnd)
        {
            base.Result(processEnd);
            if (processEnd && id >= 0)
            {
                string jump = choices[id].jump;
                (scenario as MainScenario).SafeProc(delegate (MainScenario main)
                {
                    main.BackLog.Add(new BackLog.Data(localLine, new Text.Data(new string[2]
                    {
                        "選択肢",
                        choices[id].text
                    }), null));
                });
                IEnumerator enumerator = scenario.Choices.GetEnumerator();
                try
                {
                    while (enumerator.MoveNext())
                    {
                        Transform transform = (Transform)enumerator.Current;
                        if (transform.gameObject.activeSelf)
                        {
                            UnityEngine.Object.Destroy(transform.gameObject);
                        }
                    }
                }
                finally
                {
                    IDisposable disposable;
                    if ((disposable = enumerator as IDisposable) != null)
                    {
                        disposable.Dispose();
                    }
                }
                scenario.ChoicesInit();
                scenario.SearchTagJumpOrOpenFile(jump, localLine);
            }
        }

        private static int ButtonProc(string btName)
        {
            Utils.Sound.Play(SystemSE.sel);
            int result = 0;
            try
            {
                int.TryParse(Regex.Replace(btName, "[^0-9]", string.Empty), out result);
            }
            catch (Exception)
            {
            }
            return result - 1;
        }
    }
}
