using System;
using System.Collections.Generic;
using System.Linq;

namespace ADV.Commands.Base
{
    public class Text : CommandBase
    {
        public class Data
        {
            public string name = string.Empty;

            public string text = string.Empty;

            public string colorKey
            {
                get;
                private set;
            }

            public Data(string[] args)
            {
                for (int i = 0; i < args.Length; i++)
                {
                    switch (i)
                    {
                        case 0:
                            colorKey = name = args[i];
                            break;
                        case 1:
                            text = args[i];
                            break;
                    }
                }
            }
        }

        private class Next
        {
            private int currentNo;

            private List<List<Action>> playList = new List<List<Action>>();

            private TextScenario scenario;

            private Action onChange;

            private bool voicePlayEnd;

            public Next(TextScenario scenario)
            {
                this.scenario = scenario;
                TextScenario.CurrentCharaData currentCharaData = scenario.currentCharaData;
                List<TextScenario.IMotion[]> motionList = currentCharaData.motionList;
                List<TextScenario.IExpression[]> expressionList = currentCharaData.expressionList;
                List<TextScenario.IExpressionIcon[]> expressionIconList = currentCharaData.expressionIconList;
                int cnt = 0;
                Func<bool> func = () => !motionList.IsNullOrEmpty() && motionList.SafeGet(cnt) != null;
                Func<bool> func2 = () => !expressionList.IsNullOrEmpty() && expressionList.SafeGet(cnt) != null;
                Func<bool> func3 = () => !expressionIconList.IsNullOrEmpty() && expressionIconList.SafeGet(cnt) != null;
                while (func() || func2() || func3())
                {
                    TextScenario.IMotion[] motion = func() ? motionList[cnt] : null;
                    TextScenario.IExpression[] expression = func2() ? expressionList[cnt] : null;
                    TextScenario.IExpressionIcon[] expressionIcon = func3() ? expressionIconList[cnt] : null;
                    playList.Add(new List<Action>
                    {
                        delegate
                        {
                            Play(expression);
                        },
                        delegate
                        {
                            Play(expressionIcon);
                        },
                        delegate
                        {
                            Play(motion);
                        }
                    });
                    cnt++;
                }
                if (!currentCharaData.voiceList.IsNullOrEmpty())
                {
                    onChange = delegate
                    {
                        Play();
                    };
                    scenario.VoicePlay(currentCharaData.voiceList, onChange, delegate
                    {
                        voicePlayEnd = true;
                    });
                }
            }

            private bool Play()
            {
                return playList.SafeProc(currentNo++, delegate (List<Action> p)
                {
                    p.ForEach(delegate (Action proc)
                    {
                        proc();
                    });
                });
            }

            public bool Process()
            {
                if (scenario.currentCharaData.isSkip)
                {
                    return true;
                }
                if (playList.Count <= currentNo && voicePlayEnd)
                {
                    return true;
                }
                if (onChange == null)
                {
                    bool flag = false;
                    List<TextScenario.IMotion[]> motionList = scenario.currentCharaData.motionList;
                    if (currentNo == 0)
                    {
                        flag = true;
                    }
                    else
                    {
                        bool flag2 = false;
                        if (!motionList.IsNullOrEmpty() && currentNo < motionList.Count)
                        {
                            flag2 = !motionList[currentNo].IsNullOrEmpty();
                        }
                        TextScenario.IMotion[] array = !flag2 ? null : motionList[currentNo - 1];
                        flag = array.IsNullOrEmpty() || MotionEndCheck(array);
                    }
                    if (flag && !Play())
                    {
                        bool lastMotionEnd = true;
                        bool flag3 = !motionList.IsNullOrEmpty();
                        if (flag3)
                        {
                            flag3 = motionList.LastOrDefault().SafeProc(delegate (TextScenario.IMotion[] last)
                            {
                                lastMotionEnd = MotionEndCheck(last);
                            });
                        }
                        if (flag3 && !lastMotionEnd)
                        {
                            return false;
                        }
                        return true;
                    }
                }
                return false;
            }

            public void Result()
            {
                while (currentNo < playList.Count)
                {
                    Play();
                }
            }

            private void Play(TextScenario.IMotion[] motionList)
            {
                if (!motionList.IsNullOrEmpty())
                {
                    scenario.CrossFadeStart();
                    foreach (TextScenario.IMotion motion in motionList)
                    {
                        motion.Play(scenario);
                    }
                }
            }

            private void Play(TextScenario.IExpression[] expressionList)
            {
                if (!expressionList.IsNullOrEmpty())
                {
                    foreach (TextScenario.IExpression expression in expressionList)
                    {
                        expression.Play(scenario);
                    }
                }
            }

            private void Play(TextScenario.IExpressionIcon[] expressionIconList)
            {
                foreach (CharaData value in scenario.commandController.Characters.Values)
                {
                    value.faceIcon.Release();
                }
                if (!expressionIconList.IsNullOrEmpty())
                {
                    foreach (TextScenario.IExpressionIcon expressionIcon in expressionIconList)
                    {
                        expressionIcon.Play(scenario);
                    }
                }
            }

            private bool MotionEndCheck(TextScenario.IMotion[] list)
            {
                Func<ChaControl, bool> endCheck = chaCtrl => chaCtrl.getAnimatorStateInfo(0).normalizedTime >= 1f;
                return list.All(motion => endCheck(motion.GetChara(scenario).chaCtrl));
            }
        }

        private Next next;

        public override string[] ArgsLabel
        {
            get
            {
                return new string[2]
                {
                    "Name",
                    "Text"
                };
            }
        }

        public override string[] ArgsDefault
        {
            get
            {
                return null;
            }
        }

        public override void Convert(string fileName, ref string[] args)
        {
            int index = 1;
            string text = args.SafeGet(index);
            if (!text.IsNullOrEmpty())
            {
                args[index++] = string.Join("\n", text.Split(new string[1]
                {
                    "@br"
                }, StringSplitOptions.None));
            }
        }

        public override void Do()
        {
            base.Do();
            Data outPut = new Data(args);
            scenario.fontColorKey = outPut.name;
            if (outPut.name != string.Empty)
            {
                outPut.name = scenario.ReplaceText(outPut.name, false);
            }
            if (outPut.text != string.Empty)
            {
                outPut.text = scenario.ReplaceText(outPut.text, true);
            }
            TextScenario.CurrentCharaData currentCharaData = scenario.currentCharaData;
            if (currentCharaData.isSkip || Manager.Config.TextData.NextVoiceStop)
            {
                scenario.VoicePlay(null, null, null);
            }
            currentCharaData.isSkip = false;
            scenario.textController.Set(outPut.name, outPut.text);
            (scenario as MainScenario).SafeProc(delegate (MainScenario main)
            {
                main.BackLog.Add(new BackLog.Data(localLine, outPut, !scenario.currentCharaData.isKaraoke ? scenario.currentCharaData.voiceList : null));
            });
            next = new Next(scenario);
        }

        public override bool Process()
        {
            base.Process();
            return next.Process();
        }

        public override void Result(bool processEnd)
        {
            base.Result(processEnd);
            next.Result();
        }
    }
}
