using UnityEngine;

namespace ADV.Commands.Movie
{
    public class Play : CommandBase
    {
        private MoviePlayerGUI player;

        private bool skipReg;

        public override string[] ArgsLabel
        {
            get
            {
                return new string[3]
                {
                    "Bundle",
                    "Asset",
                    "isSkipReg"
                };
            }
        }

        public override string[] ArgsDefault
        {
            get
            {
                return new string[3]
                {
                    string.Empty,
                    string.Empty,
                    bool.FalseString
                };
            }
        }

        public override void Do()
        {
            base.Do();
            player = null;
            int num = 0;
            MovieTexture asset = AssetBundleManager.LoadAsset(args[num++], args[num++], typeof(MovieTexture)).GetAsset<MovieTexture>();
            if (!bool.TryParse(args.SafeGet(num++), out skipReg))
            {
                skipReg = false;
            }
            if (skipReg)
            {
                MainScenario mainScenario = scenario as MainScenario;
                if (mainScenario != null && mainScenario.mode == MainScenario.Mode.Movie)
                {
                    scenario.regulate.AddRegulate(Regulate.Control.Auto);
                }
                else
                {
                    scenario.regulate.AddRegulate((Regulate.Control)13);
                }
            }
            GameObject gameObject = new GameObject("MoviePlaying");
            player = gameObject.AddComponent<MoviePlayerGUI>();
            player.movie = asset;
        }

        public override bool Process()
        {
            base.Process();
            return !player.movie.isPlaying;
        }

        public override void Result(bool processEnd)
        {
            base.Result(processEnd);
            Destroy();
        }

        private void Destroy()
        {
            UnityEngine.Object.Destroy(player.gameObject);
            if (skipReg)
            {
                MainScenario mainScenario = scenario as MainScenario;
                if (mainScenario != null && mainScenario.mode == MainScenario.Mode.Movie)
                {
                    mainScenario.regulate.SubRegulate(Regulate.Control.Auto);
                }
                else
                {
                    scenario.regulate.SetRegulate(0);
                }
            }
        }
    }
}
