using ADV;
using Illusion.Extensions;
using Manager;
using System;
using System.Collections;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.AI;

namespace ActionGame.Chara
{
    public sealed class Fix : Base
    {
        private BoolReactiveProperty _isPlayerAction = new BoolReactiveProperty();

        [SerializeField]
        private SpritesAnim _spritesAnim;

        [SerializeField]
        private bl_MiniMapItem miniMapIcon;

        [SerializeField]
        private SpeakChara _speak;

        [SerializeField]
        private NavMeshObstacle _obstacle;

        private MotionVoice motionVoice = new MotionVoice();

        [SerializeField]
        private Transform _advCamPos;

        public override bool isAction
        {
            get
            {
                return true;
            }
        }

        public FixEventScheduler.Result eventSchedulerResult
        {
            get;
            private set;
        }

        public bool isPlayerAction
        {
            get
            {
                return _isPlayerAction.Value;
            }
            set
            {
                _isPlayerAction.Value = value;
            }
        }

        public SpeakChara speak
        {
            get
            {
                return _speak;
            }
        }

        public NavMeshObstacle obstacle
        {
            get
            {
                return _obstacle;
            }
        }

        public Transform advCamPos
        {
            get
            {
                return _advCamPos;
            }
        }

        public static Fix Create(Transform parent, SaveData.Heroine heroine)
        {
            GameObject asset = AssetBundleManager.LoadAsset("action/chara.unity3d", "Fix", typeof(GameObject)).GetAsset<GameObject>();
            Fix component = Instantiate(asset, parent, false).GetComponent<Fix>();
            component.charaData = heroine;
            component.name = GetObjectName(heroine);
            component.gameObject.SetActive(true);
            return component;
        }

        private static string GetObjectName(SaveData.Heroine heroine)
        {
            return string.Format("Fix({0})_{1}_{2}", heroine.FixCharaIDOrPersonality, Cycle.GetClassRoomName(heroine.schoolClass), heroine.schoolClassIndex);
        }

        public override void PlayAnimation()
        {
            if (!(chaCtrl == null))
            {
                ItemClear();
                ResetKind();
                if (isActive && isArrival)
                {
                    PlayAnimationItemKindSet(true);
                }
                else
                {
                    motion.state = "Idle";
                }
                SetParameterHeightAnimation();
                motion.Play(animator);
            }
        }

        public override void LoadAnimator()
        {
            CharaInfo.Param param = actScene.charaInfoDic[heroine.fixCharaID];
            motion.bundle = param.MotionBundle;
            motion.asset = param.MotionAsset;
            motion.LoadAnimator(animator);
        }

        public override void ChangeNowCoordinate()
        {
            if (!(chaCtrl == null))
            {
                ChaFileDefine.CoordinateType type = ChaFileDefine.CoordinateType.School01;
                if (!heroine.isTeacher && eventSchedulerResult.param != null && !eventSchedulerResult.param.Coordinate.IsNullOrEmpty())
                {
                    string coordinate = eventSchedulerResult.param.Coordinate;
                    int num = coordinate.Check(true, Enum.GetNames(typeof(ChaFileDefine.CoordinateType)));
                    type = (ChaFileDefine.CoordinateType)num;
                }
                chaCtrl.ChangeCoordinateTypeAndReload(type);
            }
        }

        public override void Replace(SaveData.CharaData charaData)
        {
            base.Replace(charaData);
            name = GetObjectName(heroine);
            miniMapIcon.iconItem.text = charaData.Name;
            motionVoice.Init(heroine);
        }

        public void SetEventSchedulerResult(FixEventScheduler.Result result)
        {
            eventSchedulerResult = result;
        }

        public void Speak(string bundleName, string assetName)
        {
            if (speak.isOpen)
            {
                speak.Close();
            }
            speak.bundleName = bundleName;
            speak.assetName = assetName;
            speak.Open();
        }

        protected override void Awake()
        {
            base.Awake();
        }

        protected override IEnumerator Start()
        {
            yield return new WaitWhile(() => actScene.Player.chaCtrl == null);
            yield return new WaitUntil(() => actScene.Player.chaCtrl.loadEnd);
            yield return StartCoroutine(base.Start());
            if (_isCharaLoad)
            {
            }
            _isPlayerAction.Subscribe(delegate (bool value)
            {
                if (!(actScene == null))
                {
                    if (!value)
                    {
                        actScene.actionChangeUI.Remove(ActionChangeUI.ActionType.CharaEvent);
                    }
                    else if (!heroine.isTeacher)
                    {
                        actScene.actionChangeUI.Set(ActionChangeUI.ActionType.CharaEvent);
                    }
                }
            }).AddTo(disposables);
            GameObject ag = _actionIcon.gameObject;
            Transform at = _actionIcon.transform;
            (from active in OnActiveChangeObservable.TakeUntilDestroy(actScene)
             where !active
             select active).Subscribe(delegate (bool active)
         {
             if (actScene.Player.actionTarget == this)
             {
                 actScene.Player.actionTarget = null;
             }
             ag.SetActive(active);
         }).AddTo(disposables);
            ag.OnEnableAsObservable().Subscribe(delegate
            {
                if (heroine.isTeacher)
                {
                    _actionIcon.sprite = null;
                    _spritesAnim.isEnable = true;
                    _spritesAnim.Play(0);
                }
                else
                {
                    _spritesAnim.isEnable = false;
                    _actionIcon.sprite = _sprites[1];
                }
            });
            if (isCharaLoad)
            {
                _isPlayerAction.Subscribe(delegate (bool isOn)
                {
                    ag.SetActiveIfDifferent(isOn);
                }).AddTo(disposables);
                (from _ in this.UpdateAsObservable()
                 where ag.activeSelf
                 select _).Subscribe(delegate
             {
                 Vector3 headPos = HeadPos;
                 headPos.y += 0.5f;
                 at.position = headPos;
             });
            }
            else
            {
                OnActiveChangeObservable.Subscribe(delegate (bool active)
                {
                    ag.SetActiveIfDifferent(active);
                }).AddTo(disposables);
            }
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where isActive
             select _).Subscribe(delegate
         {
             LookForTarget(_isPlayerAction.Value ? actScene.Player : null);
         });
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                _obstacle.enabled = active && _isCharaLoad;
            }).AddTo(disposables);
            yield return new WaitWhile(() => miniMapIcon.iconItem == null);
            miniMapIcon.iconItem.text = charaData.Name;
            Sprites minimapIconSprites = GetComponents<Sprites>()[1];
            (from _ in this.UpdateAsObservable()
             where heroine != null
             select !heroine.isTeacher ? 1 : 0).DistinctUntilChanged().Subscribe(delegate (int index)
         {
             miniMapIcon.iconItem.SetIcon(minimapIconSprites[index]);
         });
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                miniMapIcon.SetVisible(active);
            }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                speak.Close();
            }).AddTo(disposables);
            if (chaCtrl != null)
            {
                actScene.VisibleList.Add(chaCtrl);
            }
            motionVoice.Init(heroine);
            (from active in OnActiveChangeObservable
             where !active
             select active into _
             where Singleton<Voice>.IsInstance()
             select _).Subscribe(delegate
         {
             Singleton<Voice>.Instance.Stop(Head);
         }).AddTo(disposables);
            (from _ in this.UpdateAsObservable()
             where isActive
             where !Singleton<Game>.Instance.IsRegulate(true)
             where !Program.isADVProcessing
             select _).Subscribe(delegate
         {
             motionVoice.Proc();
         });
            initialized = true;
        }

        protected override void OnDestroy()
        {
            base.OnDestroy();
            if (actScene != null && actScene.VisibleList != null && chaCtrl != null)
            {
                actScene.VisibleList.Remove(chaCtrl);
            }
        }
    }
}
