using ActionGame.Chara.Mover;
using ActionGame.Place;
using ADV;
using Illusion;
using Illusion.Component;
using Illusion.Extensions;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;

namespace ActionGame.Chara
{
    public sealed class NPC : Base, IHitable
    {
        public enum ActIcon
        {
            NoActiveTalkOK = -2,
            NoActive,
            TalkOK,
            Sleep,
            PleaseTalk,
            PleaseTalkActive,
            PleaseH,
            PleaseHActive,
            Anger
        }

        public const string TalkState = "talk";

        public const string ListenState = "talk_uke";

        private const string height1 = "height1";

        private const string breast = "Breast";

        private const string breast1 = "Breast1";

        public const string HesitantlyWaitState = "hesitantly";

        private static IDisposable summoningDisposable;

        private static IDisposable summoningTimeOutDisposable;

        private static bool isFailed;

        private const float summoningTimeLimit = 60f;

        private bool _isPause;

        private BoolReactiveProperty _isPlayerAction = new BoolReactiveProperty();

        [SerializeField]
        private SpritesAnim _spritesAnim;

        [SerializeField]
        private float _inSightDistance = 10f;

        [SerializeField]
        private TriggerEnterExitEvent sightArea;

        private bool isPhysicsInSight;

        [SerializeField]
        private bl_MiniMapItem miniMapIcon;

        [SerializeField]
        private SpeakChara _speak;

        [SerializeField]
        private TriggerEnterExitEvent noticeArea;

        private bool _isDefenseSpeak;

        private HashSet<Area> _placeAreaList = new HashSet<Area>();

        private MotionVoice motionVoice = new MotionVoice();

        private BoolReactiveProperty _isLesH = new BoolReactiveProperty(false);

        public override bool isAction
        {
            get
            {
                ActIcon actIconNo = this.actIconNo;
                return actIconNo != ActIcon.NoActive && actIconNo != ActIcon.Sleep;
            }
        }

        public bool isPause
        {
            get
            {
                return _isPause;
            }
            set
            {
                _isPause = value;
            }
        }

        public bool isPlayerAction
        {
            get
            {
                return _isPlayerAction.Value;
            }
            set
            {
                _isPlayerAction.Value = value;
            }
        }

        public bool isEasyPlace
        {
            get
            {
                if (actScene.Map.Info.isOutdoors)
                {
                    return true;
                }
                float radius = 1f;
                return otherPeopleList.Any(p => (p.position - actScene.Player.position).sqrMagnitude < radius * radius);
            }
        }

        public bool isOtherPeople
        {
            get
            {
                return otherPeopleList.Any();
            }
        }

        public List<Base> otherPeopleList
        {
            get
            {
                List<Base> list = new List<Base>();
                foreach (NPC item in from p in actScene.npcList
                                     where p != this
                                     where p.mapNo == mapNo
                                     select p)
                {
                    list.Add(item);
                }
                Fix fixChara = actScene.fixChara;
                if (fixChara != null && fixChara.isCharaLoad && fixChara.mapNo == mapNo)
                {
                    list.Add(fixChara);
                }
                return list;
            }
        }

        public override bool isBodyForTarget
        {
            get
            {
                if (AI.isPatrol)
                {
                    return false;
                }
                return base.isBodyForTarget;
            }
        }

        public SpeakChara speak
        {
            get
            {
                return _speak;
            }
        }

        public Vector3 calcPosition
        {
            get;
            set;
        }

        public UnityEngine.AI.NavMeshAgent agent
        {
            get;
            private set;
        }

        public ActionGame.Chara.Backup.NavMeshAgent bkAgent
        {
            get;
            private set;
        }

        public override Vector3 position
        {
            get
            {
                return cachedTransform.localPosition;
            }
            set
            {
                Vector3 vector3 = cachedTransform.localPosition = calcPosition = value;
                agent.nextPosition = value;
            }
        }

        public AI AI
        {
            get;
            private set;
        }

        public Vector3[] corners
        {
            get;
            set;
        }

        public ActIcon actIconNo
        {
            get
            {
                ActIcon actIcon = ActIcon.NoActive;
                switch (AI.actionNo)
                {
                    case -1:
                    case 0:
                    case 1:
                    case 2:
                    case 3:
                    case 4:
                    case 6:
                    case 7:
                    case 9:
                    case 10:
                    case 11:
                    case 12:
                    case 13:
                    case 14:
                    case 15:
                    case 16:
                    case 17:
                    case 18:
                    case 19:
                    case 21:
                    case 25:
                        actIcon = ActIcon.TalkOK;
                        break;
                    case 22:
                        actIcon = isArrival ? ActIcon.Sleep : ActIcon.TalkOK;
                        break;
                    case 5:
                    case 8:
                    case 23:
                    case 24:
                        actIcon = AI.isArrival ? ActIcon.NoActive : ActIcon.TalkOK;
                        break;
                    case 26:
                        actIcon = isArrival && motion.state == wpData.motion.motion ? ActIcon.NoActive : ActIcon.TalkOK;
                        break;
                    case 27:
                        actIcon = isArrival ? ActIcon.NoActive : ActIcon.TalkOK;
                        break;
                    case 28:
                        actIcon = _isPlayerAction.Value ? ActIcon.PleaseTalkActive : ActIcon.PleaseTalk;
                        break;
                    case 29:
                        actIcon = _isPlayerAction.Value ? ActIcon.PleaseHActive : ActIcon.PleaseH;
                        break;
                    case 30:
                        actIcon = ActIcon.NoActiveTalkOK;
                        break;
                }
                if (actIcon != ActIcon.NoActive && actIcon != ActIcon.Sleep)
                {
                    if (motion.state.CompareParts("sleep", true))
                    {
                        actIcon = ActIcon.Sleep;
                    }
                    else if (heroine.isAnger)
                    {
                        actIcon = ActIcon.Anger;
                    }
                }
                return actIcon;
            }
        }

        public override bool isLookForTarget
        {
            get
            {
                if (heroine.isAnger || AI.isArrival && AI.ActionNoCheck(4, 27))
                {
                    return false;
                }
                return base.isLookForTarget;
            }
        }

        public bool isOnanism
        {
            get
            {
                return AI.isArrival && AI.actionNo == 4;
            }
        }

        public bool isOnanismInSight
        {
            get;
            private set;
        }

        public bool isLesbian
        {
            get
            {
                return AI.actionNo == 27 && isArrival;
            }
        }

        public bool isDefenseSpeak
        {
            get
            {
                return _isDefenseSpeak;
            }
            set
            {
                _isDefenseSpeak = value;
            }
        }

        public bool isSightInPlayer
        {
            get
            {
                if (!isPhysicsInSight && !sightArea.isHit)
                {
                    return false;
                }
                RaycastHit[] array = InSightHits();
                return array.Any() && array[0].collider.CompareTag("Player");
            }
        }

        public bool isNotice
        {
            get
            {
                return noticeArea.isHit;
            }
        }

        public string[] PlaceNames
        {
            get
            {
                return _placeAreaList.Where(p => p != null).SelectMany(p => p.names).Concat(new string[1]
                {
                    actScene.Map.Info.MapName
                })
                    .Distinct()
                    .ToArray();
            }
        }

        public bool isLesH
        {
            get
            {
                return _isLesH.Value;
            }
        }

        public bool isTalking
        {
            get
            {
                return new string[3]
                {
                    "talk",
                    "talk_uke",
                    "hesitantly"
                }.Contains(motion.state);
            }
        }

        public static NPC Create(Transform parent, SaveData.Heroine heroine)
        {
            GameObject asset = AssetBundleManager.LoadAsset("action/chara.unity3d", "NPC", typeof(GameObject)).GetAsset<GameObject>();
            NPC component = Instantiate(asset, parent, false).GetComponent<NPC>();
            component.charaData = heroine;
            component.name = GetObjectName(heroine);
            component.gameObject.SetActive(true);
            return component;
        }

        public static NPC Summon(SaveData.Heroine heroine, ActionScene actScene, int? mapNo = null)
        {
            Action<NPC> summoning = delegate (NPC npc)
            {
                isFailed = false;
                if (summoningDisposable != null)
                {
                    summoningDisposable.Dispose();
                }
                if (summoningTimeOutDisposable != null)
                {
                    summoningTimeOutDisposable.Dispose();
                }
                actScene.actionChangeUI.Set(ActionChangeUI.ActionType.SummonEvent);
                summoningTimeOutDisposable = (from _ in npc.UpdateAsObservable()
                                              where false
                                              select _).Timeout(TimeSpan.FromSeconds(60.0)).Catch((TimeoutException ex) => Observable.Empty<Unit>()).Take(1)
                    .Subscribe(delegate
                    {
                    }, (Action)delegate
                    {
                        if (actScene != null && actScene.Player.isActionNow)
                        {
                            npc.AI.NextActionNoTarget();
                            actScene.Player.isActionNow = false;
                        }
                        isFailed = true;
                    });
                summoningDisposable = npc.UpdateAsObservable().TakeUntilDestroy(actScene).TakeWhile(_ => npc.AI.actionNo == 24 && !isFailed)
                    .Subscribe(delegate
                    {
                        actScene.ShortcutKeyRegulate(false);
                        Vector3 position = npc.position;
                        actScene.CameraState.SetAngle(Quaternion.LookRotation(position - actScene.Player.position).eulerAngles);
                    }, (Action)delegate
                    {
                        if (actScene != null)
                        {
                            if (!isFailed && summoningTimeOutDisposable != null)
                            {
                                summoningTimeOutDisposable.Dispose();
                            }
                            actScene.actionChangeUI.Remove(ActionChangeUI.ActionType.SummonEvent);
                            actScene.ShortcutKeyRegulate(true);
                        }
                    });
            };
            actScene.Player.ChaserEnd();
            actScene.Player.isActionNow = true;
            NPC target = heroine.charaBase as NPC;
            if (target == null)
            {
                target = Create(actScene.transform, heroine);
                target.gameObject.SetActive(true);
                target.LoadStart(true);
                actScene.npcList.Add(target);
                Observable.FromCoroutine(_ => new WaitUntil(() => target.initialized)).TakeUntilDestroy(target).Subscribe(delegate
                {
                    target.isPopOK = true;
                    target.ItemClear();
                    target.AI.FirstAction();
                    actScene.Map.PlayerMapNearWarp(target);
                    target.ReStart();
                    target.AI.TogetherAction(mapNo);
                    target.AI.GotoTarget(actScene.Player);
                    summoning(target);
                });
            }
            else
            {
                if (!target.initialized || target.AI.result == null || target.AI.actionNo == 24)
                {
                    return target;
                }
                if (target.mapNo != actScene.Player.mapNo)
                {
                    actScene.Map.PlayerMapNearWarp(target);
                }
                target.AI.TogetherAction(mapNo);
                target.AI.GotoTarget(actScene.Player);
                summoning(target);
            }
            return target;
        }

        private static string GetObjectName(SaveData.Heroine heroine)
        {
            return string.Format("NPC({0})_{1}_{2}", heroine.FixCharaIDOrPersonality, Cycle.GetClassRoomName(heroine.schoolClass), heroine.schoolClassIndex);
        }

        public override void PlayAnimation()
        {
            if (AI.actionNo == 27 && AI.isArrival && isActive)
            {
                if (actScene != null && actScene.Player != null && actScene.Player.isLesMotionPlay)
                {
                    if (AI.target != null && AI.target.isArrival && otherPeopleList.Count == 1)
                    {
                        chaCtrl.hideMoz = false;
                        chaCtrl.ChangeToiletStateLowPoly();
                        wpData = AI.target.wpData;
                        Vector3 pos = wpData.wp.position + wpData.motion.offsetPos;
                        Vector3 ang = wpData.wp.angle + wpData.motion.offsetAngle;
                        SetPositionAndRotation(pos, ang);
                        WaitPoint.Parameter.Motion motion = wpData.motion;
                        base.motion.state = motion.motion + "_sub";
                        state = motion.state;
                        ItemClear();
                        motion.ItemSet(itemObjList, chaCtrl);
                        SetParameterHeightAnimation();
                        LesParam();
                        animator.SetFloat("height1", AI.target.Height);
                        animator.SetFloat("Breast1", AI.target.BustSize);
                        base.motion.normalizedTime = 0f;
                        base.motion.Play(chaCtrl.animBody);
                        return;
                    }
                }
                else if (isArrival)
                {
                    AI.TimeEnd();
                }
            }
            ItemClear();
            ResetKind();
            if (isActive && isArrival && AI.target == null)
            {
                bool isNormalMotion = true;
                NPC target = null;
                if (AI.actionNo == 26)
                {
                    isNormalMotion = false;
                    if (LesMotion(out target))
                    {
                        return;
                    }
                }
                PlayAnimationItemKindSet(isNormalMotion);
                if (AI.actionNo == 4 && motion.state == "Masturbation4")
                {
                    foreach (GameObject itemObj in itemObjList)
                    {
                        if (itemObj.name == "p_item_hardle")
                        {
                            Animator component = itemObj.GetComponent<Animator>();
                            if (component != null)
                            {
                                component.SetFloat(hashHeight, Height);
                            }
                        }
                    }
                }
                if (target != null && isBodyForTarget)
                {
                    motion.state = "talk_uke";
                }
            }
            else
            {
                move.AnimeUpdate();
                state = 0;
            }
            SetParameterHeightAnimation();
            if (isTalking)
            {
                ItemClear();
                motion.normalizedTime = 0f;
            }
            else
            {
                motion.normalizedTime = Mathf.Min(UnityEngine.Random.value, 0.9f);
            }
            motion.Play(chaCtrl.animBody);
            UpdateMotionSpeed();
        }

        public override void LoadAnimator()
        {
            CharaInfo.Param param = actScene.charaInfoDic[charaData.personality];
            motion.bundle = param.MotionBundle;
            motion.asset = param.MotionAsset;
            motion.LoadAnimator(animator);
        }

        public override void ChangeNowCoordinate()
        {
            chaCtrl.ChangeCoordinateTypeAndReload((ChaFileDefine.CoordinateType)heroine.coordinates.SafeGet(0));
        }

        public override void Replace(SaveData.CharaData charaData)
        {
            base.Replace(charaData);
            name = GetObjectName(heroine);
            miniMapIcon.iconItem.text = charaData.Name;
            motionVoice.Init(heroine);
        }

        public void ReStart()
        {
            if (heroine.coordinates.SafeGet(0) != chaCtrl.fileStatus.coordinateType)
            {
                ChangeNowCoordinate();
            }
            SetActive(actScene.Map.IsPop(this));
            AI.Start();
        }

        public void Pause(bool isPause)
        {
            if (isPause)
            {
                if (AI.isRunning)
                {
                    _isPause = true;
                    AI.Pause();
                }
            }
            else if (_isPause)
            {
                AI.Start();
                _isPause = false;
            }
        }

        public void Stop()
        {
            SetActive(false);
            AI.Reset(true);
            move.animParamSpeed = 0f;
        }

        public void SynchroCoordinate(bool isRemove)
        {
            if (!(chaCtrl == null) && Singleton<Character>.IsInstance())
            {
                int nowCoordinate = heroine.NowCoordinate;
                int coordinateType = chaCtrl.fileStatus.coordinateType;
                if (coordinateType != nowCoordinate)
                {
                    Singleton<Character>.Instance.enableCharaLoadGCClear = false;
                    chaCtrl.ChangeCoordinateTypeAndReload((ChaFileDefine.CoordinateType)nowCoordinate);
                    Singleton<Character>.Instance.enableCharaLoadGCClear = true;
                }
                if (isRemove)
                {
                    chaCtrl.RandomChangeOfClothesLowPoly(heroine.lewdness);
                }
            }
        }

        public void Backup(UnityEngine.AI.NavMeshAgent agent)
        {
            bkAgent = new ActionGame.Chara.Backup.NavMeshAgent(agent);
        }

        void IHitable.Enter(Area area)
        {
            _placeAreaList.Add(area);
        }

        void IHitable.Exit(Area area)
        {
            _placeAreaList.Remove(area);
        }

        public void Speak(string bundleName, string assetName)
        {
            if (bundleName != null)
            {
                if (speak.isOpen)
                {
                    speak.Close();
                }
                int result;
                if (int.TryParse(assetName, out result))
                {
                    heroine.talkEvent.Add(result);
                }
                speak.bundleName = bundleName;
                speak.assetName = assetName;
                speak.Open();
            }
        }

        public void UpdateMotionSpeed()
        {
            if (!animator.gameObject.activeInHierarchy)
            {
                return;
            }
            float value;
            if (heroine.motionSpeed.TryGetValue(motion.state, out value))
            {
                bool flag = Singleton<Manager.Communication>.IsInstance();
                if (flag && Singleton<Manager.Communication>.Instance.CheckMoveMotion(motion.state, heroine.personality))
                {
                    float shapeBodyValue = heroine.chaCtrl.GetShapeBodyValue(0);
                    value *= !(shapeBodyValue >= 0.5f) ? Mathf.Lerp(0.8f, 1f, Mathf.InverseLerp(0f, 0.5f, shapeBodyValue)) : Mathf.Lerp(1f, 1.2f, Mathf.InverseLerp(0.5f, 1f, shapeBodyValue));
                }
                animator.SetFloat("MotionSpeed", value);
                if (flag && Singleton<Manager.Communication>.Instance.CheckChangeMoveSpeed(motion.state, heroine.personality) && heroine.motionSpeed.TryGetValue(motion.state, out value))
                {
                    float shapeBodyValue2 = chaCtrl.GetShapeBodyValue(0);
                    value *= !(shapeBodyValue2 >= 0.5f) ? Mathf.Lerp(0.8f, 1f, Mathf.InverseLerp(0f, 0.5f, shapeBodyValue2)) : Mathf.Lerp(1f, 1.2f, Mathf.InverseLerp(0.5f, 1f, shapeBodyValue2));
                    AI.speeder.walkRev = value;
                    AI.speeder.runRev = value;
                }
            }
        }

        public int GetOnanismID(bool isMotion, bool isCheck = true)
        {
            if (isCheck && !isOnanism)
            {
                return -1;
            }
            switch (isMotion ? motion.state : GetOnanismMotion())
            {
                case "Masturbation":
                    return 1012;
                case "Masturbation2":
                    return 1011;
                case "Masturbation3":
                    return 1010;
                case "Masturbation4":
                    return 1013;
                default:
                    return 1012;
            }
        }

        public string GetOnanismMotion()
        {
            switch (state)
            {
                case 1:
                    return "Masturbation2";
                case 6:
                    return "Masturbation3";
                case 7:
                    return "Masturbation4";
                default:
                    return "Masturbation";
            }
        }

        public void LesHEndForce()
        {
            _isLesH.Value = false;
        }

        public int GetLesbianID(bool isMotion, bool isCheck = true)
        {
            if (isCheck && !isLesbian)
            {
                return -1;
            }
            string text = isMotion ? motion.state : GetLesbianMotion();
            switch (text.Replace("_sub", string.Empty))
            {
                case "Lesbian":
                    return 1100;
                case "Lesbian2":
                    return 1101;
                case "Lesbian3":
                    return 1102;
                default:
                    return 1100;
            }
        }

        public string GetLesbianMotion()
        {
            switch (state)
            {
                case 1:
                    return "Lesbian2";
                case 2:
                    return "Lesbian";
                default:
                    return "Lesbian3";
            }
        }

        public void SetTalker()
        {
            motion.state = "talk";
            base.SetWaitPoint(null);
            corners = null;
        }

        public void SetListener()
        {
            motion.state = "talk_uke";
            if (AI.actionNo == 26)
            {
                PlayAnimation();
            }
            else
            {
                base.SetWaitPoint(null);
            }
            corners = null;
        }

        public void SetHesitantly()
        {
            motion.state = "hesitantly";
            base.SetWaitPoint(null);
            corners = null;
        }

        public override void SetWaitPoint(WaitPointData wpData)
        {
            base.SetWaitPoint(wpData);
            corners = null;
            AI.isArrival = wpData != null;
            if (isActive)
            {
                if (wpData != null && wpData.wp.isNavMeshOffsetPoint)
                {
                    agent.enabled = false;
                }
                else
                {
                    agent.enabled = true;
                }
            }
        }

        public void AgentVelocityMoveAnimeUpdate()
        {
            float animParamSpeed = move.animParamSpeed;
            move.animParamSpeed = agent.velocity.magnitude;
            if (Mathf.Approximately(animParamSpeed, move.animParamSpeed))
            {
                move.MoveUpdate();
            }
        }

        protected override void Awake()
        {
            base.Awake();
            agent = GetComponent<UnityEngine.AI.NavMeshAgent>();
            move = GetComponent<NPCMover>();
            Backup(agent);
            AI = new AI(this);
        }

        protected override IEnumerator Start()
        {
            yield return new WaitWhile(() => actScene.Player == null);
            yield return new WaitWhile(() => actScene.Player.chaCtrl == null);
            yield return new WaitUntil(() => actScene.Player.initialized);
            yield return StartCoroutine(base.Start());
            _mapNo.Subscribe(delegate
            {
                corners = null;
            });
            ActionChangeUI changeUI = actScene.actionChangeUI;
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where isActive
             select _).Subscribe(delegate
         {
             if (isArrival && AI.actionNo == 4 && motion.state == "Masturbation4")
             {
                 chaCtrl.ChangeLookEyesPtn(0);
                 chaCtrl.ChangeLookNeckPtn(0);
             }
             else
             {
                 Base @base;
                 if (_isPlayerAction.Value && heroine.relation >= 1)
                 {
                     @base = actScene.Player;
                 }
                 else
                 {
                     @base = AI.isArrival ? AI.target : null;
                     @base = @base ?? AI.TargetReceiver(p => p.isArrival);
                 }
                 bool isForce = false;
                 if (@base is NPC)
                 {
                     isForce = AI.actionNo == 26;
                 }
                 LookForTarget(@base, isForce);
             }
         });
            GameObject ag = _actionIcon.gameObject;
            Transform at = _actionIcon.transform;
            (from active in OnActiveChangeObservable.TakeUntilDestroy(actScene)
             where !active
             select active).Subscribe(delegate (bool active)
         {
             if (actScene.Player.actionTarget == this)
             {
                 actScene.Player.actionTarget = null;
                 if (isOnanismInSight)
                 {
                     changeUI.Remove(ActionChangeUI.ActionType.Onanism);
                 }
             }
             ag.SetActiveIfDifferent(active);
         }).AddTo(disposables);
            _isPlayerAction.Subscribe(delegate (bool isOn)
            {
                ag.SetActiveIfDifferent(isOn);
                if (isOnanism && isOn)
                {
                    changeUI.Set(ActionChangeUI.ActionType.Onanism);
                }
                if (!isOn)
                {
                    changeUI.Remove(ActionChangeUI.ActionType.Onanism);
                }
            }).AddTo(disposables);
            (from active in OnActiveChangeObservable
             where !active
             where AI.isPlayerResWaiting
             select active).Subscribe(delegate
         {
             AI.TimeEnd();
         });
            (from _ in this.UpdateAsObservable()
             where isActive && !_isPlayerAction.Value
             select AI.isPlayerResWaiting).Subscribe(delegate (bool isOn)
         {
             ag.SetActiveIfDifferent(isOn);
         });
            (from _ in this.UpdateAsObservable()
             where ag.activeSelf
             select _).Subscribe(delegate
         {
             Vector3 headPos = HeadPos;
             headPos.y += 0.5f;
             at.position = headPos;
             int actIconNo = (int)this.actIconNo;
             int num = _spritesAnim.names.Length;
             if ((uint)actIconNo < num)
             {
                 _spritesAnim.isEnable = true;
                 _spritesAnim.Play(actIconNo);
             }
             else
             {
                 _spritesAnim.isEnable = false;
                 _actionIcon.sprite = actIconNo >= 0 ? _sprites[actIconNo - num] : null;
             }
         });
            _isPlayerAction.Subscribe(delegate (bool value)
            {
                if (AI.isTalkSuccess && !(AI.TargetReceiver(p => p.isArrival) != null))
                {
                    if (!value || heroine.isAnger || AI.isPatrol || actScene.Map.Info.isWarning || heroine.talkTime <= 0 && Manager.Config.AddData.TalkTimeNoneWalkStop)
                    {
                        bool isStop = false;
                        if (AI.isPlayerResWaiting)
                        {
                            isStop = true;
                        }
                        move.isStop = isStop;
                        AI.Start();
                    }
                    else if (heroine.relation >= 1)
                    {
                        move.isStop = true;
                        AI.Pause();
                    }
                }
            });
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                if (active)
                {
                    if (isArrival && wpData.wp.isNavMeshOffsetPoint)
                    {
                        agent.enabled = false;
                    }
                    else
                    {
                        agent.enabled = true;
                    }
                }
                else
                {
                    agent.enabled = false;
                }
            }).AddTo(disposables);
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where isActive
             where agent.enabled
             where !agent.isOnNavMesh
             where !actScene.Map.isMapLoading
             select _).Subscribe(delegate
         {
             agent.enabled = false;
             agent.enabled = true;
             AI.isOnNavMeshReset = true;
         });
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                if (active)
                {
                    position = calcPosition;
                }
                else
                {
                    calcPosition = position;
                }
            }).AddTo(disposables);
            OnActiveChangeObservable.Subscribe(delegate
            {
                SynchroCoordinate(false);
            }).AddTo(disposables);
            yield return new WaitWhile(() => miniMapIcon.iconItem == null);
            miniMapIcon.iconItem.text = charaData.Name;
            Sprites minimapIconSprites = GetComponents<Sprites>()[1];
            (from _ in this.UpdateAsObservable()
             where heroine != null
             select heroine.relation into relation
             select Mathf.Max(0, relation)).DistinctUntilChanged().Subscribe(delegate (int index)
         {
             miniMapIcon.iconItem.SetIcon(minimapIconSprites[index]);
         });
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                miniMapIcon.SetVisible(active);
            }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                speak.Close();
            }).AddTo(disposables);
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                sightArea.enabled = active;
            }).AddTo(disposables);
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                noticeArea.enabled = active;
            }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                isPhysicsInSight = false;
            }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                isOnanismInSight = false;
            }).AddTo(disposables);
            (from _ in OnActiveChangeObservable.TakeUntilDestroy(actScene)
             where isOnanism || isLesbian
             select _).Subscribe(delegate (bool active)
         {
             actScene.OtherAIPause(this, active);
         }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                _isDefenseSpeak = false;
            }).AddTo(disposables);
            actScene.VisibleList.Add(chaCtrl);
            Transform sightAreaTransform = sightArea.transform;
            (from _ in sightArea.UpdateAsObservable()
             where sightArea.enabled
             select _).Subscribe(delegate
         {
             Transform head = Head;
             if (head != null)
             {
                 Vector3 position = head.position;
                 float y = position.y;
                 Vector3 localScale = sightAreaTransform.localScale;
                 position.y = y - localScale.y;
                 sightAreaTransform.SetPositionAndRotation(position, head.rotation);
             }
             else
             {
                 sightAreaTransform.SetPositionAndRotation(Vector3.zero, rotation);
             }
         });
            motionVoice.Init(heroine);
            (from active in OnActiveChangeObservable
             where !active
             select active into _
             where Singleton<Voice>.IsInstance()
             select _).Subscribe(delegate
         {
             Singleton<Voice>.Instance.Stop(Head);
         }).AddTo(disposables);
            (from _ in this.UpdateAsObservable()
             where isActive
             where !Singleton<Game>.Instance.IsRegulate(true)
             where !Program.isADVProcessing
             where !speak.isOpen
             select _).Subscribe(delegate
         {
             motionVoice.Proc();
         });
            (from _ in this.UpdateAsObservable()
             where isActive
             where AI.isArrival
             where !actScene.isEventNow
             select _).Subscribe(delegate
         {
             switch (AI.actionNo)
             {
                 case 4:
                     if (isOtherPeople)
                     {
                         changeUI.Remove(ActionChangeUI.ActionType.Onanism);
                         AI.NextActionNoTarget();
                     }
                     else if (isSightInPlayer)
                     {
                         isPhysicsInSight = true;
                         if (!isOnanismInSight)
                         {
                             isOnanismInSight = true;
                             if (heroine.HExperience <= SaveData.Heroine.HExperienceKind.不慣れ || heroine.lewdness < 80)
                             {
                                 motionVoice.actionKind = MotionVoice.ActionKind.masturbation_found;
                                 changeUI.Remove(ActionChangeUI.ActionType.Onanism);
                                 AI.EscapeAction(false);
                             }
                         }
                     }
                     break;
                 case 26:
                     if (motion.state == wpData.motion.motion)
                     {
                         bool flag = false;
                         foreach (Base otherPeople in otherPeopleList)
                         {
                             if (otherPeople is Fix)
                             {
                                 flag = true;
                                 break;
                             }
                             NPC nPC = otherPeople as NPC;
                             if (nPC.AI.actionNo != 27 || nPC.AI.target != this)
                             {
                                 flag = true;
                                 break;
                             }
                         }
                         if (flag)
                         {
                             NPC nPC2 = AI.TargetReceiver(p => p.actionNo == 27);
                             if (nPC2 != null)
                             {
                                 nPC2.AI.SetLesbianDesire(AI.LesbianResult.Success);
                                 nPC2.AI.NextActionNoTarget();
                             }
                             AI.SetLesbianDesire(AI.LesbianResult.Success);
                             AI.NextActionNoTarget();
                         }
                         else if (isSightInPlayer)
                         {
                             isPhysicsInSight = true;
                             NPC nPC3 = AI.TargetReceiver(p => p.actionNo == 27);
                             if (nPC3 != null)
                             {
                                 nPC3.AI.EscapeAction(false);
                             }
                             AI.EscapeAction(false);
                         }
                     }
                     break;
                 case 27:
                     if (isArrival)
                     {
                         if (otherPeopleList.Any(p => p != AI.target))
                         {
                             AI aI = (AI.target as NPC).AI;
                             aI.SetLesbianDesire(AI.LesbianResult.Success);
                             AI.SetLesbianDesire(AI.LesbianResult.Success);
                             aI.NextActionNoTarget();
                             AI.NextActionNoTarget();
                         }
                         else if (isSightInPlayer)
                         {
                             isPhysicsInSight = true;
                             (AI.target as NPC).AI.EscapeAction(false);
                             AI.EscapeAction(false);
                         }
                     }
                     break;
             }
         });
            int defenseNo = 25;
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where !actScene.isEventNow
             where actScene.Map.Info.isWarning
             where !_isDefenseSpeak
             where isActive
             where !AI.isArrival || AI.actionNo != 4
             where AI.actionNo != 26 || !isArrival || !(wpData.motion.motion == motion.state)
             where AI.actionNo != 27 || !isArrival
             where isAction
             where !AI.ActionNoCheck(20, 23, defenseNo)
             where !AI.isPlayerResWaiting
             select _).Subscribe(delegate
         {
             if (isSightInPlayer)
             {
                 isPhysicsInSight = true;
                 AI.ShyAction();
                 int encounterADVNo = GetEncounterADVNo();
                 Speak(Program.FindADVBundleFilePath(encounterADVNo, heroine), encounterADVNo.ToString());
                 _isDefenseSpeak = true;
             }
         });
            (from _ in this.UpdateAsObservable()
             where isActive
             where AI.actionNo == defenseNo
             select _).Subscribe(delegate
         {
             move.isStop = true;
             Utils.Math.TargetFor(cachedTransform, actScene.Player.cachedTransform);
         });
            (from active in OnActiveChangeObservable
             where !active
             select active into _
             where AI.actionNo == defenseNo
             select _).Subscribe(delegate
         {
             AI.NextActionNoTarget();
         }).AddTo(disposables);
            _isLesH.Subscribe(delegate (bool isOn)
            {
                if (isOn)
                {
                    changeUI.Set(ActionChangeUI.ActionType.Lesbian);
                }
                else
                {
                    changeUI.Remove(ActionChangeUI.ActionType.Lesbian);
                }
            });
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                _isLesH.Value = false;
            }).AddTo(disposables);
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where _isLesH.Value
             where ActionInput.isAction
             where !Singleton<Game>.Instance.IsRegulate(true)
             where !Program.isADVProcessing
             select _).Subscribe(delegate
         {
             actScene.SceneEvent(this);
         });
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(actScene)
             where isActive && isLesbian
             select Vector3.Distance(position, actScene.Player.position) < actScene.lesbianDistance).DistinctUntilChanged().Subscribe(delegate (bool isOn)
         {
             _isLesH.Value = isOn;
         });
            AI.ActionStart();
            initialized = true;
        }

        private void OnDrawGizmos()
        {
            if (!isActive || !(chaCtrl != null) || !(Head != null))
            {
                return;
            }
            if (isPhysicsInSight)
            {
                RaycastHit[] source = InSightHits();
                if (source.Any())
                {
                    Gizmos.DrawRay(HeadPos, (actScene.Player.HeadPos - HeadPos).normalized * _inSightDistance);
                }
            }
            if (AI.actionNo == 27 && isArrival)
            {
                Color cyan = Color.cyan;
                cyan.a = 0.5f;
                Gizmos.color = cyan;
                Gizmos.DrawSphere(position, actScene.lesbianDistance);
            }
        }

        protected override void OnDestroy()
        {
            base.OnDestroy();
            if (actScene != null && actScene.VisibleList != null)
            {
                actScene.VisibleList.Remove(chaCtrl);
            }
        }

        public int GetEncounterADVNo()
        {
            if (heroine.isAnger)
            {
                return 36;
            }
            switch (heroine.relation)
            {
                case 2:
                    if (isOtherPeople)
                    {
                        return 37;
                    }
                    switch (mapNo)
                    {
                        case 14:
                        case 15:
                        case 16:
                            return 40;
                        case 45:
                            return 39;
                        case 46:
                            return 38;
                        default:
                            return 37;
                    }
                case 1:
                    return 37;
                default:
                    return 35;
            }
        }

        private RaycastHit[] InSightHits()
        {
            Vector3 headPos = HeadPos;
            Vector3 headPos2 = actScene.Player.HeadPos;
            float num = Vector3.Dot(Head.forward, (headPos2 - headPos).normalized);
            if (num < 0f)
            {
                return new RaycastHit[0];
            }
            return (from p in Physics.RaycastAll(headPos, (headPos2 - headPos).normalized, _inSightDistance, 10240)
                    where p.collider.gameObject.layer == 11 || p.collider.CompareTag("Player")
                    orderby p.distance
                    select p).ToArray();
        }

        private bool LesMotion(out NPC target)
        {
            target = AI.TargetReceiver(p => p.actionNo == 27 && p.isArrival);
            if (target == null)
            {
                return false;
            }
            if (actScene == null || actScene.Player == null || !actScene.Player.isLesMotionPlay)
            {
                return false;
            }
            if (otherPeopleList.Count != 1)
            {
                return false;
            }
            Vector3 pos = wpData.wp.position + wpData.motion.offsetPos;
            Vector3 ang = wpData.wp.angle + wpData.motion.offsetAngle;
            SetPositionAndRotation(pos, ang);
            PlayAnimationItemKindSet(true);
            SetParameterHeightAnimation();
            LesParam();
            animator.SetFloat("height1", target.Height);
            animator.SetFloat("Breast1", target.BustSize);
            motion.normalizedTime = 0f;
            motion.Play(chaCtrl.animBody);
            chaCtrl.hideMoz = false;
            chaCtrl.ChangeToiletStateLowPoly();
            return true;
        }

        private void LesParam()
        {
            animator.SetFloat("Breast", BustSize);
            float weight = 0f;
            float weight2 = 0f;
            if (motion.state.IndexOf("Lesbian3") != -1)
            {
                if (motion.state.IndexOf("_sub") == -1)
                {
                    weight = 1f;
                    weight2 = 0f;
                }
                else
                {
                    weight = 0f;
                    weight2 = 1f;
                }
            }
            animator.SetLayerWeight(1, weight);
            animator.SetLayerWeight(2, weight2);
        }
    }
}
