using ActionGame.Chara.Mover;
using ActionGame.Point;
using Illusion.Component;
using Illusion.Game.Elements.EasyLoader;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.AI;

namespace ActionGame.Chara
{
    public class Player : Base
    {
        private bool _isActionNow;

        private ReactiveProperty<Base> _actionTarget = new ReactiveProperty<Base>();

        [SerializeField]
        private TriggerEnterExitEvent noticeArea;

        private bool _isChaseWarning;

        private HashSet<Component> _actionPointList = new HashSet<Component>();

        [SerializeField]
        private List<Item.Data> itemList = new List<Item.Data>();

        private float height;

        public override bool isAction
        {
            get
            {
                return !_isActionNow && actScene.isCursorLock;
            }
        }

        public bool isActionNow
        {
            get
            {
                return _isActionNow || !actScene.isCursorLock;
            }
            set
            {
                _isActionNow = value;
            }
        }

        public bool isActionNow_Origin
        {
            get
            {
                return _isActionNow;
            }
        }

        public Base actionTarget
        {
            get
            {
                return _actionTarget.Value;
            }
            set
            {
                _actionTarget.Value = value;
            }
        }

        public NPC chaser
        {
            get;
            private set;
        }

        public bool isChaseWarning
        {
            get
            {
                bool isChaseWarning = _isChaseWarning;
                _isChaseWarning = false;
                return isChaseWarning;
            }
        }

        public bool isActionPointHit
        {
            get
            {
                return isActionPointToPlayerHit || isActionPointToMapHit;
            }
        }

        public bool isActionPointToPlayerHit
        {
            get
            {
                return _actionPointList.Where(o => o != null).Any(o => o.GetComponent<PlayerActionPoint>() != null);
            }
        }

        public bool isActionPointToMapHit
        {
            get
            {
                return (from o in _actionPointList
                        where o != null
                        select o.GetComponent<ActionPoint>() into o
                        where o != null
                        select o).Any(o => o.isIconDraw);
            }
        }

        public HashSet<Component> actionPointList
        {
            get
            {
                return _actionPointList;
            }
        }

        public bool isLesMotionPlay
        {
            get;
            set;
        }

        public NavMeshAgent agent
        {
            get;
            private set;
        }

        public override Vector3 position
        {
            get
            {
                return cachedTransform.localPosition;
            }
            set
            {
                cachedTransform.localPosition = value;
                agent.nextPosition = value;
            }
        }

        public Vector3 fpsPos
        {
            get
            {
                Vector3 position = this.position;
                position.y += height;
                return position;
            }
        }

        public static Player Create(Transform parent, SaveData.Player player)
        {
            GameObject asset = AssetBundleManager.LoadAsset("action/chara.unity3d", "Player", typeof(GameObject)).GetAsset<GameObject>();
            Player component = Instantiate(asset, parent, false).GetComponent<Player>();
            component.charaData = player;
            component.name = asset.name;
            component.gameObject.SetActive(true);
            return component;
        }

        public override void Replace(SaveData.CharaData charaData)
        {
            base.Replace(charaData);
        }

        public override void PlayAnimation()
        {
            move.AnimeUpdate();
            motion.Play(animator);
        }

        public override void LoadAnimator()
        {
            motion.bundle = "action/animator/00.unity3d";
            motion.asset = "player";
            motion.LoadAnimator(animator);
        }

        public override void ChangeNowCoordinate()
        {
            ChaFileDefine.CoordinateType type = ChaFileDefine.CoordinateType.School01;
            if (player.changeClothesType < 0)
            {
                if (actScene.Cycle.nowWeek != Cycle.Week.Saturday)
                {
                    switch (actScene.Cycle.nowType)
                    {
                        case Cycle.Type.LunchTime:
                            switch (actScene.Cycle.GetNowLessones(charaData.schoolClass)[0])
                            {
                                case "グラウンド":
                                case "体育館":
                                    type = ChaFileDefine.CoordinateType.Gym;
                                    break;
                                case "プール":
                                    type = ChaFileDefine.CoordinateType.Swim;
                                    break;
                                default:
                                    type = ChaFileDefine.CoordinateType.School01;
                                    break;
                            }
                            break;
                        case Cycle.Type.StaffTime:
                            type = ChaFileDefine.CoordinateType.Club;
                            break;
                        case Cycle.Type.AfterSchool:
                            type = ChaFileDefine.CoordinateType.School02;
                            break;
                    }
                }
                else
                {
                    switch (actScene.Cycle.nowType)
                    {
                        case Cycle.Type.LunchTime:
                            type = ChaFileDefine.CoordinateType.Plain;
                            break;
                        case Cycle.Type.StaffTime:
                            type = ChaFileDefine.CoordinateType.Club;
                            break;
                        case Cycle.Type.AfterSchool:
                            type = ChaFileDefine.CoordinateType.Plain;
                            break;
                    }
                }
            }
            else
            {
                type = (ChaFileDefine.CoordinateType)player.changeClothesType;
            }
            chaCtrl.ChangeCoordinateTypeAndReload(type);
        }

        public void ChaserSet(NPC npc)
        {
            ChaserEnd();
            chaser = npc;
            npc.AI.Stop();
            npc.move.Change(new Chase(npc.move as NPCMover)
            {
                target = this
            });
            npc.AI.ChaseAction();
            npc.move.isStop = false;
        }

        public void ChaserEnd()
        {
            if (chaser != null)
            {
                chaser.move.Change(new None(chaser.move));
                chaser.AI.Start();
                chaser.AI.NextActionNoTarget();
            }
            chaser = null;
        }

        public bool isChaseCancel(int mapNo)
        {
            _isChaseWarning = false;
            if (chaser == null)
            {
                return false;
            }
            SaveData.Heroine heroine = chaser.heroine;
            if (heroine.relation <= 0)
            {
                _isChaseWarning = true;
                return true;
            }
            if (heroine.relation == 1)
            {
                List<Base> list = new List<Base>();
                foreach (NPC item in from p in actScene.npcList
                                     where p != chaser
                                     where p.mapNo == mapNo
                                     select p)
                {
                    list.Add(item);
                }
                Fix fixChara = actScene.fixChara;
                if (fixChara != null && fixChara.isCharaLoad && fixChara.mapNo == mapNo)
                {
                    list.Add(fixChara);
                }
                if (list.Any())
                {
                    _isChaseWarning = true;
                    return true;
                }
            }
            return false;
        }

        public void SetPhone()
        {
            ItemClear();
            string stateName = "call";
            motion.state = stateName;
            itemObjList.Add(itemList[0].Load(chaCtrl));
            motion.Play(animator);
            (from _ in this.UpdateAsObservable()
             where motion.state != stateName
             select _).Take(1).Subscribe(delegate
         {
             ItemClear();
         });
        }

        protected override void Awake()
        {
            base.Awake();
            agent = GetComponent<NavMeshAgent>();
            move = GetComponent<PlayerMover>();
        }

        protected override IEnumerator Start()
        {
            enabled = false;
            actScene.follow.followTarget = cachedTransform;
            yield return StartCoroutine(base.Start());
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                agent.enabled = active;
            }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => active).Subscribe(delegate
            {
                actScene.follow.Sync();
            }).AddTo(disposables);
            OnActiveChangeObservable.Subscribe(delegate (bool active)
            {
                noticeArea.enabled = active;
            }).AddTo(disposables);
            (from active in OnActiveChangeObservable.TakeUntilDestroy(actScene)
             where !active
             select active).Subscribe(delegate
         {
             actScene.actionChangeUI.Set(ActionChangeUI.ActionType.None);
         }).AddTo(disposables);
            OnActiveChangeObservable.Where(active => !active).Subscribe(delegate
            {
                _actionPointList.Clear();
            }).AddTo(disposables);
            noticeArea.onTriggerEnter += delegate (Collider collider)
            {
                Base component3 = collider.GetComponent<Base>();
                _actionTarget.Value = component3;
            };
            noticeArea.onTriggerExit += delegate (Collider collider)
            {
                Base component2 = collider.GetComponent<Base>();
                if (_actionTarget.Value == component2)
                {
                    _actionTarget.Value = null;
                }
                if (isActive && _actionTarget.Value == null && noticeArea.hitList.Any())
                {
                    noticeArea.hitList.RemoveWhere(p => p == null);
                    Base[] source2 = (from col in noticeArea.hitList
                                      select col.GetComponent<Base>() into p
                                      where p.isActive
                                      select p).ToArray();
                    _actionTarget.Value = source2.OrderBy(p => (p.position - this.position).sqrMagnitude).FirstOrDefault();
                }
            };
            _actionTarget.Scan(delegate (Base prev, Base current)
            {
                if (prev is NPC)
                {
                    (prev as NPC).isPlayerAction = false;
                }
                else if (prev is Fix)
                {
                    (prev as Fix).isPlayerAction = false;
                }
                return current;
            }).Subscribe(delegate (Base target)
            {
                LookForTarget(target);
                if (target != null)
                {
                    if (target is NPC)
                    {
                        (target as NPC).isPlayerAction = true;
                    }
                    else if (target is Fix)
                    {
                        (target as Fix).isPlayerAction = true;
                    }
                }
                if (actScene != null)
                {
                    actScene.paramUI.SetHeroine(!(target == null) ? target.heroine : null);
                }
            }).AddTo(disposables);
            this.UpdateAsObservable().Subscribe(delegate
            {
                if (!isActionNow)
                {
                    if (ActionInput.isCrouch)
                    {
                        move.agentSpeeder.mode = AgentSpeeder.Mode.Crouch;
                    }
                    else if (ActionInput.isWalk)
                    {
                        move.agentSpeeder.mode = AgentSpeeder.Mode.Walk;
                    }
                    else
                    {
                        move.agentSpeeder.mode = AgentSpeeder.Mode.Run;
                    }
                }
            });
            Vector3 position = Head.position;
            float y = position.y;
            Vector3 position2 = this.position;
            height = y - position2.y;
            (from _ in this.LateUpdateAsObservable()
             select move.agentSpeeder.mode == AgentSpeeder.Mode.Crouch).DistinctUntilChanged().Subscribe(delegate
         {
             Player player = this;
             Vector3 position3 = Head.position;
             float y2 = position3.y;
             Vector3 position4 = this.position;
             player.height = y2 - position4.y;
         });
            this.UpdateAsObservable().Subscribe(delegate
            {
                actScene.MiniMapTarget.SetPositionAndRotation(this.position, actScene.cameraTransform.rotation);
            });
            (from _ in this.UpdateAsObservable()
             where isActive
             where actScene.Map.no >= 0 && actScene.Map.Info.isWarning
             select _).Subscribe(delegate
         {
             Collider[] source = Physics.OverlapSphere(this.position, (baseCollider as CapsuleCollider).radius, 2048);
             foreach (Collider item in source.Where(p => p.name.IndexOf("DoorCloseEvent") != -1))
             {
                 if (!(item == null))
                 {
                     Door component = item.GetComponent<Door>();
                     if (component != null)
                     {
                         if (component.IsClose())
                         {
                             component.OpenForce();
                         }
                         break;
                     }
                 }
             }
         });
            enabled = true;
            initialized = true;
        }

        protected override void OnDestroy()
        {
            base.OnDestroy();
            if (actScene != null)
            {
                ChaserEnd();
            }
        }
    }
}
