using ActionGame.Chara;
using ActionGame.H;
using ActionGame.MapObject;
using ADV;
using DG.Tweening;
using Illusion.Extensions;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;

namespace ActionGame.Point
{
    public class ActionPoint : MonoBehaviour
    {
        public enum Type
        {
            なし,
            部活メニュー,
            部活レポート,
            時間割,
            覗きトイレ,
            覗きシャワー,
            特殊H,
            寝る,
            寝るv2,
            ダークネス
        }

        public int MapNo;

        [SerializeField]
        [Header("種類")]
        private Type type;

        [SerializeField]
        [Header("特殊Hタイプ(Hのカテゴリーのこと)")]
        private List<int> hType = new List<int>();

        [Header("特殊マップ読み込む？")]
        [SerializeField]
        private bool isLoadPeepLoom;

        [SerializeField]
        [Header("Hずらし位置")]
        private Vector3 hOffsetPos;

        [SerializeField]
        [Header("Hずらし回転")]
        private Vector3 hOffsetAngle;

        [SerializeField]
        private Collider col;

        [SerializeField]
        private Transform icon;

        [SerializeField]
        private VisibleChange visibleChange;

        [SerializeField]
        private List<GameObject> eventPointList = new List<GameObject>();

        [SerializeField]
        private float distance = -1f;

        private Transform _camTrans;

        private SpriteRenderer _render;

        private BoolReactiveProperty _isIconDraw;

        private static readonly string[] wakeUpWords = new string[1]
        {
            "「ふぁぁ……、すっきりした」"
        };

        private Player player;

        private bool executable = true;

        public Type category
        {
            get
            {
                return type;
            }
        }

        public List<int> HType
        {
            get
            {
                return hType;
            }
        }

        public bool IsLoadPeepLoom
        {
            get
            {
                return isLoadPeepLoom;
            }
        }

        public Vector3 HOffsetPos
        {
            get
            {
                return hOffsetPos;
            }
        }

        public Vector3 HOffsetAngle
        {
            get
            {
                return hOffsetAngle;
            }
        }

        private Transform camTrans
        {
            get
            {
                if (_camTrans == null && Camera.main != null)
                {
                    _camTrans = Camera.main.transform;
                }
                return _camTrans;
            }
        }

        public bool isIconDraw
        {
            get
            {
                return _isIconDraw.Value;
            }
            private set
            {
                _isIconDraw.Value = value;
            }
        }

        private bool isVisible
        {
            get
            {
                if (!Singleton<Game>.IsInstance())
                {
                    return false;
                }
                if (Singleton<Game>.Instance.IsRegulate(true))
                {
                    return false;
                }
                ActionScene actScene = Singleton<Game>.Instance.actScene;
                if (actScene == null)
                {
                    return false;
                }
                if (actScene.regulate != 0)
                {
                    return false;
                }
                if (actScene.AdvScene != null && actScene.AdvScene.gameObject.activeSelf)
                {
                    return false;
                }
                return true;
            }
        }

        private bool InitializeHitVisible()
        {
            switch (type)
            {
                default:
                    return false;
            }
        }

        private string[] InitializeHitTagNames()
        {
            switch (type)
            {
                case Type.なし:
                case Type.部活メニュー:
                case Type.部活レポート:
                case Type.時間割:
                case Type.覗きトイレ:
                case Type.覗きシャワー:
                case Type.特殊H:
                case Type.寝る:
                case Type.寝るv2:
                case Type.ダークネス:
                    return new string[1]
                    {
                    "Player"
                    };
                default:
                    return null;
            }
        }

        private bool VisibleIcon()
        {
            switch (type)
            {
                case Type.部活メニュー:
                    if (Singleton<Game>.Instance.actScene.npcList.Any(npc => npc.isActive && (npc.isOnanism || npc.isLesbian)))
                    {
                        return false;
                    }
                    break;
                case Type.覗きトイレ:
                case Type.覗きシャワー:
                    foreach (GameObject eventPoint in eventPointList)
                    {
                        Door component = eventPoint.GetComponent<Door>();
                        if (component != null && component.IsClose())
                        {
                            return true;
                        }
                    }
                    return false;
                case Type.特殊H:
                    {
                        if (player == null || player.chaser == null || player.chaser.heroine.hCount == 0)
                        {
                            return false;
                        }
                        if (hType.Contains(1002) && player.chaser.heroine.HExperience < SaveData.Heroine.HExperienceKind.慣れ)
                        {
                            return false;
                        }
                        BoxCollider boxCollider2 = col as BoxCollider;
                        if (boxCollider2 != null)
                        {
                            Transform transform2 = boxCollider2.transform;
                            if (Physics.OverlapBox(transform2.position, transform2.localScale * 0.5f, transform2.rotation, 8192).Any(col => col.CompareTag("NPC") && col.transform != player.chaser.cachedTransform))
                            {
                                return false;
                            }
                        }
                        return true;
                    }
                case Type.寝る:
                case Type.寝るv2:
                    {
                        if (player.chaser != null)
                        {
                            return false;
                        }
                        BoxCollider boxCollider3 = col as BoxCollider;
                        if (boxCollider3 == null)
                        {
                            return false;
                        }
                        Transform transform3 = boxCollider3.transform;
                        if (Physics.OverlapBox(transform3.position, transform3.localScale * 0.5f, transform3.rotation, 8192).Any(p => p.CompareTag("NPC")))
                        {
                            return false;
                        }
                        break;
                    }
                case Type.ダークネス:
                    {
                        if (player == null || player.chaser == null || player.chaser.heroine.hCount == 0)
                        {
                            return false;
                        }
                        BoxCollider boxCollider = col as BoxCollider;
                        if (boxCollider != null)
                        {
                            Transform transform = boxCollider.transform;
                            if (Physics.OverlapBox(transform.position, transform.localScale * 0.5f, transform.rotation, 8192).Any(col => (col.CompareTag("NPC") || col.CompareTag("FixChara")) && col.transform != player.chaser.cachedTransform))
                            {
                                return false;
                            }
                        }
                        return true;
                    }
            }
            return true;
        }

        private bool CheckTag(Collider other, string[] tagNames)
        {
            if (!enabled)
            {
                return false;
            }
            return tagNames.IsNullOrEmpty() || tagNames.Any(tag => other.CompareTag(tag));
        }

        private IEnumerator Execute(CancellationToken cancel)
        {
            visibleChange.visible = !visibleChange.visible;
            player.isActionNow = true;
            switch (type)
            {
                case Type.部活メニュー:
                    Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                    {
                        assetBundleName = "action/menu/staffroom/staffroommenu.unity3d",
                        levelName = "StaffRoomMenu",
                        isAdd = true,
                        isAsync = true,
                        isFade = true
                    }, false);
                    yield return new WaitWhile(() => Singleton<Scene>.Instance.IsNowLoadingFade);
                    break;
                case Type.部活レポート:
                    Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                    {
                        assetBundleName = "action/menu/clubreport.unity3d",
                        levelName = "ClubReport",
                        isAdd = true,
                        isAsync = true,
                        isFade = true
                    }, false);
                    yield return new WaitWhile(() => Singleton<Scene>.Instance.IsNowLoadingFade);
                    break;
                case Type.時間割:
                    Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                    {
                        assetBundleName = "action/menu/classschedulemenu.unity3d",
                        levelName = "ClassScheduleMenu",
                        isAdd = true,
                        isAsync = true
                    }, false);
                    yield return new WaitWhile(() => Singleton<Scene>.Instance.IsNowLoadingFade);
                    break;
                case Type.覗きトイレ:
                    {
                        Door door = eventPointList.Select(item => item.GetComponent<Door>()).FirstOrDefault(item => item != null);
                        if (!(door != null))
                        {
                            break;
                        }
                        NPC npc2 = door.hitList.Select(item => item.gameObject.GetComponent<NPC>()).FirstOrDefault(item => item != null && item.AI.actionNo != 23);
                        if (npc2 != null)
                        {
                            int id2 = npc2.GetOnanismID(true);
                            if (id2 == -1)
                            {
                                id2 = 2000;
                                Singleton<Game>.Instance.rankSaveData.peepToiletCount++;
                            }
                            else
                            {
                                id2 = 2003;
                            }
                            yield return Observable.FromCoroutine(_ => Singleton<Game>.Instance.actScene.ChangeHSpecial(new List<int>
                    {
                        id2
                    }, true, transform, null, npc2)).StartAsCoroutine(cancel);
                        }
                        break;
                    }
                case Type.覗きシャワー:
                    {
                        Door door2 = eventPointList.Select(item => item.GetComponent<Door>()).FirstOrDefault(item => item != null);
                        if (!(door2 != null))
                        {
                            break;
                        }
                        NPC npc = door2.hitList.Select(item => item.gameObject.GetComponent<NPC>()).FirstOrDefault(item => item != null && item.AI.actionNo != 23);
                        if (npc != null)
                        {
                            int id = npc.GetOnanismID(true);
                            if (id == -1)
                            {
                                id = 2001;
                                Singleton<Game>.Instance.rankSaveData.peepShowerCount++;
                            }
                            else
                            {
                                id = 2004;
                            }
                            yield return Observable.FromCoroutine(_ => Singleton<Game>.Instance.actScene.ChangeHSpecial(new List<int>
                    {
                        id
                    }, true, transform, null, npc)).StartAsCoroutine(cancel);
                        }
                        break;
                    }
                case Type.特殊H:
                    if (player.chaser != null)
                    {
                        string prevBGM3 = string.Empty;
                        float prevVolume3 = 1f;
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetBGMandVolume(delegate (string bgm, float volume)
                        {
                            prevBGM3 = bgm;
                            prevVolume3 = volume;
                        })).StartAsCoroutine(cancel);
                        Kind kind = eventPointList.Select(item => item.GetComponent<Kind>()).FirstOrDefault(item => item != null);
                        if (!hType.Any())
                        {
                            yield return Observable.FromCoroutine(_ => Singleton<Game>.Instance.actScene.ChangeH(kind, true, isLoadPeepLoom, player.chaser)).StartAsCoroutine(cancel);
                        }
                        else
                        {
                            yield return Observable.FromCoroutine(_ => Singleton<Game>.Instance.actScene.ChangeHSpecial(this, kind, player.chaser)).StartAsCoroutine(cancel);
                        }
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetFadePlayerWhileNull(prevBGM3, prevVolume3)).StartAsCoroutine(cancel);
                    }
                    break;
                case Type.寝る:
                    {
                        string prevBGM2 = string.Empty;
                        float prevVolume2 = 1f;
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetBGMandVolume(delegate (string bgm, float volume)
                        {
                            prevBGM2 = bgm;
                            prevVolume2 = volume;
                        })).StartAsCoroutine(cancel);
                        ActionScene actScene2 = Singleton<Game>.Instance.actScene;
                        Base target2 = GetSleepEventChara(actScene2, Tuple.Create(20, -1, 4));
                        if (target2 != null)
                        {
                            SetCharaActiveFalse(actScene2, target2);
                            List<int> selecter = new List<int>
                    {
                        0
                    };
                            HashSet<int> hash;
                            if (Singleton<Game>.Instance.saveData.clubContents.TryGetValue(1, out hash) && hash.Contains(2004))
                            {
                                selecter.Add(1);
                            }
                            int select = selecter.Shuffle().First();
                            int advNo2;
                            if (target2.heroine.isTeacher)
                            {
                                advNo2 = 6 + select;
                            }
                            else
                            {
                                advNo2 = 94 + select;
                            }
                            Singleton<Scene>.Instance.SetFadeColor(Color.black);
                            yield return Observable.FromCoroutine(_ => EventADVStart(advNo2, target2.heroine, true)).StartAsCoroutine(cancel);
                            Singleton<Scene>.Instance.SetFadeColorDefault();
                            Tuple<int, Vector3, Quaternion> data2 = GetEventValueAndNull(actScene2);
                            int appoint2 = data2.Item1;
                            Vector3 pos2 = data2.Item2;
                            Quaternion rot2 = data2.Item3;
                            yield return Observable.FromCoroutine(_ => actScene2.ChangeHEventWithActionPoint(pos2, rot2, appoint2, target2)).StartAsCoroutine(cancel);
                        }
                        else
                        {
                            Singleton<Scene>.Instance.SetFadeColor(Color.black);
                            if (Illusion.Game.Utils.Scene.SafeFadeIn() == Scene.Data.FadeType.In)
                            {
                                yield return Observable.FromCoroutine(_ => Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In)).StartAsCoroutine(cancel);
                            }
                            yield return Observable.FromCoroutine(_ => MonologueEvent(Tuple.Create("[P名]", wakeUpWords.Shuffle().First()))).StartAsCoroutine(cancel);
                        }
                        Singleton<Scene>.Instance.SetFadeColorDefault();
                        actScene2.Cycle.AddTimer(0.1f);
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetFadePlayerWhileNull(prevBGM2, prevVolume2)).StartAsCoroutine(cancel);
                        break;
                    }
                case Type.寝るv2:
                    {
                        string prevBGM = string.Empty;
                        float prevVolume = 1f;
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetBGMandVolume(delegate (string bgm, float volume)
                        {
                            prevBGM = bgm;
                            prevVolume = volume;
                        })).StartAsCoroutine(cancel);
                        ActionScene actScene = Singleton<Game>.Instance.actScene;
                        Base target = GetSleepEventChara(actScene, Tuple.Create(36, -2, 4));
                        if (target != null)
                        {
                            SetCharaActiveFalse(actScene, target);
                            List<int> selecter2 = new List<int>
                    {
                        2
                    };
                            HashSet<int> hash2;
                            if (Singleton<Game>.Instance.saveData.clubContents.TryGetValue(1, out hash2) && hash2.Contains(1027))
                            {
                                selecter2.Add(3);
                            }
                            int select2 = selecter2.Shuffle().First();
                            int advNo;
                            if (target.heroine.isTeacher)
                            {
                                advNo = 6 + select2;
                            }
                            else
                            {
                                advNo = 94 + select2;
                            }
                            Singleton<Scene>.Instance.SetFadeColor(Color.black);
                            yield return Observable.FromCoroutine(_ => EventADVStart(advNo, target.heroine, true)).StartAsCoroutine(cancel);
                            Singleton<Scene>.Instance.SetFadeColorDefault();
                            Tuple<int, Vector3, Quaternion> data3 = GetEventValueAndNull(actScene);
                            int appoint = data3.Item1;
                            Vector3 pos = data3.Item2;
                            Quaternion rot = data3.Item3;
                            yield return Observable.FromCoroutine(_ => actScene.ChangeHEventWithActionPoint(pos, rot, appoint, target)).StartAsCoroutine(cancel);
                        }
                        else
                        {
                            Singleton<Scene>.Instance.SetFadeColor(Color.black);
                            if (Illusion.Game.Utils.Scene.SafeFadeIn() == Scene.Data.FadeType.In)
                            {
                                yield return Observable.FromCoroutine(_ => Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In)).StartAsCoroutine(cancel);
                            }
                            yield return Observable.FromCoroutine(_ => MonologueEvent(Tuple.Create("[P名]", wakeUpWords.Shuffle().First()))).StartAsCoroutine(cancel);
                        }
                        Singleton<Scene>.Instance.SetFadeColorDefault();
                        actScene.Cycle.AddTimer(0.1f);
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetFadePlayerWhileNull(prevBGM, prevVolume)).StartAsCoroutine(cancel);
                        break;
                    }
                case Type.ダークネス:
                    {
                        NPC target3 = player.chaser;
                        if (target3 == null)
                        {
                            break;
                        }
                        ActionScene actScene3 = Singleton<Game>.Instance.actScene;
                        string prevBGM4 = string.Empty;
                        float prevVolume4 = 1f;
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetBGMandVolume(delegate (string bgm, float volume)
                        {
                            prevBGM4 = bgm;
                            prevVolume4 = volume;
                        })).StartAsCoroutine(cancel);
                        SetCharaActiveFalse(actScene3, target3);
                        Singleton<Scene>.Instance.SetFadeColor(Color.black);
                        var backupPlayerPos = new
                        {
                            player.position,
                            player.rotation
                        };
                        var backupChaserPos = new
                        {
                            target3.position,
                            target3.rotation
                        };
                        int mapNo = 70;
                        List<int> hType = HType;
                        yield return Observable.FromCoroutine(_ => EventADVStart("darkness_start", target3.heroine, false, mapNo, cancel)).StartAsCoroutine(cancel);
                        Vector3 pos3 = Vector3.zero;
                        Quaternion rot3 = Quaternion.identity;
                        bool useNull = false;
                        Dictionary<string, ValData> vars = actScene3.AdvScene.Scenario.Vars;
                        ValData value;
                        if (vars.TryGetValue("hPos", out value))
                        {
                            string text = (string)value.o;
                            if (!text.IsNullOrEmpty() && vars.TryGetValue("nullNo", out value))
                            {
                                int version = (int)value.o;
                                useNull = Program.GetNull(version, actScene3.Map.ConvertMapName(mapNo), text, actScene3.Map, out pos3, out rot3);
                            }
                        }
                        if (!useNull)
                        {
                            Transform transform = actScene3.Map.mapRoot.transform.Find("h_free") ?? actScene3.Map.mapRoot.transform;
                            pos3 = transform.position;
                            rot3 = transform.rotation;
                        }
                        if (player != null)
                        {
                            player.SetPositionAndRotation(backupPlayerPos.position, backupPlayerPos.rotation);
                        }
                        if (target3 != null)
                        {
                            target3.SetPositionAndRotation(backupChaserPos.position, backupChaserPos.rotation);
                        }
                        yield return Observable.FromCoroutine(_ => actScene3.ChangeHSpecial(hType, false, pos3, rot3, delegate (OpenHData.Data data)
                        {
                            data.isDarkness = true;
                        }, target3)).StartAsCoroutine(cancel);
                        Singleton<Scene>.Instance.SetFadeColorDefault();
                        yield return Observable.FromCoroutine(_ => Illusion.Game.Utils.Sound.GetFadePlayerWhileNull(prevBGM4, prevVolume4)).StartAsCoroutine(cancel);
                        break;
                    }
            }
            player.isActionNow = false;
        }

        private Base GetSleepEventChara(ActionScene actScene, Tuple<int, int, int>? fixCondition)
        {
            Base @base = null;
            if (fixCondition.HasValue && @base == null && actScene.fixChara != null)
            {
                SaveData.Heroine heroine = actScene.fixChara.heroine;
                Tuple<int, int, int> value = fixCondition.Value;
                int item = value.Item1;
                int item2 = value.Item2;
                int item3 = value.Item3;
                if (actScene.fixChara.mapNo == item && heroine.fixCharaID == item2 && (item3 == -1 || heroine.talkEvent.Contains(item3)))
                {
                    @base = actScene.fixChara;
                }
            }
            if (@base == null)
            {
                @base = actScene.npcList.Shuffle().Where(delegate (NPC p)
                {
                    SaveData.Heroine heroine2 = p.heroine;
                    if (heroine2.hCount == 0 || heroine2.lewdness < 50 || heroine2.isAnger)
                    {
                        return false;
                    }
                    int desire = actScene.actCtrl.GetDesire(5, heroine2);
                    return desire > 30 && Illusion.Utils.ProbabilityCalclator.DetectFromPercent(desire);
                }).FirstOrDefault();
            }
            return @base;
        }

        private void SetCharaActiveFalse(ActionScene actScene, Base target)
        {
            actScene.Player.SetActive(false);
            actScene.npcList.ForEach(delegate (NPC p)
            {
                p.SetActive(false);
            });
            actScene.npcList.ForEach(delegate (NPC p)
            {
                p.Pause(true);
            });
            if (actScene.fixChara != null)
            {
                actScene.fixChara.SetActive(false);
            }
            NPC nPC = target as NPC;
            if (nPC != null)
            {
                if (nPC.mapNo != actScene.Player.mapNo)
                {
                    nPC.mapNo = actScene.Player.mapNo;
                }
                Transform transform = base.transform.Find("EndPos");
                if (transform != null)
                {
                    nPC.SetPositionAndRotation(transform);
                }
                else
                {
                    Vector3 position = actScene.Player.position;
                    Vector3 forward = Vector3.forward;
                    Vector3 b = actScene.Player.rotation * forward;
                    nPC.SetPositionAndRotation(position + b, actScene.Player.eulerAngles);
                }
            }
            target.chaCtrl.RandomChangeOfClothesLowPolyEnd();
        }

        private Tuple<int, Vector3, Quaternion> GetEventValueAndNull(ActionScene actScene)
        {
            int item = -1;
            Dictionary<string, ValData> vars = actScene.AdvScene.Scenario.Vars;
            ValData value;
            if (vars.TryGetValue("appoint", out value))
            {
                item = (int)value.o;
            }
            string text = null;
            if (vars.TryGetValue("hPos", out value))
            {
                text = (string)value.o;
            }
            int version = 0;
            if (vars.TryGetValue("nullNo", out value))
            {
                version = (int)value.o;
            }
            Vector3 position;
            Quaternion rotation;
            if (text.IsNullOrEmpty())
            {
                position = transform.position;
                rotation = transform.rotation;
            }
            else
            {
                ActionMap map = actScene.Map;
                Program.GetNull(version, map.ConvertMapName(map.no), text, map, out position, out rotation);
            }
            return Tuple.Create(item, position, rotation);
        }

        private IEnumerator EventADVStart(int advNo, SaveData.Heroine heroine, bool isManCreate)
        {
            Scene.Data.FadeType fadeType = Illusion.Game.Utils.Scene.SafeFadeIn();
            if (fadeType == Scene.Data.FadeType.In)
            {
                yield return StartCoroutine(Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In));
            }
            bool isOpenADV = false;
            ActionScene actScene = Singleton<Game>.Instance.actScene;
            yield return StartCoroutine(Program.Open(new Data
            {
                fadeInTime = 0f,
                position = Vector3.zero,
                rotation = Quaternion.identity,
                camera = null,
                heroineList = new List<SaveData.Heroine>
                {
                    heroine
                },
                scene = actScene,
                transferList = EventADV(advNo, heroine, isManCreate)
            }, new Program.OpenDataProc
            {
                onLoad = delegate
                {
                    isOpenADV = true;
                }
            }));
            yield return new WaitUntil(() => isOpenADV);
            yield return Program.Wait(string.Empty);
            heroine.talkEvent.Add(advNo);
            Singleton<Scene>.Instance.SetFadeColorDefault();
        }

        private IEnumerator EventADVStart(string asset, SaveData.Heroine heroine, bool isManCreate, int mapNo, CancellationToken cancel)
        {
            Scene.Data.FadeType fadeType = Illusion.Game.Utils.Scene.SafeFadeIn();
            if (fadeType == Scene.Data.FadeType.In)
            {
                yield return StartCoroutine(Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In));
            }
            bool isOpenADV = false;
            ActionScene actScene = Singleton<Game>.Instance.actScene;
            actScene.Map.Change(mapNo, Illusion.Game.Utils.Scene.SafeFadeIn());
            yield return new WaitWhile(() => actScene.Map.isMapLoading);
            yield return Observable.FromCoroutine(_ => Program.Open(new Data
            {
                fadeInTime = 0f,
                position = Vector3.zero,
                rotation = Quaternion.identity,
                camera = null,
                heroineList = new List<SaveData.Heroine>
                {
                    heroine
                },
                scene = actScene,
                transferList = EventADV(asset, heroine, isManCreate)
            }, new Program.OpenDataProc
            {
                onLoad = delegate
                {
                    isOpenADV = true;
                }
            })).StartAsCoroutine(cancel);
            yield return new WaitUntil(() => isOpenADV);
            yield return Program.Wait(string.Empty);
            Singleton<Scene>.Instance.SetFadeColorDefault();
        }

        private List<Program.Transfer> EventADV(int advNo, SaveData.Heroine heroine, bool isManCreate)
        {
            return EventADV(advNo.ToString(), heroine, isManCreate);
        }

        private List<Program.Transfer> EventADV(string asset, SaveData.Heroine heroine, bool isManCreate)
        {
            List<Program.Transfer> list = Program.Transfer.NewList();
            Program.SetParam(Singleton<Game>.Instance.Player, heroine, list);
            list.Add(Program.Transfer.Create(false, Command.CameraSetFov, "23"));
            list.Add(Program.Transfer.Create(false, Command.CharaCreate, "0", "-2"));
            if (isManCreate)
            {
                list.Add(Program.Transfer.Create(false, Command.CharaCreate, "-1", "-1"));
            }
            list.Add(Program.Transfer.Open(Program.FindADVBundleFilePath(asset, heroine), asset, bool.TrueString));
            return list;
        }

        private IEnumerator MonologueEvent(params Tuple<string, string>[] words)
        {
            List<Program.Transfer> transferList = Program.Transfer.NewList();
            Program.SetParam(Singleton<Game>.Instance.Player, transferList);
            transferList.Add(Program.Transfer.Create(false, Command.Regulate, "Add", Regulate.Control.Next.ToString()));
            transferList.Add(Program.Transfer.Create(false, Command.Wait, "1"));
            transferList.Add(Program.Transfer.Create(false, Command.SceneFade, "out", "1"));
            transferList.Add(Program.Transfer.Create(false, Command.Regulate, "Sub", Regulate.Control.Next.ToString()));
            for (int i = 0; i < words.Length; i++)
            {
                Tuple<string, string> tuple = words[i];
                transferList.Add(Program.Transfer.Text(tuple.Item1, tuple.Item2));
            }
            ActionScene actScene = Singleton<Game>.Instance.actScene;
            Transform camT = Camera.main.transform;
            transferList.Add(Program.Transfer.Close());
            bool isOpenADV = false;
            yield return StartCoroutine(Program.Open(new Data
            {
                position = actScene.Player.position,
                rotation = actScene.Player.rotation,
                scene = actScene,
                camera = new OpenData.CameraData
                {
                    position = camT.position,
                    rotation = camT.rotation
                },
                transferList = transferList
            }, new Program.OpenDataProc
            {
                onLoad = delegate
                {
                    isOpenADV = true;
                }
            }));
            yield return new WaitUntil(() => isOpenADV);
            yield return Program.ADVProcessingCheck();
        }

        private void Awake()
        {
            if (icon != null)
            {
                _render = icon.GetComponent<SpriteRenderer>();
            }
        }

        private void Start()
        {
            if (!Singleton<Game>.IsInstance())
            {
                return;
            }
            if (Singleton<Game>.Instance.actScene != null)
            {
                player = Singleton<Game>.Instance.actScene.Player;
            }
            if (type == Type.ダークネス)
            {
                (from _ in this.UpdateAsObservable()
                 where player != null
                 select player.chaser).DistinctUntilChanged().Subscribe(delegate (NPC chaser)
             {
                 executable = chaser != null && AssetBundleCheck.IsFile("adv/scenario/" + chaser.heroine.ChaName + "/darkness.unity3d", string.Empty);
             });
            }
            bool isHitVisible = InitializeHitVisible();
            string[] tagNames = InitializeHitTagNames();
            _isIconDraw = new BoolReactiveProperty(!isHitVisible);
            if (_render == null)
            {
                _isIconDraw.Dispose();
            }
            else
            {
                _isIconDraw.TakeUntilDestroy(this).Subscribe(delegate (bool isOn)
                {
                    _render.enabled = isOn;
                });
            }
            bool isHit = false;
            (from other in col.OnTriggerEnterAsObservable()
             where CheckTag(other, tagNames)
             select other).Subscribe(delegate (Collider other)
         {
             isHit = true;
             if (other.CompareTag("Player"))
             {
                 other.GetComponent<Player>().actionPointList.Add(this);
             }
         });
            (from other in col.OnTriggerExitAsObservable()
             where CheckTag(other, tagNames)
             select other).Subscribe(delegate (Collider other)
         {
             isHit = false;
             if (other.CompareTag("Player"))
             {
                 other.GetComponent<Player>().actionPointList.Remove(this);
             }
         });
            Tween iconPosTween = null;
            Vector3 iconPos = icon.localPosition;
            this.UpdateAsObservable().Subscribe(delegate
            {
                if (isHit)
                {
                    if (iconPosTween == null)
                    {
                        iconPosTween = icon.DOLocalJump(iconPos, 0.1f, 1, 3f).SetEase(Ease.InOutQuad).SetLoops(-1, LoopType.Restart);
                    }
                }
                else if (iconPosTween != null)
                {
                    iconPosTween.Kill();
                    iconPosTween = null;
                    icon.localPosition = iconPos;
                }
            });
            if (_render != null)
            {
                Color iconColor = _render.color;
                (from _ in this.UpdateAsObservable()
                 select isHit).DistinctUntilChanged().Subscribe(delegate (bool hit)
             {
                 _render.color = !hit ? iconColor : Color.yellow;
             });
            }
            this.UpdateAsObservable().Subscribe(delegate
            {
                bool flag = executable && (!isHitVisible || isHit) && isVisible && VisibleIcon();
                if (flag && col != null && distance > 0f && player != null)
                {
                    flag &= (player.position - col.transform.position).sqrMagnitude < distance * distance;
                }
                isIconDraw = flag;
                Billboard();
            });
            if (player != null)
            {
                ActionChangeUI actionChangeUI = player.actScene.actionChangeUI;
                Type type = this.type;
                if (type == Type.覗きトイレ || type == Type.覗きシャワー)
                {
                    ActionChangeUI.ActionType iconType = ActionChangeUI.ActionType.Peeping;
                    (from _ in this.UpdateAsObservable().TakeUntilDestroy(actionChangeUI)
                     select isIconDraw && isHit).DistinctUntilChanged().Subscribe(delegate (bool active)
                 {
                     if (active)
                     {
                         actionChangeUI.Set(iconType);
                     }
                     else
                     {
                         actionChangeUI.Remove(iconType);
                     }
                 }, (Action)delegate
                 {
                     if (actionChangeUI != null)
                     {
                         actionChangeUI.Remove(iconType);
                     }
                 });
                }
            }
            (from _ in this.UpdateAsObservable().TakeUntilDestroy(player)
             where isHit
             where isIconDraw
             where !player.isActionNow
             where Singleton<Scene>.Instance.AddSceneName.IsNullOrEmpty()
             where ActionInput.isAction
             select _).Subscribe(delegate
         {
             Observable.FromCoroutine(Execute).Subscribe();
         });
        }

        private void OnDrawGizmos()
        {
            if (col != null && distance > 0f)
            {
                Color blue = Color.blue;
                blue.a = 0.5f;
                Gizmos.color = blue;
                Gizmos.DrawSphere(col.transform.position, distance);
            }
            if (hOffsetPos.magnitude > 1E-05f || hOffsetAngle.magnitude > 1E-05f)
            {
                Gizmos.color = new Color(1f, 1f, 0f, 0.5f);
                Vector3 vector = transform.position + hOffsetPos;
                float num = 0.25f;
                Gizmos.DrawSphere(vector, num);
                Illusion.Utils.Gizmos.Axis(vector, Quaternion.Euler(transform.eulerAngles + hOffsetAngle), num);
            }
        }

        private void Billboard()
        {
            if (isIconDraw && !(camTrans == null))
            {
                Vector3 position = camTrans.position;
                Vector3 position2 = icon.position;
                position.y = position2.y;
                icon.LookAt(position);
                icon.Rotate(new Vector3(0f, 180f, 0f));
            }
        }
    }
}
