using ChaCustom;
using Illusion.Component.UI;
using Illusion.Extensions;
using Illusion.Game;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

namespace ActionGame
{
    public class ClassRoomFileListCtrl : MonoBehaviour
    {
        [SerializeField]
        private Text textDrawName;

        [SerializeField]
        private RectTransform rtfScrollRect;

        [SerializeField]
        private RectTransform rtfContant;

        [SerializeField]
        private GameObject objListContent;

        [SerializeField]
        private GameObject objLineTemp;

        [SerializeField]
        private Toggle tglAddInfo;

        [SerializeField]
        private Button btnSortClub;

        [SerializeField]
        private Button btnSortPersonality;

        [SerializeField]
        private Button btnSortName;

        [SerializeField]
        private Button btnSortData;

        [SerializeField]
        private bool multiSelect;

        [SerializeField]
        [Tooltip("OFFだと最低でも一つは選択されてる必要がある")]
        private bool allowSwitchOff = true;

        private StringReactiveProperty selName = new StringReactiveProperty(string.Empty);

        private string selectDrawName = string.Empty;

        private List<CustomFileInfo> lstFileInfo = new List<CustomFileInfo>();

        private bool ascendName;

        private bool ascendData;

        private bool ascendClub;

        private bool ascendPersonality;

        private byte lastSort;

        public event Action<CustomFileInfo> OnPointerEnter = delegate
        {
        };

        public event Action<CustomFileInfo> OnPointerExit = delegate
        {
        };

        public event Action<CustomFileInfo> OnPointerClick = delegate
        {
        };

        public void SortDate(bool ascend)
        {
            ascendData = ascend;
            if (!lstFileInfo.Any())
            {
                return;
            }
            lastSort = 1;
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.time.CompareTo(b.time));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.time.CompareTo(a.time));
            }
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (!(gameObject == null))
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortName(bool ascend)
        {
            ascendName = ascend;
            if (!lstFileInfo.Any())
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.name.CompareTo(b.name));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.name.CompareTo(a.name));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (!(gameObject == null))
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortClub(bool ascend)
        {
            ascendClub = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.club.CompareTo(b.club));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.club.CompareTo(a.club));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortPersonality(bool ascend)
        {
            ascendPersonality = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.personality.CompareTo(b.personality));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.personality.CompareTo(a.personality));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void ClearList()
        {
            lstFileInfo.Clear();
        }

        public void AddList(int index, string name, string club, string personality, string fullpath, string filename, DateTime time, bool disable = false, bool disvisible = false)
        {
            CustomFileInfo customFileInfo = new CustomFileInfo();
            customFileInfo.index = index;
            customFileInfo.name = name;
            customFileInfo.club = club;
            customFileInfo.personality = personality;
            customFileInfo.FullPath = fullpath;
            customFileInfo.FileName = filename;
            customFileInfo.time = time;
            customFileInfo.disable = disable;
            customFileInfo.disvisible = disvisible;
            lstFileInfo.Add(customFileInfo);
        }

        public void Create(Action<CustomFileInfoComponent> infoListProc)
        {
            objListContent.Children().ForEach(Destroy);
            ToggleGroup tglGrp = objListContent.GetComponent<ToggleGroup>();
            tglGrp.allowSwitchOff = allowSwitchOff;
            Transform parent = objListContent.transform;
            lstFileInfo.ForEach(delegate (CustomFileInfo item)
            {
                GameObject gameObject = Instantiate(objLineTemp, parent, false);
                CustomFileInfoComponent component = gameObject.GetComponent<CustomFileInfoComponent>();
                component.info = item;
                component.info.fic = component;
                if (!multiSelect)
                {
                    component.tgl.group = tglGrp;
                }
                if ((bool)component.tmpTexClub)
                {
                    component.tmpTexClub.text = component.info.club;
                }
                if ((bool)component.tmpTexPersonality)
                {
                    component.tmpTexPersonality.text = component.info.personality;
                }
                SetToggleHandler(gameObject);
                component.Disvisible(item.disvisible);
                component.Disable(item.disable);
                infoListProc.Call(component);
            });
            SortClub(true);
            SortPersonality(true);
            SortName(true);
            SortDate(false);
            ToggleAllOff();
        }

        public void UpdateSort()
        {
            if (lastSort == 0)
            {
                SortClub(ascendClub);
                SortPersonality(ascendPersonality);
                SortDate(ascendData);
                SortName(ascendName);
            }
            else if (lastSort == 1)
            {
                SortClub(ascendClub);
                SortPersonality(ascendPersonality);
                SortName(ascendName);
                SortDate(ascendData);
            }
            else if (lastSort == 2)
            {
                SortPersonality(ascendPersonality);
                SortName(ascendName);
                SortDate(ascendData);
                SortClub(ascendClub);
            }
            else
            {
                SortClub(ascendClub);
                SortName(ascendName);
                SortDate(ascendData);
                SortPersonality(ascendPersonality);
            }
        }

        public CustomFileInfo GetFileInfoFromIndex(int index)
        {
            return lstFileInfo.Find(item => item.index == index);
        }

        public CustomFileInfo GetFileInfoFromName(string name)
        {
            return lstFileInfo.Find(item => item.name == name);
        }

        public string GetNameFormIndex(int index)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            return customFileInfo != null ? customFileInfo.name : string.Empty;
        }

        public int GetIndexFromName(string name)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            return customFileInfo != null ? customFileInfo.index : -1;
        }

        public int[] GetSelectIndex()
        {
            return (from item in lstFileInfo
                    where item.fic.gameObject.activeSelf
                    where item.fic.tgl.interactable
                    where item.fic.tgl.isOn
                    select item.index).ToArray();
        }

        public CustomFileInfoComponent GetSelectTopItem()
        {
            int[] selectIndex = GetSelectIndex();
            if (!selectIndex.Any())
            {
                return null;
            }
            CustomFileInfo fileInfoFromIndex = GetFileInfoFromIndex(selectIndex[0]);
            return fileInfoFromIndex != null ? fileInfoFromIndex.fic : null;
        }

        public CustomFileInfoComponent GetSelectableTopItem()
        {
            CustomFileInfo customFileInfo = (from item in lstFileInfo
                                             where item.fic.gameObject.activeSelf
                                             where item.fic.tgl.interactable
                                             orderby item.fic.gameObject.transform.GetSiblingIndex()
                                             select item).FirstOrDefault();
            return customFileInfo != null ? customFileInfo.fic : null;
        }

        public int GetDrawOrderFromIndex(int index)
        {
            var anon = (from item in lstFileInfo
                        where item.fic.gameObject.activeSelf
                        where item.fic.tgl.interactable
                        orderby item.fic.gameObject.transform.GetSiblingIndex()
                        select item).Select((item, idx) => new
                        {
                            item,
                            idx
                        }).FirstOrDefault(p => p.item.index == index);
            return anon != null ? anon.idx : -1;
        }

        public int GetInclusiveCount()
        {
            return lstFileInfo.Count(item => item.fic.gameObject.activeSelf);
        }

        public void SelectItem(int index)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            customFileInfo.fic.tgl.isOn = true;
            ChangeItem(customFileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void SelectItem(string name)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            customFileInfo.fic.tgl.isOn = true;
            ChangeItem(customFileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void UpdateScrollPosition()
        {
        }

        public void ChangeItem(GameObject obj)
        {
            CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
            CustomFileInfo customFileInfo = null;
            if (component != null)
            {
                customFileInfo = !component.tgl.isOn ? null : component.info;
            }
            OnPointerClick(customFileInfo);
            selectDrawName = customFileInfo != null ? customFileInfo.name : string.Empty;
        }

        public void ToggleAllOff()
        {
            lstFileInfo.ForEach(delegate (CustomFileInfo item)
            {
                item.fic.tgl.isOn = false;
            });
        }

        public void ToggleAllOn()
        {
            lstFileInfo.ForEach(delegate (CustomFileInfo item)
            {
                item.fic.tgl.isOn = true;
            });
        }

        public void DisableItem(int index, bool disable)
        {
            DisableItem(lstFileInfo.Find(item => item.index == index), disable);
        }

        public void DisableItem(string name, bool disable)
        {
            DisableItem(lstFileInfo.Find(item => item.name == name), disable);
        }

        public void DisvisibleItem(int index, bool disvisible)
        {
            DisvisibleItem(lstFileInfo.Find(item => item.index == index), disvisible);
        }

        public void DisvisibleItem(string name, bool disvisible)
        {
            DisvisibleItem(lstFileInfo.Find(item => item.name == name), disvisible);
        }

        public void DisableAddInfo()
        {
            if ((bool)tglAddInfo)
            {
                tglAddInfo.interactable = false;
            }
            if ((bool)btnSortClub)
            {
                btnSortClub.interactable = false;
            }
            if ((bool)btnSortPersonality)
            {
                btnSortPersonality.interactable = false;
            }
        }

        private void SetToggleHandler(GameObject obj)
        {
            PointerClickCheck orAddComponent = obj.GetOrAddComponent<PointerClickCheck>();
            orAddComponent.buttonType = PointerClickCheck.ButtonType.Left;
            orAddComponent.onPointerClick.AddListener(delegate
            {
                CheckToggleEvent(obj, delegate
                {
                    ChangeItem(obj);
                });
            });
            Selectable component = obj.GetComponent<Selectable>();
            component.OnPointerEnterAsObservable().Subscribe(delegate
            {
                CheckToggleEvent(obj, delegate (CustomFileInfoComponent fic)
                {
                    selName.Value = fic.info.name;
                    OnPointerEnter(fic.info);
                });
            });
            component.OnPointerExitAsObservable().Subscribe(delegate
            {
                CheckToggleEvent(obj, delegate (CustomFileInfoComponent fic)
                {
                    selName.Value = selectDrawName;
                    OnPointerExit(fic.info);
                });
            });
        }

        private void Start()
        {
            if (textDrawName != null)
            {
                selName.SubscribeToText(textDrawName);
            }
            if (btnSortData != null)
            {
                btnSortData.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortDate(!ascendData);
                });
            }
            if (btnSortName != null)
            {
                btnSortName.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortName(!ascendName);
                });
            }
            if (btnSortClub != null)
            {
                btnSortClub.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortClub(!ascendClub);
                });
            }
            if (btnSortPersonality != null)
            {
                btnSortPersonality.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortPersonality(!ascendPersonality);
                });
            }
            if ((bool)tglAddInfo)
            {
                tglAddInfo.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
                {
                    for (int i = 0; i < lstFileInfo.Count; i++)
                    {
                        lstFileInfo[i].fic.imgAddInfo.gameObject.SetActiveIfDifferent(isOn);
                    }
                });
            }
            this.UpdateAsObservable().Subscribe(delegate
            {
                foreach (CustomFileInfo item in lstFileInfo.Where(item => item.fic.gameObject.activeSelf))
                {
                    RectTransform rectTransform = item.fic.transform as RectTransform;
                    Vector2 anchoredPosition = rectTransform.anchoredPosition;
                    float y = anchoredPosition.y;
                    float height = rectTransform.rect.height;
                    Vector2 anchoredPosition2 = rtfContant.anchoredPosition;
                    float num = 0f - anchoredPosition2.y;
                    float num2 = num - rtfScrollRect.rect.height;
                    float num3 = y;
                    float num4 = y - height;
                    item.show = num4 <= num && num3 >= num2;
                    item.UpdateThumb(item.fic.imgThumb);
                }
            });
        }

        private static void CheckToggleEvent(GameObject obj, Action<CustomFileInfoComponent> act)
        {
            if (!(obj == null))
            {
                CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
                if (!(component == null) && component.tgl.interactable)
                {
                    act.Call(component);
                }
            }
        }

        private static void DisableItem(CustomFileInfo fi, bool disable)
        {
            if (fi != null)
            {
                fi.disable = disable;
                fi.fic.Disable(disable);
            }
        }

        private static void DisvisibleItem(CustomFileInfo fi, bool disvisible)
        {
            if (fi != null)
            {
                fi.disvisible = disvisible;
                fi.fic.Disvisible(disvisible);
            }
        }
    }
}
