using ActionGame.Chara;
using Illusion.CustomAttributes;
using Illusion.Extensions;
using Illusion.Game;
using Illusion.Game.Extensions;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace ActionGame
{
    public class ClassRoomList : BaseLoader
    {
        [Serializable]
        private class RandomCheck : ToggleCheck
        {
            [SerializeField]
            [Label("ネットから")]
            private Toggle _netToggle;

            [SerializeField]
            [Label("フォルダから")]
            private Toggle _directoryToggle;

            protected override Toggle[] toggles
            {
                get
                {
                    return new Toggle[2]
                    {
                        _netToggle,
                        _directoryToggle
                    };
                }
            }
        }

        [Serializable]
        private class CardInformation : ToggleCheck
        {
            [SerializeField]
            [Label("情報表示なし")]
            private Toggle _noneToggle;

            [SerializeField]
            [Label("あだ名表示")]
            private Toggle _callNameToggle;

            [SerializeField]
            [Label("性格名表示")]
            private Toggle _personalNameToggle;

            protected override Toggle[] toggles
            {
                get
                {
                    return new Toggle[3]
                    {
                        _noneToggle,
                        _callNameToggle,
                        _personalNameToggle
                    };
                }
            }

            public PreviewClassData.VisibleMode Mode
            {
                get
                {
                    if (_callNameToggle.isOn)
                    {
                        return PreviewClassData.VisibleMode.CallName;
                    }
                    if (_personalNameToggle.isOn)
                    {
                        return PreviewClassData.VisibleMode.Personality;
                    }
                    return PreviewClassData.VisibleMode.None;
                }
            }
        }

        [Serializable]
        private class EmptyTargetCheck : ToggleCheck
        {
            [SerializeField]
            [Label("全体")]
            private Toggle _allToggle;

            [SerializeField]
            [Label("クラス")]
            private Toggle _classToggle;

            protected override Toggle[] toggles
            {
                get
                {
                    return new Toggle[2]
                    {
                        _allToggle,
                        _classToggle
                    };
                }
            }
        }

        private abstract class ToggleCheck
        {
            protected abstract Toggle[] toggles
            {
                get;
            }

            public ReadOnlyReactiveProperty<int> ToReadOnlyReactiveProperty()
            {
                return (from list in toggles.Select(UnityUIComponentExtensions.OnValueChangedAsObservable).CombineLatest()
                        select list.IndexOf(true) into i
                        where i >= 0
                        select i).ToReadOnlyReactiveProperty();
            }
        }

        private class SwapData
        {
            public readonly int schoolClass;

            public readonly int schoolClassIndex;

            public SwapData(int schoolClass, int schoolClassIndex)
            {
                this.schoolClass = schoolClass;
                this.schoolClassIndex = schoolClassIndex;
            }
        }

        [SerializeField]
        private ClassRoomSelectScene classRoomSelectScene;

        [SerializeField]
        private Transform PreviewRoot;

        [Label("選択中の色")]
        [SerializeField]
        private Color selColor = Color.yellow;

        [Label("クラスタブ")]
        [SerializeField]
        private RectTransform classTab;

        [SerializeField]
        [Header("ランダムチェック")]
        private RandomCheck randomCheck;

        [SerializeField]
        [Header("学生証のあだ名枠関連")]
        private CardInformation cardInformation;

        [Label("登校人数テキスト")]
        [SerializeField]
        private TextMeshProUGUI textAttendanceNum;

        [Label("登校人数スライダー")]
        [SerializeField]
        private Slider sldAttendanceNum;

        [SerializeField]
        [Label("登校人数MAX")]
        private Button btnAttendanceNum;

        [SerializeField]
        [Label("クラスの登録人数を最大にする")]
        private Toggle tglOtherClassRegisterMax;

        [SerializeField]
        [Label("カスタム")]
        private Button customSceneButton;

        [SerializeField]
        [Label("キャラボタン1")]
        private Button chara1Button;

        [SerializeField]
        private string[] chara1Strs = new string[2];

        [Label("キャラボタン2")]
        [SerializeField]
        private Button chara2Button;

        [SerializeField]
        private string[] chara2Strs = new string[2];

        [SerializeField]
        [Label("決定ボタン")]
        private Button enterButton;

        [SerializeField]
        [Label("戻るボタン")]
        private Button returnButton;

        [SerializeField]
        [Header("対象チェック")]
        private EmptyTargetCheck emptyTargetCheck;

        [Label("ランダムボタン")]
        [SerializeField]
        private Button randomButton;

        [SerializeField]
        [Label("除外ボタン")]
        private Button removeButton;

        private BoolReactiveProperty canvasEnabled = new BoolReactiveProperty(true);

        private int? _pageMax;

        [SerializeField]
        private IntReactiveProperty _page = new IntReactiveProperty(1);

        [SerializeField]
        private CvsAttribute cvsAttribute;

        private bool isNetworkRandom;

        private bool isEmptyTargetAll;

        private List<PreviewClassData> _charaPreviewList;

        private List<SaveData.CharaData> charaList = new List<SaveData.CharaData>();

        private ReactiveProperty<PreviewClassData> enterPreview = new ReactiveProperty<PreviewClassData>();

        private CompositeDisposable disposables = new CompositeDisposable();

        public bool isCustomScene
        {
            get;
            private set;
        }

        public bool isVisible
        {
            get
            {
                return canvasEnabled.Value;
            }
            set
            {
                canvasEnabled.Value = value;
            }
        }

        private int pageMax
        {
            get
            {
                if (!_pageMax.HasValue)
                {
                    _pageMax = classTab.childCount - 1;
                }
                return _pageMax.Value;
            }
        }

        private int page
        {
            get
            {
                return _page.Value;
            }
        }

        private int emptyRegisterSum
        {
            get
            {
                return isEmptyTargetAll ? Enumerable.Range(0, pageMax).Select(GetEmptyRegisterSum).Sum() : GetEmptyRegisterSum(page);
            }
        }

        private List<PreviewClassData> charaPreviewList
        {
            get
            {
                return this.GetCache(ref _charaPreviewList, () => new List<PreviewClassData>(from o in PreviewRoot.gameObject.Children()
                                                                                             select new PreviewClassData(o)));
            }
        }

        private SaveData.Player player
        {
            get
            {
                return Singleton<Game>.Instance.Player;
            }
        }

        private List<SaveData.Heroine> heroineList
        {
            get
            {
                return Singleton<Game>.Instance.HeroineList;
            }
        }

        private int GetEmptyRegisterSum(int schoolClass)
        {
            if (schoolClass == pageMax)
            {
                return 0;
            }
            int num = 0;
            int classRegisterMax = GetClassRegisterMax(schoolClass);
            bool flag = isPlayerSchoolClass(schoolClass);
            for (int i = 0; i < classRegisterMax; i++)
            {
                if ((!flag || i != player.schoolClassIndex && i != player.schoolClassIndex + 1) && FindCharaIndex(schoolClass, charaPreviewList[i].classIndex) == -1)
                {
                    num++;
                }
            }
            return num;
        }

        private int GetClassRegisterMax(int schoolClass)
        {
            if (schoolClass == player.schoolClass || Manager.Config.AddData.OtherClassRegisterMax)
            {
                return charaPreviewList.Count;
            }
            return 5;
        }

        private bool isPlayerSchoolClass(int schoolClass)
        {
            return player.schoolClass == schoolClass;
        }

        private static bool isCharaFind(SaveData.CharaData chara, int schoolClass, int classIndex)
        {
            return chara.schoolClass == schoolClass && chara.schoolClassIndex == classIndex;
        }

        private SaveData.CharaData FindChara(int schoolClass, int classIndex)
        {
            return charaList.Find(chara => isCharaFind(chara, schoolClass, classIndex));
        }

        private int FindCharaIndex(int schoolClass, int classIndex)
        {
            return charaList.FindIndex(chara => isCharaFind(chara, schoolClass, classIndex));
        }

        private void PreviewUpdate()
        {
            PreviewClassData value = enterPreview.Value;
            enterPreview.Value = null;
            enterPreview.Value = value;
        }

        private void PreviewUpdateAll()
        {
            PreviewUpdate();
            charaPreviewList.ForEach(delegate (PreviewClassData p)
            {
                p.visibleMode = cardInformation.Mode;
            });
            if (enterPreview.Value != null && !enterPreview.Value.isVisible)
            {
                enterPreview.Value = null;
            }
        }

        private void RemoveChara()
        {
            int index = FindCharaIndex(page, enterPreview.Value.classIndex);
            SaveData.CharaData chara = charaList[index];
            SaveData saveData = Singleton<Game>.Instance.saveData;
            if (saveData.withHeroine.Check(chara))
            {
                saveData.withHeroine.Set(null);
            }
            if (saveData.dateHeroine.Check(chara))
            {
                saveData.dateHeroine.Set(null);
            }
            enterPreview.Value.Clear();
            charaList.RemoveAt(index);
            PreviewUpdate();
        }

        private void RemoveChara(bool isCheckScene)
        {
            if (!isCheckScene)
            {
                RemoveChara();
                return;
            }
            CheckScene.Parameter param = new CheckScene.Parameter();
            param.Yes = delegate
            {
                RemoveChara();
                Singleton<Scene>.Instance.UnLoad();
            };
            param.No = delegate
            {
                Singleton<Scene>.Instance.UnLoad();
            };
            param.Title = "転校させますか？";
            Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param, observer)).StartAsCoroutine();
        }

        private SwapData CreateSwapData(PreviewClassData preview)
        {
            return new SwapData(page, preview.classIndex);
        }

        private bool IsSwapData(PreviewClassData preview)
        {
            return preview != null && IsSwapData(page, preview.classIndex);
        }

        private bool IsSwapData(SwapData swapData)
        {
            return swapData != null && IsSwapData(swapData.schoolClass, swapData.schoolClassIndex);
        }

        private bool IsSwapData(int schoolClass, int schoolClassIndex)
        {
            if (schoolClass == pageMax)
            {
                return false;
            }
            if (schoolClassIndex >= GetClassRegisterMax(schoolClass))
            {
                return false;
            }
            if (FindChara(schoolClass, schoolClassIndex) is SaveData.Player)
            {
                return false;
            }
            return true;
        }

        private void Swap(SwapData mData, SwapData tData)
        {
            SaveData.CharaData charaData = FindChara(mData.schoolClass, mData.schoolClassIndex);
            SaveData.CharaData charaData2 = FindChara(tData.schoolClass, tData.schoolClassIndex);
            if (charaData != null)
            {
                charaData.schoolClass = tData.schoolClass;
                charaData.schoolClassIndex = tData.schoolClassIndex;
            }
            if (charaData2 != null)
            {
                charaData2.schoolClass = mData.schoolClass;
                charaData2.schoolClassIndex = mData.schoolClassIndex;
            }
            foreach (var item in new[]
            {
                new
                {
                    chara = charaData,
                    index = tData.schoolClassIndex,
                    isUpdate = true
                },
                new
                {
                    chara = charaData2,
                    index = mData.schoolClassIndex,
                    isUpdate = mData.schoolClass == tData.schoolClass
                }
            }.Where(item => item.isUpdate))
            {
                PreviewClassData previewClassData = charaPreviewList[item.index];
                if (item.chara != null)
                {
                    previewClassData.Set(item.chara);
                }
                else
                {
                    previewClassData.Clear();
                }
            }
        }

        private void Start()
        {
            heroineList.AddRange(Game.CreateFixCharaList((from p in heroineList
                                                          select p.fixCharaID into id
                                                          where id < 0
                                                          select id).ToArray()));
            charaList.AddRange(heroineList.OfType<SaveData.CharaData>());
            charaList.Add(player);
            string prevBGM = string.Empty;
            Utils.Sound.GetBGM().SafeProc(delegate (LoadSound bgm)
            {
                prevBGM = bgm.assetBundleName;
            });
            CanvasGroup canvasGroup = GetComponent<CanvasGroup>();
            this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
            {
                canvasGroup.interactable = isOn;
            });
            Canvas canvas = GetComponent<Canvas>();
            canvasEnabled.Subscribe(delegate (bool isOn)
            {
                canvas.enabled = isOn;
            }).AddTo(disposables);
            bool isPlaySE = true;
            Action<SystemSE> PlaySE = delegate (SystemSE se)
            {
                if (isPlaySE)
                {
                    Utils.Sound.Play(se);
                }
            };
            SwapData swap = null;
            charaPreviewList.ForEach(delegate (PreviewClassData item)
            {
                IObservable<Unit> source11 = item.button.OnClickAsObservable().Share();
                source11.Subscribe(delegate
                {
                    PlaySE(SystemSE.sel);
                }).AddTo(disposables);
                source11.Where(_ => Input.GetKey(KeyCode.LeftControl) || Input.GetKey(KeyCode.RightControl)).Subscribe(delegate
                {
                    if (IsSwapData(swap) && IsSwapData(item))
                    {
                        Swap(swap, CreateSwapData(item));
                    }
                }).AddTo(disposables);
                source11.Subscribe(delegate
                {
                    enterPreview.Value = item;
                }).AddTo(disposables);
                IObservable<IList<double>> source12 = source11.DoubleInterval(250f);
                source12.Subscribe(delegate
                {
                    if (enterPreview.Value.data == null)
                    {
                        classRoomSelectScene.OpenPreview();
                    }
                    else
                    {
                        SaveData.Heroine heroine5 = enterPreview.Value.data as SaveData.Heroine;
                        if (heroine5 != null && heroine5.fixCharaID >= 0)
                        {
                            RemoveChara(true);
                        }
                    }
                }).AddTo(disposables);
            });
            ColorBlock backupButtonColor = charaPreviewList[0].button.colors;
            enterPreview.Scan(delegate (PreviewClassData acc, PreviewClassData cur)
            {
                if (acc != null)
                {
                    acc.button.colors = backupButtonColor;
                }
                return cur;
            }).Subscribe(delegate (PreviewClassData preview)
            {
                if (preview != null)
                {
                    ColorBlock colors = preview.button.colors;
                    Color color3 = colors.highlightedColor = colors.normalColor = selColor;
                    preview.button.colors = colors;
                }
            }).AddTo(disposables);
            IObservable<PreviewClassData> source = enterPreview.Where(preview => preview != null).Share();
            source.Subscribe(delegate (PreviewClassData preview)
            {
                preview.visibleMode = cardInformation.Mode;
            }).AddTo(disposables);
            source.Subscribe(delegate (PreviewClassData preview)
            {
                swap = CreateSwapData(preview);
            }).AddTo(disposables);
            IObservable<bool> source2 = source.Select(preview => preview.data != null).Share();
            TextMeshProUGUI componentInChildren = chara1Button.GetComponentInChildren<TextMeshProUGUI>();
            componentInChildren.text = chara1Strs[0];
            source2.SubscribeWithState(componentInChildren, delegate (bool isAlive, TextMeshProUGUI t)
            {
                t.text = chara1Strs[isAlive ? 1 : 0];
            }).AddTo(disposables);
            TextMeshProUGUI componentInChildren2 = chara2Button.GetComponentInChildren<TextMeshProUGUI>();
            componentInChildren2.text = chara2Strs[0];
            source2.SubscribeWithState(componentInChildren2, delegate (bool isAlive, TextMeshProUGUI t)
            {
                t.text = chara2Strs[isAlive ? 1 : 0];
            }).AddTo(disposables);
            SaveData.Heroine[] fixHeroines = heroineList.Where(p => p.fixCharaID < 0).ToArray();
            enterPreview.Select(delegate (PreviewClassData preview)
            {
                if (preview == null)
                {
                    return false;
                }
                if (preview.data == null)
                {
                    return true;
                }
                if (preview.data == player)
                {
                    return false;
                }
                return !fixHeroines.Contains(preview.data);
            }).SubscribeToInteractable(chara1Button).AddTo(disposables);
            SaveData.Heroine mankenHeroine = heroineList.FirstOrDefault(p => p.fixCharaID == -8);
            enterPreview.Select(delegate (PreviewClassData preview)
            {
                if (preview == null)
                {
                    return false;
                }
                if (preview.data == null)
                {
                    return true;
                }
                if (mankenHeroine == preview.data)
                {
                    return true;
                }
                return !fixHeroines.Contains(preview.data);
            }).SubscribeToInteractable(chara2Button).AddTo(disposables);
            enterPreview.Select(p => p != null && p.data != null).SubscribeToInteractable(customSceneButton).AddTo(disposables);
            enterPreview.Subscribe(delegate (PreviewClassData preview)
            {
                if (preview == null || preview.data == null || !(preview.data is SaveData.Heroine) || (preview.data as SaveData.Heroine).fixCharaID < 0)
                {
                    cvsAttribute.chaFile = null;
                }
                else
                {
                    cvsAttribute.chaFile = preview.data.charFile;
                }
            }).AddTo(disposables);
            Toggle[] source3 = (from t in classTab.Children()
                                select t.GetComponent<Toggle>()).ToArray();
            (from list in source3.Select(UnityUIComponentExtensions.OnValueChangedAsObservable).CombineLatest()
             select list.IndexOf(true) into i
             where i >= 0
             select i).Subscribe(delegate (int i)
         {
             _page.Value = i;
         }).AddTo(disposables);
            ReadOnlyReactiveProperty<int> source4 = randomCheck.ToReadOnlyReactiveProperty();
            source4.Subscribe(delegate (int i)
            {
                isNetworkRandom = i == 0;
            }).AddTo(disposables);
            source4.Skip(1).Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
            }).AddTo(disposables);
            ReadOnlyReactiveProperty<int> source5 = emptyTargetCheck.ToReadOnlyReactiveProperty();
            source5.Subscribe(delegate (int i)
            {
                isEmptyTargetAll = i == 0;
            }).AddTo(disposables);
            source5.Skip(1).Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
            }).AddTo(disposables);
            PreviewClassData playerSideSheet = charaPreviewList[player.schoolClassIndex + 1];
            List<GameObject> source6 = playerSideSheet.rootObj.Children();
            GameObject imgSwim = source6.Last();
            UI_OnMouseOverMessage uiOnMouseOverMessage = source6.First().GetComponent<UI_OnMouseOverMessage>();
            _page.Select(i => !isPlayerSchoolClass(i)).Subscribe(delegate (bool isOn)
            {
                playerSideSheet.button.interactable = isOn;
                imgSwim.SetActiveIfDifferent(!isOn);
                uiOnMouseOverMessage.enabled = !isOn;
            }).AddTo(disposables);
            _page.Skip(1).Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
            }).AddTo(disposables);
            _page.Subscribe(delegate (int schoolClass)
            {
                charaPreviewList.ForEach(delegate (PreviewClassData preview)
                {
                    preview.Clear();
                });
                Resources.UnloadUnusedAssets();
                if (schoolClass == pageMax)
                {
                    List<SaveData.CharaData> list3 = charaList.Where(p => p.schoolClass == -1).ToList();
                    int count2 = list3.Count;
                    for (int m = 0; m < charaPreviewList.Count; m++)
                    {
                        charaPreviewList[m].isVisible = m < count2;
                    }
                    for (int n = 0; n < charaPreviewList.Count && n < count2; n++)
                    {
                        charaPreviewList[n].Set(list3[n]);
                    }
                }
                else
                {
                    List<SaveData.CharaData> list4 = charaList.Where(p => p.schoolClass == schoolClass).ToList();
                    int classRegisterMax2 = GetClassRegisterMax(schoolClass);
                    for (int num2 = 0; num2 < charaPreviewList.Count; num2++)
                    {
                        charaPreviewList[num2].isVisible = num2 < classRegisterMax2;
                    }
                    int l = default(int);
                    for (l = 0; l < charaPreviewList.Count; l++)
                    {
                        PreviewClassData previewClassData = charaPreviewList[l];
                        int num3 = list4.FindIndex(p => p.schoolClassIndex == l);
                        if (num3 != -1)
                        {
                            SaveData.CharaData charaData3 = list4[num3];
                            list4.RemoveAt(num3);
                            previewClassData.Set(charaData3);
                        }
                    }
                }
                if (swap != null && swap.schoolClass == schoolClass)
                {
                    enterPreview.Value = charaPreviewList[swap.schoolClassIndex];
                }
                else
                {
                    enterPreview.Value = null;
                }
                PreviewUpdateAll();
            }).AddTo(disposables);
            chara1Button.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
                if (enterPreview.Value.data != null)
                {
                    RemoveChara(true);
                }
                else
                {
                    classRoomSelectScene.OpenPreview();
                }
            }).AddTo(disposables);
            chara2Button.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
                PreviewClassData preview2 = enterPreview.Value;
                if (preview2.data != null)
                {
                    string levelName2 = "NickNameSetting";
                    Scene.Data data2 = new Scene.Data
                    {
                        levelName = levelName2,
                        isAdd = true,
                        isAsync = true,
                        onLoad = delegate
                        {
                            NickNameSettingScene rootComponent2 = Scene.GetRootComponent<NickNameSettingScene>(levelName2);
                            if (!(rootComponent2 == null))
                            {
                                canvas.enabled = false;
                                rootComponent2.OnDestroyAsObservable().TakeUntilDestroy(this).Subscribe(delegate
                                {
                                    canvas.enabled = true;
                                });
                                rootComponent2.charaData = preview2.data;
                                rootComponent2.onEnter += delegate
                                {
                                    Singleton<Scene>.Instance.UnLoad();
                                    preview2.Update();
                                    if (preview2.data is SaveData.Player)
                                    {
                                        charaPreviewList.ForEach(delegate (PreviewClassData item)
                                        {
                                            item.Update();
                                        });
                                    }
                                };
                                rootComponent2.onCancel += delegate
                                {
                                    Singleton<Scene>.Instance.UnLoad();
                                };
                            }
                        }
                    };
                    Singleton<Scene>.Instance.LoadReserve(data2, true);
                }
                else
                {
                    SaveData.Heroine heroine4 = new SaveData.Heroine(true);
                    heroine4.schoolClass = page;
                    heroine4.schoolClassIndex = preview2.classIndex;
                    Observable.FromCoroutine(__ => RandomNetChara(heroine4)).Subscribe(delegate
                    {
                        PreviewUpdate();
                    });
                }
            }).AddTo(disposables);
            ReadOnlyReactiveProperty<int> source7 = cardInformation.ToReadOnlyReactiveProperty();
            source7.Select(i => (PreviewClassData.VisibleMode)i).Subscribe(delegate (PreviewClassData.VisibleMode mode)
            {
                charaPreviewList.ForEach(delegate (PreviewClassData p)
                {
                    p.visibleMode = mode;
                });
            }).AddTo(disposables);
            source7.Skip(1).Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
            }).AddTo(disposables);
            customSceneButton.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE(SystemSE.ok_s);
                isCustomScene = true;
                string levelName = "CustomScene";
                Scene.Data data = new Scene.Data
                {
                    levelName = levelName,
                    isAdd = true,
                    isFade = true,
                    isAsync = true,
                    onLoad = delegate
                    {
                        CustomScene rootComponent = Scene.GetRootComponent<CustomScene>(levelName);
                        if (!(rootComponent == null))
                        {
                            classRoomSelectScene.gameObject.SetActive(false);
                            SaveData.CharaData charaData = enterPreview.Value.data;
                            rootComponent.modeNew = false;
                            rootComponent.chaFileCtrl = charaData.charFile;
                            Camera nowCamea = null;
                            if (Singleton<Game>.IsInstance())
                            {
                                ActionScene actScene = Singleton<Game>.Instance.actScene;
                                if (actScene != null)
                                {
                                    actScene.PlayerDelete();
                                    actScene.npcList.ForEach(delegate (NPC npc)
                                    {
                                        Destroy(npc.gameObject);
                                    });
                                    actScene.npcList.Clear();
                                    actScene.FixCharaDelete();
                                    actScene.actCtrl.Refresh();
                                    actScene.Map.mapRoot.SetActive(false);
                                    nowCamea = Camera.main;
                                    if (nowCamea != null)
                                    {
                                        nowCamea.gameObject.SetActive(false);
                                    }
                                }
                                if (Singleton<Character>.IsInstance())
                                {
                                    Singleton<Character>.Instance.EndLoadAssetBundle();
                                }
                            }
                            rootComponent.OnDestroyAsObservable().TakeUntilDestroy(this).Subscribe(delegate
                            {
                                Utils.Sound.Play(new Utils.Sound.SettingBGM(prevBGM));
                                if (Singleton<Character>.IsInstance() && classRoomSelectScene != null)
                                {
                                    classRoomSelectScene.gameObject.SetActive(true);
                                    classRoomSelectScene.previewCharaList.CharFile.InitializeList();
                                    if (Singleton<Character>.Instance.editChara != null)
                                    {
                                        ChaFileControl editChara = Singleton<Character>.Instance.editChara;
                                        SaveData.CharaData charaData2 = charaList.Find(p => p == charaData);
                                        charaData2.SetCharFile(editChara);
                                        if (!(charaData2 is SaveData.Heroine) || (charaData2 as SaveData.Heroine).fixCharaID < 0)
                                        {
                                            cvsAttribute.chaFile = null;
                                        }
                                        else
                                        {
                                            cvsAttribute.chaFile = editChara;
                                        }
                                        enterPreview.Value.Set(charaData2);
                                    }
                                    if (nowCamea != null)
                                    {
                                        nowCamea.gameObject.SetActive(true);
                                    }
                                    if (Singleton<Game>.IsInstance())
                                    {
                                        ActionScene actScene2 = Singleton<Game>.Instance.actScene;
                                        if (actScene2 != null)
                                        {
                                            actScene2.Map.mapRoot.SetActive(true);
                                        }
                                    }
                                    Singleton<Character>.Instance.editChara = null;
                                }
                            });
                        }
                    }
                };
                Singleton<Scene>.Instance.LoadReserve(data, true);
            }).AddTo(disposables);
            classRoomSelectScene.previewCharaList.onEnter += delegate (ChaFileControl chaFile)
            {
                int page = this.page;
                int classIndex = enterPreview.Value.classIndex;
                SaveData.Heroine heroine3 = new SaveData.Heroine(chaFile, true)
                {
                    charFileInitialized = true,
                    schoolClass = page,
                    schoolClassIndex = classIndex
                };
                charaList.Add(heroine3);
                enterPreview.Value.Set(heroine3);
                PreviewUpdate();
                classRoomSelectScene.OpenClassRoom();
            };
            classRoomSelectScene.previewCharaList.onCancel += delegate
            {
                classRoomSelectScene.OpenClassRoom();
            };
            IObservable<bool> source8 = (from _ in this.UpdateAsObservable()
                                         select isEmptyTargetAll || page != pageMax).Share();
            source8.Select(isOn => isOn && emptyRegisterSum > 0).SubscribeToInteractable(randomButton).AddTo(disposables);
            source8.Select(isOn => isOn && (!isEmptyTargetAll ? Enumerable.Range(0, charaPreviewList.Count).Any(index => FindChara(page, index) is SaveData.Heroine) : charaList.OfType<SaveData.Heroine>().Any(p => p.fixCharaID == 0))).SubscribeToInteractable(removeButton).AddTo(disposables);
            Func<int, List<SaveData.Heroine>> getRamdomList = delegate (int schoolClass)
            {
                bool isPlayerSchoolClass = this.isPlayerSchoolClass(schoolClass);
                return (from heroine in Enumerable.Range(0, GetClassRegisterMax(schoolClass)).Select(delegate (int index)
                    {
                        if (isPlayerSchoolClass && (index == player.schoolClassIndex || index == player.schoolClassIndex + 1))
                        {
                            return null;
                        }
                        return FindChara(schoolClass, index) is SaveData.Heroine ? null : new SaveData.Heroine(true)
                        {
                            schoolClass = schoolClass,
                            schoolClassIndex = index
                        };
                    })
                        where heroine != null
                        select heroine).ToList();
            };
            randomButton.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
                List<SaveData.Heroine> list2 = new List<SaveData.Heroine>();
                if (!isEmptyTargetAll)
                {
                    list2.AddRange(getRamdomList(page));
                }
                else
                {
                    for (int k = 0; k < pageMax; k++)
                    {
                        list2.AddRange(getRamdomList(k));
                    }
                }
                Observable.FromCoroutine(__ => RandomNetChara(list2)).Subscribe(delegate
                {
                    PreviewUpdateAll();
                });
            }).AddTo(disposables);
            removeButton.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE(SystemSE.sel);
                int schoolClass2 = page;
                CheckScene.Parameter param = new CheckScene.Parameter();
                param.Yes = delegate
                {
                    SaveData saveData = Singleton<Game>.Instance.saveData;
                    if (!isEmptyTargetAll)
                    {
                        (from index in Enumerable.Range(0, charaPreviewList.Count)
                         select FindChara(schoolClass2, index) as SaveData.Heroine into heroine
                         where heroine != null
                         select heroine).ToList().ForEach(delegate (SaveData.Heroine chara)
                     {
                         if (saveData.withHeroine.Check(chara))
                         {
                             saveData.withHeroine.Set(null);
                         }
                         if (saveData.dateHeroine.Check(chara))
                         {
                             saveData.dateHeroine.Set(null);
                         }
                         charaList.Remove(chara);
                     });
                    }
                    else
                    {
                        saveData.withHeroine.Set(null);
                        saveData.dateHeroine.Set(null);
                        charaList.RemoveAll(delegate (SaveData.CharaData p)
                        {
                            SaveData.Heroine heroine2 = p as SaveData.Heroine;
                            return heroine2 != null && heroine2.fixCharaID == 0;
                        });
                    }
                    int count = schoolClass2 != pageMax ? charaPreviewList.Count : 0;
                    foreach (int item in from index in Enumerable.Range(0, count)
                                         where FindCharaIndex(schoolClass2, index) == -1
                                         select index)
                    {
                        charaPreviewList[item].Clear();
                    }
                    PreviewUpdate();
                    Singleton<Scene>.Instance.UnLoad();
                };
                param.No = delegate
                {
                    Singleton<Scene>.Instance.UnLoad();
                };
                param.Title = (!isEmptyTargetAll ? "クラス内のキャラ" : "全キャラ") + "を転校させますか？";
                Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param, observer)).StartAsCoroutine();
            }).AddTo(disposables);
            GameObject enterButtonObject = enterButton.gameObject;
            if (Singleton<Scene>.Instance.AddSceneName == "ClassRoomSelect")
            {
                enterButtonObject.SetActive(false);
            }
            Action enterProc = delegate
            {
                charaList.ForEach(SaveData.CallNormalize);
                charaList.Remove(player);
                heroineList.Clear();
                heroineList.AddRange(charaList.OfType<SaveData.Heroine>());
                if (Singleton<Game>.Instance.actScene != null)
                {
                    Singleton<Game>.Instance.actScene.actCtrl.Refresh();
                    Singleton<Game>.Instance.actScene.paramUI.UpdatePlayer();
                }
                if (Singleton<Scene>.Instance.AddSceneName == "ClassRoomSelect")
                {
                    Singleton<Scene>.Instance.UnLoad();
                }
                else
                {
                    Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                    {
                        levelName = "Action",
                        fadeType = Scene.Data.FadeType.In,
                        isAsync = true
                    }, false);
                }
                PlaySE(SystemSE.ok_s);
            };
            if (enterButtonObject.activeSelf)
            {
                enterButton.OnClickAsObservable().Take(1).Subscribe(delegate
                {
                    enterProc();
                });
            }
            returnButton.OnClickAsObservable().Take(1).Subscribe(delegate
            {
                if (!enterButtonObject.activeSelf)
                {
                    enterProc();
                }
                else
                {
                    PlaySE(SystemSE.cancel);
                    Singleton<Scene>.Instance.UnLoad();
                }
            });
            ReadOnlyReactiveProperty<int> source9 = (from _ in this.UpdateAsObservable()
                                                     select Manager.Config.ActData.MaxCharaNum).ToReadOnlyReactiveProperty();
            if (textAttendanceNum != null)
            {
                source9.Select(i => i + "人").SubscribeToText(textAttendanceNum).AddTo(disposables);
            }
            if (sldAttendanceNum != null)
            {
                source9.Subscribe(delegate (int value)
                {
                    sldAttendanceNum.value = value;
                }).AddTo(disposables);
                sldAttendanceNum.maxValue = 38f;
                sldAttendanceNum.onValueChanged.AsObservable().Subscribe(delegate (float value)
                {
                    Manager.Config.ActData.MaxCharaNum = (int)value;
                }).AddTo(disposables);
                sldAttendanceNum.OnScrollAsObservable().Select(delegate (PointerEventData evData)
                {
                    Vector2 scrollDelta = evData.scrollDelta;
                    return scrollDelta.y;
                }).Subscribe(delegate (float value)
                {
                    int num = 0;
                    if (value < 0f)
                    {
                        num++;
                    }
                    else if (value > 0f)
                    {
                        num--;
                    }
                    sldAttendanceNum.value = Mathf.Clamp((int)sldAttendanceNum.value + num, 0, 38);
                })
                    .AddTo(disposables);
                (from evData in sldAttendanceNum.OnPointerDownAsObservable()
                 where evData.button == PointerEventData.InputButton.Left
                 select evData).Subscribe(delegate
             {
                 PlaySE(SystemSE.sel);
             }).AddTo(disposables);
            }
            if (btnAttendanceNum != null)
            {
                btnAttendanceNum.OnClickAsObservable().Subscribe(delegate
                {
                    PlaySE(SystemSE.sel);
                    Manager.Config.ActData.MaxCharaNum = 38;
                }).AddTo(disposables);
            }
            ReadOnlyReactiveProperty<bool> source10 = (from _ in this.UpdateAsObservable()
                                                       select Manager.Config.AddData.OtherClassRegisterMax).ToReadOnlyReactiveProperty();
            if (tglOtherClassRegisterMax != null)
            {
                source10.Subscribe(delegate (bool isOn)
                {
                    tglOtherClassRegisterMax.isOn = isOn;
                }).AddTo(disposables);
                tglOtherClassRegisterMax.onValueChanged.AsObservable().Subscribe(delegate (bool isOn)
                {
                    Manager.Config.AddData.OtherClassRegisterMax = isOn;
                }).AddTo(disposables);
                (from evData in tglOtherClassRegisterMax.OnPointerClickAsObservable()
                 where evData.button == PointerEventData.InputButton.Left
                 select evData).Subscribe(delegate
             {
                 PlaySE(SystemSE.sel);
             }).AddTo(disposables);
            }
            source10.Where(_ => page != pageMax).Subscribe(delegate
            {
                int classRegisterMax = GetClassRegisterMax(page);
                for (int j = 0; j < charaPreviewList.Count; j++)
                {
                    charaPreviewList[j].isVisible = j < classRegisterMax;
                }
                PreviewUpdateAll();
            }).AddTo(disposables);
        }

        private void OnDestroy()
        {
            if (Singleton<Character>.IsInstance())
            {
                Singleton<Character>.Instance.lstProductId.Clear();
            }
            disposables.Clear();
        }

        private IEnumerator RandomNetChara(SaveData.Heroine heroine)
        {
            ChaFileControl[] result2;
            if (isNetworkRandom)
            {
                yield return StartCoroutine(LoadRandomSelectScene(1));
                result2 = Singleton<Character>.Instance.netRandChara;
            }
            else
            {
                result2 = ChaControl.GetRandomFemaleCard(1);
            }
            if (!result2.IsNullOrEmpty())
            {
                heroine.SetCharFile(result2.First());
                heroine.charFileInitialized = true;
                charaList.Add(heroine);
                enterPreview.Value.Set(heroine);
            }
        }

        private IEnumerator RandomNetChara(List<SaveData.Heroine> randomHeroineList)
        {
            ChaFileControl[] result;
            if (isNetworkRandom)
            {
                yield return StartCoroutine(LoadRandomSelectScene(emptyRegisterSum));
                result = Singleton<Character>.Instance.netRandChara;
            }
            else
            {
                result = ChaControl.GetRandomFemaleCard(emptyRegisterSum);
            }
            if (!result.IsNullOrEmpty())
            {
                int cnt = 0;
                foreach (SaveData.Heroine item in randomHeroineList.Shuffle().Take(result.Length))
                {
                    item.SetCharFile(result[cnt++]);
                    item.charFileInitialized = true;
                    charaList.Add(item);
                    if (item.schoolClass == page)
                    {
                        charaPreviewList[item.schoolClassIndex].Set(item);
                    }
                }
            }
        }

        private IEnumerator LoadRandomSelectScene(int num)
        {
            bool isLoaded = false;
            string levelName = "NetworkCheckScene";
            Singleton<Character>.Instance.nextNetworkScene = "RandomNetChara";
            Singleton<Character>.Instance.netRandCharaNum = num;
            Singleton<Character>.Instance.netRandChara = null;
            Scene.Data data = new Scene.Data
            {
                levelName = levelName,
                isAdd = true,
                isFade = false,
                isAsync = true,
                onLoad = delegate
                {
                    isLoaded = true;
                    NetworkCheckScene rootComponent = Scene.GetRootComponent<NetworkCheckScene>(levelName);
                    if (rootComponent == null)
                    {
                        isVisible = true;
                    }
                    else
                    {
                        isVisible = false;
                        rootComponent.OnDestroyAsObservable().TakeUntilDestroy(this).Subscribe(delegate
                        {
                            isVisible = true;
                        });
                    }
                }
            };
            Singleton<Scene>.Instance.LoadReserve(data, true);
            yield return new WaitUntil(() => isLoaded);
            yield return new WaitUntil(() => isVisible);
        }
    }
}
