using ActionGame.Chara;
using ADV;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

namespace ActionGame
{
    public class MapCharaIconDraw : MonoBehaviour
    {
        [SerializeField]
        private Canvas _canvas;

        [SerializeField]
        private Button _button;

        private ActionScene _actScene;

        public Canvas canvas
        {
            get
            {
                return _canvas;
            }
        }

        public Button button
        {
            get
            {
                return _button;
            }
        }

        private ActionScene actScene
        {
            get
            {
                if (_actScene == null && Singleton<Game>.IsInstance())
                {
                    _actScene = Singleton<Game>.Instance.actScene;
                }
                return _actScene;
            }
        }

        private void CreateCharaIcon()
        {
            float canvasWidth = canvas.pixelRect.width;
            float initX = 10f;
            Rect set = new Rect(initX, 10f, 10f, 10f);
            bool isInit = false;
            Action<Image, SaveData.CharaData> iconSet = delegate (Image image, SaveData.CharaData data)
            {
                image.GetComponentInChildren<Text>().text = data.Name;
                if (isInit)
                {
                    set.x += set.width + image.rectTransform.rect.width;
                    if (set.x >= canvasWidth)
                    {
                        set.x = initX;
                        set.y += set.height + image.rectTransform.rect.height;
                    }
                }
                isInit = true;
                image.rectTransform.anchoredPosition = new Vector2(set.x, set.y);
            };
            List<NPC> list = actScene.npcList.Where(p => actScene.Map.no == p.mapNo).ToList();
            list.ForEach(delegate (NPC item)
            {
                SaveData.Heroine heroine = item.heroine;
                Button copy = Instantiate(button, canvas.transform, false);
                copy.gameObject.SetActive(true);
                copy.interactable = item.initialized;
                if (!copy.interactable)
                {
                    copy.UpdateAsObservable().SkipWhile(_ => !item.initialized).Take(1)
                        .Subscribe(delegate
                        {
                            copy.interactable = true;
                        });
                }
                copy.OnClickAsObservable().Subscribe(delegate
                {
                    actScene.SceneEvent(item);
                });
                iconSet(copy.image, heroine);
            });
        }

        private IEnumerator Start()
        {
            enabled = false;
            canvas.enabled = false;
            yield return new WaitUntil(Singleton<Scene>.IsInstance);
            if (!(actScene == null) && !(actScene.Map.Info.MapName == "後門"))
            {
                CanvasGroup canvasGroup = canvas.GetComponent<CanvasGroup>();
                if (canvasGroup != null)
                {
                    this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
                    {
                        canvasGroup.interactable = isOn;
                    });
                }
                enabled = true;
            }
        }

        private void Update()
        {
            bool enabled = canvas.enabled;
            canvas.enabled = !Program.isADVProcessing && !actScene.isEventNow;
            if (enabled != canvas.enabled && canvas.enabled)
            {
                List<Button> list = new List<Button>();
                IEnumerator enumerator = canvas.transform.GetEnumerator();
                try
                {
                    while (enumerator.MoveNext())
                    {
                        Transform transform = (Transform)enumerator.Current;
                        Button component = transform.GetComponent<Button>();
                        if (!(button == component) && component != null)
                        {
                            list.Add(component);
                        }
                    }
                }
                finally
                {
                    IDisposable disposable;
                    if ((disposable = enumerator as IDisposable) != null)
                    {
                        disposable.Dispose();
                    }
                }
                list.ForEach(delegate (Button p)
                {
                    Destroy(p.gameObject);
                });
                CreateCharaIcon();
            }
        }
    }
}
