using Illusion.Extensions;
using Manager;
using System.Linq;
using UniRx;
using UnityEngine;
using UnityEngine.UI;

namespace ActionGame
{
    public class PreviewClassData
    {
        public enum VisibleMode
        {
            None,
            CallName,
            Personality
        }

        private VisibleMode _visibleMode;

        private IntReactiveProperty _cardIndex = new IntReactiveProperty(0);

        private string callName;

        private string personality;

        private readonly StudentCardControlComponent[] _cards;

        private readonly GameObject _objHeart;

        private readonly GameObject _objMember;

        public GameObject rootObj
        {
            get;
            private set;
        }

        public SaveData.CharaData data
        {
            get;
            private set;
        }

        public Button button
        {
            get;
            private set;
        }

        public int classIndex
        {
            get;
            private set;
        }

        public bool isVisible
        {
            get
            {
                return rootObj.activeSelf;
            }
            set
            {
                rootObj.SetActive(value);
            }
        }

        public VisibleMode visibleMode
        {
            set
            {
                _visibleMode = value;
                bool flag = _visibleMode != 0 && data != null && data is SaveData.Heroine;
                switch (_visibleMode)
                {
                    case VisibleMode.CallName:
                        if (flag)
                        {
                            int fixCharaID = (data as SaveData.Heroine).fixCharaID;
                            flag = fixCharaID == 0 || fixCharaID == -8;
                        }
                        break;
                    case VisibleMode.Personality:
                        if (flag)
                        {
                            flag = (data as SaveData.Heroine).fixCharaID == 0;
                        }
                        break;
                }
                int num = _cardIndex.Value - 1;
                if (num >= 0)
                {
                    StudentCardControlComponent studentCardControlComponent = _cards[num];
                    if (flag)
                    {
                        studentCardControlComponent.SetNickName(_visibleMode != VisibleMode.CallName ? personality : callName);
                    }
                    studentCardControlComponent.ChangeDrawNickName(flag);
                }
            }
        }

        public PreviewClassData(GameObject rootObj)
        {
            this.rootObj = rootObj;
            classIndex = int.Parse(rootObj.name.Split('_')[1]);
            button = rootObj.GetOrAddComponent<Button>();
            ColorBlock colors = button.colors;
            colors.disabledColor = Color.white;
            button.colors = colors;
            Navigation navigation = new Navigation
            {
                mode = Navigation.Mode.None
            };
            button.navigation = navigation;
            GameObject[] array = rootObj.Children().ToArray();
            Image component = array[0].GetComponent<Image>();
            _cards = (from p in array.Skip(1)
                      select p.GetComponent<StudentCardControlComponent>() into p
                      where p != null
                      select p).ToArray();
            Image[] targetGraphics = new Image[1]
            {
                component
            }.Concat(_cards.Select(p => p.BaseImage)).ToArray();
            var selArray = array.Take(targetGraphics.Length).Select((o, i) => new
            {
                o,
                i
            }).ToArray();
            _cardIndex.Subscribe(delegate (int index)
            {
                var array3 = selArray;
                foreach (var anon in array3)
                {
                    anon.o.SetActive(anon.i == index);
                }
                button.targetGraphic = targetGraphics[index];
            });
            GameObject[] array2 = array.Skip(targetGraphics.Length).ToArray();
            _objHeart = array2.SafeGet(0);
            _objMember = array2.SafeGet(1);
        }

        public void Update()
        {
            if (data != null)
            {
                Set(data);
            }
        }

        public void Clear()
        {
            StudentCardControlComponent[] cards = _cards;
            foreach (StudentCardControlComponent studentCardControlComponent in cards)
            {
                studentCardControlComponent.Clear(Singleton<Game>.Instance.saveData.accademyName);
            }
            _cardIndex.Value = 0;
            _objHeart.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(false);
            });
            _objMember.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(false);
            });
            data = null;
        }

        public void Set(SaveData.CharaData charaData)
        {
            SaveData.Heroine heroine = charaData as SaveData.Heroine;
            int num = heroine == null || heroine.fixCharaID == 0 ? 1 : !heroine.isTeacher ? 3 : 2;
            _cardIndex.Value = num;
            data = charaData;
            callName = data.callName = data.GetCallName();
            SaveData saveData = Singleton<Game>.Instance.saveData;
            StudentCardControlComponent studentCardControlComponent = _cards[num - 1];
            if (heroine != null)
            {
                VoiceInfo.Param value;
                if (Singleton<Voice>.Instance.voiceInfoDic.TryGetValue(heroine.FixCharaIDOrPersonality, out value))
                {
                    personality = value.Personality;
                }
                else
                {
                    personality = "不明";
                }
            }
            bool isHeart = false;
            bool isMember = false;
            if (heroine != null)
            {
                switch (num)
                {
                    case 1:
                        isHeart = heroine.isGirlfriend;
                        isMember = heroine.isStaff;
                        break;
                    case 3:
                        isHeart = heroine.isTaked;
                        break;
                }
                Transform child = studentCardControlComponent.transform.GetChild(0);
                foreach (Transform item in from o in new GameObject[2]
                    {
                        _objMember,
                        _objHeart
                    }
                                           where o != null
                                           select o.transform)
                {
                    item.SetParent(child, true);
                    item.SetSiblingIndex(1);
                }
            }
            _objHeart.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(isHeart);
            });
            _objMember.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(isMember);
            });
            studentCardControlComponent.SetCharaInfo(charaData.charFile, saveData.emblemID, saveData.accademyName, _visibleMode != VisibleMode.CallName ? personality : callName);
            if (heroine != null && heroine.isTeacher)
            {
                studentCardControlComponent.SetChargeName(personality);
            }
        }
    }
}
