using Manager;
using System.Collections;
using UnityEngine;
using UnityEngine.UI;

namespace ActionGame
{
    public class bl_MiniMapItem : MonoBehaviour
    {
        [Tooltip("UI Prefab")]
        [Separator("TARGET")]
        public GameObject GraphicPrefab;

        [Tooltip("Transform to UI Icon will be follow")]
        public Transform Target;

        [Tooltip("Custom Position from target position")]
        public Vector3 OffSet = Vector3.zero;

        [Separator("ICON")]
        public Sprite Icon;

        public Color IconColor = new Color(1f, 1f, 1f, 0.9f);

        [Range(1f, 100f)]
        public float Size = 20f;

        [Tooltip("UI can interact when press it?")]
        [Separator("ICON BUTTON")]
        [CustomToggle("is Interactable")]
        public bool isInteractable = true;

        [Tooltip("Can Icon show when is off screen?")]
        [CustomToggle("Off Screen")]
        [Separator("SETTINGS")]
        public bool OffScreen = true;

        [Range(0f, 5f)]
        public float BorderOffScreen = 0.01f;

        [Range(1f, 50f)]
        public float OffScreenSize = 10f;

        [Tooltip("Time before render/show item in minimap after instance")]
        [Range(0f, 3f)]
        public float RenderDelay = 0.3f;

        public ItemEffect m_Effect = ItemEffect.None;

        private bl_MiniMap MiniMap;

        private RectTransform RectRoot;

        private GameObject cacheItem;

        private RectTransform graphicRectTransform;

        private bl_IconItem _iconItem;

        private static int TypeHash = Animator.StringToHash("Type");

        private bl_MiniMap _minimap;

        public bl_IconItem iconItem
        {
            get
            {
                return _iconItem;
            }
        }

        public Vector3 TargetPosition
        {
            get
            {
                if (Target == null)
                {
                    return Vector3.zero;
                }
                Vector3 position = Target.position;
                float x = position.x;
                Vector3 position2 = Target.position;
                return new Vector3(x, 0f, position2.z);
            }
        }

        private bl_MiniMap m_miniMap
        {
            get
            {
                if (_minimap == null)
                {
                    _minimap = cacheItem.transform.root.GetComponentInChildren<bl_MiniMap>();
                }
                return _minimap;
            }
        }

        public void SetVisible(bool visible)
        {
            _iconItem.visible = visible;
        }

        private IEnumerator Start()
        {
            enabled = false;
            yield return new WaitWhile(() => !Singleton<Game>.IsInstance());
            yield return new WaitWhile(() => Singleton<Game>.Instance.actScene == null);
            MiniMap = Singleton<Game>.Instance.actScene.miniMap;
            if (bl_MiniMap.MapUIRoot != null)
            {
                CreateIcon();
            }
            enabled = true;
        }

        private void CreateIcon()
        {
            RectRoot = MiniMap.MMUIRoot;
            cacheItem = Instantiate(GraphicPrefab, RectRoot, false);
            cacheItem.name = GraphicPrefab.name;
            _iconItem = cacheItem.GetComponent<bl_IconItem>();
            SetVisible(false);
            Image graphic = _iconItem.graphic;
            graphicRectTransform = graphic.rectTransform;
            graphicRectTransform.anchoredPosition = Vector2.zero;
            if (Icon != null)
            {
                graphic.sprite = Icon;
                graphic.color = IconColor;
            }
            _iconItem.canvasGroup.interactable = isInteractable;
            if (Target == null)
            {
                Target = transform;
            }
            StartEffect();
            _iconItem.DelayStart(RenderDelay);
        }

        private void Update()
        {
            Vector3 position = TargetPosition + OffSet;
            Vector2 vector = bl_MiniMap.MiniMapCamera.WorldToViewportPoint(position);
            float x = vector.x;
            Vector2 sizeDelta = RectRoot.sizeDelta;
            float num = x * sizeDelta.x;
            Vector2 sizeDelta2 = RectRoot.sizeDelta;
            float x2 = num - sizeDelta2.x * 0.5f;
            float y = vector.y;
            Vector2 sizeDelta3 = RectRoot.sizeDelta;
            float num2 = y * sizeDelta3.y;
            Vector2 sizeDelta4 = RectRoot.sizeDelta;
            Vector3 v = new Vector2(x2, num2 - sizeDelta4.y * 0.5f);
            if (OffScreen)
            {
                float x3 = v.x;
                Vector2 sizeDelta5 = RectRoot.sizeDelta;
                float min = 0f - (sizeDelta5.x * 0.5f - BorderOffScreen);
                Vector2 sizeDelta6 = RectRoot.sizeDelta;
                v.x = Mathf.Clamp(x3, min, sizeDelta6.x * 0.5f - BorderOffScreen);
                float y2 = v.y;
                Vector2 sizeDelta7 = RectRoot.sizeDelta;
                float min2 = 0f - (sizeDelta7.y * 0.5f - BorderOffScreen);
                Vector2 sizeDelta8 = RectRoot.sizeDelta;
                v.y = Mathf.Clamp(y2, min2, sizeDelta8.y * 0.5f - BorderOffScreen);
            }
            float size;
            if (m_miniMap.useCompassRotation)
            {
                Vector3 zero = Vector3.zero;
                Vector3 direction = Target.position - m_miniMap.TargetPosition;
                Vector3 vector2 = bl_MiniMap.MiniMapCamera.transform.InverseTransformDirection(direction);
                vector2.z = 0f;
                vector2 = vector2.normalized / 2f;
                float num3 = Mathf.Abs(v.x);
                float num4 = Mathf.Abs(0.5f + vector2.x * m_miniMap.CompassSize);
                if (num3 >= num4)
                {
                    zero.x = 0.5f + vector2.x * m_miniMap.CompassSize;
                    zero.y = 0.5f + vector2.y * m_miniMap.CompassSize;
                    v = zero;
                    size = OffScreenSize;
                }
                else
                {
                    size = Size;
                }
            }
            else
            {
                float x4 = v.x;
                Vector2 sizeDelta9 = RectRoot.sizeDelta;
                if (x4 != sizeDelta9.x * 0.5f - BorderOffScreen)
                {
                    float y3 = v.y;
                    Vector2 sizeDelta10 = RectRoot.sizeDelta;
                    if (y3 != sizeDelta10.y * 0.5f - BorderOffScreen)
                    {
                        float x5 = v.x;
                        Vector2 sizeDelta11 = RectRoot.sizeDelta;
                        if (x5 != 0f - sizeDelta11.x * 0.5f - BorderOffScreen)
                        {
                            float num5 = 0f - v.y;
                            Vector2 sizeDelta12 = RectRoot.sizeDelta;
                            if (num5 != sizeDelta12.y * 0.5f - BorderOffScreen)
                            {
                                size = Size;
                                goto IL_031f;
                            }
                        }
                    }
                }
                size = OffScreenSize;
            }
            goto IL_031f;
        IL_031f:
            RectTransform rectTransform = graphicRectTransform;
            rectTransform.anchoredPosition = v;
            float num6 = size * MiniMap.IconMultiplier;
            rectTransform.sizeDelta = Vector2.Lerp(rectTransform.sizeDelta, new Vector2(num6, num6), Time.deltaTime * 8f);
            if (MiniMap.RotationAlwaysFront)
            {
                Quaternion identity = Quaternion.identity;
                Quaternion rotation = Target.rotation;
                identity.x = rotation.x;
                rectTransform.localRotation = identity;
            }
            else
            {
                Vector3 eulerAngles = MiniMap.transform.eulerAngles;
                Vector3 zero2 = Vector3.zero;
                Vector3 eulerAngles2 = Target.rotation.eulerAngles;
                zero2.z = 0f - eulerAngles2.y + eulerAngles.y;
                Quaternion quaternion2 = rectTransform.rotation = Quaternion.Euler(zero2);
            }
        }

        private void StartEffect()
        {
            Animator animator = _iconItem.animator;
            if (!(animator == null))
            {
                switch (m_Effect)
                {
                    case ItemEffect.Pulsing:
                        animator.SetInteger(TypeHash, 2);
                        break;
                    case ItemEffect.Fade:
                        animator.SetInteger(TypeHash, 1);
                        break;
                }
            }
        }

        public void DestroyItem()
        {
            if (!(_iconItem == null))
            {
                _iconItem.DestroyIcon();
            }
        }

        public void SetIcon(Sprite ico)
        {
            _iconItem.SetIcon(ico);
        }

        private void OnDestroy()
        {
            DestroyItem();
        }
    }
}
