using System;
using System.Collections;
using System.IO;
using System.Text;
using UnityEngine;

public class BaseLoader : MonoBehaviour
{
    [SerializeField]
    protected bool isErase;

    public const string LocalPath = "file://";

    public const string NetWorkPath = "http://";

    protected virtual void Awake()
    {
        Initialize();
        if (isErase)
        {
            Destroy(gameObject);
        }
    }

    protected void Initialize()
    {
        if (!Singleton<AssetBundleManager>.IsInstance())
        {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.Append(Application.dataPath);
            string value = "/../abdata" + '/';
            stringBuilder.Append(value);
            AssetBundleManager.Initialize(stringBuilder.ToString());
        }
    }

    public string GetRelativePath()
    {
        if (Application.isEditor)
        {
            return "file://" + Environment.CurrentDirectory.Replace("\\", "/");
        }
        if (Application.isWebPlayer)
        {
            return Path.GetDirectoryName(Application.absoluteURL).Replace("\\", "/") + "/StreamingAssets";
        }
        if (Application.isMobilePlatform || Application.isConsolePlatform)
        {
            return Application.streamingAssetsPath;
        }
        return "file://" + Application.streamingAssetsPath;
    }

    private static string GetPlatformFolderForAssetBundles(RuntimePlatform platform)
    {
        switch (platform)
        {
            case RuntimePlatform.Android:
                return "Android";
            case RuntimePlatform.IPhonePlayer:
                return "iOS";
            case RuntimePlatform.WindowsPlayer:
                return "Windows";
            case RuntimePlatform.OSXPlayer:
                return "OSX";
            default:
                return null;
        }
    }

    protected T Load<T>(string assetBundleName, string assetName, bool isClone = false, string manifestAssetBundleName = null) where T : UnityEngine.Object
    {
        T val = new AssetBundleManifestData(assetBundleName, assetName, manifestAssetBundleName).GetAsset<T>();
        if (val != null && isClone)
        {
            T val2 = Instantiate(val);
            val2.name = val.name;
            val = val2;
        }
        return val;
    }

    protected IEnumerator Load_Coroutine<T>(string assetBundleName, string assetName, Action<T> act = null, bool isClone = false, string manifestAssetBundleName = null) where T : UnityEngine.Object
    {
        T asset = (T)(object)null;
        yield return new AssetBundleManifestData(assetBundleName, assetName, manifestAssetBundleName).GetAsset<T>(delegate (T x)
        {
            asset = x;
        });
        if (asset == null)
        {
            yield break;
        }
        if (isClone)
        {
            T asset2 = Instantiate(asset);
            asset2.name = asset.name;
            asset = asset2;
        }
        act.Call(asset);
        yield break;
    }
}
