using System;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CustomAcsMoveWindow : MonoBehaviour
    {
        public enum AcsSlotNo
        {
            Slot01,
            Slot02,
            Slot03,
            Slot04,
            Slot05,
            Slot06,
            Slot07,
            Slot08,
            Slot09,
            Slot10,
            Slot11,
            Slot12,
            Slot13,
            Slot14,
            Slot15,
            Slot16,
            Slot17,
            Slot18,
            Slot19,
            Slot20
        }

        [Serializable]
        public class TypeReactiveProperty : ReactiveProperty<AcsSlotNo>
        {
            public TypeReactiveProperty()
            {
            }

            public TypeReactiveProperty(AcsSlotNo initialValue)
                : base(initialValue)
            {
            }
        }

        private readonly float[] movePosValue = new float[3]
        {
            0.1f,
            1f,
            10f
        };

        private readonly float[] moveRotValue = new float[3]
        {
            1f,
            5f,
            10f
        };

        private readonly float[] moveSclValue = new float[3]
        {
            0.01f,
            0.1f,
            1f
        };

        [SerializeField]
        private TypeReactiveProperty _slotNo = new TypeReactiveProperty(AcsSlotNo.Slot01);

        public int correctNo;

        [SerializeField]
        private TextMeshProUGUI textTitle;

        [SerializeField]
        private Toggle[] tglPosRate;

        [SerializeField]
        private Button[] btnPos;

        [SerializeField]
        private TMP_InputField[] inpPos;

        [SerializeField]
        private Button[] btnPosReset;

        [SerializeField]
        private Toggle[] tglRotRate;

        [SerializeField]
        private Button[] btnRot;

        [SerializeField]
        private TMP_InputField[] inpRot;

        [SerializeField]
        private Button[] btnRotReset;

        [SerializeField]
        private Toggle[] tglSclRate;

        [SerializeField]
        private Button[] btnScl;

        [SerializeField]
        private TMP_InputField[] inpScl;

        [SerializeField]
        private Button[] btnSclReset;

        [SerializeField]
        private Button btnCopy;

        [SerializeField]
        private Button btnPaste;

        [SerializeField]
        private Button btnAllReset;

        [SerializeField]
        private Button btnClose;

        [SerializeField]
        private Toggle tglReference;

        [SerializeField]
        private CvsAccessory[] cvsAccessory;

        public AcsSlotNo slotNo
        {
            get
            {
                return _slotNo.Value;
            }
            set
            {
                _slotNo.Value = value;
            }
        }

        public int nSlotNo
        {
            get
            {
                return (int)_slotNo.Value;
            }
        }

        private ChaControl chaCtrl
        {
            get
            {
                return Singleton<CustomBase>.Instance.chaCtrl;
            }
        }

        private ChaFileAccessory accessory
        {
            get
            {
                return chaCtrl.nowCoordinate.accessory;
            }
        }

        private ChaFileAccessory setAccessory
        {
            get
            {
                return chaCtrl.chaFile.coordinate[chaCtrl.chaFile.status.coordinateType].accessory;
            }
        }

        private ChaFileAccessory defAccessory
        {
            get
            {
                return Singleton<CustomBase>.Instance.defChaInfo.coordinate[chaCtrl.chaFile.status.coordinateType].accessory;
            }
        }

        public void UpdateWindow()
        {
            if ((bool)textTitle)
            {
                textTitle.text = string.Format("スロット{0:00}の調整{1:00}", nSlotNo + 1, correctNo + 1);
            }
            int num = Singleton<CustomBase>.Instance.customSettingSave.acsCorrectPosRate[correctNo];
            int num2 = Singleton<CustomBase>.Instance.customSettingSave.acsCorrectRotRate[correctNo];
            int num3 = Singleton<CustomBase>.Instance.customSettingSave.acsCorrectSclRate[correctNo];
            for (int i = 0; i < 3; i++)
            {
                tglPosRate[i].isOn = i == num;
                tglRotRate[i].isOn = i == num2;
                tglSclRate[i].isOn = i == num3;
            }
            UpdateCustomUI();
        }

        public void ChangeSlot(int _no, bool open)
        {
            slotNo = (AcsSlotNo)_no;
            bool isOn = tglReference.isOn;
            tglReference.isOn = false;
            if (correctNo == 0)
            {
                tglReference = cvsAccessory[nSlotNo].tglAcsMove01;
            }
            else
            {
                tglReference = cvsAccessory[nSlotNo].tglAcsMove02;
            }
            if (open && isOn)
            {
                tglReference.isOn = true;
            }
        }

        public void CloseWindow()
        {
            tglReference.isOn = false;
        }

        public void UpdateCustomUI(int param = 0)
        {
            for (int i = 0; i < 3; i++)
            {
                inpPos[i].text = accessory.parts[nSlotNo].addMove[correctNo, 0][i].ToString();
                inpRot[i].text = accessory.parts[nSlotNo].addMove[correctNo, 1][i].ToString();
                inpScl[i].text = accessory.parts[nSlotNo].addMove[correctNo, 2][i].ToString();
            }
        }

        public void UpdateDragValue(int type, int xyz, float move)
        {
            switch (type)
            {
                case 0:
                    {
                        float val3 = move * movePosValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectPosRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsPosAdd(correctNo, xyz, true, val3);
                        inpPos[xyz].text = accessory.parts[nSlotNo].addMove[correctNo, 0][xyz].ToString();
                        break;
                    }
                case 1:
                    {
                        float val2 = move * moveRotValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectRotRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsRotAdd(correctNo, xyz, true, val2);
                        inpRot[xyz].text = accessory.parts[nSlotNo].addMove[correctNo, 1][xyz].ToString();
                        break;
                    }
                case 2:
                    {
                        float val = move * moveSclValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectSclRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsSclAdd(correctNo, xyz, true, val);
                        inpScl[xyz].text = accessory.parts[nSlotNo].addMove[correctNo, 2][xyz].ToString();
                        break;
                    }
            }
            cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
        }

        protected virtual void Awake()
        {
        }

        protected virtual void Start()
        {
            for (int i = 0; i < 3; i++)
            {
                Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpPos[i]);
                Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpRot[i]);
                Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpScl[i]);
            }
            _slotNo.TakeUntilDestroy(this).Subscribe(delegate
            {
                UpdateWindow();
            });
            if ((bool)btnClose)
            {
                btnClose.OnClickAsObservable().Subscribe(delegate
                {
                    if ((bool)tglReference)
                    {
                        tglReference.isOn = false;
                    }
                });
            }
            tglPosRate.Select((p, idx) => new
            {
                toggle = p,
                index = (byte)idx
            }).ToList().ForEach(p =>
            {
                (from isOn in p.toggle.OnValueChangedAsObservable()
                 where isOn
                 select isOn).Subscribe(delegate
             {
                 Singleton<CustomBase>.Instance.customSettingSave.acsCorrectPosRate[correctNo] = p.index;
             });
            });
            tglRotRate.Select((p, idx) => new
            {
                toggle = p,
                index = (byte)idx
            }).ToList().ForEach(p =>
            {
                (from isOn in p.toggle.OnValueChangedAsObservable()
                 where isOn
                 select isOn).Subscribe(delegate
             {
                 Singleton<CustomBase>.Instance.customSettingSave.acsCorrectRotRate[correctNo] = p.index;
             });
            });
            tglSclRate.Select((p, idx) => new
            {
                toggle = p,
                index = (byte)idx
            }).ToList().ForEach(p =>
            {
                (from isOn in p.toggle.OnValueChangedAsObservable()
                 where isOn
                 select isOn).Subscribe(delegate
             {
                 Singleton<CustomBase>.Instance.customSettingSave.acsCorrectSclRate[correctNo] = p.index;
             });
            });
            float downTimeCnt = 0f;
            float loopTimeCnt = 0f;
            bool change = false;
            btnPos.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    if (!change)
                    {
                        int num17 = p.index / 2;
                        int num18 = p.index % 2 != 0 ? 1 : -1;
                        if (num17 == 0)
                        {
                            num18 *= -1;
                        }
                        float val6 = num18 * movePosValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectPosRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsPosAdd(correctNo, num17, true, val6);
                        inpPos[num17].text = accessory.parts[nSlotNo].addMove[correctNo, 0][num17].ToString();
                        cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                    }
                });
                p.btn.UpdateAsObservable().SkipUntil(p.btn.OnPointerDownAsObservable().Do(delegate
                {
                    downTimeCnt = 0f;
                    loopTimeCnt = 0f;
                    change = false;
                })).TakeUntil(p.btn.OnPointerUpAsObservable())
                    .RepeatUntilDestroy(this)
                    .Subscribe(delegate
                    {
                        int num13 = p.index / 2;
                        int num14 = p.index % 2 != 0 ? 1 : -1;
                        if (num13 == 0)
                        {
                            num14 *= -1;
                        }
                        float num15 = num14 * movePosValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectPosRate[correctNo]];
                        float num16 = 0f;
                        downTimeCnt += Time.deltaTime;
                        if (downTimeCnt > 0.3f)
                        {
                            for (loopTimeCnt += Time.deltaTime; loopTimeCnt > 0.05f; loopTimeCnt -= 0.05f)
                            {
                                num16 += num15;
                            }
                            if (num16 != 0f)
                            {
                                cvsAccessory[nSlotNo].FuncUpdateAcsPosAdd(correctNo, num13, true, num16);
                                inpPos[num13].text = accessory.parts[nSlotNo].addMove[correctNo, 0][num13].ToString();
                                change = true;
                                cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                            }
                        }
                    })
                    .AddTo(this);
            });
            inpPos.Select((p, idx) => new
            {
                inp = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.inp.onEndEdit.AsObservable().Subscribe(delegate (string value)
                {
                    int xyz3 = p.index % 3;
                    float val5 = CustomBase.ConvertValueFromTextLimit(-100f, 100f, 1, value);
                    p.inp.text = val5.ToString();
                    cvsAccessory[nSlotNo].FuncUpdateAcsPosAdd(correctNo, xyz3, false, val5);
                    cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                });
            });
            btnPosReset.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    inpPos[p.index].text = "0";
                    cvsAccessory[nSlotNo].FuncUpdateAcsPosAdd(correctNo, p.index, false, 0f);
                    cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                });
            });
            btnRot.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    if (!change)
                    {
                        int num11 = p.index / 2;
                        int num12 = p.index % 2 != 0 ? 1 : -1;
                        float val4 = num12 * moveRotValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectRotRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsRotAdd(correctNo, num11, true, val4);
                        inpRot[num11].text = accessory.parts[nSlotNo].addMove[correctNo, 1][num11].ToString();
                        cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                    }
                });
                p.btn.UpdateAsObservable().SkipUntil(p.btn.OnPointerDownAsObservable().Do(delegate
                {
                    downTimeCnt = 0f;
                    loopTimeCnt = 0f;
                    change = false;
                })).TakeUntil(p.btn.OnPointerUpAsObservable())
                    .RepeatUntilDestroy(this)
                    .Subscribe(delegate
                    {
                        int num7 = p.index / 2;
                        int num8 = p.index % 2 != 0 ? 1 : -1;
                        float num9 = num8 * moveRotValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectRotRate[correctNo]];
                        float num10 = 0f;
                        downTimeCnt += Time.deltaTime;
                        if (downTimeCnt > 0.3f)
                        {
                            for (loopTimeCnt += Time.deltaTime; loopTimeCnt > 0.05f; loopTimeCnt -= 0.05f)
                            {
                                num10 += num9;
                            }
                            if (num10 != 0f)
                            {
                                cvsAccessory[nSlotNo].FuncUpdateAcsRotAdd(correctNo, num7, true, num10);
                                inpRot[num7].text = accessory.parts[nSlotNo].addMove[correctNo, 1][num7].ToString();
                                change = true;
                                cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                            }
                        }
                    })
                    .AddTo(this);
            });
            inpRot.Select((p, idx) => new
            {
                inp = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.inp.onEndEdit.AsObservable().Subscribe(delegate (string value)
                {
                    int xyz2 = p.index % 3;
                    float val3 = CustomBase.ConvertValueFromTextLimit(0f, 360f, 0, value);
                    p.inp.text = val3.ToString();
                    cvsAccessory[nSlotNo].FuncUpdateAcsRotAdd(correctNo, xyz2, false, val3);
                    cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                });
            });
            btnRotReset.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    inpRot[p.index].text = "0";
                    cvsAccessory[nSlotNo].FuncUpdateAcsRotAdd(correctNo, p.index, false, 0f);
                    cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
                });
            });
            btnScl.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    if (!change)
                    {
                        int num5 = p.index / 2;
                        int num6 = p.index % 2 != 0 ? 1 : -1;
                        float val2 = num6 * moveSclValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectSclRate[correctNo]];
                        cvsAccessory[nSlotNo].FuncUpdateAcsSclAdd(correctNo, num5, true, val2);
                        inpScl[num5].text = accessory.parts[nSlotNo].addMove[correctNo, 2][num5].ToString();
                    }
                });
                p.btn.UpdateAsObservable().SkipUntil(p.btn.OnPointerDownAsObservable().Do(delegate
                {
                    downTimeCnt = 0f;
                    loopTimeCnt = 0f;
                    change = false;
                })).TakeUntil(p.btn.OnPointerUpAsObservable())
                    .RepeatUntilDestroy(this)
                    .Subscribe(delegate
                    {
                        int num = p.index / 2;
                        int num2 = p.index % 2 != 0 ? 1 : -1;
                        float num3 = num2 * moveSclValue[Singleton<CustomBase>.Instance.customSettingSave.acsCorrectSclRate[correctNo]];
                        float num4 = 0f;
                        downTimeCnt += Time.deltaTime;
                        if (downTimeCnt > 0.3f)
                        {
                            for (loopTimeCnt += Time.deltaTime; loopTimeCnt > 0.05f; loopTimeCnt -= 0.05f)
                            {
                                num4 += num3;
                            }
                            if (num4 != 0f)
                            {
                                cvsAccessory[nSlotNo].FuncUpdateAcsSclAdd(correctNo, num, true, num4);
                                inpScl[num].text = accessory.parts[nSlotNo].addMove[correctNo, 2][num].ToString();
                                change = true;
                            }
                        }
                    })
                    .AddTo(this);
            });
            inpScl.Select((p, idx) => new
            {
                inp = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.inp.onEndEdit.AsObservable().Subscribe(delegate (string value)
                {
                    int xyz = p.index % 3;
                    float val = CustomBase.ConvertValueFromTextLimit(0.01f, 100f, 2, value);
                    p.inp.text = val.ToString();
                    cvsAccessory[nSlotNo].FuncUpdateAcsSclAdd(correctNo, xyz, false, val);
                });
            });
            btnSclReset.Select((p, idx) => new
            {
                btn = p,
                index = idx
            }).ToList().ForEach(p =>
            {
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    inpScl[p.index].text = "1";
                    cvsAccessory[nSlotNo].FuncUpdateAcsSclAdd(correctNo, p.index, false, 1f);
                });
            });
            btnCopy.OnClickAsObservable().Subscribe(delegate
            {
                Singleton<CustomBase>.Instance.vecAcsClipBord[0] = accessory.parts[nSlotNo].addMove[correctNo, 0];
                Singleton<CustomBase>.Instance.vecAcsClipBord[1] = accessory.parts[nSlotNo].addMove[correctNo, 1];
                Singleton<CustomBase>.Instance.vecAcsClipBord[2] = accessory.parts[nSlotNo].addMove[correctNo, 2];
            });
            btnPaste.OnClickAsObservable().Subscribe(delegate
            {
                cvsAccessory[nSlotNo].FuncUpdateAcsMovePaste(correctNo, Singleton<CustomBase>.Instance.vecAcsClipBord);
                inpPos[0].text = accessory.parts[nSlotNo].addMove[correctNo, 0].x.ToString();
                inpPos[1].text = accessory.parts[nSlotNo].addMove[correctNo, 0].y.ToString();
                inpPos[2].text = accessory.parts[nSlotNo].addMove[correctNo, 0].z.ToString();
                inpRot[0].text = accessory.parts[nSlotNo].addMove[correctNo, 1].x.ToString();
                inpRot[1].text = accessory.parts[nSlotNo].addMove[correctNo, 1].y.ToString();
                inpRot[2].text = accessory.parts[nSlotNo].addMove[correctNo, 1].z.ToString();
                inpScl[0].text = accessory.parts[nSlotNo].addMove[correctNo, 2].x.ToString();
                inpScl[1].text = accessory.parts[nSlotNo].addMove[correctNo, 2].y.ToString();
                inpScl[2].text = accessory.parts[nSlotNo].addMove[correctNo, 2].z.ToString();
                cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
            });
            btnAllReset.OnClickAsObservable().Subscribe(delegate
            {
                for (int j = 0; j < 3; j++)
                {
                    inpPos[j].text = "0";
                    inpRot[j].text = "0";
                    inpScl[j].text = "1";
                }
                cvsAccessory[nSlotNo].FuncUpdateAcsAllReset(correctNo);
                cvsAccessory[nSlotNo].SetControllerTransform(correctNo);
            });
        }
    }
}
