using Illusion.Game;
using Manager;
using System;
using System.IO;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;

namespace ChaCustom
{
    public class CustomCharaFile : MonoBehaviour
    {
        [SerializeField]
        private CustomFileListCtrl listCtrl;

        [SerializeField]
        private CustomFileWindow fileWindow;

        [SerializeField]
        private CustomCheckWindow checkWindow;

        private CustomBase customBase
        {
            get
            {
                return Singleton<CustomBase>.Instance;
            }
        }

        private ChaControl chaCtrl
        {
            get
            {
                return customBase.chaCtrl;
            }
        }

        private ChaFileControl chaFile
        {
            get
            {
                return chaCtrl.chaFile;
            }
        }

        private void Start()
        {
            Initialize();
            if (!fileWindow)
            {
                return;
            }
            if (!customBase.modeNew)
            {
                fileWindow.tglChaLoadParam.interactable = false;
                Transform transform = fileWindow.tglChaLoadParam.transform.Find("textLoad");
                if ((bool)transform)
                {
                    TextMeshProUGUI component = transform.GetComponent<TextMeshProUGUI>();
                    if ((bool)component)
                    {
                        component.color = Color.gray;
                    }
                }
                fileWindow.tglChaLoadParam.isOn = false;
            }
            if ((bool)fileWindow.btnChaLoadAllOn)
            {
                fileWindow.btnChaLoadAllOn.OnClickAsObservable().Subscribe(delegate
                {
                    if ((bool)fileWindow.tglChaLoadFace)
                    {
                        fileWindow.tglChaLoadFace.isOn = true;
                    }
                    if ((bool)fileWindow.tglChaLoadBody)
                    {
                        fileWindow.tglChaLoadBody.isOn = true;
                    }
                    if ((bool)fileWindow.tglChaLoadHair)
                    {
                        fileWindow.tglChaLoadHair.isOn = true;
                    }
                    if (customBase.modeNew && fileWindow.tglChaLoadParam)
                    {
                        fileWindow.tglChaLoadParam.isOn = true;
                    }
                    if ((bool)fileWindow.tglChaLoadCoorde)
                    {
                        fileWindow.tglChaLoadCoorde.isOn = true;
                    }
                });
            }
            if ((bool)fileWindow.btnChaLoadAllOff)
            {
                fileWindow.btnChaLoadAllOff.OnClickAsObservable().Subscribe(delegate
                {
                    if ((bool)fileWindow.tglChaLoadFace)
                    {
                        fileWindow.tglChaLoadFace.isOn = false;
                    }
                    if ((bool)fileWindow.tglChaLoadBody)
                    {
                        fileWindow.tglChaLoadBody.isOn = false;
                    }
                    if ((bool)fileWindow.tglChaLoadHair)
                    {
                        fileWindow.tglChaLoadHair.isOn = false;
                    }
                    if ((bool)fileWindow.tglChaLoadParam)
                    {
                        fileWindow.tglChaLoadParam.isOn = false;
                    }
                    if ((bool)fileWindow.tglChaLoadCoorde)
                    {
                        fileWindow.tglChaLoadCoorde.isOn = false;
                    }
                });
            }
            if (fileWindow.btnChaLoadLoad && null != listCtrl)
            {
                fileWindow.btnChaLoadLoad.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.ok_s);
                    CustomFileInfoComponent selectTopItem = listCtrl.GetSelectTopItem();
                    if (null != selectTopItem)
                    {
                        ChaControl chaCtrl = Singleton<CustomBase>.Instance.chaCtrl;
                        string fullPath = selectTopItem.info.FullPath;
                        bool flag6 = fileWindow.tglChaLoadFace && fileWindow.tglChaLoadFace.isOn;
                        bool flag7 = fileWindow.tglChaLoadBody && fileWindow.tglChaLoadBody.isOn;
                        bool flag8 = fileWindow.tglChaLoadHair && fileWindow.tglChaLoadHair.isOn;
                        bool parameter = fileWindow.tglChaLoadParam && fileWindow.tglChaLoadParam.isOn;
                        bool flag9 = fileWindow.tglChaLoadCoorde && fileWindow.tglChaLoadCoorde.isOn;
                        chaFile.LoadFileLimited(fullPath, chaCtrl.sex, flag6, flag7, flag8, parameter, flag9);
                        chaCtrl.ChangeCoordinateType();
                        chaCtrl.Reload(!flag9, !flag6 && !flag9, !flag8, !flag7);
                        Singleton<CustomBase>.Instance.updateCustomUI = true;
                    }
                });
            }
            if ((bool)fileWindow.btnSave)
            {
                fileWindow.btnSave.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.window_o);
                    Singleton<CustomBase>.Instance.customCtrl.saveFileName = string.Empty;
                    Singleton<CustomBase>.Instance.customCtrl.saveFileListCtrl = listCtrl;
                    Singleton<CustomBase>.Instance.customCtrl.saveNew = true;
                    Singleton<CustomBase>.Instance.customCtrl.saveMode = true;
                });
            }
            this.UpdateAsObservable().Subscribe(delegate
            {
                int[] selectIndex = listCtrl.GetSelectIndex();
                bool interactable = selectIndex != null && selectIndex.Length > 0;
                if ((bool)fileWindow.btnOverwrite)
                {
                    fileWindow.btnOverwrite.interactable = interactable;
                }
                if ((bool)fileWindow.btnDelete)
                {
                    fileWindow.btnDelete.interactable = interactable;
                }
                bool flag = fileWindow.tglChaLoadFace && fileWindow.tglChaLoadFace.isOn;
                bool flag2 = fileWindow.tglChaLoadBody && fileWindow.tglChaLoadBody.isOn;
                bool flag3 = fileWindow.tglChaLoadHair && fileWindow.tglChaLoadHair.isOn;
                bool flag4 = fileWindow.tglChaLoadParam && fileWindow.tglChaLoadParam.isOn;
                bool flag5 = fileWindow.tglChaLoadCoorde && fileWindow.tglChaLoadCoorde.isOn;
                if (!flag && !flag2 && !flag3 && !flag4 && !flag5)
                {
                    interactable = false;
                }
                if ((bool)fileWindow.btnChaLoadLoad)
                {
                    fileWindow.btnChaLoadLoad.interactable = interactable;
                }
            });
            if ((bool)fileWindow.btnOverwrite)
            {
                fileWindow.btnOverwrite.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.window_o);
                    checkWindow.Setup(CustomCheckWindow.CheckType.CharaOverwrite, "キャラ画像を変更しますか？", null, null, OverwriteCharaFileWithCapture, OverwriteCharaFile, null);
                });
            }
            if ((bool)fileWindow.btnDelete)
            {
                fileWindow.btnDelete.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.window_o);
                    checkWindow.Setup(CustomCheckWindow.CheckType.YesNo, "本当に削除しますか？", null, null, DeleteCharaFile, null);
                });
            }
        }

        private void Initialize()
        {
            int modeSex = Singleton<CustomBase>.Instance.modeSex;
            FolderAssist folderAssist = new FolderAssist();
            string[] searchPattern = new string[1]
            {
                "*.png"
            };
            string folder = UserData.Path + (modeSex != 0 ? "chara/female/" : "chara/male/");
            folderAssist.CreateFolderInfoEx(folder, searchPattern);
            listCtrl.ClearList();
            int fileCount = folderAssist.GetFileCount();
            int num = 0;
            for (int i = 0; i < fileCount; i++)
            {
                ChaFileControl chaFileControl = new ChaFileControl();
                if (!chaFileControl.LoadCharaFile(folderAssist.lstFile[i].FullPath))
                {
                    int lastErrorCode = chaFileControl.GetLastErrorCode();
                }
                else if (chaFileControl.parameter.sex == modeSex)
                {
                    string club = string.Empty;
                    string personality = string.Empty;
                    if (modeSex != 0)
                    {
                        VoiceInfo.Param value;
                        personality = Singleton<Voice>.Instance.voiceInfoDic.TryGetValue(chaFileControl.parameter.personality, out value) ? value.Personality : "不明";
                        ClubInfo.Param value2;
                        club = Game.ClubInfos.TryGetValue(chaFileControl.parameter.clubActivities, out value2) ? value2.Name : "不明";
                    }
                    else
                    {
                        listCtrl.DisableAddInfo();
                    }
                    listCtrl.AddList(num, chaFileControl.parameter.fullname, club, personality, folderAssist.lstFile[i].FullPath, folderAssist.lstFile[i].FileName, folderAssist.lstFile[i].time);
                    num++;
                }
            }
            listCtrl.Create(OnChangeSelect);
        }

        public void DeleteCharaFile(string buf)
        {
            int[] selectIndex = listCtrl.GetSelectIndex();
            CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
            if (File.Exists(fileInfoFromIndex.FullPath))
            {
                File.Delete(fileInfoFromIndex.FullPath);
            }
            listCtrl.RemoveList(selectIndex[0]);
            listCtrl.ReCreate();
        }

        public void OverwriteCharaFileWithCapture(string buf)
        {
            int[] selectIndex = listCtrl.GetSelectIndex();
            CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
            Singleton<CustomBase>.Instance.customCtrl.saveFileName = fileInfoFromIndex.FileName;
            Singleton<CustomBase>.Instance.customCtrl.saveFileListCtrl = listCtrl;
            Singleton<CustomBase>.Instance.customCtrl.saveNew = false;
            Singleton<CustomBase>.Instance.customCtrl.saveMode = true;
        }

        public void OverwriteCharaFile(string buf)
        {
            int[] selectIndex = listCtrl.GetSelectIndex();
            CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
            ChaFileControl chaFileControl = new ChaFileControl();
            if (!chaFileControl.LoadCharaFile(fileInfoFromIndex.FullPath))
            {
            }
            if (chaFileControl.facePngData != null && chaFileControl.pngData != null)
            {
                chaFile.facePngData = chaFileControl.facePngData;
                chaFile.pngData = chaFileControl.pngData;
            }
            chaFile.SaveCharaFile(fileInfoFromIndex.FullPath);
            fileInfoFromIndex.time = DateTime.Now;
            fileInfoFromIndex.name = chaFile.parameter.fullname;
            fileInfoFromIndex.DeleteThumb();
        }

        public void OnChangeSelect(CustomFileInfo info)
        {
        }

        private void Update()
        {
        }
    }
}
