using Illusion.Game;
using IllusionUtility.SetUtility;
using MessagePack;
using System;
using System.Collections;
using System.IO;
using UniRx;
using UniRx.Triggers;
using UnityEngine;

namespace ChaCustom
{
    public class CustomCoordinateFile : MonoBehaviour
    {
        [SerializeField]
        private Camera mainCamera;

        [SerializeField]
        private Camera coordinateCamera;

        [SerializeField]
        private CustomRender customRender;

        [SerializeField]
        private CustomFileListCtrl listCtrl;

        [SerializeField]
        private CustomFileWindow fileWindow;

        [SerializeField]
        private CustomCheckWindow checkWindow;

        private bool overwrite;

        private int poseNo;

        private float backAnmPos;

        private CustomBase customBase
        {
            get
            {
                return Singleton<CustomBase>.Instance;
            }
        }

        private ChaControl chaCtrl
        {
            get
            {
                return customBase.chaCtrl;
            }
        }

        private CustomControl cmpCustomCtrl
        {
            get
            {
                return customBase.customCtrl;
            }
        }

        private CvsDrawCtrl cmpDrawCtrl
        {
            get
            {
                return cmpCustomCtrl.cmpDrawCtrl;
            }
        }

        private void Start()
        {
            Initialize();
            if ((bool)fileWindow)
            {
                if (fileWindow.btnCoordeLoadLoad && null != listCtrl)
                {
                    fileWindow.btnCoordeLoadLoad.OnClickAsObservable().Subscribe(delegate
                    {
                        CustomFileInfoComponent selectTopItem = listCtrl.GetSelectTopItem();
                        if (null != selectTopItem)
                        {
                            Utils.Sound.Play(SystemSE.ok_s);
                            string fullPath = selectTopItem.info.FullPath;
                            bool flag3 = fileWindow.tglCoordeLoadClothes && fileWindow.tglCoordeLoadClothes.isOn;
                            bool flag4 = fileWindow.tglCoordeLoadAcs && fileWindow.tglCoordeLoadAcs.isOn;
                            byte[] bytes = MessagePackSerializer.Serialize(chaCtrl.nowCoordinate.clothes);
                            byte[] bytes2 = MessagePackSerializer.Serialize(chaCtrl.nowCoordinate.accessory);
                            chaCtrl.nowCoordinate.LoadFile(fullPath);
                            if (!flag3)
                            {
                                chaCtrl.nowCoordinate.clothes = MessagePackSerializer.Deserialize<ChaFileClothes>(bytes);
                            }
                            if (!flag4)
                            {
                                chaCtrl.nowCoordinate.accessory = MessagePackSerializer.Deserialize<ChaFileAccessory>(bytes2);
                            }
                            chaCtrl.Reload(false, true, true, true);
                            chaCtrl.AssignCoordinate((ChaFileDefine.CoordinateType)chaCtrl.chaFile.status.coordinateType);
                            customBase.updateCustomUI = true;
                        }
                    });
                }
                if ((bool)fileWindow.btnSave)
                {
                    fileWindow.btnSave.OnClickAsObservable().Subscribe(delegate
                    {
                        overwrite = false;
                        Utils.Sound.Play(SystemSE.window_o);
                        checkWindow.Setup(CustomCheckWindow.CheckType.CoordinateInput, "コーディネート名を入力して下さい", null, string.Empty, CreateCoordinateFile, null);
                    });
                }
                this.UpdateAsObservable().Subscribe(delegate
                {
                    int[] selectIndex2 = listCtrl.GetSelectIndex();
                    bool interactable = selectIndex2 != null && selectIndex2.Length > 0;
                    if ((bool)fileWindow.btnOverwrite)
                    {
                        fileWindow.btnOverwrite.interactable = interactable;
                    }
                    if ((bool)fileWindow.btnDelete)
                    {
                        fileWindow.btnDelete.interactable = interactable;
                    }
                    bool flag = fileWindow.tglCoordeLoadClothes && fileWindow.tglCoordeLoadClothes.isOn;
                    bool flag2 = fileWindow.tglCoordeLoadAcs && fileWindow.tglCoordeLoadAcs.isOn;
                    if (!flag && !flag2)
                    {
                        interactable = false;
                    }
                    if ((bool)fileWindow.btnCoordeLoadLoad)
                    {
                        fileWindow.btnCoordeLoadLoad.interactable = interactable;
                    }
                });
                if ((bool)fileWindow.btnOverwrite)
                {
                    fileWindow.btnOverwrite.OnClickAsObservable().Subscribe(delegate
                    {
                        overwrite = true;
                        int[] selectIndex = listCtrl.GetSelectIndex();
                        CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
                        Utils.Sound.Play(SystemSE.window_o);
                        checkWindow.Setup(CustomCheckWindow.CheckType.CoordinateInput, "本当に上書きしますか？", null, fileInfoFromIndex.name, CreateCoordinateFile, null);
                    });
                }
                if ((bool)fileWindow.btnDelete)
                {
                    fileWindow.btnDelete.OnClickAsObservable().Subscribe(delegate
                    {
                        Utils.Sound.Play(SystemSE.window_o);
                        checkWindow.Setup(CustomCheckWindow.CheckType.YesNo, "本当に削除しますか？", null, null, DeleteCoordinateFile, null);
                    });
                }
            }
        }

        private void Initialize()
        {
            FolderAssist folderAssist = new FolderAssist();
            string[] searchPattern = new string[1]
            {
                "*.png"
            };
            string folder = UserData.Path + "coordinate/";
            folderAssist.CreateFolderInfoEx(folder, searchPattern);
            listCtrl.ClearList();
            int fileCount = folderAssist.GetFileCount();
            int num = 0;
            for (int i = 0; i < fileCount; i++)
            {
                ChaFileCoordinate chaFileCoordinate = new ChaFileCoordinate();
                if (!chaFileCoordinate.LoadFile(folderAssist.lstFile[i].FullPath))
                {
                    int lastErrorCode = chaFileCoordinate.GetLastErrorCode();
                    continue;
                }
                listCtrl.AddList(num, chaFileCoordinate.coordinateName, string.Empty, string.Empty, folderAssist.lstFile[i].FullPath, folderAssist.lstFile[i].FileName, folderAssist.lstFile[i].time);
                num++;
            }
            listCtrl.Create(OnChangeSelect);
        }

        public void DeleteCoordinateFile(string buf)
        {
            int[] selectIndex = listCtrl.GetSelectIndex();
            CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
            if (File.Exists(fileInfoFromIndex.FullPath))
            {
                File.Delete(fileInfoFromIndex.FullPath);
            }
            listCtrl.RemoveList(selectIndex[0]);
            listCtrl.ReCreate();
        }

        public void CreateCoordinateFile(string coordinateName)
        {
            CreateCoordinateFileBefore();
            StartCoroutine(CreateCoordinateFileCoroutine(coordinateName));
        }

        public void CreateCoordinateFileBefore()
        {
            chaCtrl.ChangeSettingMannequin(true);
            cmpDrawCtrl.ChangeClothesStateForCapture(true);
            string assetName = chaCtrl.sex != 0 ? "coorde_frame_back_f" : "coorde_frame_back_m";
            string assetName2 = chaCtrl.sex != 0 ? "coorde_frame_front_f" : "coorde_frame_front_m";
            Texture2D changeTex = PngAssist.LoadTexture2DFromAssetBundle("custom/custom_etc.unity3d", assetName);
            Texture2D changeTex2 = PngAssist.LoadTexture2DFromAssetBundle("custom/custom_etc.unity3d", assetName2);
            customBase.saveFrameAssist.ChangeSaveFrameBackTexture(changeTex);
            customBase.saveFrameAssist.ChangeSaveFrameFrontTexture(changeTex2);
            poseNo = cmpDrawCtrl.ddPose.value;
            Animator animBody = chaCtrl.animBody;
            if ((bool)animBody)
            {
                backAnmPos = animBody.GetCurrentAnimatorStateInfo(0).normalizedTime;
            }
            string text = "custom/custompose.unity3d";
            string text2 = "custom_pose";
            string stateName = chaCtrl.sex != 0 ? "Fmannequin_00" : "Mmannequin_00";
            if (customBase.animeAssetBundleName != text || customBase.animeAssetName != text2)
            {
                customBase.LoadAnimation(text, text2);
            }
            customBase.PlayAnimation(stateName, 0f);
            chaCtrl.resetDynamicBoneAll = true;
            customRender.update = false;
        }

        public IEnumerator CreateCoordinateFileCoroutine(string coordinateName)
        {
            yield return new WaitForEndOfFrame();
            byte[] pngData = CreateCoordinatePng();
            if (overwrite)
            {
                int[] selectIndex = listCtrl.GetSelectIndex();
                CustomFileInfo fileInfoFromIndex = listCtrl.GetFileInfoFromIndex(selectIndex[0]);
                int coordinateType = chaCtrl.chaFile.status.coordinateType;
                chaCtrl.chaFile.coordinate[coordinateType].pngData = pngData;
                chaCtrl.chaFile.coordinate[coordinateType].coordinateName = coordinateName;
                chaCtrl.chaFile.coordinate[coordinateType].SaveFile(fileInfoFromIndex.FullPath);
                fileInfoFromIndex.time = DateTime.Now;
                fileInfoFromIndex.name = coordinateName;
                fileInfoFromIndex.DeleteThumb();
            }
            else
            {
                string text = chaCtrl.sex != 0 ? "KKCoordeF_" + DateTime.Now.ToString("yyyyMMddHHmmssfff") : "KKCoordeM_" + DateTime.Now.ToString("yyyyMMddHHmmssfff");
                string text2 = UserData.Path + "coordinate/" + text + ".png";
                int coordinateType2 = chaCtrl.chaFile.status.coordinateType;
                chaCtrl.chaFile.coordinate[coordinateType2].pngData = pngData;
                chaCtrl.chaFile.coordinate[coordinateType2].coordinateName = coordinateName;
                chaCtrl.chaFile.coordinate[coordinateType2].SaveFile(text2);
                int noUseIndex = listCtrl.GetNoUseIndex();
                listCtrl.AddList(noUseIndex, coordinateName, string.Empty, string.Empty, text2, text, DateTime.Now);
                listCtrl.ReCreate();
            }
            StartCoroutine(CreateCoordinateFileAfterCoroutine());
        }

        public byte[] CreateCoordinatePng()
        {
            mainCamera.gameObject.SetActive(false);
            if ((bool)coordinateCamera)
            {
                coordinateCamera.enabled = true;
            }
            if ((bool)coordinateCamera)
            {
                coordinateCamera.gameObject.SetActive(true);
                if (chaCtrl.sex == 0)
                {
                    coordinateCamera.transform.SetPosition(0.58f, 1.1f, 4.5f);
                    coordinateCamera.transform.SetRotation(3.5f, 187.5f, 0f);
                }
                else
                {
                    coordinateCamera.transform.SetPosition(0.58f, 1.08f, 4.4f);
                    coordinateCamera.transform.SetRotation(3.5f, 187.5f, 0f);
                }
            }
            customBase.drawSaveFrameTop = true;
            bool drawSaveFrameBack = customBase.drawSaveFrameBack;
            customBase.drawSaveFrameBack = true;
            bool drawSaveFrameFront = customBase.drawSaveFrameFront;
            customBase.drawSaveFrameFront = true;
            byte[] array = cmpCustomCtrl.customCap.CapCoordinateCard(false, customBase.saveFrameAssist, coordinateCamera);
            customBase.saveFrameAssist.ChangeSaveFrameBack(2);
            customBase.saveFrameAssist.ChangeSaveFrameFront(2);
            customBase.drawSaveFrameBack = drawSaveFrameBack;
            customBase.drawSaveFrameFront = drawSaveFrameFront;
            customBase.drawSaveFrameTop = false;
            if ((bool)coordinateCamera)
            {
                coordinateCamera.enabled = false;
            }
            if ((bool)coordinateCamera)
            {
                coordinateCamera.gameObject.SetActive(false);
            }
            mainCamera.gameObject.SetActive(true);
            chaCtrl.ChangeSettingMannequin(false);
            cmpDrawCtrl.ChangeClothesStateForCapture(true);
            cmpDrawCtrl.ChangeAnimationForce(poseNo, backAnmPos);
            return array;
        }

        public IEnumerator CreateCoordinateFileAfterCoroutine()
        {
            yield return null;
            customRender.update = true;
        }

        public void OnChangeSelect(CustomFileInfo info)
        {
        }
    }
}
