using Illusion.Extensions;
using Illusion.Game;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using UGUI_AssistLibrary;
using UniRx;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CustomFileListCtrl : MonoBehaviour
    {
        public delegate void OnChangeItemFunc(CustomFileInfo info);

        [HideInInspector]
        public CanvasGroup[] canvasGrp;

        [HideInInspector]
        public Image[] imgRaycast;

        [SerializeField]
        private CustomFileWindow cfWindow;

        [SerializeField]
        private Text textDrawName;

        [SerializeField]
        private RectTransform rtfScrollRect;

        [SerializeField]
        private RectTransform rtfContant;

        [SerializeField]
        private GameObject objListContent;

        [SerializeField]
        private GameObject objLineTemp;

        [SerializeField]
        private Toggle tglAddInfo;

        [SerializeField]
        private Button btnSortClub;

        [SerializeField]
        private Button btnSortPersonality;

        [SerializeField]
        private Button btnSortName;

        [SerializeField]
        private Button btnSortData;

        [SerializeField]
        private bool multiSelect;

        [SerializeField]
        [Tooltip("OFFだと最低でも一つは選択されてる必要がある")]
        private bool allowSwitchOff = true;

        private string selectDrawName = string.Empty;

        private List<CustomFileInfo> lstFileInfo = new List<CustomFileInfo>();

        private bool ascendName;

        private bool ascendData;

        private bool ascendClub;

        private bool ascendPersonality;

        private byte lastSort;

        public OnChangeItemFunc onChangeItemFunc;

        private void GetCanvasGroup()
        {
            List<CanvasGroup> list = new List<CanvasGroup>();
            CanvasGroup component = GetComponent<CanvasGroup>();
            if (null != component)
            {
                list.Add(component);
            }
            Transform parent = transform.parent;
            if (null == parent)
            {
                return;
            }
            while (true)
            {
                component = parent.GetComponent<CanvasGroup>();
                if (null != component)
                {
                    list.Add(component);
                }
                if (null == parent.parent)
                {
                    break;
                }
                parent = parent.parent;
            }
            canvasGrp = list.ToArray();
        }

        public void ChangeRaycastTarget(bool enable)
        {
            if (imgRaycast != null)
            {
                Image[] array = imgRaycast;
                foreach (Image image in array)
                {
                    image.raycastTarget = enable;
                }
            }
        }

        public void DisableAddInfo()
        {
            if ((bool)tglAddInfo)
            {
                tglAddInfo.interactable = false;
            }
            if ((bool)btnSortClub)
            {
                btnSortClub.interactable = false;
            }
            if ((bool)btnSortPersonality)
            {
                btnSortPersonality.interactable = false;
            }
        }

        private void Start()
        {
            if ((bool)btnSortData)
            {
                btnSortData.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortDate(!ascendData);
                });
            }
            if ((bool)btnSortName)
            {
                btnSortName.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortName(!ascendName);
                });
            }
            if ((bool)btnSortClub)
            {
                btnSortClub.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortClub(!ascendClub);
                });
            }
            if ((bool)btnSortPersonality)
            {
                btnSortPersonality.OnClickAsObservable().Subscribe(delegate
                {
                    Utils.Sound.Play(SystemSE.sel);
                    SortPersonality(!ascendPersonality);
                });
            }
            if ((bool)tglAddInfo)
            {
                tglAddInfo.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
                {
                    for (int i = 0; i < lstFileInfo.Count; i++)
                    {
                        lstFileInfo[i].fic.imgAddInfo.gameObject.SetActiveIfDifferent(isOn);
                    }
                });
            }
            GetCanvasGroup();
        }

        private void Update()
        {
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                if (lstFileInfo[i].fic.gameObject.activeSelf)
                {
                    RectTransform rectTransform = lstFileInfo[i].fic.transform as RectTransform;
                    Vector2 anchoredPosition = rectTransform.anchoredPosition;
                    float y = anchoredPosition.y;
                    float height = rectTransform.rect.height;
                    Vector2 anchoredPosition2 = rtfContant.anchoredPosition;
                    float num = 0f - anchoredPosition2.y;
                    float num2 = num - rtfScrollRect.rect.height;
                    float num3 = y;
                    float num4 = y - height;
                    if (num4 <= num && num3 >= num2)
                    {
                        lstFileInfo[i].show = true;
                    }
                    else
                    {
                        lstFileInfo[i].show = false;
                    }
                    lstFileInfo[i].UpdateThumb(lstFileInfo[i].fic.imgThumb);
                }
            }
            if (canvasGrp == null || imgRaycast == null)
            {
                return;
            }
            bool enable = true;
            CanvasGroup[] array = canvasGrp;
            foreach (CanvasGroup canvasGroup in array)
            {
                if (!canvasGroup.blocksRaycasts)
                {
                    enable = false;
                    break;
                }
            }
            ChangeRaycastTarget(enable);
        }

        private void OnDestroy()
        {
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                lstFileInfo[i].DeleteThumb();
            }
        }

        public void Setup()
        {
        }

        public void SortDate(bool ascend)
        {
            ascendData = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 1;
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.time.CompareTo(b.time));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.time.CompareTo(a.time));
            }
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortName(bool ascend)
        {
            ascendName = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.name.CompareTo(b.name));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.name.CompareTo(a.name));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortClub(bool ascend)
        {
            ascendClub = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.club.CompareTo(b.club));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.club.CompareTo(a.club));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortPersonality(bool ascend)
        {
            ascendPersonality = ascend;
            if (lstFileInfo.Count == 0)
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.personality.CompareTo(b.personality));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.personality.CompareTo(a.personality));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (null != gameObject)
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void ClearList()
        {
            lstFileInfo.Clear();
        }

        public void AddList(int index, string name, string club, string personality, string fullpath, string filename, DateTime time, bool disable = false)
        {
            CustomFileInfo customFileInfo = new CustomFileInfo();
            customFileInfo.index = index;
            customFileInfo.name = name;
            customFileInfo.club = club;
            customFileInfo.personality = personality;
            customFileInfo.FullPath = fullpath;
            customFileInfo.FileName = filename;
            customFileInfo.time = time;
            customFileInfo.disable = disable;
            lstFileInfo.Add(customFileInfo);
        }

        public void RemoveList(int index)
        {
            CustomFileInfo fileInfoFromIndex = GetFileInfoFromIndex(index);
            fileInfoFromIndex.DeleteThumb();
            lstFileInfo.Remove(fileInfoFromIndex);
        }

        public void ReCreate()
        {
            Create(onChangeItemFunc);
        }

        public void Create(OnChangeItemFunc _onChangeItemFunc)
        {
            onChangeItemFunc = _onChangeItemFunc;
            List<Image> list = new List<Image>();
            for (int num = objListContent.transform.childCount - 1; num >= 0; num--)
            {
                Transform child = objListContent.transform.GetChild(num);
                Destroy(child.gameObject);
            }
            ToggleGroup component = objListContent.GetComponent<ToggleGroup>();
            component.allowSwitchOff = allowSwitchOff;
            int num2 = 0;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                lstFileInfo[i].DeleteThumb();
                GameObject gameObject = Instantiate(objLineTemp);
                CustomFileInfoComponent component2 = gameObject.GetComponent<CustomFileInfoComponent>();
                component2.info = lstFileInfo[i];
                component2.info.fic = component2;
                Transform transform = gameObject.transform.Find("ThumbBG");
                if ((bool)transform)
                {
                    Image component3 = transform.GetComponent<Image>();
                    if ((bool)component3)
                    {
                        list.Add(component3);
                    }
                }
                if (!multiSelect)
                {
                    component2.tgl.group = component;
                }
                if ((bool)component2.tmpTexClub)
                {
                    component2.tmpTexClub.text = component2.info.club;
                }
                if ((bool)component2.tmpTexPersonality)
                {
                    component2.tmpTexPersonality.text = component2.info.personality;
                }
                gameObject.transform.SetParent(objListContent.transform, false);
                SetToggleHandler(gameObject);
                component2.Disvisible(lstFileInfo[i].disvisible);
                component2.Disable(lstFileInfo[i].disable);
                num2++;
            }
            SortClub(true);
            SortPersonality(true);
            SortName(true);
            SortDate(false);
            ToggleAllOff();
            imgRaycast = list.ToArray();
        }

        public void UpdateSort()
        {
            if (lastSort == 0)
            {
                SortClub(ascendClub);
                SortPersonality(ascendPersonality);
                SortDate(ascendData);
                SortName(ascendName);
            }
            else if (lastSort == 1)
            {
                SortClub(ascendClub);
                SortPersonality(ascendPersonality);
                SortName(ascendName);
                SortDate(ascendData);
            }
            else if (lastSort == 2)
            {
                SortPersonality(ascendPersonality);
                SortName(ascendName);
                SortDate(ascendData);
                SortClub(ascendClub);
            }
            else
            {
                SortClub(ascendClub);
                SortName(ascendName);
                SortDate(ascendData);
                SortPersonality(ascendPersonality);
            }
        }

        public CustomFileInfo GetFileInfoFromIndex(int index)
        {
            return lstFileInfo.Find(item => item.index == index);
        }

        public CustomFileInfo GetFileInfoFromName(string name)
        {
            return lstFileInfo.Find(item => item.name == name);
        }

        public string GetNameFormIndex(int index)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            if (customFileInfo != null)
            {
                return customFileInfo.name;
            }
            return string.Empty;
        }

        public int GetIndexFromName(string name)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            if (customFileInfo != null)
            {
                return customFileInfo.index;
            }
            return -1;
        }

        public int[] GetSelectIndex()
        {
            List<int> list = new List<int>();
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                if (lstFileInfo[i].fic.tgl.interactable && lstFileInfo[i].fic.gameObject.activeSelf && lstFileInfo[i].fic.tgl.isOn)
                {
                    list.Add(lstFileInfo[i].index);
                }
            }
            return list.ToArray();
        }

        public int GetNoUseIndex()
        {
            int[] array = (from info in lstFileInfo
                           select info.index into idx
                           orderby idx
                           select idx).ToArray();
            for (int i = 0; i < array.Length; i++)
            {
                if (array[i] != i)
                {
                    return i;
                }
            }
            return array.Length;
        }

        public CustomFileInfoComponent GetSelectTopItem()
        {
            int[] selectIndex = GetSelectIndex();
            if (selectIndex.IsNullOrEmpty())
            {
                return null;
            }
            CustomFileInfo fileInfoFromIndex = GetFileInfoFromIndex(selectIndex[0]);
            if (fileInfoFromIndex != null)
            {
                return fileInfoFromIndex.fic;
            }
            return null;
        }

        public CustomFileInfoComponent GetSelectableTopItem()
        {
            SortedDictionary<int, CustomFileInfoComponent> sortedDictionary = new SortedDictionary<int, CustomFileInfoComponent>();
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                if (lstFileInfo[i].fic.tgl.interactable && lstFileInfo[i].fic.gameObject.activeSelf)
                {
                    sortedDictionary[lstFileInfo[i].fic.gameObject.transform.GetSiblingIndex()] = lstFileInfo[i].fic;
                }
            }
            CustomFileInfoComponent result = null;
            if (sortedDictionary.Count != 0)
            {
                result = sortedDictionary.First().Value;
            }
            return result;
        }

        public int GetDrawOrderFromIndex(int index)
        {
            SortedDictionary<int, CustomFileInfoComponent> sortedDictionary = new SortedDictionary<int, CustomFileInfoComponent>();
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                if (lstFileInfo[i].fic.gameObject.activeSelf)
                {
                    sortedDictionary[lstFileInfo[i].fic.gameObject.transform.GetSiblingIndex()] = lstFileInfo[i].fic;
                }
            }
            foreach (var item in sortedDictionary.Select((val, idx) => new
            {
                val,
                idx
            }))
            {
                if (item.val.Value.info.index == index)
                {
                    return item.idx;
                }
            }
            return -1;
        }

        public int GetInclusiveCount()
        {
            List<int> list = new List<int>();
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                if (lstFileInfo[i].fic.gameObject.activeSelf)
                {
                    list.Add(lstFileInfo[i].index);
                }
            }
            return list.Count;
        }

        public void SelectItem(int index)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            customFileInfo.fic.tgl.isOn = true;
            ChangeItem(customFileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void SelectItem(string name)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            customFileInfo.fic.tgl.isOn = true;
            ChangeItem(customFileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void UpdateScrollPosition()
        {
        }

        public void OnPointerClick(GameObject obj)
        {
            if (!(null == obj))
            {
                CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
                if (!(null == component) && component.tgl.interactable)
                {
                    ChangeItem(obj);
                }
            }
        }

        public void OnPointerEnter(GameObject obj)
        {
            if (!(null == obj))
            {
                CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
                if (!(null == component) && component.tgl.interactable && textDrawName)
                {
                    textDrawName.text = component.info.name;
                }
            }
        }

        public void OnPointerExit(GameObject obj)
        {
            if (!(null == obj))
            {
                CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
                if (!(null == component) && component.tgl.interactable && textDrawName)
                {
                    textDrawName.text = selectDrawName;
                }
            }
        }

        public void ChangeItem(GameObject obj)
        {
            CustomFileInfoComponent component = obj.GetComponent<CustomFileInfoComponent>();
            if (onChangeItemFunc != null)
            {
                onChangeItemFunc(component.info);
            }
            selectDrawName = component.info.name;
        }

        public void ToggleAllOff()
        {
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                lstFileInfo[i].fic.tgl.isOn = false;
            }
        }

        public void ToggleAllOn()
        {
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                lstFileInfo[i].fic.tgl.isOn = true;
            }
        }

        public void DisableItem(int index, bool disable)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            if (customFileInfo != null)
            {
                customFileInfo.disable = disable;
                customFileInfo.fic.Disable(disable);
            }
        }

        public void DisableItem(string name, bool disable)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            customFileInfo.disable = disable;
            customFileInfo.fic.Disable(disable);
        }

        public void DisvisibleItem(int index, bool disvisible)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.index == index);
            if (customFileInfo != null)
            {
                customFileInfo.disvisible = disvisible;
                customFileInfo.fic.Disvisible(disvisible);
            }
        }

        public void DisvisibleItem(string name, bool disvisible)
        {
            CustomFileInfo customFileInfo = lstFileInfo.Find(item => item.name == name);
            if (customFileInfo != null)
            {
                customFileInfo.disvisible = disvisible;
                customFileInfo.fic.Disvisible(disvisible);
            }
        }

        public void UpdateCategory()
        {
            if (null == cfWindow)
            {
                return;
            }
            Toggle[] tglCategory = cfWindow.tglCategory;
            if (tglCategory == null || tglCategory.Length == 0)
            {
                return;
            }
            bool[] array = tglCategory.Select(x => x.isOn).ToArray();
            bool[] array2 = array.Where(x => x).ToArray();
            List<CustomFileInfo> list = null;
            if (array2.Length == 0)
            {
                for (int i = 0; i < lstFileInfo.Count; i++)
                {
                    lstFileInfo[i].fic.Disvisible(false);
                }
                list = new List<CustomFileInfo>(lstFileInfo);
            }
            else
            {
                bool flag = false;
                list = new List<CustomFileInfo>();
                for (int j = 0; j < lstFileInfo.Count; j++)
                {
                    flag = array[lstFileInfo[j].category];
                    lstFileInfo[j].fic.Disvisible(!flag);
                    if (flag)
                    {
                        list.Add(lstFileInfo[j]);
                    }
                }
            }
            CustomFileInfo customFileInfo = list.Find(x => x.fic.tgl.isOn);
            if (customFileInfo == null)
            {
                selectDrawName = string.Empty;
            }
            else
            {
                selectDrawName = customFileInfo.fic.name;
            }
        }

        private void SetToggleHandler(GameObject obj)
        {
            UIAL_EventTrigger uIAL_EventTrigger = obj.AddComponent<UIAL_EventTrigger>();
            uIAL_EventTrigger.triggers = new List<UIAL_EventTrigger.Entry>();
            UIAL_EventTrigger.Entry entry = new UIAL_EventTrigger.Entry();
            entry.eventID = EventTriggerType.PointerClick;
            entry.buttonType = UIAL_EventTrigger.ButtonType.Left;
            entry.callback.AddListener(delegate
            {
                OnPointerClick(obj);
            });
            uIAL_EventTrigger.triggers.Add(entry);
            if ((bool)textDrawName)
            {
                entry = new UIAL_EventTrigger.Entry();
                entry.eventID = EventTriggerType.PointerEnter;
                entry.callback.AddListener(delegate
                {
                    OnPointerEnter(obj);
                });
                uIAL_EventTrigger.triggers.Add(entry);
                entry = new UIAL_EventTrigger.Entry();
                entry.eventID = EventTriggerType.PointerExit;
                entry.callback.AddListener(delegate
                {
                    OnPointerExit(obj);
                });
                uIAL_EventTrigger.triggers.Add(entry);
            }
        }
    }
}
