using Illusion.Extensions;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UnityEngine;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CustomFileWindow : MonoBehaviour
    {
        public enum FileWindowType
        {
            CharaSave,
            CharaLoad,
            CoordinateSave,
            CoordinateLoad,
            FaceSave,
            FaceLoad,
            BodySave,
            BodyLoad,
            HairSave,
            HairLoad,
            TopSave,
            TopLoad,
            BotSave,
            BotLoad,
            BraSave,
            BraLoad,
            ShortsSave,
            ShortsLoad,
            GlovesSave,
            GlovesLoad,
            PanstSave,
            PanstLoad,
            SocksSave,
            SocksLoad,
            ShoesSave,
            ShoesLoad,
            AccessorySave,
            AccessoryLoad,
            MakeupSave,
            MakeupLoad
        }

        [Serializable]
        public class TypeReactiveProperty : ReactiveProperty<FileWindowType>
        {
            public TypeReactiveProperty()
            {
            }

            public TypeReactiveProperty(FileWindowType initialValue)
                : base(initialValue)
            {
            }
        }

        [SerializeField]
        private TypeReactiveProperty _fwType = new TypeReactiveProperty(FileWindowType.CharaSave);

        [SerializeField]
        private TextMeshProUGUI textTitle;

        [SerializeField]
        private Button btnClose;

        [SerializeField]
        private GameObject objSave;

        [SerializeField]
        private GameObject objLoad;

        [SerializeField]
        private GameObject objCharaLoad;

        [SerializeField]
        private GameObject objCoordinateLoad;

        [SerializeField]
        private GameObject objClose;

        [SerializeField]
        private GameObject objTglCategory;

        [SerializeField]
        private GameObject objTglTemp;

        [HideInInspector]
        public Toggle[] tglCategory;

        [SerializeField]
        private Toggle _tglChaLoadFace;

        [SerializeField]
        private Toggle _tglChaLoadBody;

        [SerializeField]
        private Toggle _tglChaLoadHair;

        [SerializeField]
        private Toggle _tglChaLoadParam;

        [SerializeField]
        private Toggle _tglChaLoadCoorde;

        [SerializeField]
        private Button _btnChaLoadAllOn;

        [SerializeField]
        private Button _btnChaLoadAllOff;

        [SerializeField]
        private Button _btnChaLoadLoad;

        [SerializeField]
        private Button _btnDelete;

        [SerializeField]
        private Button _btnSave;

        [SerializeField]
        private Button _btnOverwrite;

        [SerializeField]
        private Toggle _tglCoordeLoadClothes;

        [SerializeField]
        private Toggle _tglCoordeLoadAcs;

        [SerializeField]
        private Button _btnCoordeLoadAllOn;

        [SerializeField]
        private Button _btnCoordeLoadAllOff;

        [SerializeField]
        private Button _btnCoordeLoadLoad;

        [SerializeField]
        private Button _btnLoad;

        public FileWindowType fwType
        {
            get
            {
                return _fwType.Value;
            }
            set
            {
                _fwType.Value = value;
            }
        }

        public Toggle tglChaLoadFace
        {
            get
            {
                return _tglChaLoadFace;
            }
        }

        public Toggle tglChaLoadBody
        {
            get
            {
                return _tglChaLoadBody;
            }
        }

        public Toggle tglChaLoadHair
        {
            get
            {
                return _tglChaLoadHair;
            }
        }

        public Toggle tglChaLoadParam
        {
            get
            {
                return _tglChaLoadParam;
            }
        }

        public Toggle tglChaLoadCoorde
        {
            get
            {
                return _tglChaLoadCoorde;
            }
        }

        public Button btnChaLoadAllOn
        {
            get
            {
                return _btnChaLoadAllOn;
            }
        }

        public Button btnChaLoadAllOff
        {
            get
            {
                return _btnChaLoadAllOff;
            }
        }

        public Button btnChaLoadLoad
        {
            get
            {
                return _btnChaLoadLoad;
            }
        }

        public Button btnDelete
        {
            get
            {
                return _btnDelete;
            }
        }

        public Button btnSave
        {
            get
            {
                return _btnSave;
            }
        }

        public Button btnOverwrite
        {
            get
            {
                return _btnOverwrite;
            }
        }

        public Toggle tglCoordeLoadClothes
        {
            get
            {
                return _tglCoordeLoadClothes;
            }
        }

        public Toggle tglCoordeLoadAcs
        {
            get
            {
                return _tglCoordeLoadAcs;
            }
        }

        public Button btnCoordeLoadAllOn
        {
            get
            {
                return _btnCoordeLoadAllOn;
            }
        }

        public Button btnCoordeLoadAllOff
        {
            get
            {
                return _btnCoordeLoadAllOff;
            }
        }

        public Button btnCoordeLoadLoad
        {
            get
            {
                return _btnCoordeLoadLoad;
            }
        }

        public Button btnLoad
        {
            get
            {
                return _btnLoad;
            }
        }

        public void Awake()
        {
        }

        public IEnumerator Start()
        {
            yield return new WaitUntil(() => null != Singleton<CustomBase>.Instance.lstFileList);
            _fwType.TakeUntilDestroy(this).Subscribe(delegate
            {
                UpdateWindow();
            });
            if ((bool)btnClose)
            {
                btnClose.OnClickAsObservable().Subscribe(delegate
                {
                    gameObject.SetActiveIfDifferent(false);
                });
            }
        }

        public void UpdateWindow()
        {
            List<ExcelData.Param> lstFileList = Singleton<CustomBase>.Instance.lstFileList;
            ExcelData.Param param = lstFileList.FirstOrDefault(x => fwType.ToString() == x.list[1]);
            if (param == null)
            {
                return;
            }
            if (null != textTitle)
            {
                textTitle.text = param.list[2];
            }
            if ((bool)objSave)
            {
                objSave.SetActiveIfDifferent("1" == param.list[3]);
            }
            if ((bool)objLoad)
            {
                objLoad.SetActiveIfDifferent("1" == param.list[4]);
            }
            if ((bool)objCharaLoad)
            {
                objCharaLoad.SetActiveIfDifferent("1" == param.list[5]);
            }
            if ((bool)objCoordinateLoad)
            {
                objCoordinateLoad.SetActiveIfDifferent("1" == param.list[6]);
            }
            if ((bool)objClose)
            {
                objClose.SetActiveIfDifferent("1" == param.list[7]);
            }
            if ((bool)objTglCategory)
            {
                objTglCategory.SetActiveIfDifferent("1" == param.list[8]);
            }
            List<Toggle> list = new List<Toggle>();
            int num = param.list.Count - 9;
            for (int i = 0; i < num && !("0" == param.list[9 + i]); i++)
            {
                GameObject gameObject = Instantiate(objTglTemp, objTglCategory.transform, false);
                if (null == gameObject)
                {
                    break;
                }
                Transform transform = gameObject.transform.Find("textCate");
                if (null == transform)
                {
                    break;
                }
                TextMeshProUGUI component = transform.GetComponent<TextMeshProUGUI>();
                if ((bool)component)
                {
                    component.text = param.list[9 + i];
                }
                Toggle component2 = gameObject.GetComponent<Toggle>();
                gameObject.SetActiveIfDifferent(true);
                list.Add(component2);
            }
            if (list.Count != 0)
            {
                tglCategory = list.ToArray();
            }
        }
    }
}
