using MessagePack;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CvsAccessoryChange : MonoBehaviour
    {
        [SerializeField]
        private CustomAcsChangeSlot cmpAcsChangeSlot;

        [SerializeField]
        private Toggle[] tglSrcKind;

        [SerializeField]
        private Toggle[] tglDstKind;

        [SerializeField]
        private TextMeshProUGUI[] textSrc;

        [SerializeField]
        private TextMeshProUGUI[] textDst;

        [SerializeField]
        private Toggle tglReverse;

        [SerializeField]
        private Button btnSlotCopy;

        [SerializeField]
        private Button[] btnCopyValue;

        [SerializeField]
        private Button[] btnCopyRevValueLR;

        [SerializeField]
        private Button[] btnCopyRevValueUD;

        private int selSrc;

        private int selDst;

        private ChaControl chaCtrl
        {
            get
            {
                return Singleton<CustomBase>.Instance.chaCtrl;
            }
        }

        private ChaFileAccessory accessory
        {
            get
            {
                return chaCtrl.nowCoordinate.accessory;
            }
        }

        private ChaFileAccessory setAccessory
        {
            get
            {
                return chaCtrl.chaFile.coordinate[chaCtrl.chaFile.status.coordinateType].accessory;
            }
        }

        public void CalculateUI()
        {
            for (int i = 0; i < 20; i++)
            {
                ListInfoBase listInfo = chaCtrl.lstCtrl.GetListInfo((ChaListDefine.CategoryNo)accessory.parts[i].type, accessory.parts[i].id);
                if (listInfo == null)
                {
                    textDst[i].text = "なし";
                    textSrc[i].text = "なし";
                }
                else
                {
                    textDst[i].text = listInfo.Name;
                    textSrc[i].text = listInfo.Name;
                }
            }
        }

        public virtual void UpdateCustomUI()
        {
            CalculateUI();
        }

        private void CopyAcs()
        {
            byte[] bytes = MessagePackSerializer.Serialize(accessory.parts[selSrc]);
            accessory.parts[selDst] = MessagePackSerializer.Deserialize<ChaFileAccessory.PartsInfo>(bytes);
            if (tglReverse.isOn)
            {
                string reverseParent = ChaAccessoryDefine.GetReverseParent(accessory.parts[selDst].parentKey);
                if (string.Empty != reverseParent)
                {
                    accessory.parts[selDst].parentKey = reverseParent;
                }
            }
            chaCtrl.AssignCoordinate((ChaFileDefine.CoordinateType)chaCtrl.fileStatus.coordinateType);
            chaCtrl.Reload(false, true, true, true);
            CalculateUI();
            cmpAcsChangeSlot.UpdateSlotNames();
            Singleton<CustomBase>.Instance.SetUpdateCvsAccessory(selDst, true);
        }

        private void CopyAcsCorrect(int correctNo)
        {
            for (int i = 0; i < 3; i++)
            {
                accessory.parts[selDst].addMove[correctNo, i] = setAccessory.parts[selDst].addMove[correctNo, i] = accessory.parts[selSrc].addMove[correctNo, i];
            }
            chaCtrl.UpdateAccessoryMoveFromInfo(selDst);
        }

        private void CopyAcsCorrectRevLR(int correctNo)
        {
            for (int i = 0; i < 3; i++)
            {
                Vector3 vector = accessory.parts[selSrc].addMove[correctNo, i];
                if (i == 1)
                {
                    vector.y += 180f;
                    if (vector.y >= 360f)
                    {
                        vector.y -= 360f;
                    }
                }
                accessory.parts[selDst].addMove[correctNo, i] = setAccessory.parts[selDst].addMove[correctNo, i] = vector;
            }
            chaCtrl.UpdateAccessoryMoveFromInfo(selDst);
        }

        private void CopyAcsCorrectRevUD(int correctNo)
        {
            for (int i = 0; i < 3; i++)
            {
                Vector3 vector = accessory.parts[selSrc].addMove[correctNo, i];
                if (i == 1)
                {
                    vector.x += 180f;
                    if (vector.x >= 360f)
                    {
                        vector.x -= 360f;
                    }
                }
                accessory.parts[selDst].addMove[correctNo, i] = setAccessory.parts[selDst].addMove[correctNo, i] = vector;
            }
            chaCtrl.UpdateAccessoryMoveFromInfo(selDst);
        }

        protected virtual void Awake()
        {
        }

        protected virtual void Start()
        {
            Singleton<CustomBase>.Instance.actUpdateCvsAccessoryChange += UpdateCustomUI;
            tglSrcKind.Select((p, index) => new
            {
                tgl = p, index
            }).ToList().ForEach(p =>
            {
                CvsAccessoryChange cvsAccessoryChange5 = this;
                (from isOn in p.tgl.OnValueChangedAsObservable()
                 where isOn
                 select isOn).Subscribe(delegate
             {
                 cvsAccessoryChange5.selSrc = p.index;
             });
            });
            tglDstKind.Select((p, index) => new
            {
                tgl = p, index
            }).ToList().ForEach(p =>
            {
                CvsAccessoryChange cvsAccessoryChange4 = this;
                (from isOn in p.tgl.OnValueChangedAsObservable()
                 where isOn
                 select isOn).Subscribe(delegate
             {
                 cvsAccessoryChange4.selDst = p.index;
             });
            });
            btnSlotCopy.OnClickAsObservable().Subscribe(delegate
            {
                CopyAcs();
            });
            btnCopyValue.Select((p, index) => new
            {
                btn = p, index
            }).ToList().ForEach(p =>
            {
                CvsAccessoryChange cvsAccessoryChange3 = this;
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    cvsAccessoryChange3.CopyAcsCorrect(p.index);
                });
            });
            btnCopyRevValueLR.Select((p, index) => new
            {
                btn = p, index
            }).ToList().ForEach(p =>
            {
                CvsAccessoryChange cvsAccessoryChange2 = this;
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    cvsAccessoryChange2.CopyAcsCorrectRevLR(p.index);
                });
            });
            btnCopyRevValueUD.Select((p, index) => new
            {
                btn = p, index
            }).ToList().ForEach(p =>
            {
                CvsAccessoryChange cvsAccessoryChange = this;
                p.btn.OnClickAsObservable().Subscribe(delegate
                {
                    cvsAccessoryChange.CopyAcsCorrectRevUD(p.index);
                });
            });
            this.UpdateAsObservable().Subscribe(delegate
            {
                btnSlotCopy.interactable = selDst != selSrc;
            });
        }
    }
}
