using System.Collections;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CvsEar : MonoBehaviour
    {
        [SerializeField]
        private Slider sldEarSize;

        [SerializeField]
        private TMP_InputField inpEarSize;

        [SerializeField]
        private Button btnEarSize;

        [SerializeField]
        private Slider sldEarRotY;

        [SerializeField]
        private TMP_InputField inpEarRotY;

        [SerializeField]
        private Button btnEarRotY;

        [SerializeField]
        private Slider sldEarRotZ;

        [SerializeField]
        private TMP_InputField inpEarRotZ;

        [SerializeField]
        private Button btnEarRotZ;

        [SerializeField]
        private Slider sldEarUpForm;

        [SerializeField]
        private TMP_InputField inpEarUpForm;

        [SerializeField]
        private Button btnEarUpForm;

        [SerializeField]
        private Slider sldEarLowForm;

        [SerializeField]
        private TMP_InputField inpEarLowForm;

        [SerializeField]
        private Button btnEarLowForm;

        private Slider[] sliders;

        private TMP_InputField[] inputs;

        private Button[] buttons;

        private readonly int[] arrIndex = new int[5]
        {
            47,
            48,
            49,
            50,
            51
        };

        private ChaControl chaCtrl
        {
            get
            {
                return Singleton<CustomBase>.Instance.chaCtrl;
            }
        }

        private ChaFileFace face
        {
            get
            {
                return chaCtrl.chaFile.custom.face;
            }
        }

        public void CalculateUI()
        {
            float[] shapeValueFace = face.shapeValueFace;
            for (int i = 0; i < sliders.Length; i++)
            {
                if (!(null == sliders[i]))
                {
                    sliders[i].value = shapeValueFace[arrIndex[i]];
                }
            }
        }

        public virtual void UpdateCustomUI()
        {
            CalculateUI();
        }

        public IEnumerator SetInputText()
        {
            yield return new WaitUntil(() => null != chaCtrl);
            float[] shape = face.shapeValueFace;
            for (int i = 0; i < inputs.Length; i++)
            {
                if (!(null == inputs[i]))
                {
                    inputs[i].text = CustomBase.ConvertTextFromRate(0, 100, shape[arrIndex[i]]);
                }
            }
        }

        protected virtual void Awake()
        {
            sliders = new Slider[5]
            {
                sldEarSize,
                sldEarRotY,
                sldEarRotZ,
                sldEarUpForm,
                sldEarLowForm
            };
            inputs = new TMP_InputField[5]
            {
                inpEarSize,
                inpEarRotY,
                inpEarRotZ,
                inpEarUpForm,
                inpEarLowForm
            };
            buttons = new Button[5]
            {
                btnEarSize,
                btnEarRotY,
                btnEarRotZ,
                btnEarUpForm,
                btnEarLowForm
            };
        }

        protected virtual void Start()
        {
            TMP_InputField[] array = inputs;
            foreach (TMP_InputField item in array)
            {
                Singleton<CustomBase>.Instance.lstTmpInputField.Add(item);
            }
            Singleton<CustomBase>.Instance.actUpdateCvsEar += UpdateCustomUI;
            sliders.Select((p, index) => new
            {
                slider = p, index
            }).ToList().ForEach(p =>
            {
                CvsEar cvsEar3 = this;
                p.slider.onValueChanged.AsObservable().Subscribe(delegate (float value)
                {
                    cvsEar3.chaCtrl.SetShapeFaceValue(cvsEar3.arrIndex[p.index], p.slider.value);
                    cvsEar3.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, value);
                });
                p.slider.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
                {
                    Slider slider = p.slider;
                    float value3 = p.slider.value;
                    Vector2 scrollDelta = scl.scrollDelta;
                    slider.value = Mathf.Clamp(value3 + scrollDelta.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                    cvsEar3.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, p.slider.value);
                });
            });
            inputs.Select((p, index) => new
            {
                input = p, index
            }).ToList().ForEach(p =>
            {
                CvsEar cvsEar2 = this;
                p.input.onEndEdit.AsObservable().Subscribe(delegate (string value)
                {
                    cvsEar2.sliders[p.index].value = CustomBase.ConvertRateFromText(0, 100, value);
                });
            });
            buttons.Select((p, index) => new
            {
                button = p, index
            }).ToList().ForEach(p =>
            {
                CvsEar cvsEar = this;
                p.button.onClick.AsObservable().Subscribe(delegate
                {
                    float value2 = Singleton<CustomBase>.Instance.defChaInfo.custom.face.shapeValueFace[cvsEar.arrIndex[p.index]];
                    cvsEar.chaCtrl.SetShapeFaceValue(cvsEar.arrIndex[p.index], value2);
                    cvsEar.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, value2);
                    cvsEar.sliders[p.index].value = value2;
                });
            });
            StartCoroutine(SetInputText());
        }
    }
}
