using Illusion.Extensions;
using Illusion.Game;
using System.Collections;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CvsFaceAll : MonoBehaviour
    {
        [SerializeField]
        private Toggle tglHeadKind;

        [SerializeField]
        private Image imgHeadKind;

        [SerializeField]
        private TextMeshProUGUI textHeadKind;

        [SerializeField]
        private CustomSelectKind customFaceHead;

        [SerializeField]
        private CanvasGroup cgHeadWin;

        [SerializeField]
        private Button btnFaceReset;

        [SerializeField]
        private Slider sldFaceBaseW;

        [SerializeField]
        private TMP_InputField inpFaceBaseW;

        [SerializeField]
        private Button btnFaceBaseW;

        [SerializeField]
        private Slider sldFaceUpZ;

        [SerializeField]
        private TMP_InputField inpFaceUpZ;

        [SerializeField]
        private Button btnFaceUpZ;

        [SerializeField]
        private Slider sldFaceUpY;

        [SerializeField]
        private TMP_InputField inpFaceUpY;

        [SerializeField]
        private Button btnFaceUpY;

        [SerializeField]
        private Slider sldFaceUpSize;

        [SerializeField]
        private TMP_InputField inpFaceUpSize;

        [SerializeField]
        private Button btnFaceUpSize;

        [SerializeField]
        private Slider sldFaceLowZ;

        [SerializeField]
        private TMP_InputField inpFaceLowZ;

        [SerializeField]
        private Button btnFaceLowZ;

        [SerializeField]
        private Slider sldFaceLowW;

        [SerializeField]
        private TMP_InputField inpFaceLowW;

        [SerializeField]
        private Button btnFaceLowW;

        [SerializeField]
        private Toggle tglDetailKind;

        [SerializeField]
        private Image imgDetailKind;

        [SerializeField]
        private TextMeshProUGUI textDetailKind;

        [SerializeField]
        private CustomSelectKind customFaceDetail;

        [SerializeField]
        private CanvasGroup cgDetailWin;

        [SerializeField]
        private Slider sldDetailPow;

        [SerializeField]
        private TMP_InputField inpDetailPow;

        [SerializeField]
        private Button btnDetailPow;

        private Slider[] sliders;

        private TMP_InputField[] inputs;

        private Button[] buttons;

        private readonly int[] arrIndex = new int[6]
        {
            0,
            1,
            2,
            3,
            4,
            5
        };

        private ChaControl chaCtrl
        {
            get
            {
                return Singleton<CustomBase>.Instance.chaCtrl;
            }
        }

        private ChaFileFace face
        {
            get
            {
                return chaCtrl.chaFile.custom.face;
            }
        }

        public void CalculateUI()
        {
            float[] shapeValueFace = face.shapeValueFace;
            for (int i = 0; i < sliders.Length; i++)
            {
                if (!(null == sliders[i]))
                {
                    sliders[i].value = shapeValueFace[arrIndex[i]];
                }
            }
            sldDetailPow.value = face.detailPower;
        }

        public virtual void UpdateCustomUI()
        {
            CalculateUI();
            if (null != customFaceDetail)
            {
                customFaceDetail.UpdateCustomUI();
            }
            if (null != customFaceHead)
            {
                customFaceHead.UpdateCustomUI();
            }
        }

        public IEnumerator SetInputText()
        {
            yield return new WaitUntil(() => null != chaCtrl);
            float[] shape = face.shapeValueFace;
            for (int i = 0; i < inputs.Length; i++)
            {
                if (!(null == inputs[i]))
                {
                    inputs[i].text = CustomBase.ConvertTextFromRate(0, 100, shape[arrIndex[i]]);
                }
            }
            inpDetailPow.text = CustomBase.ConvertTextFromRate(0, 100, face.detailPower);
        }

        public void UpdateSelectHeadKind(string name, Sprite sp, int index)
        {
            if ((bool)textHeadKind)
            {
                textHeadKind.text = name;
            }
            if ((bool)imgHeadKind)
            {
                imgHeadKind.sprite = sp;
            }
            if (face.headId != index)
            {
                chaCtrl.ChangeHead(index);
            }
        }

        public void UpdateSelectDetailKind(string name, Sprite sp, int index)
        {
            if ((bool)textDetailKind)
            {
                textDetailKind.text = name;
            }
            if ((bool)imgDetailKind)
            {
                imgDetailKind.sprite = sp;
            }
            if (face.detailId != index)
            {
                face.detailId = index;
                chaCtrl.ChangeSettingFaceDetail();
            }
        }

        protected virtual void Awake()
        {
            sliders = new Slider[6]
            {
                sldFaceBaseW,
                sldFaceUpZ,
                sldFaceUpY,
                sldFaceUpSize,
                sldFaceLowZ,
                sldFaceLowW
            };
            inputs = new TMP_InputField[6]
            {
                inpFaceBaseW,
                inpFaceUpZ,
                inpFaceUpY,
                inpFaceUpSize,
                inpFaceLowZ,
                inpFaceLowW
            };
            buttons = new Button[6]
            {
                btnFaceBaseW,
                btnFaceUpZ,
                btnFaceUpY,
                btnFaceUpSize,
                btnFaceLowZ,
                btnFaceLowW
            };
        }

        protected virtual void Start()
        {
            TMP_InputField[] array = inputs;
            foreach (TMP_InputField item in array)
            {
                Singleton<CustomBase>.Instance.lstTmpInputField.Add(item);
            }
            Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpDetailPow);
            Singleton<CustomBase>.Instance.actUpdateCvsFaceAll += UpdateCustomUI;
            tglHeadKind.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
            {
                if ((bool)cgHeadWin)
                {
                    bool flag2 = cgHeadWin.alpha != 0f;
                    if (flag2 != isOn)
                    {
                        cgHeadWin.Enable(isOn);
                        if (isOn)
                        {
                            tglDetailKind.isOn = false;
                        }
                    }
                }
            });
            sliders.Select((p, index) => new
            {
                slider = p, index
            }).ToList().ForEach(p =>
            {
                CvsFaceAll cvsFaceAll3 = this;
                p.slider.onValueChanged.AsObservable().Subscribe(delegate (float value)
                {
                    cvsFaceAll3.chaCtrl.SetShapeFaceValue(cvsFaceAll3.arrIndex[p.index], p.slider.value);
                    cvsFaceAll3.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, value);
                });
                p.slider.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
                {
                    Slider slider2 = p.slider;
                    float value4 = p.slider.value;
                    Vector2 scrollDelta2 = scl.scrollDelta;
                    slider2.value = Mathf.Clamp(value4 + scrollDelta2.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                    cvsFaceAll3.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, p.slider.value);
                });
            });
            inputs.Select((p, index) => new
            {
                input = p, index
            }).ToList().ForEach(p =>
            {
                CvsFaceAll cvsFaceAll2 = this;
                p.input.onEndEdit.AsObservable().Subscribe(delegate (string value)
                {
                    cvsFaceAll2.sliders[p.index].value = CustomBase.ConvertRateFromText(0, 100, value);
                });
            });
            buttons.Select((p, index) => new
            {
                button = p, index
            }).ToList().ForEach(p =>
            {
                CvsFaceAll cvsFaceAll = this;
                p.button.onClick.AsObservable().Subscribe(delegate
                {
                    float value3 = Singleton<CustomBase>.Instance.defChaInfo.custom.face.shapeValueFace[cvsFaceAll.arrIndex[p.index]];
                    cvsFaceAll.chaCtrl.SetShapeFaceValue(cvsFaceAll.arrIndex[p.index], value3);
                    cvsFaceAll.inputs[p.index].text = CustomBase.ConvertTextFromRate(0, 100, value3);
                    cvsFaceAll.sliders[p.index].value = value3;
                });
            });
            tglDetailKind.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
            {
                if ((bool)cgDetailWin)
                {
                    bool flag = cgDetailWin.alpha != 0f;
                    if (flag != isOn)
                    {
                        cgDetailWin.Enable(isOn);
                        if (isOn)
                        {
                            tglHeadKind.isOn = false;
                        }
                    }
                }
            });
            sldDetailPow.onValueChanged.AsObservable().Subscribe(delegate (float value)
            {
                face.detailPower = value;
                chaCtrl.ChangeSettingFaceDetailPower();
                inpDetailPow.text = CustomBase.ConvertTextFromRate(0, 100, value);
            });
            sldDetailPow.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
            {
                Slider slider = sldDetailPow;
                float value2 = sldDetailPow.value;
                Vector2 scrollDelta = scl.scrollDelta;
                slider.value = Mathf.Clamp(value2 + scrollDelta.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                inpDetailPow.text = CustomBase.ConvertTextFromRate(0, 100, sldDetailPow.value);
            });
            inpDetailPow.onEndEdit.AsObservable().Subscribe(delegate (string value)
            {
                sldDetailPow.value = CustomBase.ConvertRateFromText(0, 100, value);
            });
            btnDetailPow.onClick.AsObservable().Subscribe(delegate
            {
                float detailPower = Singleton<CustomBase>.Instance.defChaInfo.custom.face.detailPower;
                face.detailPower = detailPower;
                inpDetailPow.text = CustomBase.ConvertTextFromRate(0, 100, detailPower);
                sldDetailPow.value = detailPower;
                chaCtrl.ChangeSettingFaceDetailPower();
            });
            btnFaceReset.onClick.AsObservable().Subscribe(delegate
            {
                Utils.Sound.Play(SystemSE.ok_s);
                chaCtrl.chaFile.LoadFacePreset();
                chaCtrl.Reload(true, false, true, true);
                Singleton<CustomBase>.Instance.updateCustomUI = true;
            });
            StartCoroutine(SetInputText());
        }
    }
}
