using Illusion.Extensions;
using System.Collections;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace ChaCustom
{
    public class CvsMole : MonoBehaviour
    {
        [SerializeField]
        private Toggle tglMoleKind;

        [SerializeField]
        private Image imgMoleKind;

        [SerializeField]
        private TextMeshProUGUI textMoleKind;

        [SerializeField]
        private CustomSelectKind customMole;

        [SerializeField]
        private CanvasGroup cgMoleWin;

        [SerializeField]
        private CvsColor cvsColor;

        [SerializeField]
        private Button btnMoleColor;

        [SerializeField]
        private Image imgMoleColor;

        [SerializeField]
        private Slider sldPosX;

        [SerializeField]
        private TMP_InputField inpPosX;

        [SerializeField]
        private Button btnPosX;

        [SerializeField]
        private Slider sldPosY;

        [SerializeField]
        private TMP_InputField inpPosY;

        [SerializeField]
        private Button btnPosY;

        [SerializeField]
        private Slider sldScale;

        [SerializeField]
        private TMP_InputField inpScale;

        [SerializeField]
        private Button btnScale;

        [SerializeField]
        private Toggle tglPreset;

        [SerializeField]
        private CanvasGroup cgPreWin;

        private ChaControl chaCtrl
        {
            get
            {
                return Singleton<CustomBase>.Instance.chaCtrl;
            }
        }

        private ChaFileFace face
        {
            get
            {
                return chaCtrl.chaFile.custom.face;
            }
        }

        public void CalculateUI()
        {
            Slider slider = sldPosX;
            Vector4 moleLayout = face.moleLayout;
            slider.value = moleLayout.x;
            Slider slider2 = sldPosY;
            Vector4 moleLayout2 = face.moleLayout;
            slider2.value = moleLayout2.y;
            Slider slider3 = sldScale;
            Vector4 moleLayout3 = face.moleLayout;
            slider3.value = moleLayout3.w;
            imgMoleColor.color = face.moleColor;
        }

        public virtual void UpdateCustomUI()
        {
            CalculateUI();
            if (null != customMole)
            {
                customMole.UpdateCustomUI();
            }
            if (cvsColor.CheckConnectKind(CvsColor.ConnectColorKind.Mole))
            {
                cvsColor.SetColor(face.moleColor);
            }
        }

        public IEnumerator SetInputText()
        {
            yield return new WaitUntil(() => null != chaCtrl);
            float[] shape = face.shapeValueFace;
            TMP_InputField tMP_InputField = inpPosX;
            Vector4 moleLayout = face.moleLayout;
            tMP_InputField.text = CustomBase.ConvertTextFromRate(0, 100, moleLayout.x);
            TMP_InputField tMP_InputField2 = inpPosY;
            Vector4 moleLayout2 = face.moleLayout;
            tMP_InputField2.text = CustomBase.ConvertTextFromRate(0, 100, moleLayout2.y);
            TMP_InputField tMP_InputField3 = inpScale;
            Vector4 moleLayout3 = face.moleLayout;
            tMP_InputField3.text = CustomBase.ConvertTextFromRate(0, 100, moleLayout3.w);
        }

        public void UpdateSelectMoleKind(string name, Sprite sp, int index)
        {
            if ((bool)textMoleKind)
            {
                textMoleKind.text = name;
            }
            if ((bool)imgMoleKind)
            {
                imgMoleKind.sprite = sp;
            }
            if (face.moleId != index)
            {
                face.moleId = index;
                FuncUpdateMoleKind();
            }
        }

        public void UpdateMoleColor(Color color)
        {
            face.moleColor = color;
            imgMoleColor.color = color;
            FuncUpdateMoleColor();
        }

        public bool FuncUpdateMoleLayout()
        {
            bool flag = true;
            chaCtrl.AddUpdateCMFaceLayoutFlags(false, false, true);
            flag |= chaCtrl.CreateFaceTexture();
            return flag | chaCtrl.SetFaceBaseMaterial();
        }

        public bool FuncUpdateMoleKind()
        {
            bool flag = true;
            chaCtrl.AddUpdateCMFaceTexFlags(false, false, false, false, false, false, true);
            flag |= chaCtrl.CreateFaceTexture();
            return flag | chaCtrl.SetFaceBaseMaterial();
        }

        public bool FuncUpdateMoleColor()
        {
            bool flag = true;
            chaCtrl.AddUpdateCMFaceColorFlags(false, false, false, false, false, false, true);
            flag |= chaCtrl.CreateFaceTexture();
            return flag | chaCtrl.SetFaceBaseMaterial();
        }

        public void SetDefaultColorWindow()
        {
            cvsColor.Setup("ホクロの色", CvsColor.ConnectColorKind.Mole, face.moleColor, UpdateMoleColor, true);
        }

        protected virtual void Awake()
        {
        }

        protected virtual void Start()
        {
            Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpPosX);
            Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpPosY);
            Singleton<CustomBase>.Instance.lstTmpInputField.Add(inpScale);
            Singleton<CustomBase>.Instance.actUpdateCvsMole += UpdateCustomUI;
            tglMoleKind.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
            {
                if ((bool)cgMoleWin)
                {
                    bool flag2 = cgMoleWin.alpha != 0f;
                    if (flag2 != isOn)
                    {
                        cgMoleWin.Enable(isOn);
                        if (isOn)
                        {
                            tglPreset.isOn = false;
                        }
                    }
                }
            });
            btnMoleColor.OnClickAsObservable().Subscribe(delegate
            {
                if (cvsColor.isOpen && cvsColor.connectColorKind == CvsColor.ConnectColorKind.Mole)
                {
                    cvsColor.Close();
                }
                else
                {
                    cvsColor.Setup("ホクロの色", CvsColor.ConnectColorKind.Mole, face.moleColor, UpdateMoleColor, true);
                }
            });
            sldPosX.onValueChanged.AsObservable().Subscribe(delegate (float value)
            {
                ChaFileFace face6 = face;
                Vector4 moleLayout14 = face.moleLayout;
                float y3 = moleLayout14.y;
                Vector4 moleLayout15 = face.moleLayout;
                face6.moleLayout = new Vector4(value, y3, 0f, moleLayout15.w);
                FuncUpdateMoleLayout();
                inpPosX.text = CustomBase.ConvertTextFromRate(0, 100, value);
            });
            sldPosX.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
            {
                Slider slider3 = sldPosX;
                float value4 = sldPosX.value;
                Vector2 scrollDelta3 = scl.scrollDelta;
                slider3.value = Mathf.Clamp(value4 + scrollDelta3.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                inpPosX.text = CustomBase.ConvertTextFromRate(0, 100, sldPosX.value);
            });
            inpPosX.onEndEdit.AsObservable().Subscribe(delegate (string value)
            {
                sldPosX.value = CustomBase.ConvertRateFromText(0, 100, value);
            });
            btnPosX.onClick.AsObservable().Subscribe(delegate
            {
                Vector4 moleLayout11 = Singleton<CustomBase>.Instance.defChaInfo.custom.face.moleLayout;
                float x5 = moleLayout11.x;
                ChaFileFace face5 = face;
                Vector4 moleLayout12 = face.moleLayout;
                float y2 = moleLayout12.y;
                Vector4 moleLayout13 = face.moleLayout;
                face5.moleLayout = new Vector4(x5, y2, 0f, moleLayout13.w);
                FuncUpdateMoleLayout();
                inpPosX.text = CustomBase.ConvertTextFromRate(0, 100, x5);
                sldPosX.value = x5;
            });
            sldPosY.onValueChanged.AsObservable().Subscribe(delegate (float value)
            {
                ChaFileFace face4 = face;
                Vector4 moleLayout9 = face.moleLayout;
                float x4 = moleLayout9.x;
                Vector4 moleLayout10 = face.moleLayout;
                face4.moleLayout = new Vector4(x4, value, 0f, moleLayout10.w);
                FuncUpdateMoleLayout();
                inpPosY.text = CustomBase.ConvertTextFromRate(0, 100, value);
            });
            sldPosY.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
            {
                Slider slider2 = sldPosY;
                float value3 = sldPosY.value;
                Vector2 scrollDelta2 = scl.scrollDelta;
                slider2.value = Mathf.Clamp(value3 + scrollDelta2.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                inpPosY.text = CustomBase.ConvertTextFromRate(0, 100, sldPosY.value);
            });
            inpPosY.onEndEdit.AsObservable().Subscribe(delegate (string value)
            {
                sldPosY.value = CustomBase.ConvertRateFromText(0, 100, value);
            });
            btnPosY.onClick.AsObservable().Subscribe(delegate
            {
                Vector4 moleLayout6 = Singleton<CustomBase>.Instance.defChaInfo.custom.face.moleLayout;
                float y = moleLayout6.y;
                ChaFileFace face3 = face;
                Vector4 moleLayout7 = face.moleLayout;
                float x3 = moleLayout7.x;
                Vector4 moleLayout8 = face.moleLayout;
                face3.moleLayout = new Vector4(x3, y, 0f, moleLayout8.w);
                FuncUpdateMoleLayout();
                inpPosY.text = CustomBase.ConvertTextFromRate(0, 100, y);
                sldPosY.value = y;
            });
            sldScale.onValueChanged.AsObservable().Subscribe(delegate (float value)
            {
                ChaFileFace face2 = face;
                Vector4 moleLayout4 = face.moleLayout;
                float x2 = moleLayout4.x;
                Vector4 moleLayout5 = face.moleLayout;
                face2.moleLayout = new Vector4(x2, moleLayout5.y, 0f, value);
                FuncUpdateMoleLayout();
                inpScale.text = CustomBase.ConvertTextFromRate(0, 100, value);
            });
            sldScale.OnScrollAsObservable().Subscribe(delegate (PointerEventData scl)
            {
                Slider slider = sldScale;
                float value2 = sldScale.value;
                Vector2 scrollDelta = scl.scrollDelta;
                slider.value = Mathf.Clamp(value2 + scrollDelta.y * Singleton<CustomBase>.Instance.sliderWheelSensitive, 0f, 100f);
                inpScale.text = CustomBase.ConvertTextFromRate(0, 100, sldScale.value);
            });
            inpScale.onEndEdit.AsObservable().Subscribe(delegate (string value)
            {
                sldScale.value = CustomBase.ConvertRateFromText(0, 100, value);
            });
            btnScale.onClick.AsObservable().Subscribe(delegate
            {
                Vector4 moleLayout = Singleton<CustomBase>.Instance.defChaInfo.custom.face.moleLayout;
                float w = moleLayout.w;
                ChaFileFace face = this.face;
                Vector4 moleLayout2 = this.face.moleLayout;
                float x = moleLayout2.x;
                Vector4 moleLayout3 = this.face.moleLayout;
                face.moleLayout = new Vector4(x, moleLayout3.y, 0f, w);
                FuncUpdateMoleLayout();
                inpScale.text = CustomBase.ConvertTextFromRate(0, 100, w);
                sldScale.value = w;
            });
            tglPreset.OnValueChangedAsObservable().Subscribe(delegate (bool isOn)
            {
                if ((bool)cgPreWin)
                {
                    bool flag = cgPreWin.alpha != 0f;
                    if (flag != isOn)
                    {
                        cgPreWin.Enable(isOn);
                        if (isOn)
                        {
                            tglMoleKind.isOn = false;
                        }
                    }
                }
            });
            StartCoroutine(SetInputText());
        }
    }
}
