using Illusion.Extensions;
using Illusion.Game;
using Manager;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

public class ChaStatusScene : BaseLoader
{
    private class CultureTojaJPScope : IDisposable
    {
        private CultureInfo cultureInfo;

        public CultureTojaJPScope()
        {
            cultureInfo = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
        }

        void IDisposable.Dispose()
        {
            Thread.CurrentThread.CurrentCulture = cultureInfo;
        }
    }

    [SerializeField]
    private CanvasGroup cgStatus;

    [SerializeField]
    private ChaStatusComponent cmpMale;

    [SerializeField]
    private ChaStatusComponent cmpFix;

    [SerializeField]
    private ChaStatusComponent cmpTeacher;

    [SerializeField]
    private ChaStatusComponent cmpFemale;

    [SerializeField]
    [Header("一列目を主人公とシナリオor先生キャラに限定するためのダミー")]
    private GameObject dummy;

    [SerializeField]
    private Button btnChangeInfo;

    [SerializeField]
    private Button btnSortFavor;

    [SerializeField]
    private Button btnSortName;

    [SerializeField]
    private Button btnClose;

    private const string FORMAT_NUM = "{0} 人";

    private const string FORMAT_PERCENT = "{0} %";

    private bool isEnd;

    private bool isStatus
    {
        get
        {
            return Singleton<Scene>.Instance.NowSceneNames[0] == "Status";
        }
    }

    public void Unload()
    {
        if (isStatus && !isEnd)
        {
            isEnd = true;
            cgStatus.blocksRaycasts = false;
            Singleton<Scene>.Instance.UnLoad();
        }
    }

    private void Start()
    {
        cgStatus.blocksRaycasts = true;
        Singleton<Game>.Instance.ParameterCorrectValues();
        SaveData saveData = Singleton<Game>.Instance.saveData;
        if (cmpMale != null)
        {
            SaveData.Player player = saveData.player;
            if (cmpMale.cmpStudentCard != null)
            {
                cmpMale.cmpStudentCard.SetCharaInfo(player.charFile, saveData.emblemID, saveData.accademyName, string.Empty);
            }
            TextMeshProUGUI[] array = cmpMale.GetinfoTexts(0);
            if (array != null)
            {
                int num = 0;
                SaveData.Heroine[] heroines = saveData.heroineList.Where(h => h.fixCharaID == 0).ToArray();
                array.SafeProc(num++, delegate (TextMeshProUGUI t)
                {
                    t.text = string.Format("{0} 人", heroines.Count(h => h.isStaff));
                });
                array.SafeProc(num++, delegate (TextMeshProUGUI t)
                {
                    t.text = string.Format("{0} 人", heroines.Count(h => h.isGirlfriend));
                });
                array.SafeProc(num++, delegate (TextMeshProUGUI t)
                {
                    t.text = player.physical.ToString();
                });
                array.SafeProc(num++, delegate (TextMeshProUGUI t)
                {
                    t.text = player.intellect.ToString();
                });
                array.SafeProc(num++, delegate (TextMeshProUGUI t)
                {
                    t.text = player.hentai.ToString();
                });
            }
        }
        ActionScene actScene = Singleton<Game>.Instance.actScene;
        SaveData.Heroine fixHeroine = !(actScene.fixChara == null) ? actScene.fixChara.heroine : null;
        bool flag = fixHeroine != null;
        dummy.SetActiveIfDifferent(!flag);
        if (!flag)
        {
            cmpTeacher.gameObject.SetActiveIfDifferent(false);
            cmpFix.gameObject.SetActiveIfDifferent(false);
        }
        else
        {
            bool isTeacher = fixHeroine.isTeacher;
            cmpTeacher.gameObject.SetActiveIfDifferent(isTeacher);
            cmpFix.gameObject.SetActiveIfDifferent(!isTeacher);
            ChaStatusComponent chaStatusComponent = !isTeacher ? cmpFix : cmpTeacher;
            if (chaStatusComponent != null)
            {
                StudentCardControlComponent cmpStudentCard = chaStatusComponent.cmpStudentCard;
                if (cmpStudentCard != null)
                {
                    cmpStudentCard.SetCharaInfo(fixHeroine.charFile, saveData.emblemID, saveData.accademyName, string.Empty);
                    if (isTeacher)
                    {
                        cmpStudentCard.SetChargeName(Singleton<Voice>.Instance.voiceInfoDic[fixHeroine.voiceNo].Personality);
                    }
                }
                TextMeshProUGUI[] array2 = chaStatusComponent.GetinfoTexts(0);
                array2.SafeProc(0, delegate (TextMeshProUGUI t)
                {
                    t.text = actScene.Map.ConvertMapName(fixHeroine.charaBase.mapNo);
                });
                if (isTeacher)
                {
                    array2.SafeProc(1, delegate (TextMeshProUGUI t)
                    {
                        t.text = string.Format("{0} %", fixHeroine.lewdness);
                    });
                }
            }
        }
        Transform cmpFemaleParent = cmpFemale.transform.parent;
        string cmpFemaleBaseName = cmpFemale.name;
        ChaStatusComponent[] cmpFemales = actScene.npcList.Select(npc => npc.heroine).Select(delegate (SaveData.Heroine heroine, int i)
        {
            ChaStatusComponent chaStatusComponent4 = Instantiate(cmpFemale, cmpFemaleParent, false);
            chaStatusComponent4.name = cmpFemaleBaseName + i.ToString("00");
            chaStatusComponent4.heroine = heroine;
            return chaStatusComponent4;
        }).ToArray();
        string[] relationLabels = new string[4]
        {
            "初対面",
            "知り合い",
            "友達",
            "恋人"
        };
        ChaStatusComponent[] array3 = cmpFemales;
        foreach (ChaStatusComponent chaStatusComponent2 in array3)
        {
            SaveData.Heroine heroine2 = chaStatusComponent2.heroine;
            if (chaStatusComponent2.cmpStudentCard != null)
            {
                chaStatusComponent2.cmpStudentCard.SetCharaInfo(heroine2.charFile, saveData.emblemID, saveData.accademyName, string.Empty);
            }
            TextMeshProUGUI[] array4 = chaStatusComponent2.GetinfoTexts(0);
            int num2 = 0;
            int mapNo = heroine2.charaBase.mapNo;
            string mapName;
            switch (mapNo)
            {
                case 2:
                    mapName = "廊下１Ｆ";
                    break;
                case 3:
                    mapName = "廊下２Ｆ";
                    break;
                case 4:
                    mapName = "廊下３Ｆ";
                    break;
                default:
                    mapName = actScene.Map.ConvertMapName(mapNo);
                    break;
            }
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = mapName;
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = Singleton<Voice>.Instance.voiceInfoDic[heroine2.voiceNo].Personality;
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = !heroine2.isStaff ? "未所属" : "所属";
            });
            chaStatusComponent2.objMember.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(heroine2.isStaff);
            });
            bool isGirlfriend = heroine2.isGirlfriend;
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = relationLabels[heroine2.relation + 1];
            });
            chaStatusComponent2.objHeart.SafeProcObject(delegate (GameObject o)
            {
                o.SetActiveIfDifferent(isGirlfriend);
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = !heroine2.isDate ? "未定" : "約束あり";
            });
            array4 = chaStatusComponent2.GetinfoTexts(1);
            num2 = 0;
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = !heroine2.isAnger ? "普通" : "怒り";
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = string.Format("{0} %", heroine2.favor);
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = string.Format("{0} %", heroine2.lewdness);
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = heroine2.HExperience.ToString();
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.text = string.Format("{0} 回", heroine2.hCount);
            });
            array4 = chaStatusComponent2.GetinfoTexts(2);
            num2 = 0;
            if (!Game.isAdd20)
            {
                chaStatusComponent2.infoObject.SafeProc(2, delegate (ChaStatusComponent.InfoObject o)
                {
                    o.enabled = false;
                });
            }
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                Transform parent = t.transform.parent;
                parent.gameObject.SetActiveIfDifferent(isGirlfriend);
                if (isGirlfriend)
                {
                    Image component = parent.GetChild(0).GetComponent<Image>();
                    int intimacy = heroine2.intimacy;
                    component.fillAmount = Mathf.InverseLerp(0f, 100f, intimacy);
                    t.text = intimacy.ToString();
                }
            });
            array4.SafeProc(num2++, delegate (TextMeshProUGUI t)
            {
                t.gameObject.SetActiveIfDifferent(!isGirlfriend);
            });
            chaStatusComponent2.gameObject.SetActive(true);
        }
        (from _ in btnChangeInfo.OnClickAsObservable()
         select 1).Scan((sum, cur) => cur + sum).Subscribe(delegate (int count)
     {
         Utils.Sound.Play(SystemSE.sel);
         ChaStatusComponent[] array5 = cmpFemales;
         foreach (ChaStatusComponent chaStatusComponent3 in array5)
         {
             chaStatusComponent3.SetActiveInfoAll(count);
         }
     });
        SortFavor(cmpFemales);
        btnSortFavor.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.sel);
            SortFavor(cmpFemales);
        });
        btnSortName.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.sel);
            SortName(cmpFemales);
        });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         select _).Subscribe(delegate
     {
         Unload();
     });
        btnClose.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.cancel);
            Unload();
        });
    }

    private void SortFavor(IEnumerable<ChaStatusComponent> source)
    {
        using (new CultureTojaJPScope())
        {
            foreach (ChaStatusComponent item in from p in source
                                                orderby p.heroine.relation descending, p.heroine.favor descending, p.heroine.parameter.fullname
                                                select p)
            {
                item.transform.SetAsLastSibling();
            }
        }
    }

    private void SortName(IEnumerable<ChaStatusComponent> source)
    {
        using (new CultureTojaJPScope())
        {
            foreach (ChaStatusComponent item in from p in source
                                                orderby p.heroine.parameter.fullname, p.heroine.relation descending, p.heroine.favor descending
                                                select p)
            {
                item.transform.SetAsLastSibling();
            }
        }
    }
}
