using Manager;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UnityEngine;
using UnityEngine.UI;

namespace Config
{
    public class VoiceSetting : BaseSetting
    {
        private class SetData
        {
            public SoundData sd;

            public Toggle toggle;

            public Slider slider;
        }

        [SerializeField]
        private GameObject prefab;

        [SerializeField]
        private RectTransform rtcPcm;

        [SerializeField]
        private RectTransform node;

        private Dictionary<Transform, SetData> dic = new Dictionary<Transform, SetData>();

        public override void Init()
        {
            Dictionary<int, VoiceSystem.Voice> chara = Singleton<Voice>.Instance._Config.chara;
            Add(rtcPcm);
            for (int i = 0; i < node.transform.childCount; i++)
            {
                RectTransform trans = node.transform.GetChild(i) as RectTransform;
                Add(trans);
            }
            int num = node.transform.childCount + (node.transform.childCount % 2 != 0 ? 1 : 0);
            foreach (KeyValuePair<int, string> item in Singleton<Voice>.Instance.voiceInfoList.ToDictionary(v => v.No, v => v.Personality))
            {
                if (chara.ContainsKey(item.Key))
                {
                    Create(num++, item.Key, item.Value);
                }
            }
        }

        protected override void ValueToUI()
        {
            foreach (SetData value in dic.Values)
            {
                value.toggle.isOn = value.sd.Switch;
                value.slider.value = value.sd.Volume;
            }
        }

        private void Create(int num, int key, string name)
        {
            RectTransform component = Instantiate(prefab, node.transform).GetComponent<RectTransform>();
            component.name = key.ToString();
            component.GetComponentInChildren<TextMeshProUGUI>().text = name;
            Add(key, component);
        }

        private bool Add(Transform trans)
        {
            if (dic.ContainsKey(trans))
            {
                return false;
            }
            SetData setData = new SetData();
            setData.sd = Singleton<Voice>.Instance._Config.PCM;
            setData.slider = trans.GetComponentInChildren<Slider>();
            setData.toggle = trans.GetComponentInChildren<Toggle>();
            SetData setData2 = setData;
            AddEvent(setData2);
            dic.Add(trans, setData2);
            return true;
        }

        private bool Add(int key, Transform trans)
        {
            if (dic.ContainsKey(trans))
            {
                return false;
            }
            SetData setData = new SetData();
            setData.sd = Singleton<Voice>.Instance._Config.chara[key].sound;
            setData.slider = trans.GetComponentInChildren<Slider>();
            setData.toggle = trans.GetComponentInChildren<Toggle>();
            SetData setData2 = setData;
            AddEvent(setData2);
            dic.Add(trans, setData2);
            return true;
        }

        private void AddEvent(SetData data)
        {
            data.toggle.onValueChanged.AsObservable().Subscribe(delegate (bool isOn)
            {
                data.sd.Switch = isOn;
            });
            data.toggle.OnValueChangedAsObservable().SubscribeToInteractable(data.slider);
            (from value in data.slider.onValueChanged.AsObservable()
             select (int)value).Subscribe(delegate (int value)
         {
             data.sd.Volume = value;
         });
        }
    }
}
