using ActionGame.Chara;
using Config;
using Illusion.CustomAttributes;
using Illusion.Extensions;
using Illusion.Game;
using Manager;
using System;
using System.Collections;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

[DefaultExecutionOrder(100)]
public class ConfigScene : BaseLoader
{
    [Serializable]
    public class ShortCutGroup
    {
        public string title;

        public string name;

        public int visibleNo;

        [SerializeField]
        private RectTransform[] _trans;

        public RectTransform trans
        {
            get
            {
                return _trans[visibleNo];
            }
        }

        public void VisibleUpdate()
        {
            RectTransform[] trans = _trans;
            foreach (RectTransform rectTransform in trans)
            {
                rectTransform.gameObject.SetActiveIfDifferent(rectTransform == this.trans);
            }
        }
    }

    [SerializeField]
    private Canvas canvas;

    [SerializeField]
    [Label("コンフィグ項目を写す場所")]
    private RectTransform mainWindow;

    [SerializeField]
    [Label("ショートカットボタンの場所")]
    private RectTransform shortCutButtonBackGround;

    [Label("ショートカットボタンのプレファブ")]
    [SerializeField]
    private Button shortCutButtonPrefab;

    [Tooltip("ショートカットのリンク")]
    [SerializeField]
    private ShortCutGroup[] shortCuts;

    [Tooltip("初めに設定されているボタン")]
    [SerializeField]
    private Button[] buttons;

    private BaseSetting[] settings;

    private bool isEnd;

    private bool isConfig
    {
        get
        {
            return Singleton<Scene>.Instance.NowSceneNames[0] == "Config";
        }
    }

    public void Unload()
    {
        if (!isConfig || isEnd)
        {
            return;
        }
        isEnd = true;
        Save();
        if (Manager.Config.AddData.AINotPlayerTarget)
        {
            ActionScene actScene = Singleton<Game>.Instance.actScene;
            if (actScene != null && actScene.npcList != null)
            {
                foreach (AI item in from npc in actScene.npcList
                                    select npc.AI into p
                                    where p.ActionNoCheck(5, 8)
                                    select p)
                {
                    item.TimeEnd();
                }
            }
        }
        Singleton<Scene>.Instance.UnLoad();
    }

    private void PlaySE(SystemSE se = SystemSE.sel)
    {
        Utils.Sound.Play(se);
    }

    private IEnumerator Start()
    {
        ShortCutGroup shortCutGroup = shortCuts.FirstOrDefault(p => p.name == "Add");
        if (shortCutGroup != null && Game.isAdd20)
        {
            shortCutGroup.visibleNo = 1;
        }
        ShortCutGroup[] array = shortCuts;
        foreach (ShortCutGroup shortCutGroup2 in array)
        {
            shortCutGroup2.VisibleUpdate();
        }
        var shortCutList = shortCuts.Select(delegate (ShortCutGroup p, int i)
        {
            Button button = Instantiate(shortCutButtonPrefab, shortCutButtonBackGround, false);
            TextMeshProUGUI componentInChildren = button.GetComponentInChildren<TextMeshProUGUI>(true);
            componentInChildren.text = p.title;
            RectTransform rectTransform = button.transform as RectTransform;
            if (rectTransform != null)
            {
                Vector2 anchoredPosition2 = rectTransform.anchoredPosition;
                float x = anchoredPosition2.x;
                Vector2 sizeDelta2 = rectTransform.sizeDelta;
                anchoredPosition2.x = x + sizeDelta2.x * i;
                rectTransform.anchoredPosition = anchoredPosition2;
            }
            return new
            {
                bt = button, p.name
            };
        }).ToList();
        settings = shortCuts.Select(p => p.trans.GetComponent<BaseSetting>()).ToArray();
        while (!Manager.Config.initialized)
        {
            yield return null;
        }
        BaseSetting[] array2 = settings;
        foreach (BaseSetting baseSetting in array2)
        {
            baseSetting.Init();
            baseSetting.UIPresenter();
        }
        float spacing = mainWindow.GetComponent<VerticalLayoutGroup>().spacing;
        shortCutList.ForEach(sc =>
        {
            sc.bt.OnClickAsObservable().Subscribe(delegate
            {
                ShortCutGroup[] array3 = shortCuts.TakeWhile(p => p.name != sc.name).ToArray();
                float y = Mathf.Min(array3.Sum(delegate (ShortCutGroup p)
                {
                    Vector2 sizeDelta = p.trans.sizeDelta;
                    return sizeDelta.y;
                }) + spacing * array3.Length, mainWindow.rect.height - (mainWindow.parent as RectTransform).rect.height);
                Vector2 anchoredPosition = mainWindow.anchoredPosition;
                anchoredPosition.y = y;
                mainWindow.anchoredPosition = anchoredPosition;
            });
        });
        shortCutList.ForEach(sc =>
        {
            sc.bt.OnClickAsObservable().Subscribe(delegate
            {
                PlaySE();
            });
        });
        buttons.ToList().ForEach(delegate (Button bt)
        {
            this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).SubscribeToInteractable(bt);
        });
        Button backButton = buttons.FirstOrDefault(p => p.name == "Back");
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where isConfig
         where !Singleton<Scene>.Instance.IsOverlap && backButton.interactable
         select _).Take(1).Subscribe(delegate
     {
         Unload();
     });
        if (Singleton<GameCursor>.IsInstance())
        {
            Singleton<GameCursor>.Instance.SetCursorLock(false);
            Singleton<GameCursor>.Instance.SetCursorTexture(-1);
        }
        Canvas[] sorter = (from p in FindObjectsOfType<Canvas>()
                           where p != canvas
                           select p).ToArray();
        if (sorter.Any())
        {
            canvas.sortingOrder = sorter.Max(p => p.sortingOrder) + 1;
        }
    }

    public void OnDefault()
    {
        PlaySE();
        CheckScene.Parameter param = new CheckScene.Parameter();
        UnityAction yes = delegate
        {
            if (Singleton<KeyInput>.IsInstance())
            {
                Singleton<KeyInput>.Instance.Reset();
            }
            Singleton<Voice>.Instance.Reset();
            Singleton<Manager.Config>.Instance.Reset();
            BaseSetting[] array = settings;
            foreach (BaseSetting baseSetting in array)
            {
                baseSetting.UIPresenter();
            }
            Singleton<Scene>.Instance.UnLoad();
        };
        param.Yes = yes;
        param.No = delegate
        {
            Singleton<Scene>.Instance.UnLoad();
        };
        param.Title = "設定を初期化しますか？";
        Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param, observer)).TakeUntilDestroy(this).Subscribe();
    }

    public void OnTitle()
    {
        Save();
        Observable.FromCoroutine(_ => Utils.Scene.ReturnTitle()).TakeUntilDestroy(this).Subscribe();
        PlaySE();
    }

    public void OnGameEnd()
    {
        Utils.Scene.GameEnd();
        PlaySE();
    }

    public void OnBack()
    {
        PlaySE();
        Unload();
    }

    private void Save()
    {
        if (Singleton<KeyInput>.IsInstance())
        {
            Singleton<KeyInput>.Instance.Save();
        }
        Singleton<Voice>.Instance.Save();
        Singleton<Manager.Config>.Instance.Save();
    }
}
