using UnityEngine;
using UnityEngine.EventSystems;

namespace CustomUtility
{
    public class CustomGuideMove : CustomGuideBase, IInitializePotentialDragHandler, IPointerDownHandler, IPointerUpHandler, IEventSystemHandler
    {
        public enum MoveAxis
        {
            X,
            Y,
            Z,
            XYZ
        }

        public MoveAxis axis;

        private Vector2 oldPos = Vector2.zero;

        private Camera m_Camera;

        private Camera camera
        {
            get
            {
                if (m_Camera == null)
                {
                    m_Camera = Camera.main;
                }
                return m_Camera;
            }
        }

        public void OnInitializePotentialDrag(PointerEventData eventData)
        {
            if (!GlobalMethod.IsCameraActionFlag(guideObject.ccv2))
            {
                oldPos = eventData.pressPosition;
            }
        }

        public override void OnDrag(PointerEventData eventData)
        {
            if (!GlobalMethod.IsCameraActionFlag(guideObject.ccv2))
            {
                base.OnDrag(eventData);
                bool _snap = false;
                Vector3 vector = axis != MoveAxis.XYZ ? AxisMove(eventData.delta, ref _snap) : WorldPos(eventData.position) - WorldPos(oldPos);
                Vector3 position = guideObject.amount.position;
                position += vector;
                guideObject.amount.position = axis == MoveAxis.XYZ ? position : !_snap ? position : Parse(position);
                oldPos = eventData.position;
            }
        }

        public void OnPointerDown(PointerEventData eventData)
        {
            if (!GlobalMethod.IsCameraActionFlag(guideObject.ccv2))
            {
                GlobalMethod.SetCameraMoveFlag(guideObject.ccv2, false);
                guideObject.isDrag = true;
            }
        }

        public void OnPointerUp(PointerEventData eventData)
        {
            GlobalMethod.SetCameraMoveFlag(guideObject.ccv2, true);
            guideObject.isDrag = false;
        }

        private Vector3 WorldPos(Vector2 _screenPos)
        {
            Plane plane = new Plane(camera.transform.forward * -1f, transform.position);
            Ray ray = RectTransformUtility.ScreenPointToRay(camera, _screenPos);
            float enter;
            return !plane.Raycast(ray, out enter) ? transform.position : ray.GetPoint(enter);
        }

        private Vector3 AxisPos(Vector2 _screenPos)
        {
            Vector3 position = transform.position;
            Plane plane = new Plane(transform.forward, position);
            if (!plane.GetSide(camera.transform.position))
            {
                plane = new Plane(transform.forward * -1f, position);
            }
            Vector3 up = transform.up;
            Ray ray = RectTransformUtility.ScreenPointToRay(camera, _screenPos);
            float enter;
            return !plane.Raycast(ray, out enter) ? Vector3.Project(position, up) : Vector3.Project(ray.GetPoint(enter), up);
        }

        private Vector3 AxisMove(Vector2 _delta, ref bool _snap)
        {
            Vector3 b = camera.transform.TransformVector(_delta.x * 0.005f, _delta.y * 0.005f, 0f) * guideObject.speedMove;
            Vector3 up = transform.up;
            up.x = Mathf.Abs(up.x);
            up.y = Mathf.Abs(up.y);
            up.z = Mathf.Abs(up.z);
            return Vector3.Scale(up, b);
        }

        private Vector3 Parse(Vector3 _src)
        {
            return _src;
        }
    }
}
