using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using UnityEngine;

public class EliminateScale : MonoBehaviour
{
    public enum EliminateScaleKind
    {
        ALL,
        X,
        Y,
        Z,
        XY,
        XZ,
        YZ,
        NONE
    }

    [Serializable]
    public class ShapeMove
    {
        public int numShape;

        public Vector3 posMax;

        public Vector3 posMin;

        public Vector3 rotMax;

        public Vector3 rotMin;
    }

    [Tooltip("省きたいスケール軸")]
    public EliminateScaleKind kind;

    public List<ShapeMove> lstShape = new List<ShapeMove>();

    public ChaControl chaCtrl;

    [SerializeField]
    [Header("Debug表示")]
    private Vector3 defScale = Vector3.one;

    [SerializeField]
    private Vector3 defPositon = Vector3.zero;

    [SerializeField]
    private Quaternion defRotation = Quaternion.identity;

    [Header("Debug表示 出現時のTransform")]
    [SerializeField]
    private Vector3 oldScale = Vector3.one;

    [SerializeField]
    private Vector3 oldPositon = Vector3.zero;

    [SerializeField]
    private Quaternion oldRotation = Quaternion.identity;

    private Dictionary<int, Dictionary<int, ShapeMove>> dicEliminate = new Dictionary<int, Dictionary<int, ShapeMove>>();

    private void Awake()
    {
        defScale = transform.lossyScale;
        defPositon = transform.localPosition;
        defRotation = transform.localRotation;
    }

    private void OnEnable()
    {
        oldScale = transform.lossyScale;
        oldPositon = transform.localPosition;
        oldRotation = transform.localRotation;
    }

    private void OnDisable()
    {
        transform.localScale = defScale;
        transform.localPosition = defPositon;
        transform.localRotation = defRotation;
        oldScale = Vector3.one;
        oldPositon = Vector3.zero;
        oldRotation = Quaternion.identity;
    }

    private void LateUpdate()
    {
        Vector3 lossyScale = transform.lossyScale;
        Vector3 localScale = transform.localScale;
        transform.localScale = new Vector3(kind != 0 && kind != EliminateScaleKind.X && kind != EliminateScaleKind.XY && kind != EliminateScaleKind.XZ ? localScale.x : localScale.x / lossyScale.x * oldScale.x, kind != 0 && kind != EliminateScaleKind.Y && kind != EliminateScaleKind.XY && kind != EliminateScaleKind.YZ ? localScale.y : localScale.y / lossyScale.y * oldScale.y, kind != 0 && kind != EliminateScaleKind.Z && kind != EliminateScaleKind.XZ && kind != EliminateScaleKind.YZ ? localScale.z : localScale.z / lossyScale.z * oldScale.z);
        if (chaCtrl != null)
        {
            Vector3 zero = Vector3.zero;
            Vector3 zero2 = Vector3.zero;
            for (int i = 0; i < lstShape.Count; i++)
            {
                float shapeBodyValue = chaCtrl.GetShapeBodyValue(Mathf.Max(lstShape[i].numShape, 0));
                zero += !(shapeBodyValue >= 0.5f) ? Vector3.Lerp(lstShape[i].posMin, Vector3.zero, Mathf.InverseLerp(0f, 0.5f, shapeBodyValue)) : Vector3.Lerp(Vector3.zero, lstShape[i].posMax, Mathf.InverseLerp(0.5f, 1f, shapeBodyValue));
                zero2 += !(shapeBodyValue >= 0.5f) ? Vector3.Lerp(lstShape[i].rotMin, Vector3.zero, Mathf.InverseLerp(0f, 0.5f, shapeBodyValue)) : Vector3.Lerp(Vector3.zero, lstShape[i].rotMax, Mathf.InverseLerp(0.5f, 1f, shapeBodyValue));
            }
            transform.localPosition = oldPositon + zero;
            transform.localRotation = oldRotation * Quaternion.Euler(zero2);
        }
    }

    public bool LoadList(int _idObj)
    {
        string text = GlobalMethod.LoadAllListText("h/list/", "EliminateScale_AibuItem_" + _idObj.ToString("00"));
        dicEliminate.Clear();
        if (text.IsNullOrEmpty())
        {
            return false;
        }
        string[,] data;
        GlobalMethod.GetListString(text, out data);
        int length = data.GetLength(0);
        for (int i = 0; i < length; i++)
        {
            int num = 0;
            int result2;
            int.TryParse(data[i, num++], out result2);
            int result;
            int.TryParse(data[i, num++], out result);
            if (!dicEliminate.ContainsKey(result2))
            {
                dicEliminate.Add(result2, new Dictionary<int, ShapeMove>());
            }
            Dictionary<int, ShapeMove> dictionary = dicEliminate[result2];
            if (!dictionary.ContainsKey(result))
            {
                dictionary.Add(result, new ShapeMove());
            }
            ShapeMove shapeMove = dictionary[result];
            shapeMove.numShape = int.Parse(data[i, num++]);
            shapeMove.posMin = new Vector3(float.Parse(data[i, num++]), float.Parse(data[i, num++]), float.Parse(data[i, num++]));
            shapeMove.rotMin = new Vector3(float.Parse(data[i, num++]), float.Parse(data[i, num++]), float.Parse(data[i, num++]));
            shapeMove.posMax = new Vector3(float.Parse(data[i, num++]), float.Parse(data[i, num++]), float.Parse(data[i, num++]));
            shapeMove.rotMax = new Vector3(float.Parse(data[i, num++]), float.Parse(data[i, num++]), float.Parse(data[i, num++]));
        }
        return true;
    }

    public bool SetShapeList(int _idArea)
    {
        if (!dicEliminate.ContainsKey(_idArea))
        {
            lstShape = new List<ShapeMove>();
            return false;
        }
        lstShape = dicEliminate[_idArea].Values.ToList();
        return true;
    }

    private void SaveText(StreamWriter _writer)
    {
        foreach (ShapeMove item in lstShape)
        {
            _writer.Write(item.numShape + "\t");
            _writer.Write(item.posMin.x.ToString() + "\t");
            _writer.Write(item.posMin.y.ToString() + "\t");
            _writer.Write(item.posMin.z.ToString() + "\t");
            _writer.Write(item.rotMin.x.ToString() + "\t");
            _writer.Write(item.rotMin.y.ToString() + "\t");
            _writer.Write(item.rotMin.z.ToString() + "\t");
            _writer.Write(item.posMax.x.ToString() + "\t");
            _writer.Write(item.posMax.y.ToString() + "\t");
            _writer.Write(item.posMax.z.ToString() + "\t");
            _writer.Write(item.rotMax.x.ToString() + "\t");
            _writer.Write(item.rotMax.y.ToString() + "\t");
            _writer.Write(item.rotMax.z.ToString() + "\t");
            _writer.Write("\n");
        }
    }
}
