using CharaFiles;
using Illusion.CustomAttributes;
using Illusion.Game;
using Illusion.Game.Extensions;
using Manager;
using System;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

public class EntryPlayer : BaseLoader
{
    [SerializeField]
    [Label("キャラリスト")]
    private ChaFileListCtrl chaFileLists;

    [SerializeField]
    [Label("エンブレムリスト")]
    private EmblemSelectListCtrl emblemLists;

    [Label("学園名入力")]
    [SerializeField]
    private InputField inpAccademy;

    [SerializeField]
    [Label("学生証")]
    private StudentCardControlComponent cmpStudent;

    [SerializeField]
    [Label("キャンバス")]
    private Canvas Canvas;

    [SerializeField]
    private CanvasGroup cgEntryPlayer;

    [SerializeField]
    [Label("決定ボタン")]
    private Button btnNext;

    [Label("戻るボタン")]
    [SerializeField]
    private Button btnPrev;

    private ChaFileControl chaFileCtrl;

    private int emblemId;

    private string accademyName = "コイカツ女学園";

    private bool noPlaySSE;

    private void Start()
    {
        if (cgEntryPlayer != null)
        {
            this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
            {
                cgEntryPlayer.interactable = isOn;
                cgEntryPlayer.blocksRaycasts = isOn;
            });
        }
        this.SelectSEAdd(btnNext, btnPrev);
        btnNext.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.ok_s);
        });
        btnPrev.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.cancel);
        });
        SaveData saveData = Singleton<Game>.Instance.saveData;
        SaveData.Player player = saveData.player = new SaveData.Player(false);
        CardUpdate();
        noPlaySSE = true;
        CreateMaleList();
        chaFileLists.SelectItem(0);
        CreateEmblemList();
        emblemLists.SelectItem(emblemId);
        noPlaySSE = false;
        cmpStudent.SetAccademy(accademyName);
        inpAccademy.OnEndEditAsObservable().Subscribe(delegate (string buf)
        {
            if (buf.IsNullOrEmpty())
            {
                buf = "コイカツ女学園";
            }
            accademyName = buf;
            cmpStudent.SetAccademy(buf);
        });
        (from _ in this.UpdateAsObservable()
         select chaFileCtrl != null).SubscribeToInteractable(btnNext);
        btnNext.OnClickAsObservable().Subscribe(delegate
        {
            saveData.accademyName = accademyName;
            saveData.emblemID = emblemId;
            player.SetCharFile(chaFileCtrl);
            player.charFileInitialized = true;
            string levelName = "ClassRoomSelect";
            Singleton<Scene>.Instance.LoadReserve(new Scene.Data
            {
                levelName = levelName,
                isAsync = true,
                isFade = true
            }, true);
        });
        Action close = delegate
        {
            cgEntryPlayer.blocksRaycasts = false;
            Utils.Sound.Play(SystemSE.cancel);
            Singleton<Scene>.Instance.UnLoad();
        };
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where !Singleton<Scene>.Instance.IsNowLoadingFade
         where Singleton<Scene>.Instance.NowSceneNames[0] == "EntryPlayer"
         select _).Take(1).Subscribe(delegate
     {
         close();
     });
        btnPrev.OnClickAsObservable().Take(1).Subscribe(delegate
        {
            close();
        });
    }

    private void CreateMaleList()
    {
        FolderAssist folderAssist = new FolderAssist();
        string[] searchPattern = new string[1]
        {
            "*.png"
        };
        string folder = UserData.Path + "chara/male/";
        folderAssist.CreateFolderInfoEx(folder, searchPattern);
        chaFileLists.ClearList();
        int fileCount = folderAssist.GetFileCount();
        int num = 0;
        for (int i = 0; i < fileCount; i++)
        {
            ChaFileControl chaFileControl = new ChaFileControl();
            if (!chaFileControl.LoadCharaFile(folderAssist.lstFile[i].FullPath))
            {
                int lastErrorCode = chaFileControl.GetLastErrorCode();
            }
            else if (chaFileControl.parameter.sex == 0)
            {
                chaFileLists.AddList(num, chaFileControl, folderAssist.lstFile[i].FullPath, folderAssist.lstFile[i].FileName, folderAssist.lstFile[i].time);
                num++;
            }
        }
        chaFileLists.Create(OnChangeChaFileListSelect);
    }

    public void OnChangeChaFileListSelect(ChaFileInfo info)
    {
        if (!noPlaySSE)
        {
            Utils.Sound.Play(SystemSE.sel);
        }
        int[] selectIndex = chaFileLists.GetSelectIndex();
        if (selectIndex.Length == 0)
        {
            chaFileCtrl = null;
            cmpStudent.Clear(string.Empty);
            cmpStudent.SetAccademy(accademyName);
            cmpStudent.SetEmblem(emblemId);
        }
        else
        {
            chaFileCtrl = new ChaFileControl();
            chaFileCtrl.LoadCharaFile(info.FullPath);
            cmpStudent.SetCharaInfo(chaFileCtrl);
        }
    }

    private void CreateEmblemList()
    {
        ChaListControl chaListCtrl = Singleton<Character>.Instance.chaListCtrl;
        Dictionary<int, ListInfoBase> categoryInfo = chaListCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.mt_emblem);
        List<ListInfoBase> list = categoryInfo.Values.ToList();
        list.ForEach(delegate (ListInfoBase info)
        {
            emblemLists.AddList(info.Id, info.Name, info.GetInfo(ChaListDefine.KeyType.ThumbAB), info.GetInfo(ChaListDefine.KeyType.ThumbTex));
        });
        emblemLists.Create(OnChangeEmblemListSelect);
    }

    public void OnChangeEmblemListSelect(int index)
    {
        if (!noPlaySSE)
        {
            Utils.Sound.Play(SystemSE.sel);
        }
        emblemId = index;
        cmpStudent.SetEmblem(index);
    }

    private void CardUpdate()
    {
        cmpStudent.Clear(string.Empty);
        cmpStudent.SetCharaInfo(chaFileCtrl, emblemId, accademyName, Singleton<Game>.Instance.Player.GetCallName());
    }
}
