using FBSAssist;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

[Serializable]
public class FBSBase
{
    public FBSTargetInfo[] FBSTarget;

    protected Dictionary<int, float> dictBackFace = new Dictionary<int, float>();

    protected Dictionary<int, float> dictNowFace = new Dictionary<int, float>();

    protected float openRate;

    [Range(0f, 1f)]
    public float OpenMin;

    [Range(0f, 1f)]
    public float OpenMax = 1f;

    [Range(-0.1f, 1f)]
    public float FixedRate = -0.1f;

    private float correctOpenMax = -1f;

    protected TimeProgressCtrl blendTimeCtrl;

#if FIX
    private static readonly List<bool> _blendIdCache = new List<bool>();
    private static readonly List<float> _blendValueCache = new List<float>();
#endif

    public bool Init()
    {
        blendTimeCtrl = new TimeProgressCtrl();
        blendTimeCtrl.End();
        for (int i = 0; i < FBSTarget.Length; i++)
        {
            FBSTarget[i].SetSkinnedMeshRenderer();
        }
        dictBackFace.Clear();
        dictBackFace[0] = 1f;
        dictNowFace.Clear();
        dictNowFace[0] = 1f;
        return true;
    }

    public void SetOpenRateForce(float rate)
    {
        openRate = rate;
    }

    public int GetMaxPtn()
    {
        if (FBSTarget.Length == 0)
        {
            return 0;
        }
        return FBSTarget[0].PtnSet.Length;
    }

    public void ChangePtn(int ptn, bool blend)
    {
        if (GetMaxPtn() > ptn && (dictNowFace.Count != 1 || !dictNowFace.ContainsKey(ptn) || dictNowFace[ptn] != 1f))
        {
            Dictionary<int, float> dictionary = new Dictionary<int, float>();
            dictionary[ptn] = 1f;
            ChangeFace(dictionary, blend);
        }
    }

    public void ChangeFace(Dictionary<int, float> dictFace, bool blend)
    {
        bool flag = false;
        byte b = 0;
        float num = 0f;
        FBSTargetInfo[] fBSTarget = FBSTarget;
        foreach (FBSTargetInfo fBSTargetInfo in fBSTarget)
        {
            SkinnedMeshRenderer skinnedMeshRenderer = fBSTargetInfo.GetSkinnedMeshRenderer();
            foreach (int key in dictFace.Keys)
            {
                if (skinnedMeshRenderer.sharedMesh.blendShapeCount <= fBSTargetInfo.PtnSet[key].Close)
                {
                    b = 1;
                    break;
                }
                if (skinnedMeshRenderer.sharedMesh.blendShapeCount <= fBSTargetInfo.PtnSet[key].Open)
                {
                    b = 1;
                    break;
                }
                num += dictFace[key];
            }
            if (b != 0)
            {
                break;
            }
            if (!flag && num > 1f)
            {
                b = 2;
                break;
            }
            flag = true;
        }
        if (b != 1 && b != 2)
        {
            dictBackFace.Clear();
            foreach (int key2 in dictNowFace.Keys)
            {
                dictBackFace[key2] = dictNowFace[key2];
            }
            dictNowFace.Clear();
            foreach (int key3 in dictFace.Keys)
            {
                dictNowFace[key3] = dictFace[key3];
            }
            if (!blend)
            {
                blendTimeCtrl.End();
            }
            else
            {
                blendTimeCtrl.Start();
            }
        }
    }

    public void SetFixedRate(float value)
    {
        FixedRate = value;
    }

    public void SetCorrectOpenMax(float value)
    {
        correctOpenMax = value;
    }

#if FIX
    //Fix new Dictionary spam that caused massive allocations and garbage pileup
    public void CalculateBlendShape()
    {
        if (FBSTarget.Length == 0)
            return;

        float openMax = correctOpenMax >= 0f ? correctOpenMax : OpenMax;
        float lerpOpenRate = Mathf.Lerp(OpenMin, openMax, openRate);
        if (0f <= FixedRate)
            lerpOpenRate = FixedRate;
        float rate = 0f;
        if (blendTimeCtrl != null)
            rate = blendTimeCtrl.Calculate();

        retry:
        try
        {
            for (var index = 0; index < FBSTarget.Length; index++)
            {
                for (var i = 0; i < _blendIdCache.Count; i++)
                {
                    _blendIdCache[i] = false;
                    _blendValueCache[i] = 0f;
                }

                FBSTargetInfo targetInfo = FBSTarget[index];
                SkinnedMeshRenderer skinnedMeshRenderer = targetInfo.GetSkinnedMeshRenderer();
                int percent = (int)Mathf.Clamp(lerpOpenRate * 100f, 0f, 100f);

                for (var i = 0; i < targetInfo.PtnSet.Length; i++)
                {
                    var ptnSet = targetInfo.PtnSet[i];
                    var resultClose = 0f;
                    var resultOpen = 0f;

                    if (rate != 1f)
                    {
                        float valueBack;
                        if (dictBackFace.TryGetValue(i, out valueBack))
                        {
                            resultClose += valueBack * (100 - percent) * (1f - rate);
                            resultOpen += valueBack * percent * (1f - rate);
                        }
                    }

                    float valueNow;
                    if (dictNowFace.TryGetValue(i, out valueNow))
                    {
                        resultClose += valueNow * (100 - percent) * rate;
                        resultOpen += valueNow * percent * rate;
                    }

                    if (ptnSet.Close >= 0)
                    {
                        _blendIdCache[ptnSet.Close] = true;
                        _blendValueCache[ptnSet.Close] += resultClose;
                    }
                    if (ptnSet.Open >= 0)
                    {
                        _blendIdCache[ptnSet.Open] = true;
                        _blendValueCache[ptnSet.Open] += resultOpen;
                    }
                }

                for (var i = 0; i < _blendIdCache.Count; i++)
                {
                    if (_blendIdCache[i])
                    {
                        skinnedMeshRenderer.SetBlendShapeWeight(i, _blendValueCache[i]);
                    }
                }
            }
        }
        catch (ArgumentOutOfRangeException)
        {
            Debug.Log("Increasing blend ID cache size");
            _blendIdCache.AddRange(Enumerable.Repeat(false, 50));
            _blendValueCache.AddRange(Enumerable.Repeat(0f, 50));
            goto retry;
        }
    }
#else
    public void CalculateBlendShape()
    {
        if (FBSTarget.Length == 0)
        {
            return;
        }
        float b = !(correctOpenMax < 0f) ? correctOpenMax : OpenMax;
        float num = Mathf.Lerp(OpenMin, b, openRate);
        if (0f <= FixedRate)
        {
            num = FixedRate;
        }
        float num2 = 0f;
        if (blendTimeCtrl != null)
        {
            num2 = blendTimeCtrl.Calculate();
        }
        FBSTargetInfo[] fBSTarget = FBSTarget;
        foreach (FBSTargetInfo fBSTargetInfo in fBSTarget)
        {
            SkinnedMeshRenderer skinnedMeshRenderer = fBSTargetInfo.GetSkinnedMeshRenderer();
            Dictionary<int, float> dictionary = new Dictionary<int, float>();
            for (int j = 0; j < fBSTargetInfo.PtnSet.Length; j++)
            {
                dictionary[fBSTargetInfo.PtnSet[j].Close] = 0f;
                dictionary[fBSTargetInfo.PtnSet[j].Open] = 0f;
            }
            int num3 = (int)Mathf.Clamp(num * 100f, 0f, 100f);
            if (num2 != 1f)
            {
                foreach (int key in dictBackFace.Keys)
                {
                    dictionary[fBSTargetInfo.PtnSet[key].Close] = dictionary[fBSTargetInfo.PtnSet[key].Close] + dictBackFace[key] * (100 - num3) * (1f - num2);
                    dictionary[fBSTargetInfo.PtnSet[key].Open] = dictionary[fBSTargetInfo.PtnSet[key].Open] + dictBackFace[key] * num3 * (1f - num2);
                }
            }
            foreach (int key2 in dictNowFace.Keys)
            {
                dictionary[fBSTargetInfo.PtnSet[key2].Close] = dictionary[fBSTargetInfo.PtnSet[key2].Close] + dictNowFace[key2] * (100 - num3) * num2;
                dictionary[fBSTargetInfo.PtnSet[key2].Open] = dictionary[fBSTargetInfo.PtnSet[key2].Open] + dictNowFace[key2] * num3 * num2;
            }
            foreach (KeyValuePair<int, float> item in dictionary)
            {
                if (item.Key != -1)
                {
                    skinnedMeshRenderer.SetBlendShapeWeight(item.Key, item.Value);
                }
            }
        }
    }
#endif
}
