using ActionGame;
using ADV;
using Manager;
using System.Collections;
using System.Collections.Generic;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

namespace FixCharaEvent
{
    public class FixEventScene : BaseLoader
    {
        private class ADVParam : SceneParameter
        {
            public bool isInVisibleChara = true;

            public ADVParam(MonoBehaviour scene)
                : base(scene)
            {
            }

            public override void Init(Data data)
            {
                ADVScene advScene = SceneParameter.advScene;
                TextScenario scenario = advScene.Scenario;
                scenario.BackCamera.fieldOfView = Camera.main.fieldOfView;
                scenario.LoadBundleName = data.bundleName;
                scenario.LoadAssetName = data.assetName;
                advScene.Stand.SetPositionAndRotation(data.position, data.rotation);
                scenario.heroineList = data.heroineList;
                scenario.transferList = data.transferList;
                if (!data.heroineList.IsNullOrEmpty())
                {
                    scenario.currentChara = new CharaData(data.heroineList[0], scenario, null);
                }
                if (data.camera != null)
                {
                    scenario.BackCamera.transform.SetPositionAndRotation(data.camera.position, data.camera.rotation);
                }
                float fadeInTime = data.fadeInTime;
                if (fadeInTime > 0f)
                {
                    advScene.fadeTime = fadeInTime;
                }
                else
                {
                    isInVisibleChara = false;
                }
            }

            public override void Release()
            {
                ADVScene advScene = SceneParameter.advScene;
                advScene.gameObject.SetActive(false);
            }

            public override void WaitEndProc()
            {
            }
        }

        [SerializeField]
        private Canvas canvas;

        [SerializeField]
        private RectTransform characterView;

        [SerializeField]
        private Button enterButton;

        [SerializeField]
        private Button returnButton;

        protected override void Awake()
        {
            base.Awake();
            ParameterList.Add(new ADVParam(this));
        }

        private void OnDestroy()
        {
            ParameterList.Remove(this);
        }

        private IEnumerator Start()
        {
            enabled = false;
            ReactiveProperty<Tuple<int, EventInfo.Param>> eventParam = new ReactiveProperty<Tuple<int, EventInfo.Param>>();
            Dictionary<int, List<EventInfo.Param>> evInfos = new Dictionary<int, List<EventInfo.Param>>();
            CommonLib.GetAssetBundleNameListFromPath("action/list/event/", true).ForEach(delegate (string file)
            {
                EventInfo[] allAssets2 = AssetBundleManager.LoadAllAsset(file, typeof(EventInfo)).GetAllAssets<EventInfo>();
                foreach (EventInfo eventInfo in allAssets2)
                {
                    int result2;
                    int.TryParse(eventInfo.name, out result2);
                    List<EventInfo.Param> value;
                    if (!evInfos.TryGetValue(result2, out value))
                    {
                        value = evInfos[result2] = new List<EventInfo.Param>();
                    }
                    value.AddRange(eventInfo.param);
                }
                AssetBundleManager.UnloadAssetBundle(file, false);
            });
            ReactiveProperty<Button> enterEventColor = new ReactiveProperty<Button>();
            Color enterColor = Color.yellow;
            enterEventColor.Scan(delegate (Button prev, Button current)
            {
                if (prev != null)
                {
                    prev.colors = current.colors;
                }
                ColorBlock colors = current.colors;
                colors.normalColor = enterColor;
                colors.highlightedColor = enterColor;
                current.colors = colors;
                return current;
            }).Subscribe();
            List<SaveData.Heroine> list = Game.CreateFixCharaList();
            foreach (KeyValuePair<int, List<EventInfo.Param>> item3 in evInfos)
            {
                int key = item3.Key;
                if (key < 0)
                {
                    SaveData.Heroine heroine = list.Find(p => p.fixCharaID == key);
                    if (heroine != null)
                    {
                        GameObject gameObject = Instantiate(characterView.gameObject, characterView.parent, false);
                        gameObject.SetActive(true);
                        ImageButton component = gameObject.GetComponent<ImageButton>();
                        RawImage faceImage = component.faceImage;
                        Rect rect = faceImage.rectTransform.rect;
                        Texture2D texture2D = new Texture2D((int)rect.width, (int)rect.height);
                        texture2D.LoadImage(heroine.charFile.facePngData);
                        faceImage.texture = texture2D;
                        faceImage.enabled = true;
                        foreach (EventInfo.Param param2 in item3.Value)
                        {
                            Button button = Instantiate(component.button, component.button.transform.parent, false);
                            button.gameObject.SetActive(true);
                            button.GetComponentInChildren<Text>().text = param2.Name;
                            (from _ in button.OnPointerClickAsObservable()
                             select param2).Subscribe(delegate (EventInfo.Param p)
                         {
                             enterEventColor.Value = button;
                             eventParam.Value = Tuple.Create(key, p);
                         });
                        }
                    }
                }
            }
            eventParam.Select(param => param.Item2 != null).SubscribeToInteractable(enterButton);
            enterButton.OnClickAsObservable().Subscribe(delegate
            {
                Observable.FromCoroutine(__ => Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In)).Subscribe(delegate
                {
                    EventSystem.current.SetSelectedGameObject(null);
                    SaveData.Heroine heroine2 = new SaveData.Heroine(true);
                    TextAsset textAsset = null;
                    int item = eventParam.Value.Item1;
                    EventInfo.Param item2 = eventParam.Value.Item2;
                    foreach (string item4 in CommonLib.GetAssetBundleNameListFromPath("action/fixchara/", true))
                    {
                        TextAsset[] allAssets = AssetBundleManager.LoadAllAsset(item4, typeof(TextAsset)).GetAllAssets<TextAsset>();
                        foreach (TextAsset textAsset2 in allAssets)
                        {
                            int result;
                            int.TryParse(textAsset2.name.Replace("c", string.Empty), out result);
                            if (item == result)
                            {
                                textAsset = textAsset2;
                                break;
                            }
                        }
                        AssetBundleManager.UnloadAssetBundle(item4, false);
                        if (textAsset != null)
                        {
                            break;
                        }
                    }
                    Game.LoadFromTextAsset(item, heroine2, textAsset);
                    List<Program.Transfer> list2 = Program.Transfer.NewList();
                    Program.SetParam(heroine2, list2);
                    SaveData.Player player = Singleton<Game>.Instance.Player;
                    Game.LoadFromTextAsset(player);
                    Program.SetParam(player, list2);
                    list2.Add(Program.Transfer.Create(true, Command.CameraLock, bool.FalseString));
                    list2.Add(Program.Transfer.Open(item2.Bundle, item2.Asset));
                    StartCoroutine(Program.Open(new Data
                    {
                        position = Vector3.zero,
                        rotation = Quaternion.identity,
                        scene = this,
                        transferList = list2,
                        heroineList = new List<SaveData.Heroine>
                        {
                            heroine2
                        }
                    }, new Program.OpenDataProc
                    {
                        onLoad = delegate
                        {
                            StartCoroutine(Wait());
                        }
                    }));
                });
            });
            returnButton.OnClickAsObservable().Subscribe(delegate
            {
                Singleton<Scene>.Instance.UnLoad();
            });
            enabled = true;
            yield break;
        }

        private IEnumerator Wait()
        {
            canvas.enabled = false;
            yield return null;
            yield return new WaitWhile(() => Program.isADVScene);
            yield return StartCoroutine(Singleton<Scene>.Instance.Fade(SimpleFade.Fade.Out));
            canvas.enabled = true;
            Singleton<Manager.Sound>.Instance.StopBGM(0.8f);
        }
    }
}
