using ActionGame;
using ADV;
using CharaFiles;
using Illusion.Extensions;
using Illusion.Game;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

public class FixEventSceneEx : BaseLoader
{
    private class ADVParam : SceneParameter
    {
        public bool isInVisibleChara = true;

        public ADVParam(MonoBehaviour scene)
            : base(scene)
        {
        }

        public override void Init(Data data)
        {
            ADVScene advScene = SceneParameter.advScene;
            TextScenario scenario = advScene.Scenario;
            scenario.BackCamera.fieldOfView = Camera.main.fieldOfView;
            scenario.LoadBundleName = data.bundleName;
            scenario.LoadAssetName = data.assetName;
            advScene.Stand.SetPositionAndRotation(data.position, data.rotation);
            scenario.heroineList = data.heroineList;
            scenario.transferList = data.transferList;
            if (!data.heroineList.IsNullOrEmpty())
            {
                scenario.currentChara = new CharaData(data.heroineList[0], scenario, null);
            }
            if (data.camera != null)
            {
                scenario.BackCamera.transform.SetPositionAndRotation(data.camera.position, data.camera.rotation);
            }
            float fadeInTime = data.fadeInTime;
            if (fadeInTime > 0f)
            {
                advScene.fadeTime = fadeInTime;
            }
            else
            {
                isInVisibleChara = false;
            }
        }

        public override void Release()
        {
            ADVScene advScene = SceneParameter.advScene;
            advScene.gameObject.SetActive(false);
        }

        public override void WaitEndProc()
        {
        }
    }

    [SerializeField]
    private BGM bgm = BGM.Memories;

    [SerializeField]
    private Canvas canvas;

    [SerializeField]
    private CanvasGroup cgFixEvent;

    [SerializeField]
    private GameObject[] objCharaLoad;

    [SerializeField]
    private ChaFileListCtrl[] cmpChaFileListCtrl;

    [SerializeField]
    private Button[] btnChaLoad;

    [SerializeField]
    private Button[] btnCancel;

    private int editNo;

    [SerializeField]
    private Toggle[] tglMainMenu;

    [SerializeField]
    private GameObject[] objMenuTarget;

    [SerializeField]
    private Button[] btnCharaInit;

    [SerializeField]
    private Button[] btnEdit;

    [SerializeField]
    private GameObject[] objCard;

    private RawImage[] imgCard;

    [SerializeField]
    private GameObject[] objStand;

    [SerializeField]
    private TextMeshProUGUI[] textTitle;

    [SerializeField]
    private Toggle[] tglCha00Event;

    [SerializeField]
    private Toggle[] tglCha01Event;

    [SerializeField]
    private Toggle[] tglCha02Event;

    [SerializeField]
    private Toggle[] tglCha03Event;

    [SerializeField]
    private Button btnBack;

    [SerializeField]
    private Button btnOK;

    protected override void Awake()
    {
        base.Awake();
        ParameterList.Add(new ADVParam(this));
    }

    private void OnDestroy()
    {
        ParameterList.Remove(this);
    }

    private void Start()
    {
        cgFixEvent.blocksRaycasts = true;
        Utils.Sound.Play(new Utils.Sound.SettingBGM(bgm));
        List<SaveData.Heroine> fixInitCharaDataList = Game.CreateFixCharaList();
        MemoriesData memData = new MemoriesData();
        memData.Initialize();
        memData.Load();
        memData.CreateFixHeroines();
        CreateCharaList();
        Dictionary<int, List<EventInfo.Param>> evInfos = new Dictionary<int, List<EventInfo.Param>>();
        CommonLib.GetAssetBundleNameListFromPath("action/list/event/", true).ForEach(delegate (string file)
        {
            EventInfo[] allAssets = AssetBundleManager.LoadAllAsset(file, typeof(EventInfo)).GetAllAssets<EventInfo>();
            foreach (EventInfo eventInfo in allAssets)
            {
                int result;
                int.TryParse(eventInfo.name, out result);
                List<EventInfo.Param> value;
                if (!evInfos.TryGetValue(result, out value))
                {
                    value = evInfos[result] = new List<EventInfo.Param>();
                }
                value.AddRange(eventInfo.param);
            }
            AssetBundleManager.UnloadAssetBundle(file, false);
        });
        HashSet<int> fixCharaTaked = Singleton<Game>.Instance.glSaveData.fixCharaTaked;
        for (int j = 0; j < tglMainMenu.Length - 1; j++)
        {
            tglMainMenu[j].interactable = fixCharaTaked.Contains(ConvertIDtoFixID(j));
        }
        IntReactiveProperty mainPage = new IntReactiveProperty(tglMainMenu.Check(tgl => tgl.interactable));
        for (int k = 0; k < tglMainMenu.Length; k++)
        {
            objMenuTarget[k].SetActive(false);
            tglMainMenu[k].isOn = k == mainPage.Value;
        }
        mainPage.Skip(1).Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.sel);
        });
        mainPage.Scan(delegate (int prev, int current)
        {
            objMenuTarget[prev].SetActive(false);
            return current;
        }).Subscribe(delegate (int i)
        {
            objMenuTarget[i].SetActive(true);
        });
        (from list in tglMainMenu.Select(UnityUIComponentExtensions.OnValueChangedAsObservable).CombineLatest()
         select list.IndexOf(true) into i
         where i >= 0
         select i).Subscribe(delegate (int i)
     {
         mainPage.Value = i;
     });
        Toggle[][] subToggles = new Toggle[4][]
        {
            tglCha00Event,
            tglCha01Event,
            tglCha02Event,
            tglCha03Event
        };
        IntReactiveProperty subPage = new IntReactiveProperty(-1);
        bool isPlaySE = true;
        (from _ in subPage.Skip(1)
         where isPlaySE
         select _).Subscribe(delegate
     {
         Utils.Sound.Play(SystemSE.sel);
     });
        mainPage.Where(i => i < subToggles.Length).Subscribe(delegate (int i)
        {
            isPlaySE = false;
            subPage.Value = subToggles[i].Check(t => t.isOn);
            isPlaySE = true;
        });
        Toggle[][] array = subToggles;
        foreach (Toggle[] source in array)
        {
            (from list in source.Select(UnityUIComponentExtensions.OnValueChangedAsObservable).CombineLatest()
             select list.IndexOf(true) into i
             where i >= 0
             select i).Subscribe(delegate (int i)
         {
             subPage.Value = i;
         });
        }
        (from page in subPage.Select(_ => mainPage.Value).Merge(mainPage)
         where page < 4
         select page).Subscribe(delegate (int page)
     {
         SaveData.Heroine heroine6 = ConvertIDtoChara(page, memData) as SaveData.Heroine;
         EventInfo.Param param3 = evInfos[heroine6.fixCharaID][subPage.Value];
         textTitle[page].text = param3.Name;
     });
        imgCard = new RawImage[objCard.Length];
        for (int m = 0; m < objCard.Length; m++)
        {
            imgCard[m] = objCard[m].GetComponent<RawImage>();
            byte[] array2 = null;
            if (m == 7)
            {
                array2 = memData.etcData[-100].pngData;
            }
            else
            {
                SaveData.Heroine heroine = ConvertIDtoChara(m, memData) as SaveData.Heroine;
                array2 = memData.etcData[heroine.fixCharaID].pngData;
            }
            objCard[m].SetActiveIfDifferent(null != array2);
            objStand[m].SetActiveIfDifferent(null == array2);
            if (array2 != null)
            {
                if ((bool)imgCard[m].texture)
                {
                    Destroy(imgCard[m].texture);
                }
                Texture2D texture2D = new Texture2D(0, 0);
                texture2D.LoadImage(array2);
                imgCard[m].texture = texture2D;
            }
        }
        btnCharaInit.Select((p, i) => new
        {
            btn = p,
            index = i
        }).ToList().ForEach(p =>
        {
            p.btn.OnClickAsObservable().Subscribe(delegate
            {
                Utils.Sound.Play(SystemSE.ok_s);
                CheckScene.Parameter param2 = new CheckScene.Parameter();
                param2.Yes = delegate
                {
                    SaveData.CharaData charaData2 = ConvertIDtoChara(p.index, memData);
                    if (charaData2 is SaveData.Heroine)
                    {
                        SaveData.Heroine heroine4 = charaData2 as SaveData.Heroine;
                        SaveData.Heroine heroine5 = fixInitCharaDataList.FirstOrDefault(c => c.fixCharaID == heroine4.fixCharaID);
                        ChaFile.CopyChaFile(heroine4.charFile, heroine5.charFile);
                        memData.etcData[heroine4.fixCharaID].initialized = true;
                        memData.etcData[heroine4.fixCharaID].pngData = null;
                    }
                    else
                    {
                        memData.CreatePlayer();
                        memData.etcData[-100].initialized = true;
                        memData.etcData[-100].pngData = null;
                    }
                    memData.Save();
                    Singleton<Scene>.Instance.UnLoad();
                    objStand[p.index].SetActiveIfDifferent(true);
                    objCard[p.index].SetActiveIfDifferent(false);
                };
                param2.No = delegate
                {
                    Singleton<Scene>.Instance.UnLoad();
                };
                param2.Title = "容姿を初期化しますか？";
                Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param2, observer)).StartAsCoroutine();
            });
        });
        btnEdit.Select((p, i) => new
        {
            btn = p,
            index = i
        }).ToList().ForEach(p =>
        {
            p.btn.OnClickAsObservable().Subscribe(delegate
            {
                Utils.Sound.Play(SystemSE.window_o);
                objCharaLoad[0].SetActiveIfDifferent(7 == p.index);
                objCharaLoad[1].SetActiveIfDifferent(7 != p.index);
                editNo = p.index;
            });
        });
        Action close = delegate
        {
            cgFixEvent.blocksRaycasts = false;
            Utils.Sound.Play(SystemSE.cancel);
            Singleton<Scene>.Instance.UnLoad();
        };
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where !Singleton<Scene>.Instance.IsNowLoadingFade
         where !objCharaLoad[0].activeInHierarchy && !objCharaLoad[1].activeInHierarchy
         where Singleton<Scene>.Instance.NowSceneNames[0] == "FixEventSceneEx"
         select _).Take(1).Subscribe(delegate
     {
         close();
     });
        btnBack.OnClickAsObservable().Take(1).Subscribe(delegate
        {
            close();
        });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where !Singleton<Scene>.Instance.IsNowLoadingFade
         where objCharaLoad[0].activeInHierarchy || objCharaLoad[1].activeInHierarchy
         where Singleton<Scene>.Instance.NowSceneNames[0] == "FixEventSceneEx"
         select _).Subscribe(delegate
     {
         Utils.Sound.Play(SystemSE.cancel);
         objCharaLoad[0].SetActiveIfDifferent(false);
         objCharaLoad[1].SetActiveIfDifferent(false);
     });
        mainPage.Select(i => i < subToggles.Length).SubscribeToInteractable(btnOK);
        btnOK.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.ok_s);
            Observable.FromCoroutine(__ => Singleton<Scene>.Instance.Fade(SimpleFade.Fade.In)).Subscribe(delegate
            {
                EventSystem.current.SetSelectedGameObject(null);
                SaveData.Heroine heroine3 = ConvertIDtoChara(mainPage.Value, memData) as SaveData.Heroine;
                EventInfo.Param param = evInfos[heroine3.fixCharaID][subPage.Value];
                Singleton<Game>.Instance.HeroineList.Clear();
                Singleton<Game>.Instance.HeroineList.AddRange(memData.heroineDic.Values);
                List<Program.Transfer> list2 = Program.Transfer.NewList();
                Program.SetParam(heroine3, list2);
                SaveData.Player player = Singleton<Game>.Instance.Player;
                player.SetCharFile(memData.player.charFile);
                Program.SetParam(player, list2);
                list2.Add(Program.Transfer.Create(true, Command.CameraLock, bool.FalseString));
                list2.Add(Program.Transfer.Open(param.Bundle, param.Asset));
                StartCoroutine(Program.Open(new Data
                {
                    position = Vector3.zero,
                    rotation = Quaternion.identity,
                    scene = this,
                    transferList = list2,
                    heroineList = new List<SaveData.Heroine>
                    {
                        heroine3
                    }
                }, new Program.OpenDataProc
                {
                    onLoad = delegate
                    {
                        StartCoroutine(Wait());
                    }
                }));
            });
        });
        btnChaLoad.Select((p, i) => new
        {
            btn = p,
            index = i
        }).ToList().ForEach(p =>
        {
            p.btn.OnClickAsObservable().Subscribe(delegate
            {
                Utils.Sound.Play(SystemSE.ok_s);
                objCharaLoad[0].SetActiveIfDifferent(false);
                objCharaLoad[1].SetActiveIfDifferent(false);
                ChaFileInfoComponent selectTopItem = cmpChaFileListCtrl[p.index].GetSelectTopItem();
                if (null != selectTopItem)
                {
                    ChaFileControl chaFileControl = new ChaFileControl();
                    chaFileControl.LoadCharaFile(selectTopItem.info.FullPath, (byte)p.index);
                    SaveData.CharaData charaData = ConvertIDtoChara(editNo, memData);
                    charaData.charFile.pngData = chaFileControl.pngData;
                    charaData.charFile.SetCustomBytes(chaFileControl.GetCustomBytes(), ChaFileDefine.ChaFileCustomVersion);
                    charaData.charFile.SetCoordinateBytes(chaFileControl.GetCoordinateBytes(), ChaFileDefine.ChaFileCoordinateVersion);
                    charaData.charFile.parameter.firstname = chaFileControl.parameter.firstname;
                    charaData.charFile.parameter.lastname = chaFileControl.parameter.lastname;
                    if ((bool)imgCard[editNo].texture)
                    {
                        Destroy(imgCard[editNo].texture);
                    }
                    Texture2D texture2D2 = new Texture2D(180, 240);
                    texture2D2.LoadImage(charaData.charFile.pngData);
                    imgCard[editNo].texture = texture2D2;
                    if (p.index == 0)
                    {
                        memData.etcData[-100].initialized = false;
                        memData.etcData[-100].pngData = charaData.charFile.pngData;
                    }
                    else
                    {
                        SaveData.Heroine heroine2 = ConvertIDtoChara(editNo, memData) as SaveData.Heroine;
                        memData.etcData[heroine2.fixCharaID].initialized = false;
                        memData.etcData[heroine2.fixCharaID].pngData = charaData.charFile.pngData;
                    }
                    memData.Save();
                    objStand[editNo].SetActiveIfDifferent(false);
                    objCard[editNo].SetActiveIfDifferent(true);
                }
            });
        });
        btnCancel.Select((p, i) => new
        {
            btn = p,
            index = i
        }).ToList().ForEach(p =>
        {
            p.btn.OnClickAsObservable().Subscribe(delegate
            {
                Utils.Sound.Play(SystemSE.cancel);
                objCharaLoad[0].SetActiveIfDifferent(false);
                objCharaLoad[1].SetActiveIfDifferent(false);
            });
        });
        Observable.EveryUpdate().Subscribe(delegate
        {
            for (int n = 0; n < 2; n++)
            {
                bool interactable = !(null == cmpChaFileListCtrl[n].GetSelectTopItem());
                btnChaLoad[n].interactable = interactable;
            }
        }).AddTo(this);
    }

    private void CreateCharaList()
    {
        FolderAssist folderAssist = new FolderAssist();
        string[] searchPattern = new string[1]
        {
            "*.png"
        };
        string[] array = new string[2]
        {
            "chara/male/",
            "chara/female/"
        };
        for (int i = 0; i < 2; i++)
        {
            string folder = UserData.Path + array[i];
            folderAssist.CreateFolderInfoEx(folder, searchPattern);
            cmpChaFileListCtrl[i].ClearList();
            int fileCount = folderAssist.GetFileCount();
            int num = 0;
            for (int j = 0; j < fileCount; j++)
            {
                ChaFileControl chaFileControl = new ChaFileControl();
                if (chaFileControl.LoadCharaFile(folderAssist.lstFile[j].FullPath))
                {
                    if (chaFileControl.parameter.sex == i)
                    {
                        cmpChaFileListCtrl[i].AddList(num, chaFileControl, folderAssist.lstFile[j].FullPath,
                            folderAssist.lstFile[j].FileName, folderAssist.lstFile[j].time);
                        num++;
                    }
                }
            }
        }
        cmpChaFileListCtrl[0].Create(null);
        cmpChaFileListCtrl[1].Create(null);
    }

    private IEnumerator Wait()
    {
        canvas.enabled = false;
        yield return null;
        yield return new WaitWhile(() => Program.isADVScene);
        if (!Scene.isReturnTitle && !Scene.isGameEnd)
        {
            yield return StartCoroutine(Singleton<Scene>.Instance.Fade(SimpleFade.Fade.Out));
            canvas.enabled = true;
            Utils.Sound.Play(new Utils.Sound.SettingBGM(bgm));
        }
    }

    private void InitToggles(Toggle[] toggles, int n)
    {
        for (int i = 0; i < toggles.Length; i++)
        {
            toggles[i].isOn = i == n;
        }
    }

    private SaveData.CharaData ConvertIDtoChara(int editNo, MemoriesData memData)
    {
        int num = ConvertIDtoFixID(editNo);
        if (num == int.MaxValue)
        {
            return memData.player;
        }
        return memData.heroineDic[num];
    }

    private int ConvertIDtoFixID(int editNo)
    {
        int result = int.MaxValue;
        switch (editNo)
        {
            case 0:
                result = -5;
                break;
            case 1:
                result = -8;
                break;
            case 2:
                result = -9;
                break;
            case 3:
                result = -10;
                break;
            case 4:
                result = -1;
                break;
            case 5:
                result = -4;
                break;
            case 6:
                result = -2;
                break;
        }
        return result;
    }
}
