using System;
using System.Collections;

namespace Illusion.Extensions
{
    public static class BitArrayExtensions
    {
        public static bool Any(this BitArray array)
        {
            IEnumerator enumerator = array.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    if ((bool)enumerator.Current)
                    {
                        return true;
                    }
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            return false;
        }

        public static bool All(this BitArray array)
        {
            IEnumerator enumerator = array.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    if (!(bool)enumerator.Current)
                    {
                        return false;
                    }
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            return true;
        }

        public static bool None(this BitArray array)
        {
            return !array.Any();
        }

        public static void Flip(this BitArray array, int index)
        {
            array.Set(index, !array.Get(index));
        }
    }
}
