using System;
using System.Collections;
using System.Text;

namespace Illusion.Extensions
{
    public static class EnumExtensions
    {
        public static bool HasFlag(this Enum self, Enum flag)
        {
            ulong num = Convert.ToUInt64(flag);
            return self.AND(num) == num;
        }

        public static ulong Add(this Enum self, Enum flag)
        {
            return self.OR(flag);
        }

        public static ulong Sub(this Enum self, Enum flag)
        {
            return Convert.ToUInt64(self) & flag.NOT();
        }

        public static ulong Get(this Enum self, Enum flag)
        {
            return self.AND(flag);
        }

        public static ulong Reverse(this Enum self, Enum flag)
        {
            return self.XOR(flag);
        }

        public static ulong NOT(this Enum self)
        {
            return ~Convert.ToUInt64(self);
        }

        public static ulong AND(this Enum self, Enum flag)
        {
            return Convert.ToUInt64(self) & Convert.ToUInt64(flag);
        }

        public static ulong AND(this Enum self, ulong flag)
        {
            return Convert.ToUInt64(self) & flag;
        }

        public static ulong OR(this Enum self, Enum flag)
        {
            return Convert.ToUInt64(self) | Convert.ToUInt64(flag);
        }

        public static ulong OR(this Enum self, ulong flag)
        {
            return Convert.ToUInt64(self) | flag;
        }

        public static ulong XOR(this Enum self, Enum flag)
        {
            return Convert.ToUInt64(self) ^ Convert.ToUInt64(flag);
        }

        public static ulong XOR(this Enum self, ulong flag)
        {
            return Convert.ToUInt64(self) ^ flag;
        }

        public static string LabelFormat(this Enum self, string label)
        {
            StringBuilder stringBuilder = new StringBuilder();
            IEnumerator enumerator = Enum.GetValues(self.GetType()).GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    object current = enumerator.Current;
                    if (self.HasFlag((Enum)current))
                    {
                        stringBuilder.AppendFormat("{0} | ", current);
                    }
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            return stringBuilder.Length != 0 ? label + stringBuilder : string.Empty;
        }

        public static bool All(this Enum self)
        {
            return self.Reverse(self.Everything()) == 0;
        }

        public static bool Any(this Enum self)
        {
            return Convert.ToUInt64(self) != self.Nothing();
        }

        public static Enum Everything(this Enum self)
        {
            ulong num = 0uL;
            IEnumerator enumerator = Enum.GetValues(self.GetType()).GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    object current = enumerator.Current;
                    num += Convert.ToUInt64(current);
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            return (Enum)Enum.ToObject(self.GetType(), num);
        }

        public static ulong Nothing(this Enum self)
        {
            return 0uL;
        }

        public static ulong Normalize(this Enum self)
        {
            return (ulong)Enum.ToObject(self.GetType(), Convert.ToInt64(self) & Convert.ToInt64(self.Everything()));
        }
    }
}
