using UnityEngine;
using UnityEngine.UI;

namespace Illusion.Extensions
{
    public static class RectTransformExtensions
    {
        public static void SetPosition(this RectTransform self, Transform target3D, Vector3 setPos, Camera targetCamera = null)
        {
            if (targetCamera == null)
            {
                targetCamera = Camera.main;
            }
            Vector2 vector = RectTransformUtility.WorldToScreenPoint(targetCamera, target3D.position);
            self.position = new Vector2(vector[0] + setPos[0], vector[1] + setPos[1]);
        }

        public static void AdjustSize(this RectTransform self, Text text)
        {
            self.sizeDelta = new Vector2(text.preferredWidth, text.preferredHeight);
        }

        public static void AdjustHeight(this RectTransform self, Text text)
        {
            Vector2 sizeDelta = self.sizeDelta;
            self.sizeDelta = new Vector2(sizeDelta.x, text.preferredHeight);
        }

        public static bool IsHeightOver(this RectTransform self, Text text)
        {
            return text.preferredHeight > self.rect.height;
        }

        public static void Left(this RectTransform self, float v)
        {
            Vector2 offsetMin = self.offsetMin;
            self.offsetMin = new Vector2(v, offsetMin.y);
        }

        public static void Top(this RectTransform self, float v)
        {
            Vector2 offsetMin = self.offsetMin;
            self.offsetMin = new Vector2(offsetMin.x, v);
        }

        public static void Right(this RectTransform self, float v)
        {
            float x = 0f - v;
            Vector2 offsetMax = self.offsetMax;
            self.offsetMax = new Vector2(x, offsetMax.y);
        }

        public static void Bottom(this RectTransform self, float v)
        {
            Vector2 offsetMax = self.offsetMax;
            self.offsetMax = new Vector2(offsetMax.x, 0f - v);
        }

        public static void Width(this RectTransform self, Vector2 v)
        {
            self.Left(v.x);
            self.Right(v.y);
        }

        public static void Height(this RectTransform self, Vector2 v)
        {
            self.Top(v.x);
            self.Bottom(v.y);
        }

        public static float Left(this RectTransform self)
        {
            Vector2 offsetMin = self.offsetMin;
            return offsetMin.x;
        }

        public static float Top(this RectTransform self)
        {
            Vector2 offsetMin = self.offsetMin;
            return offsetMin.y;
        }

        public static float Right(this RectTransform self)
        {
            Vector2 offsetMax = self.offsetMax;
            return 0f - offsetMax.x;
        }

        public static float Bottom(this RectTransform self)
        {
            Vector2 offsetMax = self.offsetMax;
            return 0f - offsetMax.y;
        }

        public static Vector2 Width(this RectTransform self)
        {
            return new Vector2(self.Left(), self.Right());
        }

        public static Vector2 Height(this RectTransform self)
        {
            return new Vector2(self.Top(), self.Bottom());
        }

        public static bool IsStretchWidth(this RectTransform self)
        {
            Vector2 anchorMin = self.anchorMin;
            int result;
            if (anchorMin.x == 0f)
            {
                Vector2 anchorMax = self.anchorMax;
                result = anchorMax.x == 1f ? 1 : 0;
            }
            else
            {
                result = 0;
            }
            return (byte)result != 0;
        }

        public static bool IsStretchHeight(this RectTransform self)
        {
            Vector2 anchorMin = self.anchorMin;
            int result;
            if (anchorMin.y == 0f)
            {
                Vector2 anchorMax = self.anchorMax;
                result = anchorMax.y == 1f ? 1 : 0;
            }
            else
            {
                result = 0;
            }
            return (byte)result != 0;
        }

        public static bool IsStretch(this RectTransform self)
        {
            return self.IsStretchWidth() || self.IsStretchHeight();
        }

        public static bool IsStretchWidthHeight(this RectTransform self)
        {
            return self.IsStretchWidth() && self.IsStretchHeight();
        }
    }
}
