using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Illusion.Serialize
{
    [Serializable]
    public class TableBase<TKey, TValue, Type> where Type : KeyValuePair<TKey, TValue>, new()
    {
        [SerializeField]
        private List<Type> list = new List<Type>();

        public Dictionary<TKey, TValue> Table
        {
            get
            {
                return list.ToDictionary(v => v.Key, v => v.Value);
            }
            set
            {
                list = value.Select(delegate (System.Collections.Generic.KeyValuePair<TKey, TValue> v)
                {
                    Type val = new Type();
                    val.Key = v.Key;
                    val.Value = v.Value;
                    return val;
                }).ToList();
            }
        }

        public int Count
        {
            get
            {
                return list.Count;
            }
        }

        public TValue this[TKey key]
        {
            get
            {
                return Get(key);
            }
        }

        public TValue Get(TKey key)
        {
            Type val = list.Find(type => type.Key.Equals(key));
            return val == null ? default(TValue) : val.Value;
        }

        public void Set(Type type)
        {
            Remove(type.Key);
            Add(type);
        }

        public void Set(TKey key, TValue value)
        {
            Type val = new Type();
            val.Key = key;
            val.Value = value;
            Set(val);
        }

        public bool Add(Type type)
        {
            bool flag = !list.Any(p => p.Key.Equals(type.Key));
            if (flag)
            {
                list.Add(type);
            }
            return flag;
        }

        public bool Add(TKey key, TValue value)
        {
            Type val = new Type();
            val.Key = key;
            val.Value = value;
            return Add(val);
        }

        public bool Add(TKey key)
        {
            Type val = new Type();
            val.Key = key;
            return Add(val);
        }

        public int Remove(TKey key)
        {
            return list.RemoveAll(p => p.Key.Equals(key));
        }

        public int Remove(TValue value)
        {
            return list.RemoveAll(p => p.Value.Equals(value));
        }

        public void Clear()
        {
            list.Clear();
        }

        public IEnumerator<Type> GetEnumerator()
        {
            foreach (Type item in list)
            {
                yield return item;
            }
        }
    }
}
