using UnityEngine;

public class ImplicitSurface : ImplicitSurfaceMeshCreaterBase
{
    public MeshFilter meshFilter;

    public MeshCollider meshCollider;

    protected Vector3[] _positionMap;

    protected float[] _powerMap;

    protected float[] _powerMapMask;

    protected int _countX;

    protected int _countY;

    protected int _countZ;

    private bool _bMapsDirty = true;

    public MeshFilter MeshFilter
    {
        get
        {
            if (meshFilter == null)
            {
                meshFilter = GetComponent<MeshFilter>();
            }
            return meshFilter;
        }
    }

    public override Mesh Mesh
    {
        get
        {
            return meshFilter.sharedMesh;
        }
        set
        {
            meshFilter.sharedMesh = value;
        }
    }

    protected void ResetMaps()
    {
        int maxGridCellCount = MetaballBuilder.MaxGridCellCount;
        float num;
        if (bAutoGridSize)
        {
            int num2 = (int)(maxGridCellCount * Mathf.Clamp01(autoGridQuarity));
            Vector3 size = fixedBounds.size;
            float x = size.x;
            Vector3 size2 = fixedBounds.size;
            float num3 = x * size2.y;
            Vector3 size3 = fixedBounds.size;
            num = Mathf.Pow(num3 * size3.z / num2, 0.333333343f);
        }
        else
        {
            num = gridSize;
        }
        Vector3 extents = fixedBounds.extents;
        int num4 = Mathf.CeilToInt(extents.x / num) + 1;
        Vector3 extents2 = fixedBounds.extents;
        int num5 = Mathf.CeilToInt(extents2.y / num) + 1;
        Vector3 extents3 = fixedBounds.extents;
        int num6 = Mathf.CeilToInt(extents3.z / num) + 1;
        _countX = num4 * 2;
        _countY = num5 * 2;
        _countZ = num6 * 2;
        Vector3 b = new Vector3(num4 * num, num5 * num, num6 * num);
        Vector3 a = fixedBounds.center - b;
        int countX = _countX;
        int num7 = _countX * _countY;
        int num8 = _countX * _countY * _countZ;
        _positionMap = new Vector3[num8];
        _powerMap = new float[num8];
        _powerMapMask = new float[num8];
        for (int i = 0; i < num8; i++)
        {
            _powerMap[i] = 0f;
        }
        for (int j = 0; j < _countZ; j++)
        {
            for (int k = 0; k < _countY; k++)
            {
                for (int l = 0; l < _countX; l++)
                {
                    int num9 = l + k * countX + j * num7;
                    _positionMap[num9] = a + new Vector3(num * l, num * k, num * j);
                    if (j == 0 || j == _countZ - 1 || k == 0 || k == _countY - 1 || l == 0 || l == _countX - 1)
                    {
                        _powerMapMask[num9] = 0f;
                    }
                    else
                    {
                        _powerMapMask[num9] = 1f;
                    }
                }
            }
        }
        InitializePowerMap();
        _bMapsDirty = false;
    }

    protected virtual void InitializePowerMap()
    {
        int num = _countX * _countY * _countZ;
        for (int i = 0; i < num; i++)
        {
            _powerMap[i] = 0f;
        }
    }

    public override void CreateMesh()
    {
        if (_bMapsDirty)
        {
            ResetMaps();
        }
        Vector3 uDir;
        Vector3 vDir;
        Vector3 offset;
        GetUVBaseVector(out uDir, out vDir, out offset);
        Mesh mesh = MetaballBuilder.Instance.CreateImplicitSurfaceMesh(_countX, _countY, _countZ, _positionMap, _powerMap, bReverse, powerThreshold, uDir, vDir, offset);
        mesh.RecalculateBounds();
        Mesh = mesh;
        if (meshCollider != null)
        {
            meshCollider.sharedMesh = mesh;
        }
    }

    private void OnDrawGizmos()
    {
        Gizmos.color = Color.white;
        Gizmos.DrawWireCube(fixedBounds.center + transform.position, fixedBounds.size);
    }
}
