using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

internal class InteractableAlphaChanger : MonoBehaviour
{
    [Header("Interactable参照用トグル")]
    [SerializeField]
    private Toggle flagToggle;

    [Header("カラー変更対象")]
    [SerializeField]
    private TextMeshProUGUI targetTextMesh;

    [SerializeField]
    [Range(0f, 1f)]
    [Header("無効時の色")]
    private float offAlpha = 0.3f;

    private void Awake()
    {
        if (flagToggle == null)
        {
            Destroy(this);
            return;
        }
        if (targetTextMesh == null)
        {
            targetTextMesh = GetComponent<TextMeshProUGUI>();
        }
        if (targetTextMesh == null)
        {
            Destroy(this);
        }
    }

    private void Start()
    {
        Color baseColor = targetTextMesh.color;
        BoolReactiveProperty isInteract = new BoolReactiveProperty(flagToggle.interactable);
        isInteract.Subscribe(delegate (bool isOn)
        {
            Color color = baseColor;
            if (!isOn)
            {
                color = targetTextMesh.color;
                color.a = offAlpha;
            }
            targetTextMesh.color = color;
        });
        this.OnEnableAsObservable().Subscribe(delegate
        {
            isInteract.Value = flagToggle.interactable;
        });
        (from _ in this.UpdateAsObservable()
         select flagToggle.interactable).DistinctUntilChanged().Subscribe(delegate (bool interactable)
     {
         isInteract.Value = interactable;
     });
    }
}
