using Sound;
using System.Collections;
using UnityEngine;

public abstract class LoadAudioBase : AssetLoader
{
    public float delayTime;
    public float fadeTime;
    public int settingNo = -1;
    protected bool isReleaseClip = true;

    public AudioClip clip
    {
        get;
        private set;
    }

    public AudioSource audioSource
    {
        get;
        private set;
    }

    public void Init(AudioSource audioSource)
    {
        this.audioSource = audioSource;
        transform.SetParent(audioSource.transform, false);
        Init();
    }

    public override IEnumerator _Init()
    {
        if (initialized)
        {
            yield break;
        }
        if (isAsync)
        {
            yield return StartCoroutine(base._Init());
        }
        else
        {
            StartCoroutine(base._Init());
        }
        clip = loadObject as AudioClip;
        if (clip == null)
        {
            if (audioSource != null)
            {
                Destroy(audioSource.gameObject);
            }
            else
            {
                Destroy(gameObject);
            }
        }
    }

    protected IEnumerator Play(GameObject fadeOut)
    {
        if (audioSource.playOnAwake)
        {
            yield break;
        }
        yield return new WaitUntil(() => audioSource.clip.loadState == AudioDataLoadState.Loaded);
        if (delayTime > 0f)
        {
            yield return new WaitForSecondsRealtime(delayTime);
        }
        while (!audioSource.isActiveAndEnabled)
        {
            yield return null;
        }
        if (fadeTime > 0f)
        {
            Manager.Sound.PlayFade(fadeOut, audioSource, fadeTime);
        }
        else
        {
            audioSource.Play();
        }
        if (!audioSource.loop)
        {
            float pitch = audioSource.pitch;
            if (pitch == 0f)
            {
                pitch = 1f;
            }
            float endTime = (audioSource.clip.length - audioSource.time) * (1f / pitch);
            if (endTime > 0f)
            {
                yield return new WaitForSecondsRealtime(endTime);
            }
            FadePlayer fadePlay = audioSource.GetComponent<FadePlayer>();
            if (fadePlay != null)
            {
                fadePlay.Stop(fadeTime);
            }
            else
            {
                Destroy(audioSource.gameObject);
            }
        }
    }

    protected override void OnDestroy()
    {
        base.OnDestroy();
        if (isReleaseClip && Singleton<Manager.Sound>.IsInstance())
        {
            Singleton<Manager.Sound>.Instance.Remove(clip);
        }
    }
}
