using Illusion.Game;
using Manager;
using SaveLoad;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

public class LoadScene : BaseLoader
{
    [SerializeField]
    private CanvasGroup _canvasGroup;

    [SerializeField]
    private Button _autoButton;

    [SerializeField]
    private Button _returnButton;

    [SerializeField]
    private string _checkMessage;

    [SerializeField]
    private GameFile gameFile;

    public Action onReturn
    {
        get;
        set;
    }

    public event Action<string> onEnter = delegate
    {
    };

    private IEnumerator NextScene(string fileName)
    {
        Utils.Sound.Play(SystemSE.sel);
        CheckScene.Parameter param = new CheckScene.Parameter();
        param.Yes = delegate
        {
            onEnter(fileName);
        };
        param.No = delegate
        {
            Singleton<Scene>.Instance.UnLoad();
        };
        param.Title = _checkMessage;
        yield return Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param, observer)).StartAsCoroutine();
    }

    private void Start()
    {
        this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
        {
            _canvasGroup.interactable = isOn;
        });
        gameFile.Initialize(true);
        gameFile.fileButtons.ToList().ForEach(delegate (GameFile.FileButton fb)
        {
            LoadScene loadScene = this;
            fb.button.OnClickAsObservable().Subscribe(delegate
            {
                Observable.FromCoroutine(__ => loadScene.NextScene(fb.file)).StartAsCoroutine();
            });
        });
        if (onReturn.IsNullOrEmpty())
        {
            onReturn = delegate
            {
                Singleton<Scene>.Instance.UnLoad();
            };
        }
        _returnButton.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.sel);
            onReturn();
        });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where _returnButton.interactable
         where Singleton<Scene>.Instance.NowSceneNames[0] == "Load"
         select _).Take(1).Subscribe(delegate
     {
         onReturn();
     });
    }

    private void OnDestroy()
    {
        gameFile.Dispose();
    }

    [ContextMenu("Setup")]
    private void Setup()
    {
        Transform transform = base.transform.GetComponentsInChildren<Transform>(true).FirstOrDefault(t => t.name == "Files");
        if (transform != null)
        {
            List<GameFile.FileButton> list = new List<GameFile.FileButton>();
            IEnumerator enumerator = transform.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    Transform transform2 = (Transform)enumerator.Current;
                    list.Add(new GameFile.FileButton(transform2.GetComponent<Button>(), transform2.GetChild(0).GetComponent<Text>(), transform2.GetChild(1).GetComponent<Text>()));
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            gameFile.SetupFiles(list.ToArray());
        }
    }
}
