using Illusion.CustomAttributes;
using Illusion.Extensions;
using Illusion.Game;
using Illusion.Game.Extensions;
using Manager;
using System;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

public class NickNameSettingScene : BaseLoader
{
    private bool _isSpecialVisible = true;

    private SaveData.CharaData _charaData;

    [Label("キャンバス")]
    [SerializeField]
    private Canvas Canvas;

    [Label("個別用女情報")]
    [SerializeField]
    private GameObject ObjFemaleInfo;

    [Label("学生証コンポーネント")]
    [SerializeField]
    private StudentCardControlComponent cmpStudentCard;

    [SerializeField]
    [Label("姓表示")]
    private Text LastNameText;

    [SerializeField]
    [Label("名表示")]
    private Text FirstNameText;

    [Label("あだ名表示")]
    [SerializeField]
    private TextMeshProUGUI NickNameText;

    [SerializeField]
    [Label("コピー元ボタン")]
    private Toggle NickNameButton;

    [Label("あだ名(50音頭文字)")]
    [SerializeField]
    private RectTransform NickNameInitialContent;

    [SerializeField]
    [Label("あだ名ウィンドウ")]
    private RectTransform NickNameWindow;

    [SerializeField]
    [Label("あだ名用レイアウトコピー元")]
    private RectTransform NickNameBaseNode;

    [SerializeField]
    [Label("クリアボタン")]
    private Button Clear;

    [Label("決定ボタン")]
    [SerializeField]
    private Button Enter;

    [SerializeField]
    [Label("戻るボタン")]
    private Button Return;

    private HashSet<string> loadBundles = new HashSet<string>();

    public bool isSpecialVisible
    {
        get
        {
            return _isSpecialVisible;
        }
        set
        {
            _isSpecialVisible = value;
        }
    }

    public SaveData.CharaData charaData
    {
        get
        {
            return _charaData;
        }
        set
        {
            _charaData = value;
        }
    }

    public event Action onEnter = delegate
    {
    };

    public event Action onCancel = delegate
    {
    };

    public event Action onClear = delegate
    {
    };

    private void Start()
    {
        Singleton<Scene>.Instance.sceneFade.SortingOrder();
        CanvasGroup canvasGroup = Canvas.GetComponent<CanvasGroup>();
        this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
        {
            canvasGroup.interactable = isOn;
        });
        SaveData.Heroine heroine = _charaData as SaveData.Heroine;
        if (heroine == null)
        {
            if ((bool)ObjFemaleInfo)
            {
                ObjFemaleInfo.SetActiveIfDifferent(false);
            }
            if ((bool)Clear)
            {
                Clear.gameObject.SetActiveIfDifferent(false);
            }
        }
        else if ((bool)cmpStudentCard)
        {
            cmpStudentCard.SetCharaInfo(heroine.charFile, 0, Singleton<Game>.Instance.saveData.accademyName, string.Empty);
        }
        SaveData.Player player = Singleton<Game>.Instance.Player;
        if ((bool)LastNameText)
        {
            LastNameText.text = player.lastname;
        }
        if ((bool)FirstNameText)
        {
            FirstNameText.text = player.firstname;
        }
        this.SelectSEAdd(Enter, Return);
        List<NickName.Param> callNameList = SaveData.GetCallNameList(_charaData);
        Dictionary<int, List<NickName.Param>> dic = callNameList.ToLookup(v => v.Category, v => v).ToDictionary(v => v.Key, Enumerable.ToList);
        List<NickName.Param> value;
        Func<int, List<NickName.Param>> getList = category => !dic.TryGetValue(category, out value) ? new List<NickName.Param>() : value;
        int callID = _charaData.callMyID;
        StringReactiveProperty callName = new StringReactiveProperty(callNameList.Find(p => p.ID == callID).Name);
        callName.SubscribeWithState(NickNameText, delegate (string text, TextMeshProUGUI t)
        {
            t.text = text;
        });
        List<GameObject> destroyNickNameList = new List<GameObject>();
        Transform nickNameBaseNodeParent = NickNameBaseNode.transform.parent;
        Action<Tuple<string, int>[]> createNickNameButton = delegate (Tuple<string, int>[] categoryData)
        {
            destroyNickNameList.ForEach(delegate (GameObject o)
            {
                Destroy(o);
            });
            destroyNickNameList.Clear();
            foreach (var item in categoryData.Select(p => new
            {
                text = p.Item1,
                index = p.Item2
            }))
            {
                List<NickName.Param> list = getList(item.index);
                if (list.Any())
                {
                    RectTransform rectTransform = Instantiate(NickNameBaseNode, nickNameBaseNodeParent, false);
                    GameObject gameObject = rectTransform.gameObject;
                    gameObject.SetActive(true);
                    destroyNickNameList.Add(gameObject);
                    Transform child = rectTransform.GetChild(0);
                    child.GetComponentInChildren<TextMeshProUGUI>().text = item.text;
                    Button nickBaseNameButton = rectTransform.GetComponentInChildren<Button>(true);
                    Transform nickBaseNameButtonParent = nickBaseNameButton.transform.parent;
                    list.ForEach(delegate (NickName.Param nick)
                    {
                        Button button = Instantiate(nickBaseNameButton, nickBaseNameButtonParent, false);
                        button.gameObject.SetActive(true);
                        button.GetComponentInChildren<TextMeshProUGUI>().text = nick.Name;
                        button.OnClickAsObservable().Subscribe(delegate
                        {
                            callID = nick.ID;
                            callName.Value = nick.Name;
                            if (_charaData is SaveData.Heroine)
                            {
                                SaveData.CallFileData callFileData = SaveData.FindCallFileData(_charaData.personality, nick.ID);
                                loadBundles.Add(callFileData.bundle);
                                Singleton<Voice>.Instance.OnecePlayChara(_charaData.personality, callFileData.bundle, callFileData.GetFileName(0), _charaData.voicePitch, 0f, 0f, false, null, Voice.Type.PCM, -1, true, false);
                            }
                        });
                    });
                }
            }
        };
        string[][] source = new string[11][]
        {
            new string[1]
            {
                "特殊"
            },
            new string[5]
            {
                "あ",
                "い",
                "う",
                "え",
                "お"
            },
            new string[5]
            {
                "か",
                "き",
                "く",
                "け",
                "こ"
            },
            new string[5]
            {
                "さ",
                "し",
                "す",
                "せ",
                "そ"
            },
            new string[5]
            {
                "た",
                "ち",
                "つ",
                "て",
                "と"
            },
            new string[5]
            {
                "な",
                "に",
                "ぬ",
                "ね",
                "の"
            },
            new string[5]
            {
                "は",
                "ひ",
                "ふ",
                "へ",
                "ほ"
            },
            new string[5]
            {
                "ま",
                "み",
                "む",
                "め",
                "も"
            },
            new string[3]
            {
                "や",
                "ゆ",
                "よ"
            },
            new string[5]
            {
                "ら",
                "り",
                "る",
                "れ",
                "ろ"
            },
            new string[3]
            {
                "わ",
                "を",
                "ん"
            }
        };
        bool flag = false;
        int cnt = 0;
        Vector2 nickNameWindowResetPos = NickNameWindow.anchoredPosition;
        foreach (var item2 in source.Select(p => p.Select((text, index) => new
        {
            text,
            index = cnt++
        }).ToArray()).Skip(!_isSpecialVisible ? 1 : 0))
        {
            Toggle toggle = Instantiate(NickNameButton, NickNameInitialContent, false);
            toggle.gameObject.SetActive(true);
            toggle.GetComponentInChildren<TextMeshProUGUI>().text = item2[0].text;
            Tuple<string, int>[] data = item2.Select(p => Tuple.Create(p.text, p.index)).ToArray();
            (from isOn in toggle.OnValueChangedAsObservable()
             where isOn
             select isOn).Subscribe(delegate
         {
             NickNameWindow.anchoredPosition = nickNameWindowResetPos;
             createNickNameButton(data);
         });
            if (!flag)
            {
                toggle.isOn = true;
                createNickNameButton(data);
                flag = true;
            }
        }
        Enter.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.ok_s);
            _charaData.callID = callID;
            _charaData.callName = callName.Value;
            SaveData.CallNormalize(_charaData);
            foreach (string loadBundle in loadBundles)
            {
                AssetBundleManager.UnloadAssetBundle(loadBundle, true);
            }
            onEnter();
        });
        Clear.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.ok_s);
            NickName.Param callName2 = SaveData.GetCallName(Singleton<Game>.Instance.Player);
            callID = callName2.ID;
            callName.Value = callName2.Name;
            onClear();
        });
        Return.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.cancel);
            foreach (string loadBundle2 in loadBundles)
            {
                AssetBundleManager.UnloadAssetBundle(loadBundle2, true);
            }
            onCancel();
        });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1) && Return.interactable
         where Singleton<Scene>.Instance.AddSceneName == "NickNameSetting"
         select _).Take(1).Subscribe(delegate
     {
         foreach (string loadBundle3 in loadBundles)
         {
             AssetBundleManager.UnloadAssetBundle(loadBundle3, true);
         }
         onCancel();
     });
    }
}
