using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections.Generic;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [Icon("Shield", false)]
    [Category("Decorators")]
    [Name("Guard")]
    [Description("Protect the decorated child from running if another Guard with the same token is already guarding (Running) that token.\nGuarding is global for all of the agent's Behaviour Trees.")]
    public class Guard : BTDecorator
    {
        public enum GuardMode
        {
            ReturnFailure,
            WaitUntilReleased
        }

        public BBParameter<string> token;

        public GuardMode ifGuarded;

        private bool isGuarding;

        private static readonly Dictionary<GameObject, List<Guard>> guards = new Dictionary<GameObject, List<Guard>>();

        private static List<Guard> AgentGuards(Component agent)
        {
            return guards[agent.gameObject];
        }

        public override void OnGraphStarted()
        {
            SetGuards(graphAgent);
        }

        protected override Status OnExecute(Component agent, IBlackboard blackboard)
        {
            if (decoratedConnection == null)
            {
                return Status.Failure;
            }
            if (agent != graphAgent)
            {
                SetGuards(agent);
            }
            for (int i = 0; i < AgentGuards(agent).Count; i++)
            {
                Guard guard = AgentGuards(agent)[i];
                if (guard != this && guard.isGuarding && guard.token.value == token.value)
                {
                    return ifGuarded != 0 ? Status.Running : Status.Failure;
                }
            }
            status = decoratedConnection.Execute(agent, blackboard);
            if (status == Status.Running)
            {
                isGuarding = true;
                return Status.Running;
            }
            isGuarding = false;
            return status;
        }

        protected override void OnReset()
        {
            isGuarding = false;
        }

        private void SetGuards(Component guardAgent)
        {
            if (!guards.ContainsKey(guardAgent.gameObject))
            {
                guards[guardAgent.gameObject] = new List<Guard>();
            }
            if (!AgentGuards(guardAgent).Contains(this) && !string.IsNullOrEmpty(token.value))
            {
                AgentGuards(guardAgent).Add(this);
            }
        }
    }
}
