using NodeCanvas.Framework;
using ParadoxNotion.Design;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [Description("Repeat the child either x times or until it returns the specified status, or forever")]
    [Icon("Repeat", false)]
    [Name("Repeat")]
    [Category("Decorators")]
    public class Repeater : BTDecorator
    {
        public enum RepeaterMode
        {
            RepeatTimes,
            RepeatUntil,
            RepeatForever
        }

        public enum RepeatUntilStatus
        {
            Failure,
            Success
        }

        public RepeaterMode repeaterMode;

        public RepeatUntilStatus repeatUntilStatus = RepeatUntilStatus.Success;

        public BBParameter<int> repeatTimes = 1;

        private int currentIteration = 1;

        protected override Status OnExecute(Component agent, IBlackboard blackboard)
        {
            if (decoratedConnection == null)
            {
                return Status.Resting;
            }
            if (decoratedConnection.status == Status.Success || decoratedConnection.status == Status.Failure)
            {
                decoratedConnection.Reset();
            }
            status = decoratedConnection.Execute(agent, blackboard);
            switch (status)
            {
                case Status.Resting:
                    return Status.Running;
                case Status.Running:
                    return Status.Running;
                default:
                    switch (repeaterMode)
                    {
                        case RepeaterMode.RepeatTimes:
                            if (currentIteration >= repeatTimes.value)
                            {
                                return status;
                            }
                            currentIteration++;
                            break;
                        case RepeaterMode.RepeatUntil:
                            if (status == (Status)repeatUntilStatus)
                            {
                                return status;
                            }
                            break;
                    }
                    return Status.Running;
            }
        }

        protected override void OnReset()
        {
            currentIteration = 1;
        }
    }
}
