using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections.Generic;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [Description("Execute the child nodes in order or randonly and return Success if all children return Success, else return Failure\nIf is Dynamic, higher priority child status is revaluated. If a child returns Failure the Sequencer will bail out immediately in Failure too.")]
    [Category("Composites")]
    [Color("bf7fff")]
    [Icon("Sequencer", false)]
    public class Sequencer : BTComposite
    {
        public bool dynamic;

        public bool random;

        private int lastRunningNodeIndex;

        public override string name
        {
            get
            {
                return base.name.ToUpper();
            }
        }

        protected override Status OnExecute(Component agent, IBlackboard blackboard)
        {
            for (int i = !dynamic ? lastRunningNodeIndex : 0; i < outConnections.Count; i++)
            {
                status = outConnections[i].Execute(agent, blackboard);
                switch (status)
                {
                    case Status.Running:
                        if (dynamic && i < lastRunningNodeIndex)
                        {
                            outConnections[lastRunningNodeIndex].Reset();
                        }
                        lastRunningNodeIndex = i;
                        return Status.Running;
                    case Status.Failure:
                        if (dynamic && i < lastRunningNodeIndex)
                        {
                            outConnections[lastRunningNodeIndex].Reset();
                        }
                        return Status.Failure;
                }
            }
            return Status.Success;
        }

        protected override void OnReset()
        {
            lastRunningNodeIndex = 0;
            if (random)
            {
                outConnections = Shuffle(outConnections);
            }
        }

        public override void OnChildDisconnected(int index)
        {
            if (index != 0 && index == lastRunningNodeIndex)
            {
                lastRunningNodeIndex--;
            }
        }

        public override void OnGraphStarted()
        {
            OnReset();
        }

        private List<Connection> Shuffle(List<Connection> list)
        {
            for (int num = list.Count - 1; num > 0; num--)
            {
                int index = (int)Mathf.Floor(Random.value * (num + 1));
                Connection value = list[num];
                list[num] = list[index];
                list[index] = value;
            }
            return list;
        }
    }
}
