using NodeCanvas.Framework;
using ParadoxNotion.Design;
using UnityEngine;

namespace NodeCanvas.BehaviourTrees
{
    [Icon("Timeout", false)]
    [Category("Decorators")]
    [Description("Interupts decorated child node and returns Failure if the child node is still Running after the timeout period")]
    public class Timeout : BTDecorator
    {
        public BBParameter<float> timeout = 1f;

        private float timer;

        protected override Status OnExecute(Component agent, IBlackboard blackboard)
        {
            if (decoratedConnection == null)
            {
                return Status.Resting;
            }
            status = decoratedConnection.Execute(agent, blackboard);
            if (status == Status.Running)
            {
                timer += Time.deltaTime;
            }
            if (timer < timeout.value)
            {
                return status;
            }
            timer = 0f;
            decoratedConnection.Reset();
            return Status.Failure;
        }

        protected override void OnReset()
        {
            timer = 0f;
        }
    }
}
