using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections;
using UnityEngine;

namespace NodeCanvas.DialogueTrees
{
    [Description("Execute an Action Task for the Dialogue Actor selected.")]
    [Name("Task Action")]
    public class ActionNode : DTNode, ITaskAssignable<ActionTask>, ITaskAssignable
    {
        [SerializeField]
        private ActionTask _action;

        public ActionTask action
        {
            get
            {
                return _action;
            }
            set
            {
                _action = value;
            }
        }

        public Task task
        {
            get
            {
                return action;
            }
            set
            {
                action = (ActionTask)value;
            }
        }

        protected override Status OnExecute(Component agent, IBlackboard bb)
        {
            if (action == null)
            {
                return Error("Action is null on Dialogue Action Node");
            }
            status = Status.Running;
            StartCoroutine(UpdateAction(finalActor.transform));
            return status;
        }

        private IEnumerator UpdateAction(Component actionAgent)
        {
            Status actionStatus;
            while (true)
            {
                if (status == Status.Running)
                {
                    actionStatus = action.ExecuteAction(actionAgent, graphBlackboard);
                    if (actionStatus != Status.Running)
                    {
                        break;
                    }
                    yield return null;
                    continue;
                }
                yield break;
            }
            OnActionEnd(actionStatus == Status.Success);
        }

        private void OnActionEnd(bool success)
        {
            if (success)
            {
                status = Status.Success;
                DLGTree.Continue();
            }
            else
            {
                status = Status.Failure;
                DLGTree.Stop(false);
            }
        }

        protected override void OnReset()
        {
            if (action != null)
            {
                action.EndAction(null);
            }
        }

        public override void OnGraphPaused()
        {
            if (action != null)
            {
                action.PauseAction();
            }
        }
    }
}
