using NodeCanvas.Framework;
using ParadoxNotion.Design;
using UnityEngine;

namespace NodeCanvas.DialogueTrees
{
    [Name("Task Condition")]
    [Category("Flow Control")]
    [Description("Execute the first child node if a Condition is true, or the second one if that Condition is false. The Actor selected is used for the Condition check")]
    public class ConditionNode : DTNode, ITaskAssignable<ConditionTask>, ITaskAssignable
    {
        [SerializeField]
        private ConditionTask _condition;

        public ConditionTask condition
        {
            get
            {
                return _condition;
            }
            set
            {
                _condition = value;
            }
        }

        public Task task
        {
            get
            {
                return condition;
            }
            set
            {
                condition = (ConditionTask)value;
            }
        }

        public override int maxOutConnections
        {
            get
            {
                return 2;
            }
        }

        protected override Status OnExecute(Component agent, IBlackboard bb)
        {
            if (outConnections.Count == 0)
            {
                return Error("There are no connections on the Dialogue Condition Node");
            }
            if (condition == null)
            {
                return Error("There is no Conidition on the Dialoge Condition Node");
            }
            bool flag = condition.CheckCondition(finalActor.transform, graphBlackboard);
            status = flag ? Status.Success : Status.Failure;
            DLGTree.Continue(!flag ? 1 : 0);
            return status;
        }
    }
}
