using NodeCanvas.Framework;
using ParadoxNotion.Design;

namespace NodeCanvas.StateMachines
{
    [Name("Any State")]
    [Description("The Transitions of this node will constantly be checked. If any becomes true, the target connected State will Enter regardless of the current State. This node can have no incomming transitions.")]
    [Color("b3ff7f")]
    public class AnyState : FSMState, IUpdatable
    {
        public bool dontRetriggerStates;

        public override string name
        {
            get
            {
                return base.name.ToUpper();
            }
        }

        public override int maxInConnections
        {
            get
            {
                return 0;
            }
        }

        public override int maxOutConnections
        {
            get
            {
                return -1;
            }
        }

        public override bool allowAsPrime
        {
            get
            {
                return false;
            }
        }

        public new void Update()
        {
            if (outConnections.Count == 0)
            {
                return;
            }
            status = Status.Running;
            int num = 0;
            FSMConnection fSMConnection;
            while (true)
            {
                if (num >= outConnections.Count)
                {
                    return;
                }
                fSMConnection = (FSMConnection)outConnections[num];
                ConditionTask condition = fSMConnection.condition;
                if (fSMConnection.isActive && fSMConnection.condition != null && (!dontRetriggerStates || FSM.currentState != (FSMState)fSMConnection.targetNode))
                {
                    if (condition.CheckCondition(graphAgent, graphBlackboard))
                    {
                        break;
                    }
                    fSMConnection.status = Status.Failure;
                }
                num++;
            }
            FSM.EnterState((FSMState)fSMConnection.targetNode);
            fSMConnection.status = Status.Success;
        }
    }
}
