using System.Collections;
using UnityEngine;

namespace NodeCanvas.Tasks.Actions
{
    public class CameraFader : MonoBehaviour
    {
        private static CameraFader _current;

        private float alpha;

        private Texture2D _blackTexture;

        private Texture2D blackTexture
        {
            get
            {
                if (_blackTexture == null)
                {
                    _blackTexture = new Texture2D(1, 1);
                    _blackTexture.SetPixel(1, 1, Color.black);
                    _blackTexture.Apply();
                }
                return _blackTexture;
            }
        }

        public static CameraFader current
        {
            get
            {
                if (_current == null)
                {
                    _current = FindObjectOfType<CameraFader>();
                }
                if (_current == null)
                {
                    _current = new GameObject("_CameraFader").AddComponent<CameraFader>();
                }
                return _current;
            }
        }

        public void FadeIn(float time)
        {
            StartCoroutine(CoroutineFadeIn(time));
        }

        public void FadeOut(float time)
        {
            StartCoroutine(CoroutineFadeOut(time));
        }

        private IEnumerator CoroutineFadeIn(float time)
        {
            for (alpha = 1f; alpha > 0f; alpha -= 1f / time * Time.deltaTime)
            {
                yield return null;
            }
        }

        private IEnumerator CoroutineFadeOut(float time)
        {
            for (alpha = 0f; alpha < 1f; alpha += 1f / time * Time.deltaTime)
            {
                yield return null;
            }
        }

        private void OnGUI()
        {
            if (!(alpha <= 0f))
            {
                GUI.color = new Color(1f, 1f, 1f, alpha);
                GUI.DrawTexture(new Rect(0f, 0f, Screen.width, Screen.height), blackTexture);
                GUI.color = Color.white;
            }
        }
    }
}
