using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace NodeCanvas.Tasks.Actions
{
    [Category("GameObject")]
    public class GetAllChildGameObjects : ActionTask<Transform>
    {
        [BlackboardOnly]
        public BBParameter<List<GameObject>> saveAs;

        public bool recursive;

        protected override string info
        {
            get
            {
                return string.Format("{0} = {1} Children Of {2}", saveAs, !recursive ? "First" : "All", agentInfo);
            }
        }

        protected override void OnExecute()
        {
            List<Transform> list = new List<Transform>();
            IEnumerator enumerator = agent.transform.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    Transform transform = (Transform)enumerator.Current;
                    list.Add(transform);
                    if (recursive)
                    {
                        list.AddRange(Get(transform));
                    }
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            saveAs.value = list.Select(t => t.gameObject).ToList();
            EndAction();
        }

        private List<Transform> Get(Transform parent)
        {
            List<Transform> list = new List<Transform>();
            IEnumerator enumerator = parent.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    Transform transform = (Transform)enumerator.Current;
                    list.Add(transform);
                    list.AddRange(Get(transform));
                }
                return list;
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
        }
    }
}
