using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace NodeCanvas.Tasks.Actions
{
    [Description("Gets a lists of game objects that are in the physics overlap sphere at the position of the agent, excluding the agent")]
    [Category("Physics")]
    public class GetOverlapSphereObjects : ActionTask<Transform>
    {
        public LayerMask layerMask = -1;

        public BBParameter<float> radius = 2f;

        [BlackboardOnly]
        public BBParameter<List<GameObject>> saveObjectsAs;

        protected override void OnExecute()
        {
            Collider[] source = Physics.OverlapSphere(agent.position, radius.value, layerMask);
            saveObjectsAs.value = source.Select(c => c.gameObject).ToList();
            saveObjectsAs.value.Remove(agent.gameObject);
            if (saveObjectsAs.value.Count == 0)
            {
                EndAction(false);
            }
            else
            {
                EndAction(true);
            }
        }

        public override void OnDrawGizmosSelected()
        {
            if (agent != null)
            {
                Gizmos.color = new Color(1f, 1f, 1f, 0.2f);
                Gizmos.DrawSphere(agent.position, radius.value);
            }
        }
    }
}
