using NodeCanvas.Framework;
using ParadoxNotion.Design;
using UnityEngine;
using UnityEngine.AI;

namespace NodeCanvas.Tasks.Actions
{
    [Category("Movement")]
    [Name("Move To Target")]
    public class MoveToGameObject : ActionTask<NavMeshAgent>
    {
        [RequiredField]
        public BBParameter<GameObject> target;

        public BBParameter<float> speed = 3f;

        public float keepDistance = 0.1f;

        private Vector3? lastRequest;

        protected override string info
        {
            get
            {
                return "GoTo " + target.ToString();
            }
        }

        protected override void OnExecute()
        {
            agent.speed = speed.value;
            if ((agent.transform.position - target.value.transform.position).magnitude < agent.stoppingDistance + keepDistance)
            {
                EndAction(true);
            }
            else
            {
                Go();
            }
        }

        protected override void OnUpdate()
        {
            Go();
        }

        private void Go()
        {
            Vector3 position = target.value.transform.position;
            if (lastRequest != position && !agent.SetDestination(position))
            {
                EndAction(false);
                return;
            }
            lastRequest = position;
            if (!agent.pathPending && agent.remainingDistance <= agent.stoppingDistance + keepDistance)
            {
                EndAction(true);
            }
        }

        protected override void OnStop()
        {
            Vector3? vector = lastRequest;
            if (vector.HasValue && agent.gameObject.activeSelf)
            {
                agent.ResetPath();
            }
            lastRequest = null;
        }

        protected override void OnPause()
        {
            OnStop();
        }

        public override void OnDrawGizmos()
        {
            if (target.value != null)
            {
                Gizmos.DrawWireSphere(target.value.transform.position, keepDistance);
            }
        }
    }
}
