using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

namespace NodeCanvas.Tasks.Actions
{
    [Category("Movement")]
    [Description("Move Randomly or Progressively between various game object positions taken from the list provided")]
    public class Patrol : ActionTask<NavMeshAgent>
    {
        public enum PatrolMode
        {
            Progressive,
            Random
        }

        [RequiredField]
        public BBParameter<List<GameObject>> targetList;

        public BBParameter<PatrolMode> patrolMode = PatrolMode.Random;

        public BBParameter<float> speed = 3f;

        public float keepDistance = 0.1f;

        private int index = -1;

        private Vector3? lastRequest;

        protected override string info
        {
            get
            {
                return string.Format("{0} Patrol {1}", patrolMode, targetList);
            }
        }

        protected override void OnExecute()
        {
            if (targetList.value.Count == 0)
            {
                EndAction(false);
                return;
            }
            if (targetList.value.Count == 1)
            {
                index = 0;
            }
            else if (patrolMode.value == PatrolMode.Random)
            {
                int num;
                for (num = Random.Range(0, targetList.value.Count); num == index; num = Random.Range(0, targetList.value.Count))
                {
                }
                index = num;
            }
            else if (patrolMode.value == PatrolMode.Progressive)
            {
                index = (int)Mathf.Repeat(index + 1, targetList.value.Count);
            }
            GameObject gameObject = targetList.value[index];
            if (gameObject == null)
            {
                EndAction(false);
                return;
            }
            Vector3 position = gameObject.transform.position;
            agent.speed = speed.value;
            if ((agent.transform.position - position).magnitude < agent.stoppingDistance + keepDistance)
            {
                EndAction(true);
            }
            else
            {
                Go();
            }
        }

        protected override void OnUpdate()
        {
            Go();
        }

        private void Go()
        {
            Vector3 position = targetList.value[index].transform.position;
            if (lastRequest != position && !agent.SetDestination(position))
            {
                EndAction(false);
                return;
            }
            lastRequest = position;
            if (!agent.pathPending && agent.remainingDistance <= agent.stoppingDistance + keepDistance)
            {
                EndAction(true);
            }
        }

        protected override void OnStop()
        {
            lastRequest = null;
            if (agent.gameObject.activeSelf)
            {
                agent.ResetPath();
            }
        }

        protected override void OnPause()
        {
            OnStop();
        }

        public override void OnDrawGizmosSelected()
        {
            if (agent && targetList.value != null)
            {
                foreach (GameObject item in targetList.value)
                {
                    if ((bool)item)
                    {
                        Gizmos.DrawSphere(item.transform.position, 0.1f);
                    }
                }
            }
        }
    }
}
