using NodeCanvas.Framework;
using NodeCanvas.Framework.Internal;
using ParadoxNotion;
using ParadoxNotion.Design;
using System;
using System.Reflection;
using UnityEngine;

namespace NodeCanvas.Tasks.Actions
{
    [Category("✫ Script Control/Common")]
    [Description("Set a variable on a script")]
    public class SetField : ActionTask
    {
        [SerializeField]
        protected BBObjectParameter setValue;

        [SerializeField]
        protected Type targetType;

        [SerializeField]
        protected string fieldName;

        private FieldInfo field;

        public override Type agentType
        {
            get
            {
                return targetType ?? typeof(Transform);
            }
        }

        protected override string info
        {
            get
            {
                if (string.IsNullOrEmpty(fieldName))
                {
                    return "No Field Selected";
                }
                return string.Format("{0}.{1} = {2}", agentInfo, fieldName, setValue);
            }
        }

        protected override string OnInit()
        {
            field = agentType.RTGetField(fieldName);
            if (field == null)
            {
                return "Missing Field: " + fieldName;
            }
            return null;
        }

        protected override void OnExecute()
        {
            field.SetValue(agent, setValue.value);
            EndAction();
        }
    }
}
