using DG.Tweening;
using NodeCanvas.Framework;
using ParadoxNotion.Design;
using System;
using UnityEngine;

namespace NodeCanvas.Tasks.Actions
{
    [Icon("DOTTween", true)]
    [Category("Tween")]
    public class TweenPosition : ActionTask<Transform>
    {
        public BBParameter<Vector3> vector;

        public BBParameter<float> delay = 0f;

        public BBParameter<float> duration = 0.5f;

        public Ease easeType = Ease.Linear;

        public bool relative;

        public bool useSnapping;

        public bool waitActionFinish = true;

        private string id;

        protected override void OnExecute()
        {
            if (!relative && (agent.position - vector.value).sqrMagnitude <= 0.01f)
            {
                EndAction();
                return;
            }
            Tweener t = agent.DOMove(vector.value, duration.value, useSnapping);
            t.SetDelay(delay.value);
            t.SetEase(easeType);
            id = Guid.NewGuid().ToString();
            t.SetId(id);
            if (relative)
            {
                t.SetRelative();
            }
            if (!waitActionFinish)
            {
                EndAction();
            }
        }

        protected override void OnUpdate()
        {
            if (elapsedTime >= duration.value + delay.value)
            {
                EndAction();
            }
        }

        protected override void OnStop()
        {
            if (waitActionFinish)
            {
                DOTween.Kill(id);
            }
        }
    }
}
