using NodeCanvas.Framework;
using ParadoxNotion;
using ParadoxNotion.Design;
using System;
using System.Reflection;
using UnityEngine;

namespace NodeCanvas.Tasks.Conditions
{
    [Category("✫ Script Control/Common")]
    [Description("Check a field on a script and return if it's equal or not to a value")]
    public class CheckField : ConditionTask
    {
        [SerializeField]
        protected BBParameter checkValue;

        [SerializeField]
        protected Type targetType;

        [SerializeField]
        protected string fieldName;

        [SerializeField]
        protected CompareMethod comparison;

        private FieldInfo field;

        public override Type agentType
        {
            get
            {
                return targetType ?? typeof(Transform);
            }
        }

        protected override string info
        {
            get
            {
                if (string.IsNullOrEmpty(fieldName))
                {
                    return "No Field Selected";
                }
                return string.Format("{0}.{1}{2}", agentInfo, fieldName, checkValue.varType != typeof(bool) ? OperationTools.GetCompareString(comparison) + checkValue.ToString() : string.Empty);
            }
        }

        protected override string OnInit()
        {
            field = agentType.RTGetField(fieldName);
            if (field == null)
            {
                return "Missing Field Info";
            }
            return null;
        }

        protected override bool OnCheck()
        {
            if (checkValue.varType == typeof(float))
            {
                return OperationTools.Compare((float)field.GetValue(agent), (float)checkValue.value, comparison, 0.05f);
            }
            if (checkValue.varType == typeof(int))
            {
                return OperationTools.Compare((int)field.GetValue(agent), (int)checkValue.value, comparison);
            }
            return Equals(field.GetValue(agent), checkValue.value);
        }
    }
}
