using System;
using System.Collections;

namespace ParadoxNotion.Serialization.FullSerializer.Internal
{
    public class fsReflectedConverter : fsConverter
    {
        public override bool CanProcess(Type type)
        {
            if (type.Resolve().IsArray || typeof(ICollection).IsAssignableFrom(type))
            {
                return false;
            }
            return true;
        }

        public override fsResult TrySerialize(object instance, out fsData serialized, Type storageType)
        {
            serialized = fsData.CreateDictionary();
            fsResult success = fsResult.Success;
            fsMetaType fsMetaType = fsMetaType.Get(Serializer.Config, instance.GetType());
            fsMetaType.EmitAotData();
            object obj = null;
            if (!fsGlobalConfig.SerializeDefaultValues && !(instance is UnityEngine.Object))
            {
                obj = fsMetaType.CreateInstance();
            }
            for (int i = 0; i < fsMetaType.Properties.Length; i++)
            {
                fsMetaProperty fsMetaProperty = fsMetaType.Properties[i];
                if (fsMetaProperty.CanRead && (fsGlobalConfig.SerializeDefaultValues || obj == null || !Equals(fsMetaProperty.Read(instance), fsMetaProperty.Read(obj))))
                {
                    fsData data;
                    fsResult result = Serializer.TrySerialize(fsMetaProperty.StorageType, fsMetaProperty.OverrideConverterType, fsMetaProperty.Read(instance), out data);
                    success.AddMessages(result);
                    if (!result.Failed)
                    {
                        serialized.AsDictionary[fsMetaProperty.JsonName] = data;
                    }
                }
            }
            return success;
        }

        public override fsResult TryDeserialize(fsData data, ref object instance, Type storageType)
        {
            fsResult success = fsResult.Success;
            if ((success += CheckType(data, fsDataType.Object)).Failed)
            {
                return success;
            }
            if (data.AsDictionary.Count == 0)
            {
                return fsResult.Success;
            }
            fsMetaType fsMetaType = fsMetaType.Get(Serializer.Config, storageType);
            fsMetaType.EmitAotData();
            for (int i = 0; i < fsMetaType.Properties.Length; i++)
            {
                fsMetaProperty fsMetaProperty = fsMetaType.Properties[i];
                fsData value;
                if (fsMetaProperty.CanWrite && data.AsDictionary.TryGetValue(fsMetaProperty.JsonName, out value))
                {
                    object result = null;
                    fsResult result2 = Serializer.TryDeserialize(value, fsMetaProperty.StorageType, fsMetaProperty.OverrideConverterType, ref result);
                    success.AddMessages(result2);
                    if (!result2.Failed)
                    {
                        fsMetaProperty.Write(instance, result);
                    }
                }
            }
            return success;
        }

        public override object CreateInstance(fsData data, Type storageType)
        {
            fsMetaType fsMetaType = fsMetaType.Get(Serializer.Config, storageType);
            return fsMetaType.CreateInstance();
        }
    }
}
