using Illusion.Game;
using Manager;
using SaveLoad;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;

public class SaveScene : BaseLoader
{
    [SerializeField]
    private float resultTime = 3f;

    [SerializeField]
    private CanvasGroup _canvasGroup;

    [SerializeField]
    private Button _returnButton;

    [SerializeField]
    private string _saveMessage;

    [SerializeField]
    private string _saveMessageOverride;

    [SerializeField]
    private string _checkMessageSaved;

    [SerializeField]
    private GameFile gameFile;

    private IDisposable disposable;

    private IEnumerator Save(UnityAction yesAction, bool isOverride)
    {
        Utils.Sound.Play(SystemSE.sel);
        CheckScene.Parameter param = new CheckScene.Parameter();
        yesAction = (UnityAction)Delegate.Combine(yesAction, (UnityAction)delegate
        {
            UnLoad();
        });
        yesAction = (UnityAction)Delegate.Combine(yesAction, (UnityAction)delegate
        {
            Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(new CheckScene.Parameter
            {
                Title = _checkMessageSaved,
                Init = delegate
                {
                    disposable = (from _ in this.UpdateAsObservable()
                                  where Input.anyKeyDown
                                  select _).Timeout(TimeSpan.FromSeconds(resultTime)).Catch((TimeoutException ex) => Observable.Empty<Unit>()).Take(1)
                        .Subscribe(delegate
                        {
                        }, (Action)delegate
                        {
                            UnLoad();
                        });
                },
                End = delegate
                {
                    if (disposable != null)
                    {
                        disposable.Dispose();
                        disposable = null;
                    }
                }
            }, observer)).StartAsCoroutine();
        });
        param.Yes = yesAction;
        param.No = delegate
        {
            UnLoad();
        };
        param.Title = !isOverride ? _saveMessage : _saveMessageOverride;
        yield return Observable.FromCoroutine((IObserver<CheckScene> observer) => Utils.Scene.Check.Load(param, observer)).StartAsCoroutine();
    }

    private void UnLoad()
    {
        Singleton<Scene>.Instance.UnLoad();
    }

    private void Start()
    {
        this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
        {
            _canvasGroup.interactable = isOn;
        });
        gameFile.Initialize(false);
        gameFile.fileButtons.ToList().ForEach(delegate (GameFile.FileButton fb)
        {
            fb.button.OnClickAsObservable().Subscribe(delegate
            {
                UnityAction yesAction = delegate
                {
                    Singleton<Game>.Instance.Save(fb.file);
                    fb.SetName();
                };
                bool isOverride = Game.SaveFiles.Any(file => Path.GetFileName(file) == fb.file);
                Observable.FromCoroutine(c => Save(yesAction, isOverride)).StartAsCoroutine();
            });
        });
        GraphicRaycaster graphicRaycaster = _canvasGroup.GetComponent<GraphicRaycaster>();
        if (graphicRaycaster != null)
        {
            this.ObserveEveryValueChanged(_ => Singleton<Scene>.Instance.NowSceneNames[0] == "Save").Subscribe(delegate (bool isOn)
            {
                graphicRaycaster.enabled = isOn;
            });
        }
        Action returnProc = delegate
        {
            UnLoad();
        };
        _returnButton.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.sel);
            returnProc();
        });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where _returnButton.interactable
         where Singleton<Scene>.Instance.NowSceneNames[0] == "Save"
         select _).Take(1).Subscribe(delegate
     {
         returnProc();
     });
    }

    private void OnDestroy()
    {
        if (disposable != null)
        {
            disposable.Dispose();
            disposable = null;
        }
        gameFile.Dispose();
    }

    [ContextMenu("Setup")]
    private void Setup()
    {
        Transform transform = base.transform.GetComponentsInChildren<Transform>(true).FirstOrDefault(t => t.name == "Files");
        if (transform != null)
        {
            List<GameFile.FileButton> list = new List<GameFile.FileButton>();
            IEnumerator enumerator = transform.GetEnumerator();
            try
            {
                while (enumerator.MoveNext())
                {
                    Transform transform2 = (Transform)enumerator.Current;
                    list.Add(new GameFile.FileButton(transform2.GetComponent<Button>(), transform2.GetChild(0).GetComponent<Text>(), transform2.GetChild(1).GetComponent<Text>()));
                }
            }
            finally
            {
                IDisposable disposable;
                if ((disposable = enumerator as IDisposable) != null)
                {
                    disposable.Dispose();
                }
            }
            gameFile.SetupFiles(list.ToArray());
        }
    }
}
