using IllusionUtility.GetUtility;
using System;
using System.Collections.Generic;
using UnityEngine;

public abstract class ShapeInfoBase
{
    public class CategoryInfo
    {
        public int id;

        public string name = string.Empty;

        public bool[][] use = new bool[3][];

        public bool[] getflag = new bool[3];

        public void Initialize()
        {
            for (int i = 0; i < 3; i++)
            {
                use[i] = new bool[3];
                getflag[i] = false;
            }
        }
    }

    public class BoneInfo
    {
        public Transform trfBone;

        public Vector3 vctPos = Vector3.zero;

        public Vector3 vctRot = Vector3.zero;

        public Vector3 vctScl = Vector3.one;
    }

    private Dictionary<int, List<CategoryInfo>> dictCategory;

    protected Dictionary<int, BoneInfo> dictDst;

    protected Dictionary<int, BoneInfo> dictSrc;

    private AnimationKeyInfo anmKeyInfo = new AnimationKeyInfo();

    public bool InitEnd
    {
        get;
        protected set;
    }

    public int typeBone
    {
        get;
        set;
    }

    public int sex
    {
        get;
        set;
    }

    public ShapeInfoBase()
    {
        InitEnd = false;
        dictCategory = new Dictionary<int, List<CategoryInfo>>();
        dictDst = new Dictionary<int, BoneInfo>();
        dictSrc = new Dictionary<int, BoneInfo>();
        typeBone = 0;
    }

    public int GetKeyCount()
    {
        return anmKeyInfo != null ? anmKeyInfo.GetKeyCount() : 0;
    }

    public abstract void InitShapeInfo(string manifest, string assetBundleAnmKey, string assetBundleCategory, string anmKeyInfoName, string cateInfoName, Transform trfObj, int _sex);

    protected void InitShapeInfoBase(string manifest, string assetBundleAnmKey, string assetBundleCategory, string anmKeyInfoName, string cateInfoName, Transform trfObj, Dictionary<string, int> dictEnumDst, Dictionary<string, int> dictEnumSrc, Action<string, string> funcAssetBundleEntry = null)
    {
        anmKeyInfo.LoadInfo(manifest, assetBundleAnmKey, anmKeyInfoName, funcAssetBundleEntry);
        LoadCategoryInfoList(assetBundleCategory, cateInfoName, dictEnumSrc);
        GetDstBoneInfo(trfObj, dictEnumDst);
        GetSrcBoneInfo();
    }

    public void ReleaseShapeInfo()
    {
        InitEnd = false;
        dictCategory.Clear();
        dictDst.Clear();
        dictSrc.Clear();
    }

    private bool LoadCategoryInfoList(string assetBundleName, string assetName, Dictionary<string, int> dictEnumSrc)
    {
        if (!AssetBundleCheck.IsFile(assetBundleName, assetName))
        {
            string text = "読み込みエラー\r\nassetBundleName：" + assetBundleName + "\tassetName：" + assetName;
            return false;
        }
        AssetBundleLoadAssetOperation assetBundleLoadAssetOperation = AssetBundleManager.LoadAsset(assetBundleName, assetName, typeof(TextAsset));
        if (assetBundleLoadAssetOperation == null)
        {
            string text2 = "読み込みエラー\r\nassetName：" + assetName;
            return false;
        }
        TextAsset asset = assetBundleLoadAssetOperation.GetAsset<TextAsset>();
        if (null == asset)
        {
            return false;
        }
        string[,] data;
        YS_Assist.GetListString(asset.text, out data);
        int length = data.GetLength(0);
        int length2 = data.GetLength(1);
        dictCategory.Clear();
        if (length != 0 && length2 != 0)
        {
            for (int i = 0; i < length; i++)
            {
                CategoryInfo categoryInfo = new CategoryInfo();
                categoryInfo.Initialize();
                int num = int.Parse(data[i, 0]);
                categoryInfo.name = data[i, 1];
                int value;
                if (!dictEnumSrc.TryGetValue(categoryInfo.name, out value))
                {
                    string text3 = "SrcBone【" + categoryInfo.name + "】のIDが見つかりません";
                    continue;
                }
                categoryInfo.id = value;
                categoryInfo.use[0][0] = !(data[i, 2] == "0");
                categoryInfo.use[0][1] = !(data[i, 3] == "0");
                categoryInfo.use[0][2] = !(data[i, 4] == "0");
                if (categoryInfo.use[0][0] || categoryInfo.use[0][1] || categoryInfo.use[0][2])
                {
                    categoryInfo.getflag[0] = true;
                }
                categoryInfo.use[1][0] = !(data[i, 5] == "0");
                categoryInfo.use[1][1] = !(data[i, 6] == "0");
                categoryInfo.use[1][2] = !(data[i, 7] == "0");
                if (categoryInfo.use[1][0] || categoryInfo.use[1][1] || categoryInfo.use[1][2])
                {
                    categoryInfo.getflag[1] = true;
                }
                categoryInfo.use[2][0] = !(data[i, 8] == "0");
                categoryInfo.use[2][1] = !(data[i, 9] == "0");
                categoryInfo.use[2][2] = !(data[i, 10] == "0");
                if (categoryInfo.use[2][0] || categoryInfo.use[2][1] || categoryInfo.use[2][2])
                {
                    categoryInfo.getflag[2] = true;
                }
                List<CategoryInfo> value2;
                if (!dictCategory.TryGetValue(num, out value2))
                {
                    value2 = new List<CategoryInfo>();
                    dictCategory[num] = value2;
                }
                value2.Add(categoryInfo);
            }
        }
        AssetBundleManager.UnloadAssetBundle(assetBundleName, true);
        return true;
    }

    private bool GetDstBoneInfo(Transform trfBone, Dictionary<string, int> dictEnumDst)
    {
        dictDst.Clear();
        foreach (KeyValuePair<string, int> item in dictEnumDst)
        {
            GameObject gameObject = trfBone.FindLoop(item.Key);
            if (null != gameObject)
            {
                BoneInfo boneInfo = new BoneInfo();
                boneInfo.trfBone = gameObject.transform;
                dictDst[item.Value] = boneInfo;
            }
        }
        return true;
    }

    private void GetSrcBoneInfo()
    {
        dictSrc.Clear();
        foreach (KeyValuePair<int, List<CategoryInfo>> item in dictCategory)
        {
            int count = item.Value.Count;
            for (int i = 0; i < count; i++)
            {
                BoneInfo value;
                if (!dictSrc.TryGetValue(item.Value[i].id, out value))
                {
                    value = new BoneInfo();
                    dictSrc[item.Value[i].id] = value;
                }
            }
        }
    }

#if FIX
    //Prevent recreating these arrays every time
    private Vector3[] ChangeValue_array1 = new Vector3[3];
    private readonly bool[] ChangeValue_array2 = new bool[3];
#endif
    public bool ChangeValue(int category, float value)
    {
        if (anmKeyInfo == null)
            return false;

        if (!dictCategory.ContainsKey(category))
            return false;

        int count = dictCategory[category].Count;
        for (int i = 0; i < count; i++)
        {
            List<CategoryInfo> list = dictCategory[category];
            int num = list[i].id;
            string empty = list[i].name;
            BoneInfo value2;
            if (dictSrc.TryGetValue(num, out value2))
            {
#if FIX
                //Moved arrays outside of the method
#else
                Vector3[] ChangeValue_array1 = new Vector3[3];
                bool[] ChangeValue_array2 = new bool[3];
#endif
                for (int j = 0; j < 3; j++)
                    ChangeValue_array1[j] = Vector3.zero;
                for (int k = 0; k < 3; k++)
                    ChangeValue_array2[k] = list[i].getflag[k];
                anmKeyInfo.GetInfo(empty, value, ref ChangeValue_array1, ChangeValue_array2);
                if (list[i].use[0][0])
                    value2.vctPos.x = ChangeValue_array1[0].x;
                if (list[i].use[0][1])
                    value2.vctPos.y = ChangeValue_array1[0].y;
                if (list[i].use[0][2])
                    value2.vctPos.z = ChangeValue_array1[0].z;
                if (list[i].use[1][0])
                    value2.vctRot.x = ChangeValue_array1[1].x;
                if (list[i].use[1][1])
                    value2.vctRot.y = ChangeValue_array1[1].y;
                if (list[i].use[1][2])
                    value2.vctRot.z = ChangeValue_array1[1].z;
                if (list[i].use[2][0])
                    value2.vctScl.x = ChangeValue_array1[2].x;
                if (list[i].use[2][1])
                    value2.vctScl.y = ChangeValue_array1[2].y;
                if (list[i].use[2][2])
                    value2.vctScl.z = ChangeValue_array1[2].z;
            }
        }
        return true;
    }

    public bool ChangeValue(int category, int key01, int key02, float blend)
    {
        if (anmKeyInfo == null)
        {
            return false;
        }
        if (!dictCategory.ContainsKey(category))
        {
            return false;
        }
        int count = dictCategory[category].Count;
        for (int i = 0; i < count; i++)
        {
            List<CategoryInfo> list = dictCategory[category];
            int num = list[i].id;
            string empty = list[i].name;
            BoneInfo value;
            if (dictSrc.TryGetValue(num, out value))
            {
                Vector3[] value2 = new Vector3[3];
                for (int j = 0; j < 3; j++)
                {
                    value2[j] = Vector3.zero;
                }
                Vector3[] value3 = new Vector3[3];
                for (int k = 0; k < 3; k++)
                {
                    value3[k] = Vector3.zero;
                }
                bool[] array = new bool[3];
                for (int l = 0; l < 3; l++)
                {
                    array[l] = list[i].getflag[l];
                }
                if (!anmKeyInfo.GetInfo(empty, key01, ref value2, array))
                {
                    return false;
                }
                if (!anmKeyInfo.GetInfo(empty, key02, ref value3, array))
                {
                    return false;
                }
                Vector3 vector = Vector3.Lerp(value2[0], value3[0], blend);
                if (list[i].use[0][0])
                {
                    value.vctPos.x = vector.x;
                }
                if (list[i].use[0][1])
                {
                    value.vctPos.y = vector.y;
                }
                if (list[i].use[0][2])
                {
                    value.vctPos.z = vector.z;
                }
                vector.x = Mathf.LerpAngle(value2[1].x, value3[1].x, blend);
                vector.y = Mathf.LerpAngle(value2[1].y, value3[1].y, blend);
                vector.z = Mathf.LerpAngle(value2[1].z, value3[1].z, blend);
                if (list[i].use[1][0])
                {
                    value.vctRot.x = vector.x;
                }
                if (list[i].use[1][1])
                {
                    value.vctRot.y = vector.y;
                }
                if (list[i].use[1][2])
                {
                    value.vctRot.z = vector.z;
                }
                vector = Vector3.Lerp(value2[2], value3[2], blend);
                if (list[i].use[2][0])
                {
                    value.vctScl.x = vector.x;
                }
                if (list[i].use[2][1])
                {
                    value.vctScl.y = vector.y;
                }
                if (list[i].use[2][2])
                {
                    value.vctScl.z = vector.z;
                }
            }
        }
        return true;
    }

    public abstract void ForceUpdate();

    public abstract void Update();

    public abstract void UpdateAlways();
}
