using System;
using System.Threading;
using UnityEngine;
using UnityEngine.UI;

namespace StaffRoom
{
    public class FileInfo
    {
        private enum State
        {
            NO_TEX,
            LOADING,
            LOAD_END,
            HAS_TEX
        }

        private bool isChangeTex;

        private Thread thread;

        private State state;

        private byte[] bytes;

        public int index;

        public string FullPath = string.Empty;

        public string FileName = string.Empty;

        public DateTime time;

        public string name = string.Empty;

        public Texture2D texThumb;

        public int category;

        public bool disable;

        public bool disvisible;

        public StaffRoomInfoComponent fic;

        public bool show;

        public byte[] dataPng;

        public void DeleteThumb()
        {
            if (thread != null && thread.IsAlive)
            {
                thread.Abort();
            }
            if (null != texThumb)
            {
                UnityEngine.Object.Destroy(texThumb);
                texThumb = null;
            }
            state = State.NO_TEX;
        }

        private void LoadThumb()
        {
            if (state == State.NO_TEX && null == texThumb && (thread == null || !thread.IsAlive))
            {
                state = State.LOADING;
                thread = new Thread(LoadImageBytes);
                thread.Start();
            }
        }

        private void LoadImageBytes()
        {
            bytes = PngFile.LoadPngBytes(FullPath);
            if (bytes == null)
            {
                state = State.NO_TEX;
            }
            else
            {
                state = State.LOAD_END;
            }
        }

        private void CreateImage()
        {
            if (state == State.LOAD_END)
            {
                texThumb = new Texture2D(1, 1, TextureFormat.ARGB32, false);
                texThumb.LoadImage(bytes);
                bytes = null;
                state = State.HAS_TEX;
                isChangeTex = true;
            }
        }

        private void ReleaseThumb(RawImage img)
        {
            if (state != 0)
            {
                state = State.NO_TEX;
                bytes = null;
                if (thread != null && thread.IsAlive)
                {
                    thread.Abort();
                }
                if (null != texThumb)
                {
                    UnityEngine.Object.Destroy(texThumb);
                    texThumb = null;
                    isChangeTex = true;
                    img.color = Color.black;
                }
            }
        }

        public void UpdateThumb(RawImage img)
        {
            if (show)
            {
                if (dataPng == null)
                {
                    LoadThumb();
                }
                else if (state == State.NO_TEX)
                {
                    bytes = dataPng;
                    state = State.LOAD_END;
                }
            }
            else
            {
                ReleaseThumb(img);
            }
            if (state == State.LOAD_END)
            {
                CreateImage();
            }
            if (isChangeTex)
            {
                img.color = !fic || !fic.tgl || fic.tgl.interactable ? Color.white : Color.gray;
                img.texture = texThumb;
                isChangeTex = false;
            }
        }
    }
}
