using Illusion.Component.UI;
using Illusion.Extensions;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

namespace StaffRoom
{
    public class StaffRoomCharaListCtrl : MonoBehaviour
    {
        [SerializeField]
        private Text textDrawName;

        [SerializeField]
        private RectTransform rtfScrollRect;

        [SerializeField]
        private RectTransform rtfContant;

        [SerializeField]
        private GameObject objListContent;

        [SerializeField]
        private GameObject objLineTemp;

        [SerializeField]
        private Button btnSortName;

        [SerializeField]
        private Button btnSortData;

        [SerializeField]
        private bool multiSelect;

        [Tooltip("OFFだと最低でも一つは選択されてる必要がある")]
        [SerializeField]
        private bool allowSwitchOff = true;

        private StringReactiveProperty selName = new StringReactiveProperty(string.Empty);

        private string selectDrawName = string.Empty;

        private List<FileInfo> lstFileInfo = new List<FileInfo>();

        private bool ascendName;

        private bool ascendData;

        private byte lastSort;

        public event Action<FileInfo> OnPointerEnter = delegate
        {
        };

        public event Action<FileInfo> OnPointerExit = delegate
        {
        };

        public event Action<FileInfo> OnPointerClick = delegate
        {
        };

        public void SortDate(bool ascend)
        {
            ascendData = ascend;
            if (!lstFileInfo.Any())
            {
                return;
            }
            lastSort = 1;
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.time.CompareTo(b.time));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.time.CompareTo(a.time));
            }
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (!(gameObject == null))
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void SortName(bool ascend)
        {
            ascendName = ascend;
            if (!lstFileInfo.Any())
            {
                return;
            }
            lastSort = 0;
            CultureInfo currentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = CultureInfo.GetCultureInfo("ja-JP");
            if (ascend)
            {
                lstFileInfo.Sort((a, b) => a.name.CompareTo(b.name));
            }
            else
            {
                lstFileInfo.Sort((a, b) => b.name.CompareTo(a.name));
            }
            Thread.CurrentThread.CurrentCulture = currentCulture;
            for (int i = 0; i < lstFileInfo.Count; i++)
            {
                GameObject gameObject = lstFileInfo[i].fic.gameObject;
                if (!(gameObject == null))
                {
                    gameObject.transform.SetSiblingIndex(i);
                }
            }
        }

        public void ClearList()
        {
            lstFileInfo.Clear();
        }

        public void AddList(int index, string name, string fullpath, string filename, byte[] dataPng, bool disable = false, bool disvisible = false)
        {
            FileInfo fileInfo = new FileInfo();
            fileInfo.index = index;
            fileInfo.name = name;
            fileInfo.FullPath = fullpath;
            fileInfo.FileName = filename;
            fileInfo.dataPng = dataPng;
            fileInfo.disable = disable;
            fileInfo.disvisible = disvisible;
            lstFileInfo.Add(fileInfo);
        }

        public void Create(Action<StaffRoomInfoComponent> infoListProc)
        {
            objListContent.Children().ForEach(Destroy);
            ToggleGroup tglGrp = objListContent.GetComponent<ToggleGroup>();
            tglGrp.allowSwitchOff = allowSwitchOff;
            Transform parent = objListContent.transform;
            lstFileInfo.ForEach(delegate (FileInfo item)
            {
                GameObject gameObject = Instantiate(objLineTemp, parent, false);
                StaffRoomInfoComponent component = gameObject.GetComponent<StaffRoomInfoComponent>();
                component.info = item;
                component.info.fic = component;
                if (!multiSelect)
                {
                    component.tgl.group = tglGrp;
                }
                SetToggleHandler(gameObject);
                component.Disvisible(item.disvisible);
                component.Disable(item.disable);
                infoListProc.Call(component);
            });
            SortName(true);
            SortDate(false);
            ToggleAllOff();
        }

        public void UpdateSort()
        {
            if (lastSort == 0)
            {
                SortDate(ascendData);
                SortName(ascendName);
            }
            else
            {
                SortName(ascendName);
                SortDate(ascendData);
            }
        }

        public FileInfo GetFileInfoFromIndex(int index)
        {
            return lstFileInfo.Find(item => item.index == index);
        }

        public FileInfo GetFileInfoFromName(string name)
        {
            return lstFileInfo.Find(item => item.name == name);
        }

        public string GetNameFormIndex(int index)
        {
            FileInfo fileInfo = lstFileInfo.Find(item => item.index == index);
            return fileInfo != null ? fileInfo.name : string.Empty;
        }

        public int GetIndexFromName(string name)
        {
            FileInfo fileInfo = lstFileInfo.Find(item => item.name == name);
            return fileInfo != null ? fileInfo.index : -1;
        }

        public int[] GetSelectIndex()
        {
            return (from item in lstFileInfo
                    where item.fic.gameObject.activeSelf
                    where item.fic.tgl.interactable
                    where item.fic.tgl.isOn
                    select item.index).ToArray();
        }

        public StaffRoomInfoComponent GetSelectTopItem()
        {
            int[] selectIndex = GetSelectIndex();
            if (!selectIndex.Any())
            {
                return null;
            }
            FileInfo fileInfoFromIndex = GetFileInfoFromIndex(selectIndex[0]);
            return fileInfoFromIndex != null ? fileInfoFromIndex.fic : null;
        }

        public StaffRoomInfoComponent GetSelectableTopItem()
        {
            FileInfo fileInfo = (from item in lstFileInfo
                                 where item.fic.gameObject.activeSelf
                                 where item.fic.tgl.interactable
                                 orderby item.fic.gameObject.transform.GetSiblingIndex()
                                 select item).FirstOrDefault();
            return fileInfo != null ? fileInfo.fic : null;
        }

        public int GetDrawOrderFromIndex(int index)
        {
            var anon = (from item in lstFileInfo
                        where item.fic.gameObject.activeSelf
                        where item.fic.tgl.interactable
                        orderby item.fic.gameObject.transform.GetSiblingIndex()
                        select item).Select((item, idx) => new
                        {
                            item,
                            idx
                        }).FirstOrDefault(p => p.item.index == index);
            return anon != null ? anon.idx : -1;
        }

        public int GetInclusiveCount()
        {
            return lstFileInfo.Count(item => item.fic.gameObject.activeSelf);
        }

        public void SelectItem(int index)
        {
            FileInfo fileInfo = lstFileInfo.Find(item => item.index == index);
            fileInfo.fic.tgl.isOn = true;
            ChangeItem(fileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void SelectItem(string name)
        {
            FileInfo fileInfo = lstFileInfo.Find(item => item.name == name);
            fileInfo.fic.tgl.isOn = true;
            ChangeItem(fileInfo.fic.gameObject);
            UpdateScrollPosition();
        }

        public void UpdateScrollPosition()
        {
        }

        public void ChangeItem(GameObject obj)
        {
            StaffRoomInfoComponent component = obj.GetComponent<StaffRoomInfoComponent>();
            FileInfo fileInfo = null;
            if (component != null)
            {
                fileInfo = !component.tgl.isOn ? null : component.info;
            }
            OnPointerClick(fileInfo);
            selectDrawName = fileInfo != null ? fileInfo.name : string.Empty;
        }

        public void ToggleAllOff()
        {
            lstFileInfo.ForEach(delegate (FileInfo item)
            {
                item.fic.tgl.isOn = false;
            });
        }

        public void ToggleAllOn()
        {
            lstFileInfo.ForEach(delegate (FileInfo item)
            {
                item.fic.tgl.isOn = true;
            });
        }

        public void DisableItem(int index, bool disable)
        {
            DisableItem(lstFileInfo.Find(item => item.index == index), disable);
        }

        public void DisableItem(string name, bool disable)
        {
            DisableItem(lstFileInfo.Find(item => item.name == name), disable);
        }

        public void DisvisibleItem(int index, bool disvisible)
        {
            DisvisibleItem(lstFileInfo.Find(item => item.index == index), disvisible);
        }

        public void DisvisibleItem(string name, bool disvisible)
        {
            DisvisibleItem(lstFileInfo.Find(item => item.name == name), disvisible);
        }

        private void SetToggleHandler(GameObject obj)
        {
            PointerClickCheck orAddComponent = obj.GetOrAddComponent<PointerClickCheck>();
            orAddComponent.buttonType = PointerClickCheck.ButtonType.Left;
            orAddComponent.onPointerClick.AddListener(delegate
            {
                CheckToggleEvent(obj, delegate
                {
                    ChangeItem(obj);
                });
            });
            Selectable component = obj.GetComponent<Selectable>();
            component.OnPointerEnterAsObservable().Subscribe(delegate
            {
                CheckToggleEvent(obj, delegate (StaffRoomInfoComponent fic)
                {
                    selName.Value = fic.info.name;
                    OnPointerEnter(fic.info);
                });
            });
            component.OnPointerExitAsObservable().Subscribe(delegate
            {
                CheckToggleEvent(obj, delegate (StaffRoomInfoComponent fic)
                {
                    selName.Value = selectDrawName;
                    OnPointerExit(fic.info);
                });
            });
        }

        private void Start()
        {
            if (textDrawName != null)
            {
                selName.SubscribeToText(textDrawName);
            }
            if (btnSortData != null)
            {
                btnSortData.OnClickAsObservable().Subscribe(delegate
                {
                    SortDate(!ascendData);
                });
            }
            if (btnSortName != null)
            {
                btnSortName.OnClickAsObservable().Subscribe(delegate
                {
                    SortName(!ascendName);
                });
            }
            this.UpdateAsObservable().Subscribe(delegate
            {
                foreach (FileInfo item in lstFileInfo.Where(item => item.fic.gameObject.activeSelf))
                {
                    RectTransform rectTransform = item.fic.transform as RectTransform;
                    Vector2 anchoredPosition = rectTransform.anchoredPosition;
                    float y = anchoredPosition.y;
                    float height = rectTransform.rect.height;
                    Vector2 anchoredPosition2 = rtfContant.anchoredPosition;
                    float num = 0f - anchoredPosition2.y;
                    float num2 = num - rtfScrollRect.rect.height;
                    float num3 = y;
                    float num4 = y - height;
                    item.show = num4 <= num && num3 >= num2;
                    item.UpdateThumb(item.fic.imgThumb);
                }
            });
        }

        private static void CheckToggleEvent(GameObject obj, Action<StaffRoomInfoComponent> act)
        {
            if (!(obj == null))
            {
                StaffRoomInfoComponent component = obj.GetComponent<StaffRoomInfoComponent>();
                if (!(component == null) && component.tgl.interactable)
                {
                    act.Call(component);
                }
            }
        }

        private static void DisableItem(FileInfo fi, bool disable)
        {
            if (fi != null)
            {
                fi.disable = disable;
                fi.fic.Disable(disable);
            }
        }

        private static void DisvisibleItem(FileInfo fi, bool disvisible)
        {
            if (fi != null)
            {
                fi.disvisible = disvisible;
                fi.fic.Disvisible(disvisible);
            }
        }
    }
}
