using System;
using UnityEngine;

namespace StrayTech
{
    [ExecuteInEditMode]
    public class EnforceUnitScale : MonoBehaviour
    {
        private void Awake()
        {
            if (Application.isPlaying)
            {
                Destroy(this);
            }
        }

        private void Start()
        {
            UpdateCollider();
        }

        private void UpdateCollider()
        {
            if (transform.localScale == Vector3.one)
            {
                return;
            }
            Collider[] components = gameObject.GetComponents<Collider>();
            Collider[] array = components;
            foreach (Collider collider in array)
            {
                Type type = collider.GetType();
                if (type == typeof(BoxCollider))
                {
                    UpdateBoxCollider(collider as BoxCollider);
                }
                else if (type == typeof(SphereCollider))
                {
                    UpdateSphereCollider(collider as SphereCollider);
                }
            }
        }

        private void UpdateSphereCollider(SphereCollider sphereCollider)
        {
            Vector3 localScale = transform.localScale;
            float x = localScale.x;
            Vector3 localScale2 = transform.localScale;
            float a = Mathf.Max(x, localScale2.y);
            Vector3 localScale3 = transform.localScale;
            float num = Mathf.Max(a, localScale3.z);
            sphereCollider.radius *= num;
        }

        private void UpdateBoxCollider(BoxCollider boxCollider)
        {
            boxCollider.size = Vector3.Scale(boxCollider.size, transform.localScale);
        }

        private void Update()
        {
            transform.localScale = Vector3.one;
        }
    }
}
