using UnityEngine;

namespace StrayTech
{
    public class SmoothDampFollowTarget : MonoBehaviour
    {
        [SerializeField]
        private Transform _followTarget;

        [SerializeField]
        private Vector3 _targetOffset;

        [SerializeField]
        private float _smoothTime = 0.25f;

        [SerializeField]
        private bool _useFixedUpdate;

        private Vector3[] _vector3s = new Vector3[4];

        private void Start()
        {
            transform.position = _followTarget.position + _followTarget.rotation * _targetOffset;
        }

        private void Update()
        {
            if (!_useFixedUpdate)
            {
                DoUpdate(Time.deltaTime);
            }
        }

        private void FixedUpdate()
        {
            if (_useFixedUpdate)
            {
                DoUpdate(Time.fixedDeltaTime);
            }
        }

        private void DoUpdate(float deltaTime)
        {
            _vector3s[0] = _vector3s[1] = transform.position;
            _vector3s[1] = _followTarget.position + _followTarget.rotation * _targetOffset;
            _vector3s[3].x = Mathf.SmoothDamp(_vector3s[0].x, _vector3s[1].x, ref _vector3s[2].x, _smoothTime * deltaTime);
            _vector3s[3].y = Mathf.SmoothDamp(_vector3s[0].y, _vector3s[1].y, ref _vector3s[2].y, _smoothTime * deltaTime);
            _vector3s[3].z = Mathf.SmoothDamp(_vector3s[0].z, _vector3s[1].z, ref _vector3s[2].z, _smoothTime * deltaTime);
            transform.position = _vector3s[3];
            transform.rotation = _followTarget.rotation;
        }
    }
}
