using UnityEngine;

namespace StrayTech
{
    public class ThirdPersonCamera : ICameraState
    {
        private ThirdPersonCameraStateSettings _stateSettings;

        private Transform _cameraLookAtTransform;

        private float _orbitDistance = 1f;

        private float _mouseOrbitY;

        private float _mouseOrbitX;

        private Quaternion _currentOrbitRotation;

        public ICameraStateSettings StateSettings
        {
            get
            {
                return _stateSettings;
            }
        }

        public CameraSystem.CameraStateEnum StateType
        {
            get
            {
                return CameraSystem.CameraStateEnum.ThirdPerson;
            }
        }

        public bool AllowsModifiers
        {
            get
            {
                return true;
            }
        }

        public Vector3 Position
        {
            get;
            set;
        }

        public Quaternion Rotation
        {
            get;
            set;
        }

        public ThirdPersonCamera(ICameraStateSettings stateSettings)
        {
            _stateSettings = stateSettings as ThirdPersonCameraStateSettings;
            _cameraLookAtTransform = MonoBehaviourSingleton<CameraSystem>.Instance.CameraTarget;
            Vector2 mouseOrbitDistance = _stateSettings.MouseOrbitDistance;
            float x = mouseOrbitDistance.x;
            Vector2 mouseOrbitDistance2 = _stateSettings.MouseOrbitDistance;
            _orbitDistance = (x + mouseOrbitDistance2.y) * 0.5f;
            if (_stateSettings.MouseOrbit)
            {
                Vector3 normalized = (MonoBehaviourSingleton<CameraSystem>.Instance.CameraTarget.position - MonoBehaviourSingleton<CameraSystem>.Instance.CurrentCamera.transform.position).normalized;
                Quaternion quaternion = Quaternion.LookRotation(normalized, Vector3.up);
                Vector3 eulerAngles = quaternion.eulerAngles;
                _mouseOrbitX = eulerAngles.x;
                Vector3 eulerAngles2 = quaternion.eulerAngles;
                _mouseOrbitY = eulerAngles2.y;
            }
            UpdateCamera(100f);
        }

        public void UpdateCamera(float deltaTime)
        {
            if (!(_cameraLookAtTransform == null))
            {
                if (_stateSettings.MouseOrbit)
                {
                    float mouseOrbitY = _mouseOrbitY;
                    float axis = Input.GetAxis("Mouse X");
                    Vector2 mouseSensitivity = _stateSettings.MouseSensitivity;
                    _mouseOrbitY = mouseOrbitY + axis * mouseSensitivity.x * _orbitDistance * 0.02f;
                    float mouseOrbitX = _mouseOrbitX;
                    float axis2 = Input.GetAxis("Mouse Y");
                    Vector2 mouseSensitivity2 = _stateSettings.MouseSensitivity;
                    _mouseOrbitX = mouseOrbitX - axis2 * mouseSensitivity2.y * 0.02f * (!_stateSettings.MouseInvertY ? 1 : -1);
                    float mouseOrbitX2 = _mouseOrbitX;
                    Vector2 mousePitchRange = _stateSettings.MousePitchRange;
                    float x = mousePitchRange.x;
                    Vector2 mousePitchRange2 = _stateSettings.MousePitchRange;
                    _mouseOrbitX = ClampAngle(mouseOrbitX2, x, mousePitchRange2.y);
                    Quaternion b = Quaternion.Euler(_mouseOrbitX, _mouseOrbitY, 0f);
                    _currentOrbitRotation = Quaternion.Slerp(_currentOrbitRotation, b, _stateSettings.MotionSmoothing * deltaTime);
                    float value = _orbitDistance - Input.GetAxis("Mouse ScrollWheel") * 5f;
                    Vector2 mouseOrbitDistance = _stateSettings.MouseOrbitDistance;
                    float x2 = mouseOrbitDistance.x;
                    Vector2 mouseOrbitDistance2 = _stateSettings.MouseOrbitDistance;
                    _orbitDistance = Mathf.Clamp(value, x2, mouseOrbitDistance2.y);
                    Vector3 point = new Vector3(0f, 0f, 0f - _orbitDistance);
                    Vector3 position = _currentOrbitRotation * point + _cameraLookAtTransform.position;
                    Rotation = _currentOrbitRotation;
                    Position = position;
                }
                else
                {
                    Vector3 b2 = _cameraLookAtTransform.position + _cameraLookAtTransform.rotation * _stateSettings.TargetOffset;
                    Vector3 a = Vector3.Lerp(Position, b2, _stateSettings.MotionSmoothing * deltaTime);
                    float magnitude = (a - _cameraLookAtTransform.position).magnitude;
                    Vector3 normalized = (a - _cameraLookAtTransform.position).normalized;
                    Quaternion rotation = Quaternion.LookRotation(_cameraLookAtTransform.position - Position, Vector3.up);
                    Position = _cameraLookAtTransform.position + normalized * magnitude;
                    Rotation = rotation;
                }
            }
        }

        public void Cleanup()
        {
        }

        public static float ClampAngle(float angle, float min, float max)
        {
            if (angle < -360f)
            {
                angle += 360f;
            }
            if (angle > 360f)
            {
                angle -= 360f;
            }
            return Mathf.Clamp(angle, min, max);
        }
    }
}
