using Illusion.CustomAttributes;
using Manager;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

[ExecuteInEditMode]
public class StudentCardControlComponent : MonoBehaviour
{
    private const float drawH = 272f;

    [SerializeField]
    private RectTransform rtfResize;

    [SerializeField]
    private float drawHeight = 272f;

    [SerializeField]
    private bool updateSize = true;

    [SerializeField]
    private Image baseImage;

    [SerializeField]
    private Image imgEmblem;

    [SerializeField]
    private RawImage riFace;

    [SerializeField]
    private Text textAcademy;

    [SerializeField]
    private Text textName;

    [SerializeField]
    private Text textBirthday;

    [SerializeField]
    private Text textBlood;

    [SerializeField]
    private Text textClub;

    [SerializeField]
    private Text textApproval;

    [SerializeField]
    private GameObject objNickName;

    [SerializeField]
    private TextMeshProUGUI textNickName;

    [Space]
    [Button("Initialize", "設定の初期化", new object[]
    {

    })]
    public int initialize;

    public Image BaseImage
    {
        get
        {
            return baseImage;
        }
    }

    public void SetDrawSize(float height = 272f)
    {
        drawHeight = height;
        updateSize = true;
    }

    public void ChangeDrawNickName(bool draw)
    {
        if ((bool)objNickName)
        {
            objNickName.SetActive(draw);
        }
    }

    public void Clear(string academy)
    {
        if ((bool)imgEmblem)
        {
            if ((bool)imgEmblem.sprite)
            {
                Destroy(imgEmblem.sprite);
            }
            imgEmblem.sprite = null;
            imgEmblem.enabled = false;
        }
        if ((bool)riFace)
        {
            if ((bool)riFace.texture)
            {
                Destroy(riFace.texture);
            }
            riFace.texture = null;
            riFace.enabled = false;
        }
        if ((bool)textAcademy)
        {
            textAcademy.text = academy;
        }
        if ((bool)textName)
        {
            textName.text = string.Empty;
        }
        if ((bool)textBirthday)
        {
            textBirthday.text = string.Empty;
        }
        if ((bool)textBlood)
        {
            textBlood.text = ChaFileDefine.GetBloodTypeStr(-1);
        }
        if ((bool)textClub)
        {
            textClub.text = "不明";
        }
        if ((bool)textApproval)
        {
            textApproval.text = academy + " 学園長";
        }
        if ((bool)objNickName)
        {
            objNickName.SetActive(false);
        }
        if ((bool)textNickName)
        {
            textNickName.text = string.Empty;
        }
    }

    public void SetCharaInfo(ChaFileControl chaFileCtrl, int emblemId, string academy, string nickname = "")
    {
        SetCharaInfo(chaFileCtrl);
        SetEmblem(emblemId);
        SetAccademy(academy);
        SetNickName(nickname);
    }

    public void SetCharaInfo(ChaFileControl chaFileCtrl)
    {
        if (chaFileCtrl == null)
        {
            return;
        }
        if ((bool)riFace)
        {
            if ((bool)riFace.texture)
            {
                Destroy(riFace.texture);
            }
            riFace.texture = null;
            riFace.enabled = false;
            if (chaFileCtrl.facePngData != null)
            {
                Texture2D texture2D = new Texture2D(240, 320);
                texture2D.LoadImage(chaFileCtrl.facePngData);
                riFace.texture = texture2D;
                riFace.enabled = true;
            }
        }
        ChaFileParameter parameter = chaFileCtrl.parameter;
        if ((bool)textName)
        {
            textName.text = parameter.fullname;
        }
        if ((bool)textBirthday)
        {
            textBirthday.text = parameter.birthMonth + "月" + parameter.birthDay + "日";
        }
        if ((bool)textBlood)
        {
            textBlood.text = ChaFileDefine.GetBloodTypeStr(parameter.bloodType);
        }
        if (!textClub)
        {
            return;
        }
        if (parameter.sex == 0)
        {
            textClub.text = "コイカツ部";
            return;
        }
        ClubInfo.Param value;
        if (!Game.ClubInfos.TryGetValue(parameter.clubActivities, out value))
        {
            Game.ClubInfos.TryGetValue(0, out value);
        }
        if (value == null)
        {
            textClub.text = "不明";
        }
        else
        {
            textClub.text = value.Name;
        }
    }

    public void SetEmblem(int emblemId)
    {
        if (!imgEmblem)
        {
            return;
        }
        if ((bool)imgEmblem.sprite)
        {
            Destroy(imgEmblem.sprite);
        }
        imgEmblem.sprite = null;
        imgEmblem.enabled = false;
        ListInfoBase listInfo = Singleton<Character>.Instance.chaListCtrl.GetListInfo(ChaListDefine.CategoryNo.mt_emblem, emblemId);
        if (listInfo == null)
        {
            return;
        }
        string info = listInfo.GetInfo(ChaListDefine.KeyType.MainTexAB);
        string text = listInfo.GetInfo(ChaListDefine.KeyType.MainTex) + "_srgb";
        if ("0" != info && "0" != text)
        {
            Texture2D texture2D = CommonLib.LoadAsset<Texture2D>(info, text, false, string.Empty);
            if ((bool)texture2D)
            {
                imgEmblem.sprite = Sprite.Create(texture2D, new Rect(0f, 0f, texture2D.width, texture2D.height), new Vector2(0.5f, 0.5f));
            }
            imgEmblem.enabled = true;
        }
    }

    public void SetAccademy(string academy)
    {
        if ((bool)textAcademy)
        {
            textAcademy.text = academy;
        }
        if ((bool)textApproval)
        {
            textApproval.text = academy + " 学園長";
        }
    }

    public void SetNickName(string nickname)
    {
        if ((bool)textNickName)
        {
            textNickName.text = nickname;
        }
    }

    public void SetChargeName(string chargename)
    {
        if ((bool)textClub)
        {
            textClub.text = chargename;
        }
    }

    public void Initialize()
    {
        Transform transform = base.transform.Find("resize");
        if ((bool)transform)
        {
            rtfResize = transform as RectTransform;
        }
        transform = base.transform.Find("resize/image/emblem");
        if ((bool)transform)
        {
            imgEmblem = transform.GetComponent<Image>();
        }
        transform = base.transform.Find("resize/image/face");
        if ((bool)transform)
        {
            riFace = transform.GetComponent<RawImage>();
        }
        transform = base.transform.Find("resize/text/academy");
        if ((bool)transform)
        {
            textAcademy = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/text/name");
        if ((bool)transform)
        {
            textName = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/text/birthday");
        if ((bool)transform)
        {
            textBirthday = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/text/blood");
        if ((bool)transform)
        {
            textBlood = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/text/club");
        if ((bool)transform)
        {
            textClub = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/text/approval");
        if ((bool)transform)
        {
            textApproval = transform.GetComponent<Text>();
        }
        transform = base.transform.Find("resize/image/base");
        if ((bool)transform)
        {
            baseImage = transform.GetComponent<Image>();
        }
        transform = base.transform.Find("resize/imgNickName");
        if ((bool)transform)
        {
            objNickName = transform.gameObject;
        }
        transform = base.transform.Find("resize/imgNickName/textNickName");
        if ((bool)transform)
        {
            textNickName = transform.GetComponent<TextMeshProUGUI>();
        }
    }

    private void Reset()
    {
        Initialize();
    }

    private void Update()
    {
        if (updateSize && null != rtfResize)
        {
            float num = drawHeight / 272f;
            rtfResize.localScale = new Vector3(num, num, 1f);
            updateSize = false;
        }
    }
}
