using Illusion.Extensions;
using Illusion.Game;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using TMPro;
using UniRx;
using UniRx.Triggers;
using UnityEngine;
using UnityEngine.UI;

public class TitleScene : MonoBehaviour
{
    [Serializable]
    public class ButtonGroup
    {
        public int group;

        public Button button;
    }

    public class ButtonStack<T> : Stack<T> where T : List<Button>
    {
        public new void Push(T item)
        {
            if (Count > 0)
            {
                T val = Peek();
                val.ForEach(delegate (Button b)
                {
                    b.gameObject.SetActive(false);
                });
            }
            item.ForEach(delegate (Button b)
            {
                b.gameObject.SetActive(true);
            });
            base.Push(item);
        }

        public new T Pop()
        {
            T result = base.Pop();
            result.ForEach(delegate (Button b)
            {
                b.gameObject.SetActive(false);
            });
            if (Count > 0)
            {
                T val = Peek();
                val.ForEach(delegate (Button b)
                {
                    b.gameObject.SetActive(true);
                });
            }
            return result;
        }
    }

    [SerializeField]
    private GraphicRaycaster graphicRaycaster;

    [SerializeField]
    private CanvasGroup canvasGroup;

    [SerializeField]
    private ButtonGroup[] buttons;

    [SerializeField]
    private Button btnRanking;

    [SerializeField]
    private TextMeshProUGUI version;

    [SerializeField]
    private Image titleLogo;

    private Dictionary<int, List<Button>> buttonDic;

    private ButtonStack<List<Button>> buttonStack = new ButtonStack<List<Button>>();

    private bool isCustomMale;

    public void OnStart()
    {
        Enter("EntryPlayer");
    }

    public void OnLoad()
    {
        Enter("Load");
    }

    public void OnCustom()
    {
        buttonStack.Push(buttonDic[1]);
        Enter(string.Empty);
    }

    public void OnCustomMale()
    {
        isCustomMale = true;
        Enter("CustomScene");
    }

    public void OnCustomFemale()
    {
        isCustomMale = false;
        Enter("CustomScene");
    }

    public void OnUploader()
    {
        Enter("Uploader");
    }

    public void OnDownloader()
    {
        Enter("Downloader");
    }

    public void OnBack()
    {
        buttonStack.Pop();
        Enter(string.Empty);
    }

    public void OnOther()
    {
        buttonStack.Push(buttonDic[2]);
        Button button = buttonStack.Peek().FirstOrDefault(p => p.name.CompareParts("wedding", true));
        if (button != null)
        {
            bool active = Game.isAdd20 && Singleton<Voice>.Instance.voiceInfoList.Select(p => p.No).Intersect(Singleton<Game>.Instance.weddingData.personality).Any();
            button.gameObject.SetActiveIfDifferent(active);
        }
        Enter(string.Empty);
    }

    public void OnOtherFreeH()
    {
        Enter("FreeH");
    }

    public void OnOtherIdolLive()
    {
        Enter("LiveStage");
    }

    public void OnOtherEvent()
    {
        Enter("FixEventSceneEx");
    }

    public void OnConfig()
    {
        Enter("Config");
    }

    public void OnEnd()
    {
        Enter("Exit");
    }

    public void OnWedding()
    {
        Enter("Wedding");
    }

    private void ButtonInteractable(string name, Func<bool> func)
    {
        ButtonGroup[] array = buttons;
        int num = 0;
        ButtonGroup buttonGroup;
        while (true)
        {
            if (num < array.Length)
            {
                buttonGroup = array[num];
                if (buttonGroup.button.name.IndexOf(name) != -1)
                {
                    break;
                }
                num++;
                continue;
            }
            return;
        }
        if (!func.IsNullOrEmpty())
        {
            buttonGroup.button.interactable = func();
        }
    }

    private void Enter(string next)
    {
        if (!graphicRaycaster.enabled)
        {
            return;
        }
        if (next != null)
        {
            if (next == string.Empty || next == "Load" || next == "Config")
            {
                Utils.Sound.Play(SystemSE.sel);
                goto IL_8A;
            }
        }
        Utils.Sound.Play(SystemSE.ok_s);
    IL_8A:
        switch (next)
        {
            case "EntryPlayer":
                {
                    Scene.Data data = new Scene.Data
                    {
                        levelName = next,
                        isAsync = true,
                        isFade = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data, true);
                    break;
                }
            case "Load":
                {
                    Scene.Data data2 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = true,
                        onLoad = delegate
                        {
                            LoadScene rootComponent = Scene.GetRootComponent<LoadScene>(next);
                            if (rootComponent == null)
                            {
                                return;
                            }
                            rootComponent.onEnter += delegate (string fileName)
                            {
                                Game.SaveFileName = fileName;
                                Singleton<Game>.Instance.Load();
                                Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                                {
                                    levelName = "Action",
                                    fadeType = Scene.Data.FadeType.In
                                }, true);
                            };
                        }
                    };
                    Singleton<Scene>.Instance.LoadReserve(data2, false);
                    break;
                }
            case "CustomScene":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data3 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = false,
                        isFade = true,
                        isAsync = true,
                        onLoad = delegate
                        {
                            CustomScene rootComponent = Scene.GetRootComponent<CustomScene>(next);
                            if (rootComponent == null)
                            {
                                return;
                            }
                            rootComponent.modeNew = true;
                            rootComponent.modeSex = (byte)(!isCustomMale ? 1 : 0);
                            rootComponent.chaFileCtrl = null;
                        }
                    };
                    Singleton<Scene>.Instance.LoadReserve(data3, true);
                    break;
                }
            case "Uploader":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data4 = new Scene.Data();
                    data4.levelName = "NetworkCheckScene";
                    data4.isAdd = false;
                    data4.isFade = true;
                    data4.isAsync = true;
                    data4.onLoad = delegate ()
                    {
                        NetworkCheckScene rootComponent = Scene.GetRootComponent<NetworkCheckScene>("NetworkCheckScene");
                        if (rootComponent == null)
                        {
                            return;
                        }
                        rootComponent.nextSceneName = "Uploader";
                    };
                    Scene.Data data5 = data4;
                    Singleton<Scene>.Instance.LoadReserve(data5, true);
                    break;
                }
            case "Downloader":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data4 = new Scene.Data();
                    data4.levelName = "NetworkCheckScene";
                    data4.isAdd = false;
                    data4.isFade = true;
                    data4.isAsync = true;
                    data4.onLoad = delegate ()
                    {
                        NetworkCheckScene rootComponent = Scene.GetRootComponent<NetworkCheckScene>("NetworkCheckScene");
                        if (rootComponent == null)
                        {
                            return;
                        }
                        rootComponent.nextSceneName = "Downloader";
                    };
                    Scene.Data data6 = data4;
                    Singleton<Scene>.Instance.LoadReserve(data6, true);
                    break;
                }
            case "FixEventSceneEx":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data7 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = false,
                        isFade = true,
                        isAsync = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data7, true);
                    break;
                }
            case "Config":
                {
                    Scene.Data data8 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data8, true);
                    break;
                }
            case "FreeH":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data9 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = false,
                        isFade = true,
                        isAsync = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data9, true);
                    break;
                }
            case "LiveStage":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data10 = new Scene.Data
                    {
                        levelName = next,
                        isAdd = false,
                        isFade = true,
                        isAsync = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data10, true);
                    break;
                }
            case "Exit":
                Singleton<Scene>.Instance.GameEnd(true);
                break;
            case "Wedding":
                {
                    graphicRaycaster.enabled = false;
                    Scene.Data data11 = new Scene.Data
                    {
                        assetBundleName = "scene/20/wedding/main.unity3d",
                        levelName = next,
                        isAdd = false,
                        fadeType = Scene.Data.FadeType.In,
                        isAsync = true
                    };
                    Singleton<Scene>.Instance.LoadReserve(data11, true);
                    break;
                }
        }
    }

    private IEnumerator Start()
    {
        enabled = false;
        version.text = "CE Ver " + Game.Version;
        var versionRect = version.GetComponent<RectTransform>();
        versionRect.sizeDelta = new Vector2(200, versionRect.sizeDelta.y);
        Cursor.lockState = CursorLockMode.None;
        Cursor.visible = true;
        Cursor.SetCursor(null, Vector2.zero, CursorMode.Auto);
        if (Game.isAdd20)
        {
            string assetBundleName = "sprite/20/title.unity3d";
            if (AssetBundleCheck.IsFile(assetBundleName, string.Empty) && titleLogo != null)
            {
                Utils.Bundle.LoadSprite(assetBundleName, "title_logo_after", titleLogo, true);
            }
        }
        buttonDic = buttons.ToLookup(p => p.group, p => p.button).ToDictionary(p => p.Key, Enumerable.ToList);
        foreach (List<Button> value in buttonDic.Values)
        {
            value.ForEach(delegate (Button p)
            {
                p.gameObject.SetActive(false);
            });
        }
        buttonStack.Push(buttonDic[0]);
        this.ObserveEveryValueChanged(_ => !Singleton<Scene>.Instance.IsNowLoadingFade).Subscribe(delegate (bool isOn)
        {
            canvasGroup.interactable = isOn;
        });
        Singleton<Game>.Instance.NewGame();
        Singleton<Scene>.Instance.UnloadBaseScene();
        Singleton<Scene>.Instance.SetFadeColorDefault();
        Singleton<Scene>.Instance.UnloadAddScene();
        if (Singleton<Character>.IsInstance())
        {
            Singleton<Character>.Instance.EndLoadAssetBundle();
        }
        string[] array = AssetBundleManager.AllLoadedAssetBundleNames.Where(s => s.StartsWith("sound/data/pcm/")).ToArray();
        foreach (string assetBundleName2 in array)
        {
            AssetBundleManager.UnloadAssetBundle(assetBundleName2, true);
        }
        if (Scene.isReturnTitle && Singleton<Scene>.Instance.sceneFade._Fade == SimpleFade.Fade.In)
        {
            yield return new WaitUntil(() => Singleton<Scene>.Instance.sceneFade.IsEnd);
            yield return StartCoroutine(Singleton<Scene>.Instance.Fade(SimpleFade.Fade.Out));
        }
        Scene.isReturnTitle = false;
        ButtonInteractable("Load", () => Game.IsLoadCheck);
        ButtonInteractable("Other", () => true);
        ButtonInteractable("FixEventSceneEx", () => Singleton<Game>.Instance.glSaveData.fixCharaTaked.Any());
        bool rankEntry = Singleton<Game>.Instance.rankSaveData.userName.IsNullOrEmpty();
        TextMeshProUGUI textRanking = btnRanking.GetComponentInChildren<TextMeshProUGUI>();
        textRanking.text = !rankEntry ? "ランキング" : "ランキングに参加";
        btnRanking.OnClickAsObservable().Subscribe(delegate
        {
            Utils.Sound.Play(SystemSE.ok_s);
            if (rankEntry)
            {
                Scene.Data data = new Scene.Data
                {
                    levelName = "RankingEntryScene",
                    isAdd = true,
                    isFade = true,
                    isAsync = true,
                    onLoad = delegate
                    {
                        RankingEntryScene rootComponent = Scene.GetRootComponent<RankingEntryScene>("RankingEntryScene");
                        if (!(rootComponent == null))
                        {
                            rootComponent.backSceneName = "Title";
                        }
                    }
                };
                Singleton<Scene>.Instance.LoadReserve(data, true);
            }
            else
            {
                Singleton<Scene>.Instance.LoadReserve(new Scene.Data
                {
                    levelName = "RankingScene",
                    isFade = true
                }, false);
            }
        });
        this.ObserveEveryValueChanged(_ => Singleton<Scene>.Instance.AddSceneName).Subscribe(delegate (string sceneName)
        {
            graphicRaycaster.enabled = sceneName == string.Empty;
        });
        List<AudioClip> clipList = new List<AudioClip>();
        AssetBundleData.GetAssetBundleNameListFromPath("sound/data/systemse/titlecall/", true).ForEach(delegate (string file)
        {
            clipList.AddRange(AssetBundleManager.LoadAllAsset(file, typeof(AudioClip)).GetAllAssets<AudioClip>());
            AssetBundleManager.UnloadAssetBundle(file, true);
        });
        clipList.RemoveAll(p => p == null);
        AudioClip clip = clipList.Shuffle().FirstOrDefault();
        (from _ in (from _ in this.UpdateAsObservable()
                    where !Singleton<Scene>.Instance.IsFadeNow
                    select _).Take(1)
         where clip != null
         select _).Subscribe(delegate
         {
             Utils.Sound.Play(Manager.Sound.Type.SystemSE, clip).OnDestroyAsObservable().Subscribe(delegate
             {
                 clipList.ForEach(Resources.UnloadAsset);
                 clipList.Clear();
                 clip = null;
             });
         });
        (from _ in this.UpdateAsObservable()
         where Input.GetMouseButtonDown(1)
         where buttonStack.Count > 1
         select _).Subscribe(delegate
         {
             buttonStack.Pop();
         });
        enabled = true;
    }
}
