using UnityEngine;
using UnityEngine.EventSystems;
using UnityEngine.UI;

public class UI_DragWindow : MonoBehaviour, IPointerDownHandler, IBeginDragHandler, IDragHandler, IEndDragHandler, IPointerUpHandler, IEventSystemHandler
{
    public RectTransform rtDrag;

    public RectTransform rtMove;

    public RectTransform rtCanvas;

    private Canvas canvas;

    private CanvasScaler cscaler;

    private Vector2 dragStartPosBackup = Vector2.zero;

    private CameraControl camCtrl;

    private void Start()
    {
        if (null == rtMove)
        {
            rtMove = transform as RectTransform;
        }
        if (null == rtDrag)
        {
            rtDrag = rtMove;
        }
        if (null == rtCanvas)
        {
            SearchCanvas();
        }
        if (null != rtCanvas && null == canvas)
        {
            canvas = rtCanvas.GetComponent<Canvas>();
            if ((bool)canvas)
            {
                cscaler = rtCanvas.GetComponent<CanvasScaler>();
            }
        }
        if (camCtrl == null && Camera.main)
        {
            camCtrl = Camera.main.GetComponent<CameraControl>();
        }
    }

    private void SearchCanvas()
    {
        GameObject gameObject = base.gameObject;
        while (true)
        {
            canvas = gameObject.GetComponent<Canvas>();
            if ((bool)canvas)
            {
                break;
            }
            if (null == gameObject.transform.parent)
            {
                return;
            }
            gameObject = gameObject.transform.parent.gameObject;
        }
        rtCanvas = gameObject.transform as RectTransform;
    }

    private float GetScreenRate()
    {
        float num = Screen.width;
        float num2 = Screen.height;
        Vector2 one = Vector2.one;
        Vector2 referenceResolution = cscaler.referenceResolution;
        one.x = num / referenceResolution.x;
        Vector2 referenceResolution2 = cscaler.referenceResolution;
        one.y = num2 / referenceResolution2.y;
        return one.x * (1f - cscaler.matchWidthOrHeight) + one.y * cscaler.matchWidthOrHeight;
    }

    private void CalcDragPosOverlay(PointerEventData ped)
    {
        Vector2 anchoredPosition = ped.position - dragStartPosBackup;
        float x = anchoredPosition.x;
        Vector3 localScale = rtCanvas.localScale;
        anchoredPosition.x = x / localScale.x;
        float y = anchoredPosition.y;
        Vector3 localScale2 = rtCanvas.localScale;
        anchoredPosition.y = y / localScale2.y;
        Vector2 size = rtDrag.rect.size;
        float x2 = size.x;
        Vector2 sizeDelta = rtDrag.sizeDelta;
        float num;
        if (x2 == sizeDelta.x)
        {
            Vector2 sizeDelta2 = rtDrag.sizeDelta;
            num = sizeDelta2.x;
        }
        else
        {
            Vector2 size2 = rtDrag.rect.size;
            float x3 = size2.x;
            Vector2 sizeDelta3 = rtDrag.sizeDelta;
            num = x3 - sizeDelta3.x;
        }
        float num2 = num;
        Vector2 size3 = rtDrag.rect.size;
        float y2 = size3.y;
        Vector2 sizeDelta4 = rtDrag.sizeDelta;
        float num3;
        if (y2 == sizeDelta4.y)
        {
            Vector2 sizeDelta5 = rtDrag.sizeDelta;
            num3 = sizeDelta5.y;
        }
        else
        {
            Vector2 size4 = rtDrag.rect.size;
            float y3 = size4.y;
            Vector2 sizeDelta6 = rtDrag.sizeDelta;
            num3 = y3 - sizeDelta6.y;
        }
        float num4 = num3;
        float x4 = anchoredPosition.x;
        float num5 = Screen.width;
        Vector3 localScale3 = rtCanvas.localScale;
        anchoredPosition.x = Mathf.Clamp(x4, 0f, num5 / localScale3.x - num2);
        float value = 0f - anchoredPosition.y;
        float num6 = Screen.height;
        Vector3 localScale4 = rtCanvas.localScale;
        anchoredPosition.y = 0f - Mathf.Clamp(value, 0f, num6 / localScale4.y - num4);
        Vector2 size5 = rtMove.rect.size;
        float x5 = size5.x;
        Vector2 sizeDelta7 = rtMove.sizeDelta;
        float num7;
        if (x5 == sizeDelta7.x)
        {
            Vector2 sizeDelta8 = rtMove.sizeDelta;
            num7 = sizeDelta8.x;
        }
        else
        {
            Vector2 size6 = rtMove.rect.size;
            float x6 = size6.x;
            Vector2 sizeDelta9 = rtMove.sizeDelta;
            num7 = x6 - sizeDelta9.x;
        }
        float num8 = num7;
        Vector2 size7 = rtMove.rect.size;
        float y4 = size7.y;
        Vector2 sizeDelta10 = rtMove.sizeDelta;
        float num9;
        if (y4 == sizeDelta10.y)
        {
            Vector2 sizeDelta11 = rtMove.sizeDelta;
            num9 = sizeDelta11.y;
        }
        else
        {
            Vector2 size8 = rtMove.rect.size;
            float y5 = size8.y;
            Vector2 sizeDelta12 = rtMove.sizeDelta;
            num9 = y5 - sizeDelta12.y;
        }
        float num10 = num9;
        float x7 = anchoredPosition.x;
        Vector2 pivot = rtMove.pivot;
        anchoredPosition.x = x7 + num8 * pivot.x;
        float y6 = anchoredPosition.y;
        Vector2 pivot2 = rtMove.pivot;
        anchoredPosition.y = y6 + num10 * (pivot2.y - 1f);
        rtMove.anchoredPosition = anchoredPosition;
    }

    private void CalcDragPosScreenSpace(PointerEventData ped)
    {
        Vector2 anchoredPosition = ped.position - dragStartPosBackup;
        float screenRate = GetScreenRate();
        anchoredPosition.x /= screenRate;
        anchoredPosition.y /= screenRate;
        Vector2 size = rtDrag.rect.size;
        float x = size.x;
        Vector2 sizeDelta = rtDrag.sizeDelta;
        float num;
        if (x == sizeDelta.x)
        {
            Vector2 sizeDelta2 = rtDrag.sizeDelta;
            num = sizeDelta2.x;
        }
        else
        {
            Vector2 size2 = rtDrag.rect.size;
            float x2 = size2.x;
            Vector2 sizeDelta3 = rtDrag.sizeDelta;
            num = x2 - sizeDelta3.x;
        }
        float num2 = num;
        Vector2 size3 = rtDrag.rect.size;
        float y = size3.y;
        Vector2 sizeDelta4 = rtDrag.sizeDelta;
        float num3;
        if (y == sizeDelta4.y)
        {
            Vector2 sizeDelta5 = rtDrag.sizeDelta;
            num3 = sizeDelta5.y;
        }
        else
        {
            Vector2 size4 = rtDrag.rect.size;
            float y2 = size4.y;
            Vector2 sizeDelta6 = rtDrag.sizeDelta;
            num3 = y2 - sizeDelta6.y;
        }
        float num4 = num3;
        anchoredPosition.x = Mathf.Clamp(anchoredPosition.x, 0f, Screen.width / screenRate - num2);
        anchoredPosition.y = 0f - Mathf.Clamp(0f - anchoredPosition.y, 0f, Screen.height / screenRate - num4);
        Vector2 size5 = rtMove.rect.size;
        float x3 = size5.x;
        Vector2 sizeDelta7 = rtMove.sizeDelta;
        float num5;
        if (x3 == sizeDelta7.x)
        {
            Vector2 sizeDelta8 = rtMove.sizeDelta;
            num5 = sizeDelta8.x;
        }
        else
        {
            Vector2 size6 = rtMove.rect.size;
            float x4 = size6.x;
            Vector2 sizeDelta9 = rtMove.sizeDelta;
            num5 = x4 - sizeDelta9.x;
        }
        float num6 = num5;
        Vector2 size7 = rtMove.rect.size;
        float y3 = size7.y;
        Vector2 sizeDelta10 = rtMove.sizeDelta;
        float num7;
        if (y3 == sizeDelta10.y)
        {
            Vector2 sizeDelta11 = rtMove.sizeDelta;
            num7 = sizeDelta11.y;
        }
        else
        {
            Vector2 size8 = rtMove.rect.size;
            float y4 = size8.y;
            Vector2 sizeDelta12 = rtMove.sizeDelta;
            num7 = y4 - sizeDelta12.y;
        }
        float num8 = num7;
        float x5 = anchoredPosition.x;
        Vector2 pivot = rtMove.pivot;
        anchoredPosition.x = x5 + num6 * pivot.x;
        float y5 = anchoredPosition.y;
        Vector2 pivot2 = rtMove.pivot;
        anchoredPosition.y = y5 + num8 * (pivot2.y - 1f);
        rtMove.anchoredPosition = anchoredPosition;
    }

    private void SetClickPosOverlay(PointerEventData ped)
    {
        Vector2 zero = Vector2.zero;
        Vector2 size = rtMove.rect.size;
        float x = size.x;
        Vector2 sizeDelta = rtMove.sizeDelta;
        float num;
        if (x == sizeDelta.x)
        {
            Vector2 sizeDelta2 = rtMove.sizeDelta;
            num = sizeDelta2.x;
        }
        else
        {
            Vector2 size2 = rtMove.rect.size;
            float x2 = size2.x;
            Vector2 sizeDelta3 = rtMove.sizeDelta;
            num = x2 - sizeDelta3.x;
        }
        float num2 = num;
        Vector2 size3 = rtMove.rect.size;
        float y = size3.y;
        Vector2 sizeDelta4 = rtMove.sizeDelta;
        float num3;
        if (y == sizeDelta4.y)
        {
            Vector2 sizeDelta5 = rtMove.sizeDelta;
            num3 = sizeDelta5.y;
        }
        else
        {
            Vector2 size4 = rtMove.rect.size;
            float y2 = size4.y;
            Vector2 sizeDelta6 = rtMove.sizeDelta;
            num3 = y2 - sizeDelta6.y;
        }
        float num4 = num3;
        Vector2 anchoredPosition = rtMove.anchoredPosition;
        float x3 = anchoredPosition.x;
        Vector2 pivot = rtMove.pivot;
        float num5 = x3 - num2 * pivot.x;
        Vector3 localScale = rtCanvas.localScale;
        zero.x = num5 * localScale.x;
        Vector2 anchoredPosition2 = rtMove.anchoredPosition;
        float y3 = anchoredPosition2.y;
        Vector2 pivot2 = rtMove.pivot;
        float num6 = y3 - num4 * (pivot2.y - 1f);
        Vector3 localScale2 = rtCanvas.localScale;
        zero.y = num6 * localScale2.y;
        dragStartPosBackup = ped.position - zero;
    }

    private void SetClickPosScreenSpace(PointerEventData ped)
    {
        float screenRate = GetScreenRate();
        Vector2 zero = Vector2.zero;
        Vector2 size = rtMove.rect.size;
        float x = size.x;
        Vector2 sizeDelta = rtMove.sizeDelta;
        float num;
        if (x == sizeDelta.x)
        {
            Vector2 sizeDelta2 = rtMove.sizeDelta;
            num = sizeDelta2.x;
        }
        else
        {
            Vector2 size2 = rtMove.rect.size;
            float x2 = size2.x;
            Vector2 sizeDelta3 = rtMove.sizeDelta;
            num = x2 - sizeDelta3.x;
        }
        float num2 = num;
        Vector2 size3 = rtMove.rect.size;
        float y = size3.y;
        Vector2 sizeDelta4 = rtMove.sizeDelta;
        float num3;
        if (y == sizeDelta4.y)
        {
            Vector2 sizeDelta5 = rtMove.sizeDelta;
            num3 = sizeDelta5.y;
        }
        else
        {
            Vector2 size4 = rtMove.rect.size;
            float y2 = size4.y;
            Vector2 sizeDelta6 = rtMove.sizeDelta;
            num3 = y2 - sizeDelta6.y;
        }
        float num4 = num3;
        Vector2 anchoredPosition = rtMove.anchoredPosition;
        float x3 = anchoredPosition.x;
        Vector2 pivot = rtMove.pivot;
        zero.x = (x3 - num2 * pivot.x) * screenRate;
        Vector2 anchoredPosition2 = rtMove.anchoredPosition;
        float y3 = anchoredPosition2.y;
        Vector2 pivot2 = rtMove.pivot;
        zero.y = (y3 - num4 * (pivot2.y - 1f)) * screenRate;
        dragStartPosBackup = ped.position - zero;
    }

    public void OnPointerDown(PointerEventData ped)
    {
        switch (canvas.renderMode)
        {
            case RenderMode.ScreenSpaceCamera:
                SetClickPosScreenSpace(ped);
                break;
            case RenderMode.ScreenSpaceOverlay:
                SetClickPosOverlay(ped);
                break;
        }
        if ((bool)camCtrl)
        {
            camCtrl.NoCtrlCondition = () => true;
        }
    }

    public void OnBeginDrag(PointerEventData ped)
    {
        switch (canvas.renderMode)
        {
            case RenderMode.WorldSpace:
                break;
            case RenderMode.ScreenSpaceCamera:
                CalcDragPosScreenSpace(ped);
                break;
            case RenderMode.ScreenSpaceOverlay:
                CalcDragPosOverlay(ped);
                break;
        }
    }

    public void OnDrag(PointerEventData ped)
    {
        switch (canvas.renderMode)
        {
            case RenderMode.WorldSpace:
                break;
            case RenderMode.ScreenSpaceCamera:
                CalcDragPosScreenSpace(ped);
                break;
            case RenderMode.ScreenSpaceOverlay:
                CalcDragPosOverlay(ped);
                break;
        }
    }

    public void OnEndDrag(PointerEventData ped)
    {
        if ((bool)camCtrl)
        {
            camCtrl.NoCtrlCondition = () => false;
        }
    }

    public void OnPointerUp(PointerEventData ped)
    {
        if ((bool)camCtrl)
        {
            camCtrl.NoCtrlCondition = () => false;
        }
    }
}
