using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class UI_TreeView : MonoBehaviour
{
    public bool rootFlag;

    public bool ExpandFirst = true;

    public float topMargin = 2f;

    public float bottomMargin = 2f;

    public ScrollRect scrollRect;

    public RectTransform rtScroll;

    public bool unused;

    private Toggle minmax;

    private UI_TreeView tvRoot;

    private List<UI_TreeView> lstChild = new List<UI_TreeView>();

    private void Start()
    {
        if (rootFlag)
        {
            CreateTree(this);
            if (ExpandFirst)
            {
                ExpandAll();
            }
            else
            {
                CollapseAll();
            }
        }
    }

    private void CreateTree(UI_TreeView tvroot)
    {
        if (null == minmax)
        {
            minmax = gameObject.GetComponent<Toggle>();
        }
        if ((bool)minmax)
        {
            minmax.onValueChanged.AddListener(MinMaxChange);
        }
        tvRoot = tvroot;
        IEnumerator enumerator = gameObject.transform.GetEnumerator();
        try
        {
            while (enumerator.MoveNext())
            {
                Transform transform = (Transform)enumerator.Current;
                UI_TreeView component = transform.gameObject.GetComponent<UI_TreeView>();
                if (!(null == component))
                {
                    lstChild.Add(component);
                    component.CreateTree(tvroot);
                }
            }
        }
        finally
        {
            IDisposable disposable;
            if ((disposable = enumerator as IDisposable) != null)
            {
                disposable.Dispose();
            }
        }
    }

    public void UpdateView(ref float totalPosY, float parentPosY)
    {
        float parentPosY2 = totalPosY;
        if (rootFlag)
        {
            totalPosY = 0f - topMargin;
        }
        else
        {
            RectTransform component = gameObject.GetComponent<RectTransform>();
            if ((bool)component)
            {
                Vector2 anchoredPosition = component.anchoredPosition;
                component.anchoredPosition = new Vector2(anchoredPosition.x, totalPosY - parentPosY);
                if (gameObject.activeSelf && !unused)
                {
                    float num = totalPosY;
                    Vector2 sizeDelta = component.sizeDelta;
                    totalPosY = num - sizeDelta.y;
                }
            }
        }
        IEnumerator enumerator = gameObject.transform.GetEnumerator();
        try
        {
            while (enumerator.MoveNext())
            {
                Transform transform = (Transform)enumerator.Current;
                UI_TreeView component2 = transform.gameObject.GetComponent<UI_TreeView>();
                if (!(null == component2))
                {
                    if (!gameObject.activeSelf || component2.unused)
                    {
                        transform.gameObject.SetActive(false);
                    }
                    else if ((bool)minmax)
                    {
                        transform.gameObject.SetActive(minmax.isOn);
                    }
                    component2.UpdateView(ref totalPosY, parentPosY2);
                }
            }
        }
        finally
        {
            IDisposable disposable;
            if ((disposable = enumerator as IDisposable) != null)
            {
                disposable.Dispose();
            }
        }
        if (rootFlag && rtScroll)
        {
            RectTransform rectTransform = rtScroll;
            Vector2 sizeDelta2 = rtScroll.sizeDelta;
            rectTransform.sizeDelta = new Vector2(sizeDelta2.x, 0f - totalPosY + bottomMargin);
            if ((bool)scrollRect)
            {
                scrollRect.enabled = false;
                scrollRect.enabled = true;
            }
        }
    }

    public void ExpandAll()
    {
        if (rootFlag)
        {
            ChangeExpandOrCollapseLoop(true);
            float totalPosY = 0f;
            UpdateView(ref totalPosY, 0f);
        }
    }

    public void CollapseAll()
    {
        if (rootFlag)
        {
            ChangeExpandOrCollapseLoop(false);
            float totalPosY = 0f;
            UpdateView(ref totalPosY, 0f);
        }
    }

    private void ChangeExpandOrCollapseLoop(bool expand)
    {
        if ((bool)minmax)
        {
            minmax.isOn = expand;
        }
        IEnumerator enumerator = gameObject.transform.GetEnumerator();
        try
        {
            while (enumerator.MoveNext())
            {
                Transform transform = (Transform)enumerator.Current;
                UI_TreeView component = transform.gameObject.GetComponent<UI_TreeView>();
                if (!(null == component))
                {
                    component.ChangeExpandOrCollapseLoop(expand);
                }
            }
        }
        finally
        {
            IDisposable disposable;
            if ((disposable = enumerator as IDisposable) != null)
            {
                disposable.Dispose();
            }
        }
    }

    public void SetUnused(bool flag)
    {
        unused = flag;
        gameObject.SetActive(!unused);
    }

    private void Update()
    {
    }

    private void MinMaxChange(bool flag)
    {
        float totalPosY = 0f;
        if ((bool)tvRoot)
        {
            tvRoot.UpdateView(ref totalPosY, 0f);
        }
    }

    public void UpdateView()
    {
        float totalPosY = 0f;
        if (tvRoot && tvRoot.gameObject.activeSelf)
        {
            tvRoot.UpdateView(ref totalPosY, 0f);
        }
    }

    public UI_TreeView GetRoot()
    {
        return tvRoot;
    }
}
