using UnityEngine;
using UnityEngine.UI;

public class bl_MiniMapItem : MonoBehaviour
{
    [Tooltip("UI Prefab")]
    [Separator("TARGET")]
    public GameObject GraphicPrefab;

    [Tooltip("Transform to UI Icon will be follow")]
    public Transform Target;

    [Tooltip("Custom Position from target position")]
    public Vector3 OffSet = Vector3.zero;

    [Separator("ICON")]
    public Sprite Icon;

    public Sprite DeathIcon;

    public Color IconColor = new Color(1f, 1f, 1f, 0.9f);

    [Range(1f, 100f)]
    public float Size = 20f;

    [Separator("CIRCLE AREA")]
    public bool ShowCircleArea;

    [Range(1f, 100f)]
    public float CircleAreaRadius = 10f;

    public Color CircleAreaColor = new Color(1f, 1f, 1f, 0.9f);

    [Separator("ICON BUTTON")]
    [Tooltip("UI can interact when press it?")]
    [CustomToggle("is Interactable")]
    public bool isInteractable = true;

    [TextArea(2, 2)]
    public string InfoItem = "Info Icon here";

    [Separator("SETTINGS")]
    [Tooltip("Can Icon show when is off screen?")]
    [CustomToggle("Off Screen")]
    public bool OffScreen = true;

    [Range(0f, 5f)]
    public float BorderOffScreen = 0.01f;

    [Range(1f, 50f)]
    public float OffScreenSize = 10f;

    [Range(0f, 3f)]
    [Tooltip("Time before render/show item in minimap after instance")]
    public float RenderDelay = 0.3f;

    public ItemEffect m_Effect = ItemEffect.None;

    private Image Graphic;

    private RectTransform RectRoot;

    private GameObject cacheItem;

    private RectTransform CircleAreaRect;

    private Vector3 position;

    private bl_MiniMap MiniMap;

    private bl_MiniMap _minimap;

    public Vector3 TargetPosition
    {
        get
        {
            if (Target == null)
            {
                return Vector3.zero;
            }
            Vector3 vector = Target.position;
            float x = vector.x;
            Vector3 vector2 = Target.position;
            return new Vector3(x, 0f, vector2.z);
        }
    }

    private bl_MiniMap m_miniMap
    {
        get
        {
            if (_minimap == null)
            {
                _minimap = cacheItem.transform.root.GetComponentInChildren<bl_MiniMap>();
            }
            return _minimap;
        }
    }

    private void Start()
    {
        MiniMap = bl_MiniMapUtils.GetMiniMap();
        if (bl_MiniMap.MapUIRoot != null)
        {
            CreateIcon();
        }
    }

    private void CreateIcon()
    {
        cacheItem = Instantiate(GraphicPrefab);
        RectRoot = bl_MiniMapUtils.GetMiniMap().MMUIRoot;
        Graphic = cacheItem.GetComponent<Image>();
        if (Icon != null)
        {
            Graphic.sprite = Icon;
            Graphic.color = IconColor;
        }
        cacheItem.transform.SetParent(RectRoot.transform, false);
        Graphic.GetComponent<RectTransform>().anchoredPosition = Vector2.zero;
        cacheItem.GetComponent<CanvasGroup>().interactable = isInteractable;
        if (Target == null)
        {
            Target = GetComponent<Transform>();
        }
        StartEffect();
        bl_IconItem component = cacheItem.GetComponent<bl_IconItem>();
        component.DelayStart(RenderDelay);
        component.GetInfoItem(InfoItem);
        if (ShowCircleArea)
        {
            CircleAreaRect = component.SetCircleArea(CircleAreaRadius, CircleAreaColor);
        }
    }

    private void Update()
    {
        if (Target == null || Graphic == null)
        {
            return;
        }
        RectTransform component = Graphic.GetComponent<RectTransform>();
        Vector3 vector = TargetPosition + OffSet;
        Vector2 vector2 = bl_MiniMap.MiniMapCamera.WorldToViewportPoint(vector);
        float x = vector2.x;
        Vector2 sizeDelta = RectRoot.sizeDelta;
        float num = x * sizeDelta.x;
        Vector2 sizeDelta2 = RectRoot.sizeDelta;
        float x2 = num - sizeDelta2.x * 0.5f;
        float y = vector2.y;
        Vector2 sizeDelta3 = RectRoot.sizeDelta;
        float num2 = y * sizeDelta3.y;
        Vector2 sizeDelta4 = RectRoot.sizeDelta;
        position = new Vector2(x2, num2 - sizeDelta4.y * 0.5f);
        Vector2 anchoredPosition = position;
        if (OffScreen)
        {
            float x3 = position.x;
            Vector2 sizeDelta5 = RectRoot.sizeDelta;
            float min = 0f - (sizeDelta5.x * 0.5f - BorderOffScreen);
            Vector2 sizeDelta6 = RectRoot.sizeDelta;
            position.x = Mathf.Clamp(x3, min, sizeDelta6.x * 0.5f - BorderOffScreen);
            float y2 = position.y;
            Vector2 sizeDelta7 = RectRoot.sizeDelta;
            float min2 = 0f - (sizeDelta7.y * 0.5f - BorderOffScreen);
            Vector2 sizeDelta8 = RectRoot.sizeDelta;
            position.y = Mathf.Clamp(y2, min2, sizeDelta8.y * 0.5f - BorderOffScreen);
        }
        float size;
        if (m_miniMap.useCompassRotation)
        {
            Vector3 zero = Vector3.zero;
            Vector3 direction = Target.position - m_miniMap.TargetPosition;
            Vector3 vector3 = bl_MiniMap.MiniMapCamera.transform.InverseTransformDirection(direction);
            vector3.z = 0f;
            vector3 = vector3.normalized / 2f;
            float num3 = Mathf.Abs(position.x);
            float num4 = Mathf.Abs(0.5f + vector3.x * m_miniMap.CompassSize);
            if (num3 >= num4)
            {
                zero.x = 0.5f + vector3.x * m_miniMap.CompassSize;
                zero.y = 0.5f + vector3.y * m_miniMap.CompassSize;
                position = zero;
                size = OffScreenSize;
            }
            else
            {
                size = Size;
            }
        }
        else
        {
            float x4 = position.x;
            Vector2 sizeDelta9 = RectRoot.sizeDelta;
            if (x4 != sizeDelta9.x * 0.5f - BorderOffScreen)
            {
                float y3 = position.y;
                Vector2 sizeDelta10 = RectRoot.sizeDelta;
                if (y3 != sizeDelta10.y * 0.5f - BorderOffScreen)
                {
                    float x5 = position.x;
                    Vector2 sizeDelta11 = RectRoot.sizeDelta;
                    if (x5 != 0f - sizeDelta11.x * 0.5f - BorderOffScreen)
                    {
                        float num5 = 0f - position.y;
                        Vector2 sizeDelta12 = RectRoot.sizeDelta;
                        if (num5 != sizeDelta12.y * 0.5f - BorderOffScreen)
                        {
                            size = Size;
                            goto IL_038a;
                        }
                    }
                }
            }
            size = OffScreenSize;
        }
        goto IL_038a;
    IL_038a:
        component.anchoredPosition = position;
        if (CircleAreaRect != null)
        {
            CircleAreaRect.anchoredPosition = anchoredPosition;
        }
        float num6 = size * MiniMap.IconMultiplier;
        component.sizeDelta = Vector2.Lerp(component.sizeDelta, new Vector2(num6, num6), Time.deltaTime * 8f);
        if (MiniMap.RotationAlwaysFront)
        {
            Quaternion identity = Quaternion.identity;
            Quaternion rotation = Target.rotation;
            identity.x = rotation.x;
            component.localRotation = identity;
        }
        else
        {
            Vector3 eulerAngles = MiniMap.transform.eulerAngles;
            Vector3 zero2 = Vector3.zero;
            Vector3 eulerAngles2 = Target.rotation.eulerAngles;
            zero2.z = 0f - eulerAngles2.y + eulerAngles.y;
            Quaternion quaternion2 = component.rotation = Quaternion.Euler(zero2);
        }
    }

    private void StartEffect()
    {
        Animator component = Graphic.GetComponent<Animator>();
        if (m_Effect == ItemEffect.Pulsing)
        {
            component.SetInteger("Type", 2);
        }
        else if (m_Effect == ItemEffect.Fade)
        {
            component.SetInteger("Type", 1);
        }
    }

    public void DestroyItem(bool inmediate)
    {
        if (!(Graphic == null))
        {
            if (DeathIcon == null || inmediate)
            {
                Graphic.GetComponent<bl_IconItem>().DestroyIcon(inmediate);
            }
            else
            {
                Graphic.GetComponent<bl_IconItem>().DestroyIcon(inmediate, DeathIcon);
            }
        }
    }

    public void SetIcon(Sprite ico)
    {
        if (!(cacheItem == null))
        {
            cacheItem.GetComponent<bl_IconItem>().SetIcon(ico);
        }
    }

    public void SetCircleArea(float radius, Color AreaColor)
    {
        CircleAreaRect = cacheItem.GetComponent<bl_IconItem>().SetCircleArea(radius, AreaColor);
    }

    public void HideCircleArea()
    {
        cacheItem.GetComponent<bl_IconItem>().HideCircleArea();
        CircleAreaRect = null;
    }

    public void HideItem()
    {
        if (cacheItem != null)
        {
            cacheItem.SetActive(false);
        }
    }

    public void ShowItem()
    {
        if (cacheItem != null)
        {
            cacheItem.SetActive(true);
            cacheItem.GetComponent<bl_IconItem>().SetVisibleAlpha();
        }
    }
}
