namespace MessagePack.Formatters
{
    public class CharArrayFormatter : IMessagePackFormatter<char[]>
    {
        public static readonly CharArrayFormatter Instance = new CharArrayFormatter();

        private CharArrayFormatter()
        {
        }

        public int Serialize(ref byte[] bytes, int offset, char[] value, IFormatterResolver formatterResolver)
        {
            if (value == null)
            {
                return MessagePackBinary.WriteNil(ref bytes, offset);
            }
            int num = offset;
            offset += MessagePackBinary.WriteArrayHeader(ref bytes, offset, value.Length);
            for (int i = 0; i < value.Length; i++)
            {
                offset += MessagePackBinary.WriteChar(ref bytes, offset, value[i]);
            }
            return offset - num;
        }

        public char[] Deserialize(byte[] bytes, int offset, IFormatterResolver formatterResolver, out int readSize)
        {
            if (MessagePackBinary.IsNil(bytes, offset))
            {
                readSize = 1;
                return null;
            }
            int num = offset;
            int num2 = MessagePackBinary.ReadArrayHeader(bytes, offset, out readSize);
            offset += readSize;
            char[] array = new char[num2];
            for (int i = 0; i < array.Length; i++)
            {
                array[i] = MessagePackBinary.ReadChar(bytes, offset, out readSize);
                offset += readSize;
            }
            readSize = offset - num;
            return array;
        }
    }
}
