using MessagePack.Formatters;
using MessagePack.Unity;

namespace MessagePack.Resolvers
{
    public class StandardResolver : IFormatterResolver
    {
        private static class FormatterCache<T>
        {
            public static readonly IMessagePackFormatter<T> formatter;

            static FormatterCache()
            {
                IFormatterResolver[] resolvers = StandardResolver.resolvers;
                int num = 0;
                IMessagePackFormatter<T> messagePackFormatter;
                while (true)
                {
                    if (num < resolvers.Length)
                    {
                        IFormatterResolver formatterResolver = resolvers[num];
                        messagePackFormatter = formatterResolver.GetFormatter<T>();
                        if (messagePackFormatter != null)
                        {
                            break;
                        }
                        num++;
                        continue;
                    }
                    return;
                }
                formatter = messagePackFormatter;
            }
        }

        public static readonly IFormatterResolver Instance = new StandardResolver();

        private static readonly IFormatterResolver[] resolvers = new IFormatterResolver[7]
        {
            BuiltinResolver.Instance,
            UnityResolver.Instance,
            DynamicEnumResolver.Instance,
            DynamicGenericResolver.Instance,
            DynamicUnionResolver.Instance,
            DynamicObjectResolver.Instance,
            PrimitiveObjectResolver.Instance
        };

        private StandardResolver()
        {
        }

        public IMessagePackFormatter<T> GetFormatter<T>()
        {
            return FormatterCache<T>.formatter;
        }
    }
}
