using UnityEngine;

namespace RootMotion
{
    public static class QuaTools
    {
        public static Quaternion LinearBlend(Quaternion q, float weight)
        {
            if (weight <= 0f)
            {
                return Quaternion.identity;
            }
            if (weight >= 1f)
            {
                return q;
            }
            return Quaternion.Lerp(Quaternion.identity, q, weight);
        }

        public static Quaternion SphericalBlend(Quaternion q, float weight)
        {
            if (weight <= 0f)
            {
                return Quaternion.identity;
            }
            if (weight >= 1f)
            {
                return q;
            }
            return Quaternion.Slerp(Quaternion.identity, q, weight);
        }

        public static Quaternion FromToAroundAxis(Vector3 fromDirection, Vector3 toDirection, Vector3 axis)
        {
            Quaternion quaternion = Quaternion.FromToRotation(fromDirection, toDirection);
            Vector3 axis2;
            float angle;
            quaternion.ToAngleAxis(out angle, out axis2);
            float num = Vector3.Dot(axis2, axis);
            if (num < 0f)
            {
                angle = 0f - angle;
            }
            return Quaternion.AngleAxis(angle, axis);
        }

        public static Quaternion RotationToLocalSpace(Quaternion space, Quaternion rotation)
        {
            return Quaternion.Inverse(Quaternion.Inverse(space) * rotation);
        }

        public static Quaternion FromToRotation(Quaternion from, Quaternion to)
        {
            if (to == from)
            {
                return Quaternion.identity;
            }
            return to * Quaternion.Inverse(from);
        }

        public static Vector3 GetAxis(Vector3 v)
        {
            Vector3 vector = Vector3.right;
            bool flag = false;
            float num = Vector3.Dot(v, Vector3.right);
            float num2 = Mathf.Abs(num);
            if (num < 0f)
            {
                flag = true;
            }
            float num3 = Vector3.Dot(v, Vector3.up);
            float num4 = Mathf.Abs(num3);
            if (num4 > num2)
            {
                num2 = num4;
                vector = Vector3.up;
                flag = num3 < 0f;
            }
            float num5 = Vector3.Dot(v, Vector3.forward);
            num4 = Mathf.Abs(num5);
            if (num4 > num2)
            {
                vector = Vector3.forward;
                flag = num5 < 0f;
            }
            if (flag)
            {
                vector = -vector;
            }
            return vector;
        }
    }
}
