using System;

namespace UniRx.Operators
{
    internal class SampleFrameObservable<T> : OperatorObservableBase<T>
    {
        private class SampleFrame : OperatorObserverBase<T, T>
        {
            private class SampleFrameTick : IObserver<long>
            {
                private readonly SampleFrame parent;

                public SampleFrameTick(SampleFrame parent)
                {
                    this.parent = parent;
                }

                public void OnCompleted()
                {
                }

                public void OnError(Exception error)
                {
                }

                public void OnNext(long _)
                {
                    lock (parent.gate)
                    {
                        if (parent.isUpdated)
                        {
                            T latestValue = parent.latestValue;
                            parent.isUpdated = false;
                            parent.observer.OnNext(latestValue);
                        }
                        if (parent.isCompleted)
                        {
                            try
                            {
                                parent.observer.OnCompleted();
                            }
                            finally
                            {
                                parent.Dispose();
                            }
                        }
                    }
                }
            }

            private readonly SampleFrameObservable<T> parent;

            private readonly object gate = new object();

            private T latestValue = default(T);

            private bool isUpdated;

            private bool isCompleted;

            private SingleAssignmentDisposable sourceSubscription;

            public SampleFrame(SampleFrameObservable<T> parent, IObserver<T> observer, IDisposable cancel)
                : base(observer, cancel)
            {
                this.parent = parent;
            }

            public IDisposable Run()
            {
                sourceSubscription = new SingleAssignmentDisposable();
                sourceSubscription.Disposable = parent.source.Subscribe(this);
                IDisposable disposable = Observable.IntervalFrame(parent.frameCount, parent.frameCountType).Subscribe(new SampleFrameTick(this));
                return StableCompositeDisposable.Create(sourceSubscription, disposable);
            }

            private void OnNextTick(long _)
            {
                lock (gate)
                {
                    if (isUpdated)
                    {
                        T value = latestValue;
                        isUpdated = false;
                        observer.OnNext(value);
                    }
                    if (isCompleted)
                    {
                        try
                        {
                            observer.OnCompleted();
                        }
                        finally
                        {
                            Dispose();
                        }
                    }
                }
            }

            public override void OnNext(T value)
            {
                lock (gate)
                {
                    latestValue = value;
                    isUpdated = true;
                }
            }

            public override void OnError(Exception error)
            {
                lock (gate)
                {
                    try
                    {
                        observer.OnError(error);
                    }
                    finally
                    {
                        Dispose();
                    }
                }
            }

            public override void OnCompleted()
            {
                lock (gate)
                {
                    isCompleted = true;
                    sourceSubscription.Dispose();
                }
            }
        }

        private readonly IObservable<T> source;

        private readonly int frameCount;

        private readonly FrameCountType frameCountType;

        public SampleFrameObservable(IObservable<T> source, int frameCount, FrameCountType frameCountType)
            : base(source.IsRequiredSubscribeOnCurrentThread())
        {
            this.source = source;
            this.frameCount = frameCount;
            this.frameCountType = frameCountType;
        }

        protected override IDisposable SubscribeCore(IObserver<T> observer, IDisposable cancel)
        {
            return new SampleFrame(this, observer, cancel).Run();
        }
    }
}
