namespace UniRx
{
    public static class OptimizedObservableExtensions
    {
        public static bool IsRequiredSubscribeOnCurrentThread<T>(this IObservable<T> source)
        {
            IOptimizedObservable<T> optimizedObservable = source as IOptimizedObservable<T>;
            if (optimizedObservable == null)
            {
                return true;
            }
            return optimizedObservable.IsRequiredSubscribeOnCurrentThread();
        }

        public static bool IsRequiredSubscribeOnCurrentThread<T>(this IObservable<T> source, IScheduler scheduler)
        {
            if (scheduler == Scheduler.CurrentThread)
            {
                return true;
            }
            return source.IsRequiredSubscribeOnCurrentThread();
        }
    }
}
