using FBSAssist;
using Illusion.Extensions;
using IllusionUtility.GetUtility;
using IllusionUtility.SetUtility;
using Manager;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class ChaControl : ChaInfo
{
    public class MannequinBackInfo
    {
        public int coordinateType;
        public byte[] custom;
        public int eyesPtn;
        public int mouthPtn;
        public float mouthOpen;
        public bool mouthFixed;
        public int neckLook;
        public byte[] eyesInfo;
        public bool mannequin;

        public void Backup(ChaControl chaCtrl)
        {
            coordinateType = chaCtrl.fileStatus.coordinateType;
            custom = chaCtrl.chaFile.GetCustomBytes();
            eyesPtn = chaCtrl.fileStatus.eyesPtn;
            mouthPtn = chaCtrl.fileStatus.mouthPtn;
            mouthOpen = chaCtrl.fileStatus.mouthOpenMax;
            mouthFixed = chaCtrl.fileStatus.mouthFixed;
            neckLook = chaCtrl.fileStatus.neckLookPtn;
        }

        public void Restore(ChaControl chaCtrl)
        {
            chaCtrl.chaFile.SetCustomBytes(custom, ChaFileDefine.ChaFileCustomVersion);
            chaCtrl.Reload(true);
            chaCtrl.ChangeEyesPtn(eyesPtn, false);
            chaCtrl.ChangeMouthPtn(mouthPtn, false);
            chaCtrl.fileStatus.mouthFixed = mouthFixed;
            chaCtrl.ChangeMouthOpenMax(mouthOpen);
            chaCtrl.ChangeLookNeckPtn(neckLook);
            chaCtrl.neckLookCtrl.ForceLateUpdate();
            chaCtrl.eyeLookCtrl.ForceLateUpdate();
            chaCtrl.neckLookCtrl.neckLookScript.skipCalc = false;
            chaCtrl.resetDynamicBoneAll = true;
            chaCtrl.LateUpdateForce();
        }
    }

    public enum BustNormalKind
    {
        NmlBody,
        NmlTop,
        NmlBra,
        NmlTopPA
    }

    public enum FaceTexKind
    {
        inpBase,
        inpSub,
        inpPaint01,
        inpPaint02,
        inpCheek,
        inpLipLine,
        inpMole
    }

    public enum BodyTexKind
    {
        inpBase,
        inpSub,
        inpPaint01,
        inpPaint02,
        inpSunburn,
        inpNail
    }

    private MannequinBackInfo mannequinBackInfo = new MannequinBackInfo();
    private const string deleteHeadBoneName = "cf_J_N_FaceRoot";
    private const string deleteBodyBoneName = "cf_j_root";
    private readonly Bounds bounds = new Bounds(new Vector3(0f, -0.2f, 0f), new Vector3(2f, 2f, 2f));
    private AssignedAnotherWeights aaWeightsHead;
    private AssignedAnotherWeights aaWeightsBody;

    private Texture texBodyAlphaMask { get; set; }
    private Texture texBraAlphaMask { get; set; }
    public Texture texInnerAlphaMask { get; set; }
    public bool hideShortsBot { get; set; }
    public Dictionary<BustNormalKind, BustNormal> dictBustNormal { get; private set; }
    private byte[] siruNewLv { get; set; }
    public ChaFileCoordinate nowCoordinate { get; private set; }
    public bool notBra { get; private set; }
    public bool notBot { get; private set; }
    public bool notShorts { get; private set; }
    public bool[] hideHairAcs { get; private set; }
    public bool isChangeOfClothesRandom { get; private set; }
    public Dictionary<int, Dictionary<byte, string>> dictStateType { get; private set; }
    private int ShapeFaceNum { get; set; }
    private int ShapeBodyNum { get; set; }
    private ShapeInfoBase sibFace { get; set; }
    public ShapeInfoBase sibBody { get; set; }
    private bool changeShapeBodyMask { get; set; }
    public BustSoft bustSoft { get; private set; }
    public BustGravity bustGravity { get; private set; }
    public bool[] updateCMFaceTex { get; private set; }
    public bool[] updateCMFaceColor { get; private set; }
    public bool[] updateCMFaceLayout { get; private set; }
    public bool[] updateCMBodyTex { get; private set; }
    public bool[] updateCMBodyColor { get; private set; }
    public bool[] updateCMBodyLayout { get; private set; }
    public AudioSource asVoice { get; private set; }
    private AudioAssist fbsaaVoice { get; set; }
    private Texture texHairGloss { get; set; }
    public ShapeInfoBase sibHand { get; set; }
    private bool updateAlphaMask { get; set; }

    public byte tearsLv
    {
        get
        {
            return fileStatus.tearsLv;
        }
        set
        {
            fileStatus.tearsLv = value;
        }
    }

    public bool enableExpression
    {
        get
        {
            return !(null == expression) && expression.enable;
        }
        set
        {
            if (null != expression)
            {
                expression.enable = value;
            }
        }
    }

    public void Initialize(byte _sex, int _exType, bool _hiPoly, GameObject _objRoot, int _id, int _no, ChaFileControl _chaFile = null)
    {
        if (_chaFile != null)
        {
            if (_sex != _chaFile.parameter.sex)
            {
            }
            if (_exType == _chaFile.parameter.exType)
            {
            }
        }
        MemberInitializeAll();
        InitializeControlLoadAll();
        InitializeControlFaceAll();
        InitializeControlBodyAll();
        InitializeControlCoordinateAll();
        InitializeControlCustomAll();
        InitializeControlHandAll();
        objRoot = _objRoot;
        chaID = _id;
        loadNo = _no;
        hiPoly = _hiPoly;
        hideMoz = true;
        lstCtrl = Singleton<Character>.Instance.chaListCtrl;
        if (_chaFile == null)
        {
            chaFile = new ChaFileControl();
            LoadPreset(_sex, _exType, string.Empty);
        }
        else
        {
            chaFile = _chaFile;
        }
        chaFile.parameter.sex = _sex;
        chaFile.parameter.exType = chaFile.parameter.sex == 0 ? _exType : 0;
        InitBaseCustomTextureBody();
        InitBaseCustomTextureEtc();
        if (_chaFile == null)
            ChangeCoordinateType(ChaFileDefine.CoordinateType.School01);
        else
            ChangeCoordinateType(false);
        if (_sex == 0)
            chaFile.custom.body.shapeValueBody[0] = exType != 0 ? 1f : 0.6f;
        if (Game.IsStudio)
        {
            chaFile.status.visibleSonAlways = true;
            chaFile.status.visibleSon = true;
        }
        else
        {
            if (_sex == 1)
                chaFile.status.visibleSonAlways = false;
            else
                chaFile.status.visibleSon = false;
        }
    }

    public void ReleaseAll()
    {
        ReleaseControlLoadAll();
        ReleaseControlFaceAll();
        ReleaseControlBodyAll();
        ReleaseControlCoordinateAll();
        ReleaseControlCustomAll();
        ReleaseControlHandAll();
        ReleaseInfoAll();
    }

    public void ReleaseObject()
    {
        ReleaseControlLoadObject();
        ReleaseControlFaceObject();
        ReleaseControlBodyObject();
        ReleaseControlCoordinateObject();
        ReleaseControlCustomObject();
        ReleaseControlHandObject();
        ReleaseInfoObject();
        if (Singleton<Character>.Instance.enableCharaLoadGCClear)
        {
            Resources.UnloadUnusedAssets();
            GC.Collect();
        }
    }

    public void LoadPreset(int _sex, int _exType, string presetName = "")
    {
        string empty = sex == 0 && _exType == 1 ? string.Format("ill_Default_Male_Ex") : !presetName.IsNullOrEmpty() ? presetName : _sex != 0 ? "ill_Default_Female" : "ill_Default_Male";
        Dictionary<int, ListInfoBase> dictionary = _exType != 0 ? lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_sample_m_ex) : _sex != 0 ? lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_sample_f) : lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_sample_m);
        foreach (KeyValuePair<int, ListInfoBase> item in dictionary)
        {
            if (item.Value.GetInfo(ChaListDefine.KeyType.MainData) == empty)
            {
                chaFile.LoadFromAssetBundle(item.Value.GetInfo(ChaListDefine.KeyType.MainAB), empty);
                break;
            }
        }
    }

    public bool AssignDefaultCoordinate()
    {
        return AssignDefaultCoordinate(chaFile);
    }

    public static bool AssignDefaultCoordinate(ChaFileControl chaFile)
    {
        ChaListControl chaListCtrl = Singleton<Character>.Instance.chaListCtrl;
        Dictionary<int, ListInfoBase> dictionary = chaFile.parameter.sex != 0 ? chaListCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_default_cos_f) : chaListCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_default_cos_m);
        if (dictionary == null)
        {
            return false;
        }
        int count = dictionary.Keys.Count;
        if (count == 0)
        {
            return false;
        }
        int index = UnityEngine.Random.Range(0, count - 1);
        int key = dictionary.Keys.ToList()[index];
        ListInfoBase value;
        if (!dictionary.TryGetValue(key, out value))
        {
            return false;
        }
        string info = value.GetInfo(ChaListDefine.KeyType.MainManifest);
        string info2 = value.GetInfo(ChaListDefine.KeyType.MainAB);
        ChaListDefine.KeyType[] array = new ChaListDefine.KeyType[7]
        {
            ChaListDefine.KeyType.DefCoorde01,
            ChaListDefine.KeyType.DefCoorde02,
            ChaListDefine.KeyType.DefCoorde03,
            ChaListDefine.KeyType.DefCoorde04,
            ChaListDefine.KeyType.DefCoorde05,
            ChaListDefine.KeyType.DefCoorde06,
            ChaListDefine.KeyType.DefCoorde07
        };
        for (int i = 0; i < array.Length; i++)
        {
            string info3 = value.GetInfo(array[i]);
            TextAsset ta = CommonLib.LoadAsset<TextAsset>(info2, info3, false, info);
            chaFile.coordinate[i].LoadFile(ta);
        }
        return true;
    }

    public static ChaFileControl[] GetRandomFemaleCard(int num)
    {
        FolderAssist folderAssist = new FolderAssist();
        string[] searchPattern = new string[1] { "*.png" };
        string folder = UserData.Path + "chara/female/";
        folderAssist.CreateFolderInfoEx(folder, searchPattern);
        List<string> list = (from n in folderAssist.lstFile.Shuffle()
                             select n.FullPath).ToList();
        int num2 = Mathf.Min(list.Count, num);
        if (num2 == 0)
            return null;
        List<ChaFileControl> list2 = new List<ChaFileControl>();
        for (int i = 0; i < num2; i++)
        {
            ChaFileControl chaFileControl = new ChaFileControl();
            if (chaFileControl.LoadCharaFile(list[i], 1, true) && chaFileControl.parameter.sex != 0)
                list2.Add(chaFileControl);
        }
        return list2.ToArray();
    }

    public void SetActiveTop(bool active)
    {
        if ((bool)objTop)
        {
            objTop.SetActiveIfDifferent(active);
        }
    }

    public bool GetActiveTop()
    {
        if ((bool)objTop)
        {
            return objTop.activeSelf;
        }
        return false;
    }

    public void SetPosition(float x, float y, float z)
    {
        if (null != objTop)
            objTop.transform.localPosition = new Vector3(x, y, z);
    }

    public void SetPosition(Vector3 pos)
    {
        if (null != objTop)
            objTop.transform.localPosition = pos;
    }

    public Vector3 GetPosition()
    {
        return !(null == objTop) ? objTop.transform.localPosition : Vector3.zero;
    }

    public void SetRotation(float x, float y, float z)
    {
        if (null != objTop)
            objTop.transform.localRotation = Quaternion.Euler(x, y, z);
    }

    public void SetRotation(Vector3 rot)
    {
        if (null != objTop)
            objTop.transform.localRotation = Quaternion.Euler(rot);
    }

    public Vector3 GetRotation()
    {
        return !(null == objTop) ? objTop.transform.localRotation.eulerAngles : Vector3.zero;
    }

    public void SetTransform(Transform trf)
    {
        if (null != objTop)
        {
            objTop.transform.localPosition = trf.localPosition;
            objTop.transform.localRotation = trf.localRotation;
            objTop.transform.localScale = trf.localScale;
        }
    }

    public void ChangeSettingMannequin(bool mannequin)
    {
        if (mannequin)
        {
            if (!mannequinBackInfo.mannequin)
            {
                mannequinBackInfo.mannequin = true;
                mannequinBackInfo.Backup(this);
                string assetBundleName = "chara/oo_base.unity3d";
                string assetName = "mannequin";
                chaFile.LoadFileLimited(assetBundleName, assetName, true, true, true, false, false);
                Reload(true);
                ChangeEyesPtn(21, false);
                ChangeMouthPtn(0, false);
                fileStatus.mouthFixed = true;
                ChangeMouthOpenMax(0f);
                ChangeLookNeckPtn(3);
                neckLookCtrl.neckLookScript.skipCalc = true;
                neckLookCtrl.ForceLateUpdate();
                eyeLookCtrl.ForceLateUpdate();
                resetDynamicBoneAll = true;
                LateUpdateForce();
                GameObject referenceInfo = GetReferenceInfo(RefObjKey.ObjEyebrow);
                if ((bool)referenceInfo)
                {
                    referenceInfo.SetActiveIfDifferent(false);
                }
                GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.ObjNoseline);
                if ((bool)referenceInfo2)
                {
                    referenceInfo2.SetActiveIfDifferent(false);
                }
                GameObject referenceInfo3 = GetReferenceInfo(RefObjKey.N_FaceSpecial);
                if ((bool)referenceInfo3)
                {
                    referenceInfo3.SetActiveIfDifferent(false);
                }
            }
        }
        else if (mannequinBackInfo.mannequin)
        {
            mannequinBackInfo.Restore(this);
            GameObject referenceInfo4 = GetReferenceInfo(RefObjKey.ObjEyebrow);
            if ((bool)referenceInfo4)
            {
                referenceInfo4.SetActiveIfDifferent(true);
            }
            GameObject referenceInfo5 = GetReferenceInfo(RefObjKey.ObjNoseline);
            if ((bool)referenceInfo5)
            {
                referenceInfo5.SetActiveIfDifferent(true);
            }
            GameObject referenceInfo6 = GetReferenceInfo(RefObjKey.N_FaceSpecial);
            if ((bool)referenceInfo6)
            {
                referenceInfo6.SetActiveIfDifferent(true);
            }
            mannequinBackInfo.mannequin = false;
        }
    }

    public void RestoreMannequinHair()
    {
        ChaFileControl chaFileControl = new ChaFileControl();
        chaFileControl.SetCustomBytes(mannequinBackInfo.custom, ChaFileDefine.ChaFileCustomVersion);
        fileCustom.hair = chaFileControl.custom.hair;
        Reload(true, true, false, true);
    }

    public void OnDestroy()
    {
        if (Singleton<Character>.IsInstance())
            Singleton<Character>.Instance.DeleteChara(this, true);
        ReleaseAll();
    }

    public void UpdateForce()
    {
        if (loadEnd)
        {
            UpdateBlendShapeVoice();
            if (hiPoly)
                UpdateSiru();
        }
    }

    public void LateUpdateForce()
    {
        if (!loadEnd) return;

        UpdateVisible();
        if (resetDynamicBoneAll)
        {
            ResetDynamicBoneAll();
            resetDynamicBoneAll = false;
        }
        if (updateShapeBody)
        {
            UpdateShapeBody();
            updateShapeBody = false;
        }
        if (updateShapeFace)
        {
            UpdateShapeFace();
            updateShapeFace = false;
        }
        UpdateAlwaysShapeBody();
        UpdateAlwaysShapeHand();
        GameObject referenceInfo = GetReferenceInfo(RefObjKey.S_ANA);
        GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.a_n_ana);
        if (null != referenceInfo && null != referenceInfo2)
        {
            Vector3 localScale = referenceInfo.transform.localScale;
            float x = 1f / localScale.x;
            Vector3 localScale2 = referenceInfo.transform.localScale;
            float y = 1f / localScale2.y;
            Vector3 localScale3 = referenceInfo.transform.localScale;
            float z = 1f / localScale3.z;
            referenceInfo2.transform.localScale = new Vector3(x, y, z);
        }
        if (hiPoly && sex == 1)
        {
            float x2 = !Singleton<Character>.Instance.enableCorrectArmSize ? 1f : 0.92f;
            float y2 = !Singleton<Character>.Instance.enableCorrectHandSize ? 1f : 0.9f;
            float z2 = !Singleton<Character>.Instance.enableCorrectHandSize ? 1f : 0.9f;
            GameObject referenceInfo3 = GetReferenceInfo(RefObjKey.CORRECT_ARM_L);
            if ((bool)referenceInfo3)
            {
                referenceInfo3.transform.localScale = new Vector3(x2, 1f, 1f);
            }
            GameObject referenceInfo4 = GetReferenceInfo(RefObjKey.CORRECT_ARM_R);
            if ((bool)referenceInfo4)
            {
                referenceInfo4.transform.localScale = new Vector3(x2, 1f, 1f);
            }
            GameObject referenceInfo5 = GetReferenceInfo(RefObjKey.CORRECT_HAND_L);
            if ((bool)referenceInfo5)
            {
                referenceInfo5.transform.localScale = new Vector3(1f, y2, z2);
            }
            GameObject referenceInfo6 = GetReferenceInfo(RefObjKey.CORRECT_HAND_R);
            if ((bool)referenceInfo6)
            {
                referenceInfo6.transform.localScale = new Vector3(1f, y2, z2);
            }
        }
        if (reSetupDynamicBoneBust)
        {
            ReSetupDynamicBoneBust();
            UpdateBustSoftnessAndGravity();
            reSetupDynamicBoneBust = false;
        }
        if (updateBustSize)
        {
            int num = 4;
            float rate = 1f;
            if (0.5f > chaFile.custom.body.shapeValueBody[num])
            {
                rate = Mathf.InverseLerp(0f, 0.5f, chaFile.custom.body.shapeValueBody[num]);
            }
            foreach (KeyValuePair<BustNormalKind, BustNormal> item in dictBustNormal)
            {
                item.Value.Blend(rate);
            }
            updateBustSize = false;
        }
        if (!hiPoly)
        {
            RefObjKey[] array = new RefObjKey[2]
            {
                RefObjKey.a_n_nip_L,
                RefObjKey.a_n_nip_R
            };
            RefObjKey[] array2 = array;
            foreach (RefObjKey key in array2)
            {
                GameObject referenceInfo7 = GetReferenceInfo(key);
                if (!(null == referenceInfo7))
                {
                    Vector3 lossyScale = referenceInfo7.transform.parent.lossyScale;
                    Vector3 one = Vector3.one;
                    if (lossyScale.x != 0f)
                        one.x = 1f / lossyScale.x;
                    if (lossyScale.y != 0f)
                        one.y = 1f / lossyScale.y;
                    if (lossyScale.z != 0f)
                        one.z = 1f / lossyScale.z;
                    referenceInfo7.transform.localScale = one;
                }
            }
        }
        if (sex == 1 && (fileFace.headId == 200 || fileFace.headId == 201))
        {
            if (fileStatus.mouthPtn == 21)
            {
                GameObject referenceInfo8 = GetReferenceInfo(RefObjKey.CORRECT_TONGUE_TOP);
                if ((bool)referenceInfo8)
                {
                    Transform transform = referenceInfo8.transform;
                    Vector3 localPosition = referenceInfo8.transform.localPosition;
                    float x3 = localPosition.x;
                    Vector3 localPosition2 = referenceInfo8.transform.localPosition;
                    float y3 = localPosition2.y + 0.007f;
                    Vector3 localPosition3 = referenceInfo8.transform.localPosition;
                    transform.localPosition = new Vector3(x3, y3, localPosition3.z);
                    referenceInfo8.transform.localScale = new Vector3(0.92f, 1f, 1.05f);
                }
            }
            else if (fileStatus.mouthPtn == 22)
            {
                GameObject referenceInfo9 = GetReferenceInfo(RefObjKey.CORRECT_TONGUE_TOP);
                if ((bool)referenceInfo9)
                {
                    Transform transform2 = referenceInfo9.transform;
                    Vector3 localPosition4 = referenceInfo9.transform.localPosition;
                    float x4 = localPosition4.x;
                    Vector3 localPosition5 = referenceInfo9.transform.localPosition;
                    float y4 = localPosition5.y + 0.007f;
                    Vector3 localPosition6 = referenceInfo9.transform.localPosition;
                    transform2.localPosition = new Vector3(x4, y4, localPosition6.z);
                    referenceInfo9.transform.localScale = new Vector3(0.92f, 1f, 1.05f);
                }
                GameObject referenceInfo10 = GetReferenceInfo(RefObjKey.CORRECT_MOUTH_TARGET);
                if ((bool)referenceInfo10)
                {
                    Transform transform3 = referenceInfo10.transform;
                    Vector3 localPosition7 = referenceInfo10.transform.localPosition;
                    float x5 = localPosition7.x;
                    Vector3 localPosition8 = referenceInfo10.transform.localPosition;
                    float y5 = localPosition8.y + 0.007f;
                    Vector3 localPosition9 = referenceInfo10.transform.localPosition;
                    transform3.localPosition = new Vector3(x5, y5, localPosition9.z);
                }
                GameObject referenceInfo11 = GetReferenceInfo(RefObjKey.CORRECT_MOUTH_TARGET02);
                if ((bool)referenceInfo11)
                {
                    Transform transform4 = referenceInfo11.transform;
                    Vector3 localPosition10 = referenceInfo11.transform.localPosition;
                    float x6 = localPosition10.x;
                    Vector3 localPosition11 = referenceInfo11.transform.localPosition;
                    float y6 = localPosition11.y + 0.007f;
                    Vector3 localPosition12 = referenceInfo11.transform.localPosition;
                    transform4.localPosition = new Vector3(x6, y6, localPosition12.z);
                }
            }
        }
        SetForegroundEyesAndEyebrow();
    }

    public RuntimeAnimatorController LoadAnimation(string assetBundleName, string assetName, string manifestName = "")
    {
        if (null == animBody)
            return null;
        RuntimeAnimatorController runtimeAnimatorController = CommonLib.LoadAsset<RuntimeAnimatorController>(assetBundleName, assetName, false, manifestName);
        if (null == runtimeAnimatorController)
            return null;
        animBody.runtimeAnimatorController = runtimeAnimatorController;
        AssetBundleManager.UnloadAssetBundle(assetBundleName, true);
        return runtimeAnimatorController;
    }

    public void AnimPlay(string stateName)
    {
        if (!(null == animBody))
            animBody.Play(stateName);
    }

    public AnimatorStateInfo getAnimatorStateInfo(int _nLayer)
    {
        if (null == animBody || null == animBody.runtimeAnimatorController)
            return default(AnimatorStateInfo);
        return animBody.GetCurrentAnimatorStateInfo(_nLayer);
    }

    public bool syncPlay(AnimatorStateInfo _syncState, int _nLayer)
    {
        if (null == animBody)
            return false;
        animBody.Play(_syncState.shortNameHash, _nLayer, _syncState.normalizedTime);
        return true;
    }

    public bool syncPlay(int _nameHash, int _nLayer, float _fnormalizedTime)
    {
        if (null == animBody)
            return false;
        animBody.Play(_nameHash, _nLayer, _fnormalizedTime);
        return true;
    }

    public bool syncPlay(string _strameHash, int _nLayer, float _fnormalizedTime)
    {
        if (null == animBody)
            return false;
        animBody.Play(_strameHash, _nLayer, _fnormalizedTime);
        return true;
    }

    public bool setLayerWeight(float _fWeight, int _nLayer)
    {
        if (null == animBody)
            return false;
        animBody.SetLayerWeight(_nLayer, _fWeight);
        return true;
    }

    public bool setAllLayerWeight(float _fWeight)
    {
        if (null == animBody)
            return false;
        for (int i = 1; i < animBody.layerCount; i++)
            animBody.SetLayerWeight(i, _fWeight);
        return true;
    }

    public float getLayerWeight(int _nLayer)
    {
        if (null == animBody)
            return 0f;
        return animBody.GetLayerWeight(_nLayer);
    }

    public bool setPlay(string _strAnmName, int _nLayer)
    {
        if (null == animBody)
            return false;
        animBody.Play(_strAnmName, _nLayer);
        return true;
    }

    public void setAnimatorParamTrigger(string _strAnmName)
    {
        if (!(null == animBody))
            animBody.SetTrigger(_strAnmName);
    }

    public void setAnimatorParamResetTrigger(string _strAnmName)
    {
        if (!(null == animBody))
            animBody.ResetTrigger(_strAnmName);
    }

    public void setAnimatorParamBool(string _strAnmName, bool _bFlag)
    {
        if (!(null == animBody))
            animBody.SetBool(_strAnmName, _bFlag);
    }

    public bool getAnimatorParamBool(string _strAnmName)
    {
        if (null == animBody)
            return false;
        return animBody.GetBool(_strAnmName);
    }

    public void setAnimatorParamFloat(string _strAnmName, float _fValue)
    {
        if (animBody != null)
            animBody.SetFloat(_strAnmName, _fValue);
    }

    public void setAnimPtnCrossFade(string _strAnmName, float _fBlendTime, int _nLayer, float _fCrossStateTime)
    {
        if (!(null == animBody))
            animBody.CrossFade(_strAnmName, _fBlendTime, _nLayer, _fCrossStateTime);
    }

    public bool isBlend(int _nLayer)
    {
        if (null == animBody)
            return false;
        return animBody.IsInTransition(_nLayer);
    }

    public bool IsNextHash(int _nLayer, string _nameHash)
    {
        if (null == animBody)
            return false;
        return animBody.GetNextAnimatorStateInfo(_nLayer).IsName(_nameHash);
    }

    protected void InitializeControlBodyAll()
    {
        siruNewLv = new byte[Enum.GetNames(typeof(ChaFileDefine.SiruParts)).Length];
        for (int i = 0; i < siruNewLv.Length; i++)
            siruNewLv[i] = 0;
        InitializeControlBodyObject();
    }

    protected void InitializeControlBodyObject()
    {
        dictBustNormal = new Dictionary<BustNormalKind, BustNormal>();
        texBodyAlphaMask = null;
        texBraAlphaMask = null;
        texInnerAlphaMask = null;
        hideShortsBot = false;
    }

    protected void ReleaseControlBodyAll()
    {
        ReleaseControlBodyObject(false);
    }

    protected void ReleaseControlBodyObject(bool init = true)
    {
        if (dictBustNormal != null)
        {
            foreach (KeyValuePair<BustNormalKind, BustNormal> item in dictBustNormal)
                item.Value.Release();
            dictBustNormal.Clear();
        }
        for (int i = 0; i < siruNewLv.Length; i++)
            siruNewLv[i] = 0;
        Resources.UnloadUnusedAssets();
        if (init)
            InitializeControlBodyObject();
    }

    public int GetHeightCategory()
    {
        float shapeBodyValue = GetShapeBodyValue(0);
        if (0.25f > shapeBodyValue)
            return 0;
        if (0.6f < shapeBodyValue)
            return 2;
        return 1;
    }

    public int GetWaistCategory()
    {
        float num = (GetShapeBodyValue(ChaFileDefine.cf_CategoryWaist[0]) + GetShapeBodyValue(ChaFileDefine.cf_CategoryWaist[1]) + GetShapeBodyValue(ChaFileDefine.cf_CategoryWaist[2])) / 3f;
        if (0.35f > num)
            return 0;
        if (0.65f < num)
            return 2;
        return 1;
    }

    public int GetBustCategory()
    {
        float shapeBodyValue = GetShapeBodyValue(4);
        if (0.4f > shapeBodyValue)
            return 0;
        if (0.7f < shapeBodyValue)
            return 2;
        return 1;
    }

    public void ResetDynamicBoneAll()
    {
        DynamicBone[] componentsInChildren = objTop.GetComponentsInChildren<DynamicBone>(true);
        DynamicBone[] array = componentsInChildren;
        foreach (DynamicBone dynamicBone in array)
            dynamicBone.ResetParticlesPosition();
    }

    public DynamicBone_Ver02 getDynamicBoneBust(DynamicBoneKind _eArea)
    {
        DynamicBone_Ver02 value;
        dictDynamicBoneBust.TryGetValue(_eArea, out value);
        return value;
    }

    public bool InitDynamicBoneBust()
    {
        foreach (KeyValuePair<DynamicBoneKind, DynamicBone_Ver02> item in dictDynamicBoneBust)
            if (null != item.Value)
                item.Value.ResetParticlesPosition();
        return true;
    }

    public bool ReSetupDynamicBoneBust(int _nArea = 0)
    {
        DynamicBone_Ver02 value;
        if (_nArea == 0)
        {
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.BreastL, out value))
                value.ResetPosition();
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.BreastR, out value))
                value.ResetPosition();
        }
        else
        {
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.HipL, out value))
                value.ResetPosition();
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.HipR, out value))
                value.ResetPosition();
        }
        return true;
    }

    public bool playDynamicBoneBust(int _nArea, bool _bPlay)
    {
        DynamicBone_Ver02 value;
        if (_nArea == 0)
        {
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.BreastL, out value))
                value.enabled = _bPlay;
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.BreastR, out value))
                value.enabled = _bPlay;
        }
        else
        {
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.HipL, out value))
                value.enabled = _bPlay;
            if (dictDynamicBoneBust.TryGetValue(DynamicBoneKind.HipR, out value))
                value.enabled = _bPlay;
        }
        return true;
    }

    public bool playDynamicBoneBust(DynamicBoneKind _eArea, bool _bPlay)
    {
        DynamicBone_Ver02 value;
        if (dictDynamicBoneBust.TryGetValue(_eArea, out value))
            value.enabled = _bPlay;
        return true;
    }

    public bool ChangeNipRate(float rate)
    {
        chaFile.status.nipStandRate = rate;
        changeShapeBodyMask = true;
        updateShapeBody = true;
        return true;
    }

    public void ChangeHitBustBlendShapeValue(byte typeSex)
    {
        if (null != hitBodyCtrlCmp)
            hitBodyCtrlCmp.ChangeBustBlendShapeValue(typeSex != 0 ? 0f : 90f);
    }

    public void EnableExpressionIndex(int indexNo, bool enable)
    {
        if (null != expression)
            expression.EnableIndex((ExpressionBone.ExpressionIndex)indexNo, enable);
    }

    public void EnableExpressionCategory(int categoryNo, bool enable)
    {
        if (null != expression)
            expression.EnableCategory((ExpressionBone.ExpressionCategory)categoryNo, enable);
    }

    public void SetSiruFlags(ChaFileDefine.SiruParts parts, byte lv)
    {
        siruNewLv[(int)parts] = lv;
    }

    public byte GetSiruFlags(ChaFileDefine.SiruParts parts)
    {
        return chaFile.status.siruLv[(int)parts];
    }

#if FIX
    //Reuse this array to reduce GC allocs
    private static readonly ChaFileDefine.SiruParts[] _siruParts = {
        ChaFileDefine.SiruParts.SiruFrontUp,
        ChaFileDefine.SiruParts.SiruFrontDown,
        ChaFileDefine.SiruParts.SiruBackUp,
        ChaFileDefine.SiruParts.SiruBackDown
    };
#endif

    private bool UpdateSiru(bool forceChange = false)
    {
        if (!hiPoly)
            return true;

        if (customMatFace != null)
        {
            int num = 0;
            if (forceChange || fileStatus.siruLv[num] != siruNewLv[num])
            {
                fileStatus.siruLv[num] = siruNewLv[num];
                customMatFace.SetFloat(ChaShader._liquidface, fileStatus.siruLv[num]);
            }
        }
        if (!(null == customMatBody))
        {
#if FIX
            //Array moved outside of method and reused
#else
            ChaFileDefine.SiruParts[] array = new ChaFileDefine.SiruParts[4]
            {
                ChaFileDefine.SiruParts.SiruFrontUp,
                ChaFileDefine.SiruParts.SiruFrontDown,
                ChaFileDefine.SiruParts.SiruBackUp,
                ChaFileDefine.SiruParts.SiruBackDown
            };
            List<string> list = new List<string>();
#endif
            bool flag = false;
            for (int i = 0; i < _siruParts.Length; i++)
            {
                if (forceChange || fileStatus.siruLv[(int)_siruParts[i]] != siruNewLv[(int)_siruParts[i]])
                    flag = true;
#if FIX
                //Pointlessly checking something and adding it to a list which does nothing
#else
                if (siruNewLv[(int)_siruParts[i]] != 0)
                {
                    string item = _siruParts[i].ToString() + siruNewLv[(int)_siruParts[i]].ToString("00");
                    list.Add(item);
                }
#endif
                fileStatus.siruLv[(int)_siruParts[i]] = siruNewLv[(int)_siruParts[i]];
            }
            if (flag)
            {
                float[] array2 = new float[4]
                {
                    fileStatus.siruLv[(int)_siruParts[0]],
                    fileStatus.siruLv[(int)_siruParts[1]],
                    fileStatus.siruLv[(int)_siruParts[2]],
                    fileStatus.siruLv[(int)_siruParts[3]]
                };
                customMatBody.SetFloat(ChaShader._liquidftop, array2[0]);
                customMatBody.SetFloat(ChaShader._liquidfbot, array2[1]);
                customMatBody.SetFloat(ChaShader._liquidbtop, array2[2]);
                customMatBody.SetFloat(ChaShader._liquidbbot, array2[3]);
                int[] array3 = new int[5] { 0, 1, 2, 3, 5 };
                for (int j = 0; j < array3.Length; j++)
                    UpdateClothesSiru(j, array2[0], array2[1], array2[2], array2[3]);
            }
        }
        return true;
    }

#if FIX
    //Rewrote the method to use a fixed number of parameters instead of an array which creates garbage
    public void ChangeAlphaMask(byte state1, byte state2)
    {
        if ((bool)customMatBody)
        {
            customMatBody.SetFloat(ChaShader._alpha_a, state1);
            customMatBody.SetFloat(ChaShader._alpha_b, state2);
        }
        if (rendBra != null)
        {
            for (int i = 0; i < 2; i++)
            {
                if ((bool)rendBra[i])
                {
                    Material material = rendBra[i].material;
                    if ((bool)material)
                    {
                        material.SetFloat(ChaShader._alpha_a, state1);
                        material.SetFloat(ChaShader._alpha_b, state2);
                    }
                }
            }
        }

        if (rendInner == null) return;

        for (int j = 0; j < 2; j++)
        {
            if ((bool)rendInner[j])
            {
                Material material2 = rendInner[j].material;
                if ((bool)material2)
                {
                    material2.SetFloat(ChaShader._alpha_a, state1);
                    material2.SetFloat(ChaShader._alpha_b, state2);
                }
            }
        }
    }
#else
    public void ChangeAlphaMask(params byte[] state)
    {
        if ((bool)customMatBody)
        {
            customMatBody.SetFloat(ChaShader._alpha_a, state[0]);
            customMatBody.SetFloat(ChaShader._alpha_b, state[1]);
        }
        if (rendBra != null)
        {
            for (int i = 0; i < 2; i++)
            {
                if ((bool)rendBra[i])
                {
                    Material material = rendBra[i].material;
                    if ((bool)material)
                    {
                        material.SetFloat(ChaShader._alpha_a, state[0]);
                        material.SetFloat(ChaShader._alpha_b, state[1]);
                    }
                }
            }
        }

        if (rendInner == null) return;

        for (int j = 0; j < 2; j++)
        {
            if ((bool)rendInner[j])
            {
                Material material2 = rendInner[j].material;
                if ((bool)material2)
                {
                    material2.SetFloat(ChaShader._alpha_a, state[0]);
                    material2.SetFloat(ChaShader._alpha_b, state[1]);
                }
            }
        }
    }
#endif

    public void ChangeSimpleBodyDraw(bool drawSimple)
    {
        fileStatus.visibleSimple = drawSimple;
    }

    public void ChangeSimpleBodyColor(Color color)
    {
        fileStatus.simpleColor = color;
        if ((bool)rendSimpleBody)
        {
            Material material = rendSimpleBody.material;
            if ((bool)material)
                material.SetColor(ChaShader._Color, color);
        }
        if ((bool)rendSimpleTongue)
        {
            Material material2 = rendSimpleTongue.material;
            if ((bool)material2)
                material2.SetColor(ChaShader._Color, color);
        }
    }

#if FIX
    //Move these arrays outside of the method so they aren't created every time
    private readonly bool[,] UpdateVisible_array1 = {
        {false, false, false},
        {true, false, false},
        {false, true, false},
        {false, false, true}
    };
    private readonly int[] UpdateVisible_array2 = { 0, 1, 2 };
    private readonly RefObjKey[,] UpdateVisible_array3 = {
        {
            RefObjKey.S_TPARTS_00_DEF, RefObjKey.S_TPARTS_00_NUGE
        },
        {
            RefObjKey.S_TPARTS_01_DEF, RefObjKey.S_TPARTS_01_NUGE
        },
        {
            RefObjKey.S_TPARTS_02_DEF, RefObjKey.S_TPARTS_02_NUGE
        }
    };
    private readonly byte[,] UpdateVisible_array4 = {
        {1, 1},
        {0, 1},
        {0, 1},
        {0, 0}
    };

    //Avoid use of SetActiveControl which creates new arrays and generally rewrite the entire method to avoid garbage creation
    private void UpdateVisible()
    {
        bool visibleSon = true;
        bool visibleBody = true;
        bool visibleSimple = false;
        for (var index = 0; index < Singleton<Scene>.Instance.NowSceneNames.Count; index++)
        {
            var s = Singleton<Scene>.Instance.NowSceneNames[index];
            if (s == "H")
            {
                visibleSon = sex != 0 || Manager.Config.EtcData.VisibleSon;
                visibleBody = sex != 0 || Manager.Config.EtcData.VisibleBody;
                visibleSimple = sex == 0 && fileStatus.visibleSimple;
                break;
            }
        }
        if (Game.IsStudio)
            visibleSimple = fileStatus.visibleSimple;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_TongueB), visibleAll && fileStatus.tongueState == 1 && visibleBody && !visibleSimple && fileStatus.visibleHeadAlways && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_MOZ_ALL), !hideMoz))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.ObjBody), visibleAll && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        bool flag4 = IsClothesStateKind(1) && 0 == fileStatus.clothesState[1];
        bool flag5 = IsClothesStateKind(3) && 0 == fileStatus.clothesState[3];
        bool flag6 = visibleSimple || !flag4 && !flag5 || fileStatus.visibleSon;
        if (exType == 1)
            flag6 = !IsClothesStateKind(0) || 0 != fileStatus.clothesState[0] || fileStatus.visibleSon;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_Son), visibleAll && flag6 && visibleSon && fileStatus.visibleSonAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_GOMU), fileStatus.visibleGomu))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_SimpleTop), visibleAll && visibleBody && fileStatus.visibleBodyAlways && visibleSimple))
            updateShape = true;

        if (YS_Assist.SetActiveControl(objHead, visibleAll && visibleBody && !visibleSimple && fileStatus.visibleHeadAlways && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_TongueF), fileStatus.tongueState == 0))
            updateShape = true;

        GameObject[] array2 = new GameObject[3]
        {
            GetReferenceInfo(RefObjKey.S_TEARS_01),
            GetReferenceInfo(RefObjKey.S_TEARS_02),
            GetReferenceInfo(RefObjKey.S_TEARS_03)
        };
        for (int i = 0; i < 3; i++)
            if (!(null == array2[i]) && array2[i].SetActiveIfDifferent(UpdateVisible_array1[fileStatus.tearsLv, i]))
                updateShape = true;

        if (YS_Assist.SetActiveControl(objTongueEx, visibleAll && fileStatus.tongueState == 2 && visibleBody && !visibleSimple && fileStatus.visibleHeadAlways && fileStatus.visibleBodyAlways))
            updateShape = true;
        bool flag7 = CheckHideHair();
        for (var index = 0; index < objHair.Length; index++)
            if (YS_Assist.SetActiveControl(objHair[index], visibleAll && visibleBody && !flag7 && !visibleSimple && fileStatus.visibleHeadAlways && fileStatus.visibleBodyAlways))
                updateShape = true;

        if (YS_Assist.SetActiveControl(objClothes[0], visibleAll && visibleBody && !visibleSimple && 3 != fileStatus.clothesState[0] && fileStatus.visibleBodyAlways))
            updateShape = true;
        bool flag8 = false;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CTOP_T_DEF), fileStatus.clothesState[0] == 0))
        {
            updateShape = true;
            flag8 = true;
        }

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CTOP_T_NUGE), fileStatus.clothesState[0] == 1))
        {
            updateShape = true;
            flag8 = true;
        }

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CTOP_B_DEF), fileStatus.clothesState[1] == 0 && fileStatus.clothesState[0] != 3))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CTOP_B_NUGE), fileStatus.clothesState[1] != 0 && fileStatus.clothesState[0] != 3))
            updateShape = true;
        DrawOption(ChaFileDefine.ClothesKind.top);
        DrawSode();

        int num = 0;
        ListInfoBase listInfoBase = infoClothes[0];
        if (listInfoBase != null)
            num = listInfoBase.Kind;
        for (int k = 0; k < UpdateVisible_array2.Length; k++)
        {
            if (YS_Assist.SetActiveControl(objParts[UpdateVisible_array2[k]], visibleAll && fileStatus.clothesState[0] != 3 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
                updateShape = true;

            if (YS_Assist.SetActiveControl(GetReferenceInfo(UpdateVisible_array3[k, 0]), fileStatus.clothesState[0] == 0))
            {
                updateShape = true;
                switch (k)
                {
                    case 0:
                        flag8 = true;
                        break;
                    case 1:
                        if (num == 2)
                            flag8 = true;
                        break;
                }
            }

            if (!YS_Assist.SetActiveControl(GetReferenceInfo(UpdateVisible_array3[k, 1]), fileStatus.clothesState[0] == 1))
                continue;

            updateShape = true;
            switch (k)
            {
                case 0:
                    flag8 = true;
                    break;
                case 1:
                    if (num == 2)
                        flag8 = true;
                    break;
            }
        }

        if (flag8 || updateAlphaMask)
        {
            byte b = fileStatus.clothesState[0];
            ChangeAlphaMask(UpdateVisible_array4[b, 0], UpdateVisible_array4[b, 1]);
            updateAlphaMask = false;
        }

        if (YS_Assist.SetActiveControl(objClothes[1], visibleAll && !notBot && visibleBody && !visibleSimple && 3 != fileStatus.clothesState[1] && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CBOT_B_DEF), fileStatus.clothesState[1] == 0))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CBOT_B_NUGE), fileStatus.clothesState[1] == 1))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CBOT_T_DEF), fileStatus.clothesState[0] == 0 && fileStatus.clothesState[1] != 2))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_CBOT_T_NUGE), fileStatus.clothesState[0] != 0 && fileStatus.clothesState[1] != 2))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.bot);
        if (YS_Assist.SetActiveControl(objClothes[2], visibleAll && visibleBody && !visibleSimple && !notBra && 3 != fileStatus.clothesState[2] && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWT_T_DEF), fileStatus.clothesState[2] == 0))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWT_T_NUGE), fileStatus.clothesState[2] == 1))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWT_B_DEF), fileStatus.clothesState[3] == 0 && fileStatus.clothesState[2] != 3))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWT_B_NUGE), fileStatus.clothesState[3] != 0 && fileStatus.clothesState[2] != 3))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.bra);
        bool flag9 = true;
        if (hideShortsBot && objClothes[1] && objClothes[1].activeSelf && fileStatus.clothesState[1] == 0)
            flag9 = false;
        if (YS_Assist.SetActiveControl(objClothes[3], visibleAll && flag9 && !notShorts && visibleBody && !visibleSimple && 3 != fileStatus.clothesState[3] && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWB_B_DEF), fileStatus.clothesState[3] == 0))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWB_B_NUGE), fileStatus.clothesState[3] == 1))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_UWB_B_NUGE2), fileStatus.clothesState[3] == 2))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.shorts);
        if (YS_Assist.SetActiveControl(objClothes[4], visibleAll && fileStatus.clothesState[4] == 0 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.gloves);
        byte b2 = fileStatus.clothesState[5];
        if (YS_Assist.SetActiveControl(objClothes[5], visibleAll && b2 != 3 && b2 != 2 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_PANST_DEF), fileStatus.clothesState[5] == 0))
            updateShape = true;

        if (YS_Assist.SetActiveControl(GetReferenceInfo(RefObjKey.S_PANST_NUGE), fileStatus.clothesState[5] == 1))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.panst);
        if (YS_Assist.SetActiveControl(objClothes[6], visibleAll && fileStatus.clothesState[6] == 0 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.socks);
        if (YS_Assist.SetActiveControl(objClothes[7], visibleAll && 0 == fileStatus.shoesType && fileStatus.clothesState[7] == 0 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.shoes_inner);
        if (YS_Assist.SetActiveControl(objClothes[8], visibleAll && 1 == fileStatus.shoesType && fileStatus.clothesState[8] == 0 && visibleBody && !visibleSimple && fileStatus.visibleBodyAlways))
            updateShape = true;

        DrawOption(ChaFileDefine.ClothesKind.shoes_outer);
        for (int l = 0; l < objAccessory.Length; l++)
        {
            bool flag10 = false;
            if (!fileStatus.visibleHeadAlways && nowCoordinate.accessory.parts[l].partsOfHead)
                flag10 = true;
            if (!fileStatus.visibleBodyAlways || !visibleBody)
                flag10 = true;
            if (YS_Assist.SetActiveControl(objAccessory[l], visibleAll && fileStatus.showAccessory[l] && !visibleSimple && !flag10))
                updateShape = true;
        }

        GameObject referenceInfo8 = GetReferenceInfo(RefObjKey.DB_SKIRT_BOT);
        GameObject referenceInfo9 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOP);
        GameObject referenceInfo10 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOPA);
        GameObject referenceInfo11 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOPB);
        bool active = !referenceInfo8;
        if (referenceInfo8 && fileStatus.clothesState[1] == 3)
            active = true;
        if ((bool)referenceInfo9)
            referenceInfo9.SetActiveIfDifferent(active);
        if ((bool)referenceInfo10)
            referenceInfo10.SetActiveIfDifferent(active);
        if ((bool)referenceInfo11)
            referenceInfo11.SetActiveIfDifferent(active);
    }
#else
    private void UpdateVisible()
    {
        bool flag = true;
        bool flag2 = true;
        bool flag3 = false;
        if (Singleton<Scene>.Instance.NowSceneNames.Any(s => s == "H"))
        {
            flag = sex != 0 || Manager.Config.EtcData.VisibleSon;
            flag2 = sex != 0 || Manager.Config.EtcData.VisibleBody;
            flag3 = sex == 0 && fileStatus.visibleSimple;
        }
        GameObject referenceInfo = GetReferenceInfo(RefObjKey.S_TongueB);
        if (YS_Assist.SetActiveControl(referenceInfo, visibleAll, fileStatus.tongueState == 1, flag2, !flag3, fileStatus.visibleHeadAlways, fileStatus.visibleBodyAlways))
            updateShape = true;
        GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.S_MOZ_ALL);
        if (YS_Assist.SetActiveControl(referenceInfo2, !hideMoz))
            updateShape = true;
        GameObject referenceInfo3 = GetReferenceInfo(RefObjKey.ObjBody);
        if (YS_Assist.SetActiveControl(referenceInfo3, visibleAll, flag2, !flag3, fileStatus.visibleBodyAlways))
            updateShape = true;
        bool flag4 = IsClothesStateKind(1) && 0 == fileStatus.clothesState[1];
        bool flag5 = IsClothesStateKind(3) && 0 == fileStatus.clothesState[3];
        bool flag6 = flag3 || !flag4 && !flag5 || fileStatus.visibleSon;
        if (exType == 1)
            flag6 = !IsClothesStateKind(0) || 0 != fileStatus.clothesState[0] || fileStatus.visibleSon;
        GameObject referenceInfo4 = GetReferenceInfo(RefObjKey.S_Son);
        if (YS_Assist.SetActiveControl(referenceInfo4, visibleAll, flag6, flag, fileStatus.visibleSonAlways))
            updateShape = true;
        GameObject referenceInfo5 = GetReferenceInfo(RefObjKey.S_GOMU);
        if (YS_Assist.SetActiveControl(referenceInfo5, fileStatus.visibleGomu))
            updateShape = true;
        GameObject referenceInfo6 = GetReferenceInfo(RefObjKey.S_SimpleTop);
        if (YS_Assist.SetActiveControl(referenceInfo6, visibleAll, flag2, fileStatus.visibleBodyAlways, flag3))
            updateShape = true;
        if (YS_Assist.SetActiveControl(objHead, visibleAll, flag2, !flag3, fileStatus.visibleHeadAlways, fileStatus.visibleBodyAlways))
            updateShape = true;
        GameObject referenceInfo7 = GetReferenceInfo(RefObjKey.S_TongueF);
        if (YS_Assist.SetActiveControl(referenceInfo7, fileStatus.tongueState == 0))
            updateShape = true;
        bool[,] array = new bool[4, 3]
        {
            {false, false, false},
            {true, false, false},
            {false, true, false},
            {false, false, true}
        };
        GameObject[] array2 = new GameObject[3]
        {
            GetReferenceInfo(RefObjKey.S_TEARS_01),
            GetReferenceInfo(RefObjKey.S_TEARS_02),
            GetReferenceInfo(RefObjKey.S_TEARS_03)
        };
        for (int i = 0; i < 3; i++)
        {
            if (!(null == array2[i]) && array2[i].SetActiveIfDifferent(array[fileStatus.tearsLv, i]))
                updateShape = true;
        }
        if (YS_Assist.SetActiveControl(objTongueEx, visibleAll, fileStatus.tongueState == 2, flag2, !flag3, fileStatus.visibleHeadAlways, fileStatus.visibleBodyAlways))
            updateShape = true;
        bool flag7 = CheckHideHair();
        GameObject[] objHair = base.objHair;
        foreach (GameObject obj in objHair)
        {
            if (YS_Assist.SetActiveControl(obj, visibleAll, flag2, !flag7, !flag3, fileStatus.visibleHeadAlways, fileStatus.visibleBodyAlways))
                updateShape = true;
        }
        bool[] array3 = new bool[4] { true, true, true, true };
        bool[] array4 = new bool[4] { true, true, true, true };
        bool[] array5 = new bool[4] { true, true, true, true };
        bool[] array6 = new bool[5] { true, true, true, true, true };
        if (YS_Assist.SetActiveControl(objClothes[0], visibleAll, flag2, !flag3, 3 != fileStatus.clothesState[0], fileStatus.visibleBodyAlways))
            updateShape = true;
        bool flag8 = false;
        GameObject gameObject = null;
        bool[] flags = new bool[1]
        {
            fileStatus.clothesState[0] == 0
        };
        array3[0] = YS_Assist.CheckFlagsArray(flags);
        gameObject = GetReferenceInfo(RefObjKey.S_CTOP_T_DEF);
        if (YS_Assist.SetActiveControl(gameObject, array3[0]))
        {
            updateShape = true;
            flag8 = true;
        }
        bool[] flags2 = new bool[1]
        {
            fileStatus.clothesState[0] == 1
        };
        array3[1] = YS_Assist.CheckFlagsArray(flags2);
        gameObject = GetReferenceInfo(RefObjKey.S_CTOP_T_NUGE);
        if (YS_Assist.SetActiveControl(gameObject, array3[1]))
        {
            updateShape = true;
            flag8 = true;
        }
        bool[] flags3 = new bool[2]
        {
            fileStatus.clothesState[1] == 0,
            fileStatus.clothesState[0] != 3
        };
        array3[2] = YS_Assist.CheckFlagsArray(flags3);
        gameObject = GetReferenceInfo(RefObjKey.S_CTOP_B_DEF);
        if (YS_Assist.SetActiveControl(gameObject, array3[2]))
        {
            updateShape = true;
        }
        bool[] flags4 = new bool[2]
        {
            fileStatus.clothesState[1] != 0,
            fileStatus.clothesState[0] != 3
        };
        array3[3] = YS_Assist.CheckFlagsArray(flags4);
        gameObject = GetReferenceInfo(RefObjKey.S_CTOP_B_NUGE);
        if (YS_Assist.SetActiveControl(gameObject, array3[3]))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.top);
        DrawSode();
        int[] array7 = new int[3] { 0, 1, 2 };
        RefObjKey[,] array8 = new RefObjKey[3, 2]
        {
            {
                RefObjKey.S_TPARTS_00_DEF,
                RefObjKey.S_TPARTS_00_NUGE
            },
            {
                RefObjKey.S_TPARTS_01_DEF,
                RefObjKey.S_TPARTS_01_NUGE
            },
            {
                RefObjKey.S_TPARTS_02_DEF,
                RefObjKey.S_TPARTS_02_NUGE
            }
        };
        int num = 0;
        ListInfoBase listInfoBase = infoClothes[0];
        if (listInfoBase != null)
            num = listInfoBase.Kind;
        for (int k = 0; k < array7.Length; k++)
        {
            if (YS_Assist.SetActiveControl(objParts[array7[k]], visibleAll, fileStatus.clothesState[0] != 3, flag2, !flag3, fileStatus.visibleBodyAlways))
            {
                updateShape = true;
            }
            GameObject gameObject2 = null;
            bool[] flags5 = new bool[1]
            {
                fileStatus.clothesState[0] == 0
            };
            gameObject2 = GetReferenceInfo(array8[k, 0]);
            if (YS_Assist.SetActiveControl(gameObject2, flags5))
            {
                updateShape = true;
                switch (k)
                {
                    case 0:
                        flag8 = true;
                        break;
                    case 1:
                        if (num == 2)
                        {
                            flag8 = true;
                        }
                        break;
                }
            }
            bool[] flags6 = new bool[1]
            {
                fileStatus.clothesState[0] == 1
            };
            gameObject2 = GetReferenceInfo(array8[k, 1]);
            if (!YS_Assist.SetActiveControl(gameObject2, flags6))
                continue;
            updateShape = true;
            switch (k)
            {
                case 0:
                    flag8 = true;
                    break;
                case 1:
                    if (num == 2)
                    {
                        flag8 = true;
                    }
                    break;
            }
        }
        if (flag8 || updateAlphaMask)
        {
            byte b = fileStatus.clothesState[0];
            byte[,] array9 = new byte[4, 2]
            {
                {1, 1},
                {0, 1},
                {0, 1},
                {0, 0}
            };
            ChangeAlphaMask(array9[b, 0], array9[b, 1]);
            updateAlphaMask = false;
        }
        if (YS_Assist.SetActiveControl(objClothes[1], visibleAll, !notBot, flag2, !flag3, 3 != fileStatus.clothesState[1], fileStatus.visibleBodyAlways))
            updateShape = true;
        GameObject gameObject3 = null;
        bool[] flags7 = new bool[1]
        {
            fileStatus.clothesState[1] == 0
        };
        array4[0] = YS_Assist.CheckFlagsArray(flags7);
        gameObject3 = GetReferenceInfo(RefObjKey.S_CBOT_B_DEF);
        if (YS_Assist.SetActiveControl(gameObject3, array4[0]))
        {
            updateShape = true;
        }
        bool[] flags8 = new bool[1]
        {
            fileStatus.clothesState[1] == 1
        };
        array4[1] = YS_Assist.CheckFlagsArray(flags8);
        gameObject3 = GetReferenceInfo(RefObjKey.S_CBOT_B_NUGE);
        if (YS_Assist.SetActiveControl(gameObject3, array4[1]))
        {
            updateShape = true;
        }
        bool[] flags9 = new bool[2]
        {
            fileStatus.clothesState[0] == 0,
            fileStatus.clothesState[1] != 2
        };
        array4[2] = YS_Assist.CheckFlagsArray(flags9);
        gameObject3 = GetReferenceInfo(RefObjKey.S_CBOT_T_DEF);
        if (YS_Assist.SetActiveControl(gameObject3, array4[2]))
        {
            updateShape = true;
        }
        bool[] flags10 = new bool[2]
        {
            fileStatus.clothesState[0] != 0,
            fileStatus.clothesState[1] != 2
        };
        array4[3] = YS_Assist.CheckFlagsArray(flags10);
        gameObject3 = GetReferenceInfo(RefObjKey.S_CBOT_T_NUGE);
        if (YS_Assist.SetActiveControl(gameObject3, array4[3]))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.bot);
        if (YS_Assist.SetActiveControl(objClothes[2], visibleAll, flag2, !flag3, !notBra, 3 != fileStatus.clothesState[2], fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        GameObject gameObject4 = null;
        bool[] flags11 = new bool[1]
        {
            fileStatus.clothesState[2] == 0
        };
        array5[0] = YS_Assist.CheckFlagsArray(flags11);
        gameObject4 = GetReferenceInfo(RefObjKey.S_UWT_T_DEF);
        if (YS_Assist.SetActiveControl(gameObject4, array5[0]))
        {
            updateShape = true;
        }
        bool[] flags12 = new bool[1]
        {
            fileStatus.clothesState[2] == 1
        };
        array5[1] = YS_Assist.CheckFlagsArray(flags12);
        gameObject4 = GetReferenceInfo(RefObjKey.S_UWT_T_NUGE);
        if (YS_Assist.SetActiveControl(gameObject4, array5[1]))
        {
            updateShape = true;
        }
        bool[] flags13 = new bool[2]
        {
            fileStatus.clothesState[3] == 0,
            fileStatus.clothesState[2] != 3
        };
        array5[2] = YS_Assist.CheckFlagsArray(flags13);
        gameObject4 = GetReferenceInfo(RefObjKey.S_UWT_B_DEF);
        if (YS_Assist.SetActiveControl(gameObject4, array5[2]))
        {
            updateShape = true;
        }
        bool[] flags14 = new bool[2]
        {
            fileStatus.clothesState[3] != 0,
            fileStatus.clothesState[2] != 3
        };
        array5[3] = YS_Assist.CheckFlagsArray(flags14);
        gameObject4 = GetReferenceInfo(RefObjKey.S_UWT_B_NUGE);
        if (YS_Assist.SetActiveControl(gameObject4, array5[3]))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.bra);
        bool flag9 = true;
        if (hideShortsBot && objClothes[1] && objClothes[1].activeSelf && fileStatus.clothesState[1] == 0)
        {
            flag9 = false;
        }
        if (YS_Assist.SetActiveControl(objClothes[3], visibleAll, flag9, !notShorts, flag2, !flag3, 3 != fileStatus.clothesState[3], fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        GameObject gameObject5 = null;
        bool[] flags15 = new bool[1]
        {
            fileStatus.clothesState[3] == 0
        };
        array6[0] = YS_Assist.CheckFlagsArray(flags15);
        gameObject5 = GetReferenceInfo(RefObjKey.S_UWB_B_DEF);
        if (YS_Assist.SetActiveControl(gameObject5, array6[0]))
        {
            updateShape = true;
        }
        bool[] flags16 = new bool[1]
        {
            fileStatus.clothesState[3] == 1
        };
        array6[1] = YS_Assist.CheckFlagsArray(flags16);
        gameObject5 = GetReferenceInfo(RefObjKey.S_UWB_B_NUGE);
        if (YS_Assist.SetActiveControl(gameObject5, array6[1]))
        {
            updateShape = true;
        }
        bool[] flags17 = new bool[1]
        {
            fileStatus.clothesState[3] == 2
        };
        array6[2] = YS_Assist.CheckFlagsArray(flags17);
        gameObject5 = GetReferenceInfo(RefObjKey.S_UWB_B_NUGE2);
        if (YS_Assist.SetActiveControl(gameObject5, array6[2]))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.shorts);
        if (YS_Assist.SetActiveControl(objClothes[4], visibleAll, fileStatus.clothesState[4] == 0, flag2, !flag3, fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.gloves);
        byte b2 = fileStatus.clothesState[5];
        if (YS_Assist.SetActiveControl(objClothes[5], visibleAll, b2 != 3 && b2 != 2, flag2, !flag3, fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        GameObject gameObject6 = null;
        gameObject6 = GetReferenceInfo(RefObjKey.S_PANST_DEF);
        if (YS_Assist.SetActiveControl(gameObject6, fileStatus.clothesState[5] == 0))
        {
            updateShape = true;
        }
        gameObject6 = GetReferenceInfo(RefObjKey.S_PANST_NUGE);
        if (YS_Assist.SetActiveControl(gameObject6, fileStatus.clothesState[5] == 1))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.panst);
        if (YS_Assist.SetActiveControl(objClothes[6], visibleAll, fileStatus.clothesState[6] == 0, flag2, !flag3, fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.socks);
        if (YS_Assist.SetActiveControl(objClothes[7], visibleAll, 0 == fileStatus.shoesType, fileStatus.clothesState[7] == 0, flag2, !flag3, fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.shoes_inner);
        if (YS_Assist.SetActiveControl(objClothes[8], visibleAll, 1 == fileStatus.shoesType, fileStatus.clothesState[8] == 0, flag2, !flag3, fileStatus.visibleBodyAlways))
        {
            updateShape = true;
        }
        DrawOption(ChaFileDefine.ClothesKind.shoes_outer);
        for (int l = 0; l < objAccessory.Length; l++)
        {
            bool flag10 = false;
            if (!fileStatus.visibleHeadAlways && nowCoordinate.accessory.parts[l].partsOfHead)
            {
                flag10 = true;
            }
            if (!fileStatus.visibleBodyAlways || !flag2)
            {
                flag10 = true;
            }
            if (YS_Assist.SetActiveControl(objAccessory[l], visibleAll, fileStatus.showAccessory[l], !flag3, !flag10))
            {
                updateShape = true;
            }
        }
        GameObject referenceInfo8 = GetReferenceInfo(RefObjKey.DB_SKIRT_BOT);
        GameObject referenceInfo9 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOP);
        GameObject referenceInfo10 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOPA);
        GameObject referenceInfo11 = GetReferenceInfo(RefObjKey.DB_SKIRT_TOPB);
        bool active = !referenceInfo8;
        if (referenceInfo8 && fileStatus.clothesState[1] == 3)
            active = true;
        if ((bool)referenceInfo9)
            referenceInfo9.SetActiveIfDifferent(active);
        if ((bool)referenceInfo10)
            referenceInfo10.SetActiveIfDifferent(active);
        if ((bool)referenceInfo11)
            referenceInfo11.SetActiveIfDifferent(active);
    }
#endif

    public void DrawOption(ChaFileDefine.ClothesKind kind)
    {
        ChaClothesComponent chaClothesComponent = cusClothesCmp[(int)kind];
        if (null == chaClothesComponent)
            return;
        bool flag = !nowCoordinate.clothes.parts[(int)kind].hideOpt[0];
        bool flag2 = !nowCoordinate.clothes.parts[(int)kind].hideOpt[1];
        if (kind == ChaFileDefine.ClothesKind.bra)
        {
            flag = !nowCoordinate.clothes.hideBraOpt[0];
            flag2 = !nowCoordinate.clothes.hideBraOpt[1];
        }
        if (kind == ChaFileDefine.ClothesKind.shorts)
        {
            flag = !nowCoordinate.clothes.hideShortsOpt[0];
            flag2 = !nowCoordinate.clothes.hideShortsOpt[1];
        }
        if (chaClothesComponent.objOpt01 != null)
        {
            GameObject[] objOpt = chaClothesComponent.objOpt01;
            foreach (GameObject obj in objOpt)
            {
                YS_Assist.SetActiveControl(obj, flag);
            }
        }
        if (chaClothesComponent.objOpt02 != null)
        {
            GameObject[] objOpt2 = chaClothesComponent.objOpt02;
            foreach (GameObject obj2 in objOpt2)
                YS_Assist.SetActiveControl(obj2, flag2);
        }
    }

    public void DrawSode()
    {
        int num = 0;
        ChaClothesComponent chaClothesComponent = cusClothesCmp[num];
        int sleevesType = nowCoordinate.clothes.parts[num].sleevesType;
        if (null == chaClothesComponent)
            return;
        if (chaClothesComponent.objSleeves01 != null)
        {
            GameObject[] objSleeves = chaClothesComponent.objSleeves01;
            foreach (GameObject obj in objSleeves)
                YS_Assist.SetActiveControl(obj, 0 == sleevesType);
        }
        if (chaClothesComponent.objSleeves02 != null)
        {
            GameObject[] objSleeves2 = chaClothesComponent.objSleeves02;
            foreach (GameObject obj2 in objSleeves2)
                YS_Assist.SetActiveControl(obj2, 1 == sleevesType);
        }
        if (chaClothesComponent.objSleeves03 != null)
        {
            GameObject[] objSleeves3 = chaClothesComponent.objSleeves03;
            foreach (GameObject obj3 in objSleeves3)
                YS_Assist.SetActiveControl(obj3, 2 == sleevesType);
        }
    }

    protected void InitializeControlCoordinateAll()
    {
        nowCoordinate = new ChaFileCoordinate();
        InitializeControlCoordinateObject();
    }

    protected void InitializeControlCoordinateObject()
    {
        notBra = false;
        notBot = false;
        notShorts = false;
        hideHairAcs = new bool[20];
        isChangeOfClothesRandom = false;
        dictStateType = new Dictionary<int, Dictionary<byte, string>>();
    }

    protected void ReleaseControlCoordinateAll()
    {
        ReleaseControlCoordinateObject(false);
    }

    protected void ReleaseControlCoordinateObject(bool init = true)
    {
        if (init)
            InitializeControlCoordinateObject();
    }

    public int GetNowClothesType()
    {
        int num = 0;
        int num2 = 1;
        int num3 = 2;
        int num4 = 3;
        int num5 = !IsClothesStateKind(num) ? 3 : fileStatus.clothesState[num];
        int num6 = !IsClothesStateKind(num2) ? 3 : fileStatus.clothesState[num2];
        int num7 = !IsClothesStateKind(num3) ? 3 : fileStatus.clothesState[num3];
        int num8 = !IsClothesStateKind(num4) ? 3 : fileStatus.clothesState[num4];
        bool flag = true;
        bool flag2 = true;
        if (infoClothes[num3] != null)
            flag = infoClothes[num3].Kind == 1;
        if (infoClothes[num4] != null)
            flag2 = infoClothes[num4].Kind == 1;
        if (notShorts)
        {
            num8 = num7;
            flag2 = flag;
        }
        if (num5 == 0)
        {
            if (num6 == 0)
                return 0;
            if (num8 == 0)
                return flag2 ? 1 : 2;
            return 3;
        }
        if (num6 == 0)
        {
            if (num7 == 0)
                return flag ? 1 : 2;
            return 3;
        }
        if (num7 == 0)
        {
            if (num8 == 0)
            {
                if (flag)
                    return 1;
                return flag2 ? 1 : 2;
            }
            return 3;
        }
        return 3;
    }

    public bool IsKokanHide()
    {
        bool result = false;
        int[] array = new int[4] { 0, 1, 2, 3 };
        int[] array2 = new int[4] { 1, 1, 3, 3 };
        for (int i = 0; i < array.Length; i++)
        {
            int num = array[i];
            if (IsClothes(num) && (i != 0 && i != 2 || !("2" != infoClothes[num].GetInfo(ChaListDefine.KeyType.Coordinate))) && "1" == infoClothes[num].GetInfo(ChaListDefine.KeyType.KokanHide) && fileStatus.clothesState[array2[i]] == 0)
            {
                result = true;
                break;
            }
        }
        return result;
    }

    public bool AssignCoordinate(ChaFileDefine.CoordinateType type, string path)
    {
        ChaFileCoordinate chaFileCoordinate = new ChaFileCoordinate();
        string path2 = ChaFileControl.ConvertCoordinateFilePath(path);
        if (!chaFileCoordinate.LoadFile(path2))
            return false;
        return chaFile.AssignCoordinate(type, chaFileCoordinate);
    }

    public bool AssignCoordinate(ChaFileDefine.CoordinateType type, ChaFileCoordinate srcCoorde)
    {
        return chaFile.AssignCoordinate(type, srcCoorde);
    }

    public bool AssignCoordinate(ChaFileDefine.CoordinateType type)
    {
        return chaFile.AssignCoordinate(type, nowCoordinate);
    }

    public bool SetNowCoordinate(string path)
    {
        ChaFileCoordinate chaFileCoordinate = new ChaFileCoordinate();
        string path2 = ChaFileControl.ConvertCoordinateFilePath(path);
        if (!chaFileCoordinate.LoadFile(path2))
            return false;
        return SetNowCoordinate(chaFileCoordinate);
    }

    public bool SetNowCoordinate(ChaFileCoordinate srcCoorde)
    {
        byte[] data = srcCoorde.SaveBytes();
        return nowCoordinate.LoadBytes(data, srcCoorde.loadVersion);
    }

    public bool ChangeCoordinateType(bool changeBackCoordinateType = true)
    {
        return ChangeCoordinateType((ChaFileDefine.CoordinateType)fileStatus.coordinateType, changeBackCoordinateType);
    }

    public bool ChangeCoordinateType(ChaFileDefine.CoordinateType type, bool changeBackCoordinateType = true)
    {
        if (chaFile.coordinate.Length <= (int)type)
            return false;
        byte[] data = chaFile.coordinate[(int)type].SaveBytes();
        if (nowCoordinate.LoadBytes(data, ChaFileDefine.ChaFileCoordinateVersion))
        {
            if (changeBackCoordinateType)
                fileStatus.backCoordinateType = fileStatus.coordinateType;
            fileStatus.coordinateType = (int)type;
            return true;
        }
        return false;
    }

    public bool ChangeCoordinateTypeAndReload(bool changeBackCoordinateType = true)
    {
        if (ChangeCoordinateType(changeBackCoordinateType))
        {
            Reload(false, true, true, true);
            return true;
        }
        return false;
    }

    public bool ChangeCoordinateTypeAndReload(ChaFileDefine.CoordinateType type, bool changeBackCoordinateType = true)
    {
        if (ChangeCoordinateType(type, changeBackCoordinateType))
        {
            Reload(false, true, true, true);
            return true;
        }
        return false;
    }

    public bool AddClothesStateKind(int clothesKind, string stateType)
    {
        ChaFileDefine.ClothesKind clothesKind2 = (ChaFileDefine.ClothesKind)Enum.ToObject(typeof(ChaFileDefine.ClothesKind), clothesKind);
        switch (clothesKind2)
        {
            case ChaFileDefine.ClothesKind.gloves:
                dictStateType.Remove(4);
                AddClothesStateKindSub(4, byte.Parse(stateType));
                break;
            case ChaFileDefine.ClothesKind.panst:
                dictStateType.Remove(5);
                AddClothesStateKindSub(5, byte.Parse(stateType));
                break;
            case ChaFileDefine.ClothesKind.socks:
                dictStateType.Remove(6);
                AddClothesStateKindSub(6, byte.Parse(stateType));
                break;
            case ChaFileDefine.ClothesKind.shoes_inner:
                dictStateType.Remove(7);
                AddClothesStateKindSub(7, byte.Parse(stateType));
                break;
            case ChaFileDefine.ClothesKind.shoes_outer:
                dictStateType.Remove(8);
                AddClothesStateKindSub(8, byte.Parse(stateType));
                break;
            case ChaFileDefine.ClothesKind.top:
            case ChaFileDefine.ClothesKind.bot:
                {
                    dictStateType.Remove(0);
                    dictStateType.Remove(1);
                    ListInfoBase listInfoBase3 = infoClothes[0];
                    byte b3 = 3;
                    if (listInfoBase3 != null)
                    {
                        b3 = byte.Parse(listInfoBase3.GetInfo(ChaListDefine.KeyType.StateType));
                    }
                    byte b4 = 3;
                    ListInfoBase listInfoBase4 = infoClothes[1];
                    if (listInfoBase4 != null)
                    {
                        b4 = byte.Parse(listInfoBase4.GetInfo(ChaListDefine.KeyType.StateType));
                    }
                    if (b4 != 0)
                    {
                        GameObject referenceInfo3 = GetReferenceInfo(RefObjKey.S_CTOP_B_DEF);
                        if ((bool)referenceInfo3)
                        {
                            switch (b3)
                            {
                                case 0:
                                    b4 = 0;
                                    break;
                                case 1:
                                    if (b4 == 2)
                                    {
                                        b4 = 1;
                                    }
                                    break;
                            }
                        }
                    }
                    if (b3 != 0)
                    {
                        GameObject referenceInfo4 = GetReferenceInfo(RefObjKey.S_CBOT_T_DEF);
                        if ((bool)referenceInfo4)
                        {
                            switch (b4)
                            {
                                case 0:
                                    b3 = 0;
                                    break;
                                case 1:
                                    if (b3 == 2)
                                    {
                                        b3 = 1;
                                    }
                                    break;
                            }
                        }
                    }
                    AddClothesStateKindSub(0, b3);
                    AddClothesStateKindSub(1, b4);
                    if (clothesKind2 == ChaFileDefine.ClothesKind.top)
                    {
                        AddClothesStateKind(2, string.Empty);
                    }
                    break;
                }
            case ChaFileDefine.ClothesKind.bra:
            case ChaFileDefine.ClothesKind.shorts:
                {
                    dictStateType.Remove(2);
                    dictStateType.Remove(3);
                    byte b = 3;
                    if (!notBra)
                    {
                        ListInfoBase listInfoBase = infoClothes[2];
                        if (listInfoBase != null)
                        {
                            b = byte.Parse(listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
                        }
                    }
                    byte b2 = 3;
                    if (!notBra || !notShorts)
                    {
                        ListInfoBase listInfoBase2 = infoClothes[3];
                        if (listInfoBase2 != null)
                        {
                            b2 = byte.Parse(listInfoBase2.GetInfo(ChaListDefine.KeyType.StateType));
                        }
                    }
                    if (b2 != 0 && b2 != 2)
                    {
                        GameObject referenceInfo = GetReferenceInfo(RefObjKey.S_UWT_B_DEF);
                        if ((bool)referenceInfo)
                        {
                            switch (b)
                            {
                                case 0:
                                    b2 = 0;
                                    break;
                                case 1:
                                    if (b2 == 3)
                                    {
                                        b2 = 1;
                                    }
                                    break;
                            }
                        }
                    }
                    if (b != 0 && b != 2)
                    {
                        GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.S_UWB_T_DEF);
                        if ((bool)referenceInfo2)
                        {
                            switch (b2)
                            {
                                case 0:
                                    b = 0;
                                    break;
                                case 1:
                                    if (b == 3)
                                    {
                                        b = 1;
                                    }
                                    break;
                            }
                        }
                    }
                    AddClothesStateKindSub(2, b);
                    AddClothesStateKindSub(3, b2);
                    break;
                }
        }
        return true;
    }

    private bool AddClothesStateKindSub(int clothesKind, byte type)
    {
        if (!MathfEx.RangeEqualOn(0, type, 2))
            return false;
        Dictionary<byte, string> dictionary = new Dictionary<byte, string>();
        switch (type)
        {
            case 0:
                dictionary[0] = "着衣";
                dictionary[1] = "半脱";
                dictionary[3] = "脱衣";
                break;
            case 1:
                dictionary[0] = "着衣";
                dictionary[3] = "脱衣";
                break;
            default:
                dictionary[0] = "着衣";
                dictionary[1] = "半脱１";
                dictionary[2] = "半脱２";
                dictionary[3] = "脱衣";
                break;
        }
        dictStateType[clothesKind] = dictionary;
        return true;
    }

    public void RemoveClothesStateKind(int clothesKind)
    {
        switch ((ChaFileDefine.ClothesKind)Enum.ToObject(typeof(ChaFileDefine.ClothesKind), clothesKind))
        {
            case ChaFileDefine.ClothesKind.gloves:
                dictStateType.Remove(4);
                break;
            case ChaFileDefine.ClothesKind.panst:
                dictStateType.Remove(5);
                break;
            case ChaFileDefine.ClothesKind.socks:
                dictStateType.Remove(6);
                break;
            case ChaFileDefine.ClothesKind.shoes_inner:
                dictStateType.Remove(7);
                break;
            case ChaFileDefine.ClothesKind.shoes_outer:
                dictStateType.Remove(8);
                break;
            case ChaFileDefine.ClothesKind.top:
            case ChaFileDefine.ClothesKind.bot:
                AddClothesStateKind(0, string.Empty);
                break;
            case ChaFileDefine.ClothesKind.bra:
            case ChaFileDefine.ClothesKind.shorts:
                AddClothesStateKind(2, string.Empty);
                break;
        }
    }

    public bool IsClothes(int clothesKind)
    {
        if (!IsClothesStateKind(clothesKind)) return false;
        if (null == objClothes[clothesKind]) return false;
        if (infoClothes[clothesKind] == null) return false;
        return true;
    }

    public bool IsClothesStateKind(int clothesKind)
    {
        return dictStateType.ContainsKey(clothesKind);
    }

    public bool IsShoesStateKind()
    {
        int clothesKind = fileStatus.shoesType != 0 ? 8 : 7;
        return IsClothesStateKind(clothesKind);
    }

    public Dictionary<byte, string> GetClothesStateKind(int clothesKind)
    {
        Dictionary<byte, string> value;
        dictStateType.TryGetValue(clothesKind, out value);
        return value;
    }

    public bool IsClothesStateType(int clothesKind, byte stateType)
    {
        Dictionary<byte, string> value;
        dictStateType.TryGetValue(clothesKind, out value);
        if (value == null) return false;
        return value.ContainsKey(stateType);
    }

    public bool IsAccessory(int slotNo)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
            return false;
        return !(null == objAccessory[slotNo]);
    }

    public void SetClothesState(int clothesKind, byte state, bool next = true)
    {
        if (next)
        {
            byte b = fileStatus.clothesState[clothesKind];
            do
            {
                if (!IsClothesStateKind(clothesKind))
                {
                    fileStatus.clothesState[clothesKind] = state;
                    break;
                }
                if (IsClothesStateType(clothesKind, state))
                {
                    fileStatus.clothesState[clothesKind] = state;
                    break;
                }
                state = (byte)((state + 1) % 4);
            }
            while (b != state);
        }
        else
        {
            byte b2 = fileStatus.clothesState[clothesKind];
            do
            {
                if (!IsClothesStateKind(clothesKind))
                {
                    fileStatus.clothesState[clothesKind] = state;
                    break;
                }
                if (IsClothesStateType(clothesKind, state))
                {
                    fileStatus.clothesState[clothesKind] = state;
                    break;
                }
                state = (byte)((state + 3) % 4);
            }
            while (b2 != state);
        }
        switch (clothesKind)
        {
            case 4:
            case 5:
            case 6:
                break;
            case 0:
                if (notBot)
                {
                    if (fileStatus.clothesState[clothesKind] == 3)
                    {
                        fileStatus.clothesState[1] = 3;
                    }
                    else if (fileStatus.clothesState[1] == 3)
                    {
                        fileStatus.clothesState[1] = state;
                    }
                }
                break;
            case 1:
                if (notBot)
                {
                    if (fileStatus.clothesState[clothesKind] == 3)
                        fileStatus.clothesState[0] = 3;
                    else if (fileStatus.clothesState[0] == 3)
                        fileStatus.clothesState[0] = state;
                }
                break;
            case 2:
                if (notShorts)
                {
                    if (fileStatus.clothesState[clothesKind] == 3)
                        fileStatus.clothesState[3] = 3;
                    else if (fileStatus.clothesState[3] == 3)
                        fileStatus.clothesState[3] = state;
                }
                break;
            case 3:
                if (notShorts)
                {
                    if (fileStatus.clothesState[clothesKind] == 3)
                        fileStatus.clothesState[2] = 3;
                    else if (fileStatus.clothesState[2] == 3)
                        fileStatus.clothesState[2] = state;
                }
                break;
            case 7:
                if (state == 0)
                {
                    if (IsClothesStateKind(8))
                        fileStatus.clothesState[8] = state;
                }
                else
                    fileStatus.clothesState[8] = state;
                break;
            case 8:
                if (state == 0)
                {
                    if (IsClothesStateKind(7))
                        fileStatus.clothesState[7] = state;
                }
                else
                {
                    fileStatus.clothesState[7] = state;
                }
                break;
        }
    }

    public void SetClothesStatePrev(int clothesKind)
    {
        byte b = fileStatus.clothesState[clothesKind];
        b = (byte)((b + 3) % 4);
        SetClothesState(clothesKind, b, false);
    }

    public void SetClothesStateNext(int clothesKind)
    {
        byte b = fileStatus.clothesState[clothesKind];
        b = (byte)((b + 1) % 4);
        SetClothesState(clothesKind, b);
    }

    public void SetClothesStateAll(byte state)
    {
        int num = Enum.GetNames(typeof(ChaFileDefine.ClothesKind)).Length;
        for (int i = 0; i < num; i++)
        {
            SetClothesState(i, state);
        }
    }

    public void UpdateClothesStateAll()
    {
        int num = Enum.GetNames(typeof(ChaFileDefine.ClothesKind)).Length;
        for (int i = 0; i < num; i++)
        {
            SetClothesState(i, fileStatus.clothesState[i]);
        }
    }

    public void RandomChangeOfClothesLowPoly(int h)
    {
        if (!isChangeOfClothesRandom)
        {
            isChangeOfClothesRandom = true;
            byte[,] array = new byte[6, 9]
            {
                {0, 0, 0, 0, 0, 0, 0, 0, 0},
                {3, 3, 3, 3, 3, 3, 3, 3, 3},
                {3, 3, 0, 0, 3, 3, 3, 3, 3},
                {3, 0, 0, 0, 0, 0, 0, 3, 3},
                {3, 0, 3, 0, 0, 0, 0, 3, 3},
                {3, 3, 3, 0, 3, 0, 0, 3, 3}
            };
            int num = (int)(h * 0.5f);
            int[] obj = new int[6] { 10, 0, 50, 50, 50, 50 };
            obj[1] = 5 + num;
            int[] weightTable = obj;
            int randomIndex = ChaRandom.GetRandomIndex(weightTable);
            int num2 = Enum.GetNames(typeof(ChaFileDefine.ClothesKind)).Length;
            for (int i = 0; i < num2; i++)
                SetClothesState(i, array[randomIndex, i]);
        }
    }

    public void RandomChangeOfClothesLowPolyEnd()
    {
        SetClothesStateAll(0);
        isChangeOfClothesRandom = false;
    }

    public void ChangeToiletStateLowPoly()
    {
        SetClothesState(1, 3);
        SetClothesState(3, 3);
        SetClothesState(5, 3);
    }

    public void SetAccessoryState(int slotNo, bool show)
    {
        if (fileStatus.showAccessory.Length > slotNo)
            fileStatus.showAccessory[slotNo] = show;
    }

    public void SetAccessoryStateAll(bool show)
    {
        for (int i = 0; i < fileStatus.showAccessory.Length; i++)
            fileStatus.showAccessory[i] = show;
    }

    public void SetAccessoryStateCategory(int cateNo, bool show)
    {
        if (cateNo != 0 && cateNo != 1)
            return;
        for (int i = 0; i < fileStatus.showAccessory.Length; i++)
            if (nowCoordinate.accessory.parts[i].hideCategory == cateNo)
                fileStatus.showAccessory[i] = show;
    }

    public int GetAccessoryCategoryCount(int cateNo)
    {
        if (cateNo != 0 && cateNo != 1)
            return -1;
        int num = 0;
        for (int i = 0; i < fileStatus.showAccessory.Length; i++)
        {
            if (nowCoordinate.accessory.parts[i].hideCategory == cateNo)
                num++;
        }
        return num;
    }

    public string GetAccessoryDefaultParentStr(int type, int id)
    {
        int num = Enum.GetNames(typeof(ChaListDefine.CategoryNo)).Length;
        if (!MathfEx.RangeEqualOn(0, type, num - 1))
            return string.Empty;
        Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)type);
        ListInfoBase value;
        if (!categoryInfo.TryGetValue(id, out value))
            return string.Empty;
        return value.GetInfo(ChaListDefine.KeyType.Parent);
    }

    public string GetAccessoryDefaultParentStr(int slotNo)
    {
        GameObject gameObject = objAccessory[slotNo];
        if (null == gameObject)
            return string.Empty;
        ListInfoComponent component = gameObject.GetComponent<ListInfoComponent>();
        return component.data.GetInfo(ChaListDefine.KeyType.Parent);
    }

    public bool ChangeAccessoryParent(int slotNo, string parentStr)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19)) return false;
        GameObject gameObject = objAccessory[slotNo];
        if (null == gameObject) return false;
        if ("none" == parentStr)
        {
            gameObject.transform.SetParent(null, false);
            return true;
        }
        ListInfoComponent component = gameObject.GetComponent<ListInfoComponent>();
        ListInfoBase data = component.data;
        if ("0" == data.GetInfo(ChaListDefine.KeyType.Parent))
        {
            return false;
        }
        try
        {
            RefObjKey key = (RefObjKey)Enum.Parse(typeof(RefObjKey), parentStr);
            GameObject referenceInfo = GetReferenceInfo(key);
            if (null == referenceInfo)
            {
                return false;
            }
            gameObject.transform.SetParent(referenceInfo.transform, false);
            nowCoordinate.accessory.parts[slotNo].parentKey = parentStr;
            nowCoordinate.accessory.parts[slotNo].partsOfHead = ChaAccessoryDefine.CheckPartsOfHead(parentStr);
        }
        catch (ArgumentException)
        {
            return false;
        }
        return true;
    }

    public bool SetAccessoryPos(int slotNo, int correctNo, float value, bool add, int flags = 7)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19)) return false;
        GameObject gameObject = objAcsMove[slotNo, correctNo];
        if (null == gameObject) return false;
        ChaFileAccessory accessory = nowCoordinate.accessory;
        if ((flags & 1) != 0)
        {
            float value2 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 0].x) + value).ToString("f1"));
            accessory.parts[slotNo].addMove[correctNo, 0].x = Mathf.Clamp(value2, -100f, 100f);
        }
        if ((flags & 2) != 0)
        {
            float value3 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 0].y) + value).ToString("f1"));
            accessory.parts[slotNo].addMove[correctNo, 0].y = Mathf.Clamp(value3, -100f, 100f);
        }
        if ((flags & 4) != 0)
        {
            float value4 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 0].z) + value).ToString("f1"));
            accessory.parts[slotNo].addMove[correctNo, 0].z = Mathf.Clamp(value4, -100f, 100f);
        }
        Dictionary<int, ListInfoBase> dictionary = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)accessory.parts[slotNo].type);
        ListInfoBase value5;
        dictionary.TryGetValue(accessory.parts[slotNo].id, out value5);
        if (value5.GetInfoInt(ChaListDefine.KeyType.HideHair) == 1)
            gameObject.transform.localPosition = new Vector3(0f, 0f, 0f);
        else
            gameObject.transform.localPosition = new Vector3(accessory.parts[slotNo].addMove[correctNo, 0].x * 0.01f, accessory.parts[slotNo].addMove[correctNo, 0].y * 0.01f, accessory.parts[slotNo].addMove[correctNo, 0].z * 0.01f);
        return true;
    }

    public bool SetAccessoryRot(int slotNo, int correctNo, float value, bool add, int flags = 7)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
            return false;
        GameObject gameObject = objAcsMove[slotNo, correctNo];
        if (null == gameObject)
            return false;
        ChaFileAccessory accessory = nowCoordinate.accessory;
        if ((flags & 1) != 0)
        {
            float t = (int)((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 1].x) + value);
            accessory.parts[slotNo].addMove[correctNo, 1].x = Mathf.Repeat(t, 360f);
        }
        if ((flags & 2) != 0)
        {
            float t2 = (int)((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 1].y) + value);
            accessory.parts[slotNo].addMove[correctNo, 1].y = Mathf.Repeat(t2, 360f);
        }
        if ((flags & 4) != 0)
        {
            float t3 = (int)((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 1].z) + value);
            accessory.parts[slotNo].addMove[correctNo, 1].z = Mathf.Repeat(t3, 360f);
        }
        Dictionary<int, ListInfoBase> dictionary = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)accessory.parts[slotNo].type);
        ListInfoBase value2;
        dictionary.TryGetValue(accessory.parts[slotNo].id, out value2);
        if (value2.GetInfoInt(ChaListDefine.KeyType.HideHair) == 1)
            gameObject.transform.localEulerAngles = new Vector3(0f, 0f, 0f);
        else
            gameObject.transform.localEulerAngles = new Vector3(accessory.parts[slotNo].addMove[correctNo, 1].x, accessory.parts[slotNo].addMove[correctNo, 1].y, accessory.parts[slotNo].addMove[correctNo, 1].z);
        return true;
    }

    public bool SetAccessoryScl(int slotNo, int correctNo, float value, bool add, int flags = 7)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
            return false;
        GameObject gameObject = objAcsMove[slotNo, correctNo];
        if (null == gameObject)
            return false;
        ChaFileAccessory accessory = nowCoordinate.accessory;
        if ((flags & 1) != 0)
        {
            float value2 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 2].x) + value).ToString("f2"));
            accessory.parts[slotNo].addMove[correctNo, 2].x = Mathf.Clamp(value2, 0.01f, 100f);
        }
        if ((flags & 2) != 0)
        {
            float value3 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 2].y) + value).ToString("f2"));
            accessory.parts[slotNo].addMove[correctNo, 2].y = Mathf.Clamp(value3, 0.01f, 100f);
        }
        if ((flags & 4) != 0)
        {
            float value4 = float.Parse(((!add ? 0f : accessory.parts[slotNo].addMove[correctNo, 2].z) + value).ToString("f2"));
            accessory.parts[slotNo].addMove[correctNo, 2].z = Mathf.Clamp(value4, 0.01f, 100f);
        }
        Dictionary<int, ListInfoBase> dictionary = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)accessory.parts[slotNo].type);
        ListInfoBase value5;
        dictionary.TryGetValue(accessory.parts[slotNo].id, out value5);
        if (value5.GetInfoInt(ChaListDefine.KeyType.HideHair) == 1)
            gameObject.transform.localScale = new Vector3(1f, 1f, 1f);
        else
            gameObject.transform.localScale = new Vector3(accessory.parts[slotNo].addMove[correctNo, 2].x, accessory.parts[slotNo].addMove[correctNo, 2].y, accessory.parts[slotNo].addMove[correctNo, 2].z);
        return true;
    }

    public bool ResetAccessoryMove(int slotNo, int correctNo, int type = 7)
    {
        bool flag = true;
        if ((type & 1) != 0)
            flag &= SetAccessoryPos(slotNo, correctNo, 0f, false);
        if ((type & 2) != 0)
            flag &= SetAccessoryRot(slotNo, correctNo, 0f, false);
        if ((type & 4) != 0)
            flag &= SetAccessoryScl(slotNo, correctNo, 1f, false);
        return flag;
    }

    public bool UpdateAccessoryMoveFromInfo(int slotNo)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
        {
            return false;
        }
        ChaFileAccessory accessory = nowCoordinate.accessory;
        Dictionary<int, ListInfoBase> dictionary = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)accessory.parts[slotNo].type);
        ListInfoBase value;
        dictionary.TryGetValue(accessory.parts[slotNo].id, out value);
        if (value.GetInfoInt(ChaListDefine.KeyType.HideHair) == 1)
        {
            for (int i = 0; i < 2; i++)
            {
                GameObject gameObject = objAcsMove[slotNo, i];
                if (!(null == gameObject))
                {
                    gameObject.transform.localPosition = new Vector3(0f, 0f, 0f);
                    gameObject.transform.localEulerAngles = new Vector3(0f, 0f, 0f);
                    gameObject.transform.localScale = new Vector3(1f, 1f, 1f);
                }
            }
        }
        else
        {
            for (int j = 0; j < 2; j++)
            {
                GameObject gameObject2 = objAcsMove[slotNo, j];
                if (!(null == gameObject2))
                {
                    gameObject2.transform.localPosition = new Vector3(accessory.parts[slotNo].addMove[j, 0].x * 0.01f, accessory.parts[slotNo].addMove[j, 0].y * 0.01f, accessory.parts[slotNo].addMove[j, 0].z * 0.01f);
                    gameObject2.transform.localEulerAngles = new Vector3(accessory.parts[slotNo].addMove[j, 1].x, accessory.parts[slotNo].addMove[j, 1].y, accessory.parts[slotNo].addMove[j, 1].z);
                    gameObject2.transform.localScale = new Vector3(accessory.parts[slotNo].addMove[j, 2].x, accessory.parts[slotNo].addMove[j, 2].y, accessory.parts[slotNo].addMove[j, 2].z);
                }
            }
        }
        return true;
    }

    public bool UpdateAccessoryMoveAllFromInfo()
    {
        for (int i = 0; i < 20; i++)
            UpdateAccessoryMoveFromInfo(i);
        return true;
    }

    public bool ChangeAccessoryColor(int slotNo)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
            return false;
        ChaAccessoryComponent chaAccessoryComponent = cusAcsCmp[slotNo];
        ChaFileAccessory.PartsInfo partsInfo = nowCoordinate.accessory.parts[slotNo];
        if (null == chaAccessoryComponent)
            return false;
        if (chaAccessoryComponent.rendNormal != null)
        {
            foreach (Renderer renderer in chaAccessoryComponent.rendNormal)
            {
                if (chaAccessoryComponent.useColor01)
                    renderer.material.SetColor(ChaShader._Color, partsInfo.color[0]);
                if (chaAccessoryComponent.useColor02)
                    renderer.material.SetColor(ChaShader._Color2, partsInfo.color[1]);
                if (chaAccessoryComponent.useColor03)
                    renderer.material.SetColor(ChaShader._Color3, partsInfo.color[2]);
            }
        }
        if (chaAccessoryComponent.rendAlpha != null)
        {
            foreach (Renderer renderer2 in chaAccessoryComponent.rendAlpha)
            {
                renderer2.material.SetColor(ChaShader._Color4, partsInfo.color[3]);
                renderer2.gameObject.SetActiveIfDifferent(partsInfo.color[3].a != 0f);
            }
        }
        if (chaAccessoryComponent.rendHair != null)
        {
            foreach (Renderer renderer3 in chaAccessoryComponent.rendHair)
            {
                renderer3.material.SetColor(ChaShader._Color, fileHair.parts[0].startColor);
                renderer3.material.SetColor(ChaShader._Color2, fileHair.parts[0].startColor);
                renderer3.material.SetColor(ChaShader._Color3, fileHair.parts[0].startColor);
            }
        }
        return true;
    }

    public bool GetAccessoryDefaultColor(ref Color color, int slotNo, int no)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19)) return false;
        ChaAccessoryComponent chaAccessoryComponent = cusAcsCmp[slotNo];
        if (null == chaAccessoryComponent) return false;
        if (no == 0 && chaAccessoryComponent.useColor01)
        {
            color = chaAccessoryComponent.defColor01;
            return true;
        }
        if (no == 1 && chaAccessoryComponent.useColor02)
        {
            color = chaAccessoryComponent.defColor02;
            return true;
        }
        if (no == 2 && chaAccessoryComponent.useColor03)
        {
            color = chaAccessoryComponent.defColor03;
            return true;
        }
        if (no == 3 && chaAccessoryComponent.rendAlpha != null && chaAccessoryComponent.rendAlpha.Length != 0)
        {
            color = chaAccessoryComponent.defColor04;
            return true;
        }
        return false;
    }

    public void SetAccessoryDefaultColor(int slotNo)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19)) return;
        ChaAccessoryComponent chaAccessoryComponent = cusAcsCmp[slotNo];
        if (!(null == chaAccessoryComponent))
        {
            if (chaAccessoryComponent.useColor01)
                nowCoordinate.accessory.parts[slotNo].color[0] = chaAccessoryComponent.defColor01;
            if (chaAccessoryComponent.useColor02)
                nowCoordinate.accessory.parts[slotNo].color[1] = chaAccessoryComponent.defColor02;
            if (chaAccessoryComponent.useColor03)
                nowCoordinate.accessory.parts[slotNo].color[2] = chaAccessoryComponent.defColor03;
            if (chaAccessoryComponent.rendAlpha != null && chaAccessoryComponent.rendAlpha.Length != 0)
                nowCoordinate.accessory.parts[slotNo].color[3] = chaAccessoryComponent.defColor04;
        }
    }

    public void ChangeShakeAccessory(int slotNo)
    {
        if ((bool)objAccessory[slotNo])
        {
            DynamicBone[] componentsInChildren = objAccessory[slotNo].GetComponentsInChildren<DynamicBone>(true);
            DynamicBone[] array = componentsInChildren;
            foreach (DynamicBone dynamicBone in array)
                dynamicBone.enabled = !nowCoordinate.accessory.parts[slotNo].noShake;
        }
    }

    public void SetHideHairAccessory()
    {
        for (int i = 0; i < hideHairAcs.Length; i++)
            hideHairAcs[i] = false;
        for (int j = 0; j < infoAccessory.Length; j++)
            if (infoAccessory[j] != null && infoAccessory[j].GetInfoInt(ChaListDefine.KeyType.HideHair) != 0)
                hideHairAcs[j] = true;
    }

    public bool CheckHideHair()
    {
        for (int i = 0; i < hideHairAcs.Length; i++)
            if (hideHairAcs[i] && fileStatus.showAccessory[i])
                return true;
        return false;
    }

    public bool ChangeCustomClothes(bool main, int kind, bool updateColor, bool updateTex01, bool updateTex02, bool updateTex03, bool updateTex04)
    {
        CustomTextureCreate[] array = new CustomTextureCreate[3]
        {
            !main ? ctCreateClothesSub[kind, 0] : ctCreateClothes[kind, 0],
            !main ? ctCreateClothesSub[kind, 1] : ctCreateClothes[kind, 1],
            !main ? ctCreateClothesSub[kind, 2] : ctCreateClothes[kind, 2]
        };
        if (array[0] == null) return false;
        ChaClothesComponent chaClothesComponent = !main ? GetCustomClothesSubComponent(kind) : GetCustomClothesComponent(kind);
        if (null == chaClothesComponent) return false;
        ChaFileClothes.PartsInfo partsInfo = !main ? nowCoordinate.clothes.parts[0] : nowCoordinate.clothes.parts[kind];
        if (main)
        {
            if (!updateColor && !updateTex01 && !updateTex02 && !updateTex03)
                return false;
        }
        else if (!updateColor && !updateTex01 && !updateTex02 && !updateTex03 && !updateTex04)
        {
            return false;
        }
        bool result = true;
        int[] array2 = new int[4]
        {
            ChaShader._PatternMask1,
            ChaShader._PatternMask2,
            ChaShader._PatternMask3,
            ChaShader._PatternMask1
        };
        bool[] array3 = new bool[4]
        {
            updateTex01,
            updateTex02,
            updateTex03,
            updateTex04
        };
        int num = main || kind != 2 ? 3 : 4;
        for (int i = 0; i < num; i++)
        {
            if (!array3[i])
                continue;
            Texture2D texture2D = null;
            ListInfoBase listInfo = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.mt_pattern, partsInfo.colorInfo[i].pattern);
            if (listInfo != null)
            {
                string info = listInfo.GetInfo(ChaListDefine.KeyType.MainTexAB);
                string text = listInfo.GetInfo(ChaListDefine.KeyType.MainTex);
                if ("0" != info && "0" != text)
                {
                    if (!hiPoly)
                        text += "_low";
                    texture2D = CommonLib.LoadAsset<Texture2D>(info, text, false, string.Empty);
                    Singleton<Character>.Instance.AddLoadAssetBundle(info, string.Empty);
                }
            }
            if (null != texture2D)
            {
                CustomTextureCreate[] array4 = array;
                foreach (CustomTextureCreate customTextureCreate in array4)
                {
                    if (customTextureCreate != null)
                        customTextureCreate.SetTexture(array2[i], texture2D);
                }
                continue;
            }
            CustomTextureCreate[] array5 = array;
            foreach (CustomTextureCreate customTextureCreate2 in array5)
            {
                if (customTextureCreate2 != null)
                    customTextureCreate2.SetTexture(array2[i], null);
            }
        }
        if (updateColor)
        {
            CustomTextureCreate[] array6 = array;
            foreach (CustomTextureCreate customTextureCreate3 in array6)
            {
                if (customTextureCreate3 != null)
                {
                    if (!main && kind == 2)
                    {
                        customTextureCreate3.SetColor(ChaShader._Color, partsInfo.colorInfo[3].baseColor);
                        customTextureCreate3.SetColor(ChaShader._Color1_2, partsInfo.colorInfo[3].patternColor);
                        int patternScale1u = ChaShader._PatternScale1u;
                        Vector2 tiling = partsInfo.colorInfo[3].tiling;
                        customTextureCreate3.SetFloat(patternScale1u, tiling.x);
                        int patternScale1v = ChaShader._PatternScale1v;
                        Vector2 tiling2 = partsInfo.colorInfo[3].tiling;
                        customTextureCreate3.SetFloat(patternScale1v, tiling2.y);
                    }
                    else
                    {
                        customTextureCreate3.SetColor(ChaShader._Color, partsInfo.colorInfo[0].baseColor);
                        customTextureCreate3.SetColor(ChaShader._Color1_2, partsInfo.colorInfo[0].patternColor);
                        int patternScale1u2 = ChaShader._PatternScale1u;
                        Vector2 tiling3 = partsInfo.colorInfo[0].tiling;
                        customTextureCreate3.SetFloat(patternScale1u2, tiling3.x);
                        int patternScale1v2 = ChaShader._PatternScale1v;
                        Vector2 tiling4 = partsInfo.colorInfo[0].tiling;
                        customTextureCreate3.SetFloat(patternScale1v2, tiling4.y);
                    }
                    customTextureCreate3.SetColor(ChaShader._Color2, partsInfo.colorInfo[1].baseColor);
                    customTextureCreate3.SetColor(ChaShader._Color2_2, partsInfo.colorInfo[1].patternColor);
                    int patternScale2u = ChaShader._PatternScale2u;
                    Vector2 tiling5 = partsInfo.colorInfo[1].tiling;
                    customTextureCreate3.SetFloat(patternScale2u, tiling5.x);
                    int patternScale2v = ChaShader._PatternScale2v;
                    Vector2 tiling6 = partsInfo.colorInfo[1].tiling;
                    customTextureCreate3.SetFloat(patternScale2v, tiling6.y);
                    customTextureCreate3.SetColor(ChaShader._Color3, partsInfo.colorInfo[2].baseColor);
                    customTextureCreate3.SetColor(ChaShader._Color3_2, partsInfo.colorInfo[2].patternColor);
                    int patternScale3u = ChaShader._PatternScale3u;
                    Vector2 tiling7 = partsInfo.colorInfo[2].tiling;
                    customTextureCreate3.SetFloat(patternScale3u, tiling7.x);
                    int patternScale3v = ChaShader._PatternScale3v;
                    Vector2 tiling8 = partsInfo.colorInfo[2].tiling;
                    customTextureCreate3.SetFloat(patternScale3v, tiling8.y);
                }
            }
        }
        bool flag = chaClothesComponent.rendNormal01 != null && 0 != chaClothesComponent.rendNormal01.Length;
        bool flag2 = chaClothesComponent.rendAlpha01 != null && 0 != chaClothesComponent.rendAlpha01.Length;
        if (flag || flag2)
        {
            Texture texture = array[0].RebuildTextureAndSetMaterial();
            if (null != texture)
            {
                if (flag)
                {
                    for (int m = 0; m < chaClothesComponent.rendNormal01.Length; m++)
                    {
                        if ((bool)chaClothesComponent.rendNormal01[m])
                            chaClothesComponent.rendNormal01[m].material.SetTexture(ChaShader._MainTex, texture);
                        else
                            result = false;
                    }
                }
                if (flag2)
                {
                    for (int n = 0; n < chaClothesComponent.rendAlpha01.Length; n++)
                    {
                        if ((bool)chaClothesComponent.rendAlpha01[n])
                            chaClothesComponent.rendAlpha01[n].material.SetTexture(ChaShader._MainTex, texture);
                        else
                            result = false;
                    }
                }
            }
            else
                result = false;
        }
        else
        {
            result = false;
        }
        if (chaClothesComponent.rendNormal02 != null && chaClothesComponent.rendNormal02.Length != 0 && array[1] != null)
        {
            Texture texture2 = array[1].RebuildTextureAndSetMaterial();
            if (null != texture2)
            {
                for (int num2 = 0; num2 < chaClothesComponent.rendNormal02.Length; num2++)
                {
                    if ((bool)chaClothesComponent.rendNormal02[num2])
                        chaClothesComponent.rendNormal02[num2].material.SetTexture(ChaShader._MainTex, texture2);
                    else
                        result = false;
                }
            }
            else
                result = false;
        }
        if (chaClothesComponent.rendNormal03 != null && chaClothesComponent.rendNormal03.Length != 0 && array[2] != null)
        {
            Texture texture3 = array[2].RebuildTextureAndSetMaterial();
            if (null != texture3)
            {
                for (int num3 = 0; num3 < chaClothesComponent.rendNormal03.Length; num3++)
                {
                    if ((bool)chaClothesComponent.rendNormal03[num3])
                        chaClothesComponent.rendNormal03[num3].material.SetTexture(ChaShader._MainTex, texture3);
                    else
                        result = false;
                }
            }
            else
                result = false;
        }
        if (null != chaClothesComponent.rendAccessory && array[0] != null)
        {
            Texture texture4 = array[0].RebuildTextureAndSetMaterial();
            if (null != texture4)
            {
                if ((bool)chaClothesComponent.rendAccessory)
                    chaClothesComponent.rendAccessory.material.SetTexture(ChaShader._MainTex, texture4);
                else
                    result = false;
            }
            else
                result = false;
        }
        return result;
    }

    public bool UpdateClothesSiru(int kind, float frontTop, float frontBot, float downTop, float downBot)
    {
        int num = 0;
        if (kind == 0)
        {
            Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
            ListInfoBase value;
            if (categoryInfo.TryGetValue(nowCoordinate.clothes.parts[0].id, out value))
                num = value.Kind;
        }
        ChaClothesComponent[] array;
        switch (num)
        {
            case 1:
                array = new ChaClothesComponent[1]
                {
                GetCustomClothesSubComponent(0)
                };
                if (null == array[0])
                    return false;
                break;
            case 2:
                array = new ChaClothesComponent[2]
                {
                GetCustomClothesSubComponent(0),
                GetCustomClothesSubComponent(1)
                };
                if (null == array[0] && null == array[1])
                    return false;
                break;
            default:
                array = new ChaClothesComponent[1]
                {
                GetCustomClothesComponent(kind)
                };
                if (null == array[0])
                    return false;
                break;
        }
        ChaClothesComponent[] array2 = array;
        foreach (ChaClothesComponent chaClothesComponent in array2)
        {
            if (null == chaClothesComponent) continue;
            if (chaClothesComponent.rendNormal01 != null && chaClothesComponent.rendNormal01.Length != 0)
            {
                Renderer[] rendNormal = chaClothesComponent.rendNormal01;
                foreach (Renderer renderer in rendNormal)
                {
                    if (!(null == renderer))
                    {
                        renderer.material.SetFloat(ChaShader._liquidftop, frontTop);
                        renderer.material.SetFloat(ChaShader._liquidfbot, frontBot);
                        renderer.material.SetFloat(ChaShader._liquidbtop, downTop);
                        renderer.material.SetFloat(ChaShader._liquidbbot, downBot);
                    }
                }
            }
            if (chaClothesComponent.rendNormal02 != null && chaClothesComponent.rendNormal02.Length != 0)
            {
                Renderer[] rendNormal2 = chaClothesComponent.rendNormal02;
                foreach (Renderer renderer2 in rendNormal2)
                {
                    if (!(null == renderer2))
                    {
                        renderer2.material.SetFloat(ChaShader._liquidftop, frontTop);
                        renderer2.material.SetFloat(ChaShader._liquidfbot, frontBot);
                        renderer2.material.SetFloat(ChaShader._liquidbtop, downTop);
                        renderer2.material.SetFloat(ChaShader._liquidbbot, downBot);
                    }
                }
            }
            if (chaClothesComponent.rendNormal03 == null || chaClothesComponent.rendNormal03.Length == 0)
            {
                continue;
            }
            Renderer[] rendNormal3 = chaClothesComponent.rendNormal03;
            foreach (Renderer renderer3 in rendNormal3)
            {
                if (!(null == renderer3))
                {
                    renderer3.material.SetFloat(ChaShader._liquidftop, frontTop);
                    renderer3.material.SetFloat(ChaShader._liquidfbot, frontBot);
                    renderer3.material.SetFloat(ChaShader._liquidbtop, downTop);
                    renderer3.material.SetFloat(ChaShader._liquidbbot, downBot);
                }
            }
        }
        return true;
    }

    public Color GetClothesDefaultColor(int kind, int no)
    {
        int num = 0;
        if (kind == 0)
        {
            Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
            ListInfoBase value;
            if (categoryInfo.TryGetValue(nowCoordinate.clothes.parts[0].id, out value))
            {
                num = value.Kind;
            }
        }
        if (num == 1 || num == 2)
        {
            return ChaFileDefine.defClothesSubColor[no];
        }
        ChaClothesComponent customClothesComponent = GetCustomClothesComponent(kind);
        if (null != customClothesComponent)
        {
            switch (no)
            {
                case 0:
                    return customClothesComponent.defMainColor01;
                case 1:
                    return customClothesComponent.defMainColor02;
                case 2:
                    return customClothesComponent.defMainColor03;
            }
        }
        return Color.white;
    }

    public void SetClothesDefaultColor(int kind)
    {
        int num = 0;
        if (kind == 0)
        {
            Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
            ListInfoBase value;
            if (categoryInfo.TryGetValue(nowCoordinate.clothes.parts[0].id, out value))
            {
                num = value.Kind;
            }
        }
        if (num == 1 || num == 2)
        {
            for (int i = 0; i < 4; i++)
            {
                nowCoordinate.clothes.parts[kind].colorInfo[i].baseColor = ChaFileDefine.defClothesSubColor[i];
            }
            return;
        }
        ChaClothesComponent customClothesComponent = GetCustomClothesComponent(kind);
        if (null != customClothesComponent)
        {
            nowCoordinate.clothes.parts[kind].colorInfo[0].baseColor = customClothesComponent.defMainColor01;
            nowCoordinate.clothes.parts[kind].colorInfo[1].baseColor = customClothesComponent.defMainColor02;
            nowCoordinate.clothes.parts[kind].colorInfo[2].baseColor = customClothesComponent.defMainColor03;
        }
    }

    public bool IsEmblem(int kind, int arrNo)
    {
        int num = 0;
        if (kind == 0)
        {
            Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
            ListInfoBase value;
            if (categoryInfo.TryGetValue(nowCoordinate.clothes.parts[0].id, out value))
            {
                num = value.Kind;
            }
        }
        ChaClothesComponent[] array;
        if (num != 1 && num != 2)
        {
            array = new ChaClothesComponent[1]
            {
                GetCustomClothesComponent(kind)
            };
        }
        else
        {
            array = new ChaClothesComponent[Enum.GetNames(typeof(ChaFileDefine.ClothesSubKind)).Length];
            for (int i = 0; i < array.Length; i++)
            {
                array[i] = GetCustomClothesSubComponent(i);
            }
        }
        return IsEmblem(array, arrNo);
    }

    public bool IsEmblem(ChaClothesComponent[] ccc, int arrNo)
    {
        if (ccc == null || ccc.Length == 0)
        {
            return false;
        }
        foreach (ChaClothesComponent chaClothesComponent in ccc)
        {
            if (null == chaClothesComponent)
            {
                continue;
            }
            switch (arrNo)
            {
                case 0:
                    if (null == chaClothesComponent.rendEmblem01 && (chaClothesComponent.exRendEmblem01 == null || chaClothesComponent.exRendEmblem01.Length == 0))
                    {
                        continue;
                    }
                    break;
                case 1:
                    if (chaClothesComponent.exRendEmblem02 == null || chaClothesComponent.exRendEmblem02.Length == 0)
                    {
                        continue;
                    }
                    break;
            }
            return true;
        }
        return false;
    }

    public bool ChangeCustomEmblem(int kind, int arrNo)
    {
        int num = 0;
        ListInfoBase value;
        if (kind == 0)
        {
            Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
            if (categoryInfo.TryGetValue(nowCoordinate.clothes.parts[0].id, out value))
            {
                num = value.Kind;
            }
        }
        ChaClothesComponent[] array;
        if (num != 1 && num != 2)
        {
            array = new ChaClothesComponent[1]
            {
                GetCustomClothesComponent(kind)
            };
        }
        else
        {
            array = new ChaClothesComponent[Enum.GetNames(typeof(ChaFileDefine.ClothesSubKind)).Length];
            for (int i = 0; i < array.Length; i++)
            {
                array[i] = GetCustomClothesSubComponent(i);
            }
        }
        if (!IsEmblem(array, arrNo))
        {
            return false;
        }
        ChaFileClothes.PartsInfo partsInfo = nowCoordinate.clothes.parts[kind];
        Texture2D value2 = null;
        int[] array2 = new int[2]
        {
            partsInfo.emblemeId,
            partsInfo.emblemeId2
        };
        value = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.mt_emblem, array2[arrNo]);
        if (value != null)
        {
            string info = value.GetInfo(ChaListDefine.KeyType.MainTexAB);
            string text = value.GetInfo(ChaListDefine.KeyType.MainTex);
            if ("0" != info && "0" != text)
            {
                if (!hiPoly)
                {
                    text += "_low";
                }
                value2 = CommonLib.LoadAsset<Texture2D>(info, text, false, string.Empty);
                Singleton<Character>.Instance.AddLoadAssetBundle(info, string.Empty);
            }
        }
        ChaClothesComponent[] array3 = array;
        foreach (ChaClothesComponent chaClothesComponent in array3)
        {
            if (null == chaClothesComponent)
            {
                continue;
            }
            switch (arrNo)
            {
                case 0:
                    if (null != chaClothesComponent.rendEmblem01)
                    {
                        chaClothesComponent.rendEmblem01.material.SetTexture(ChaShader._MainTex, value2);
                    }
                    if (null != chaClothesComponent.rendEmblem02)
                    {
                        chaClothesComponent.rendEmblem02.material.SetTexture(ChaShader._MainTex, value2);
                    }
                    if (chaClothesComponent.exRendEmblem01 != null)
                    {
                        Renderer[] exRendEmblem2 = chaClothesComponent.exRendEmblem01;
                        foreach (Renderer renderer2 in exRendEmblem2)
                        {
                            renderer2.material.SetTexture(ChaShader._MainTex, value2);
                        }
                    }
                    break;
                case 1:
                    if (chaClothesComponent.exRendEmblem02 != null)
                    {
                        Renderer[] exRendEmblem = chaClothesComponent.exRendEmblem02;
                        foreach (Renderer renderer in exRendEmblem)
                        {
                            renderer.material.SetTexture(ChaShader._MainTex, value2);
                        }
                    }
                    break;
            }
        }
        return true;
    }

    protected void InitializeControlCustomAll()
    {
        ShapeFaceNum = ChaFileDefine.cf_headshapename.Length;
        ShapeBodyNum = ChaFileDefine.cf_bodyshapename.Length;
        InitializeControlCustomObject();
    }

    protected void InitializeControlCustomObject()
    {
        sibFace = new ShapeHeadInfoFemale();
        sibBody = new ShapeBodyInfoFemale();
        changeShapeBodyMask = false;
        bustSoft = new BustSoft(this);
        bustGravity = new BustGravity(this);
        updateCMFaceTex = new bool[Enum.GetNames(typeof(FaceTexKind)).Length];
        updateCMFaceColor = new bool[Enum.GetNames(typeof(FaceTexKind)).Length];
        updateCMFaceLayout = new bool[Enum.GetNames(typeof(FaceTexKind)).Length];
        updateCMBodyTex = new bool[Enum.GetNames(typeof(BodyTexKind)).Length];
        updateCMBodyColor = new bool[Enum.GetNames(typeof(BodyTexKind)).Length];
        updateCMBodyLayout = new bool[Enum.GetNames(typeof(BodyTexKind)).Length];
    }

    protected void ReleaseControlCustomAll()
    {
        ReleaseControlCustomObject();
    }

    protected void ReleaseControlCustomObject(bool init = true)
    {
        if (sibFace != null)
        {
            sibFace.ReleaseShapeInfo();
        }
        if (sibBody != null)
        {
            sibBody.ReleaseShapeInfo();
        }
        if (init)
        {
            InitializeControlCustomObject();
        }
    }

    private bool SetBaseMaterial(Renderer rend, Material mat)
    {
        if (null == mat || null == rend)
        {
            return false;
        }
        int num = rend.materials.Length;
        Material[] array;
        if (num == 0)
        {
            array = new Material[1]
            {
                mat
            };
        }
        else
        {
            array = new Material[num];
            for (int i = 0; i < num; i++)
            {
                array[i] = rend.materials[i];
            }
            Material material = array[0];
            array[0] = mat;
            if (material != mat)
            {
                Destroy(material);
            }
        }
        rend.materials = array;
        return true;
    }

    private bool SetCreateTexture(CustomTextureCreate ctc, bool main, ChaListDefine.CategoryNo type, int id, ChaListDefine.KeyType assetBundleKey, ChaListDefine.KeyType assetKey, int propertyID)
    {
        ListInfoBase listInfo = lstCtrl.GetListInfo(type, id);
        if (listInfo != null)
        {
            string info = listInfo.GetInfo(assetBundleKey);
            string text = listInfo.GetInfo(assetKey);
            Texture2D texture2D = null;
            if ("0" != info && "0" != text)
            {
                if (!hiPoly)
                {
                    text += "_low";
                }
                texture2D = CommonLib.LoadAsset<Texture2D>(info, text, false, string.Empty);
                Singleton<Character>.Instance.AddLoadAssetBundle(info, string.Empty);
            }
            if (main)
            {
                ctc.SetMainTexture(texture2D);
            }
            else
            {
                ctc.SetTexture(propertyID, texture2D);
            }
            return true;
        }
        return false;
    }

    private void ChangeTexture(Renderer rend, ChaListDefine.CategoryNo type, int id, ChaListDefine.KeyType assetBundleKey, ChaListDefine.KeyType assetKey, int propertyID, string addStr = "")
    {
        if (!(null == rend))
        {
            ChangeTexture(rend.material, type, id, assetBundleKey, assetKey, propertyID, addStr);
        }
    }

    private void ChangeTexture(Material mat, ChaListDefine.CategoryNo type, int id, ChaListDefine.KeyType assetBundleKey, ChaListDefine.KeyType assetKey, int propertyID, string addStr = "")
    {
        if (!(null == mat))
        {
            Texture2D texture = GetTexture(type, id, assetBundleKey, assetKey, addStr);
            mat.SetTexture(propertyID, texture);
        }
    }

    private Texture2D GetTexture(ChaListDefine.CategoryNo type, int id, ChaListDefine.KeyType assetBundleKey, ChaListDefine.KeyType assetKey, string addStr = "")
    {
        ListInfoBase listInfo = lstCtrl.GetListInfo(type, id);
        if (listInfo != null)
        {
            string text = listInfo.GetInfo(assetBundleKey);
            string text2 = listInfo.GetInfo(assetKey);
            Texture2D texture2D = null;
            if ("0" != text && "0" != text2)
            {
                if (!addStr.IsNullOrEmpty())
                {
                    string str = text.Replace(".unity3d", string.Empty);
                    text = YS_Assist.GetRemoveStringRight(str, 2) + "50.unity3d";
                    string text3 = text2 + addStr;
                    if (!hiPoly)
                    {
                        text3 += "_low";
                    }
                    texture2D = CommonLib.LoadAsset<Texture2D>(text, text3, false, string.Empty);
                }
                if (null == texture2D)
                {
                    if (!hiPoly)
                    {
                        text2 += "_low";
                    }
                    texture2D = CommonLib.LoadAsset<Texture2D>(text, text2, false, string.Empty);
                }
                Singleton<Character>.Instance.AddLoadAssetBundle(text, string.Empty);
            }
            return texture2D;
        }
        return null;
    }

    public void AddUpdateCMFaceTexFlags(bool inpBase, bool inpSub, bool inpPaint01, bool inpPaint02, bool inpCheek, bool inpLipLine, bool inpMole)
    {
        if (inpBase)
        {
            updateCMFaceTex[0] = inpBase;
        }
        if (inpSub)
        {
            updateCMFaceTex[1] = inpSub;
        }
        if (inpPaint01)
        {
            updateCMFaceTex[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMFaceTex[3] = inpPaint02;
        }
        if (inpCheek)
        {
            updateCMFaceTex[4] = inpCheek;
        }
        if (inpLipLine)
        {
            updateCMFaceTex[5] = inpLipLine;
        }
        if (inpMole)
        {
            updateCMFaceTex[6] = inpMole;
        }
    }

    public void AddUpdateCMFaceColorFlags(bool inpBase, bool inpSub, bool inpPaint01, bool inpPaint02, bool inpCheek, bool inpLipLine, bool inpMole)
    {
        if (inpBase)
        {
            updateCMFaceColor[0] = inpBase;
        }
        if (inpSub)
        {
            updateCMFaceColor[1] = inpSub;
        }
        if (inpPaint01)
        {
            updateCMFaceColor[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMFaceColor[3] = inpPaint02;
        }
        if (inpCheek)
        {
            updateCMFaceColor[4] = inpCheek;
        }
        if (inpLipLine)
        {
            updateCMFaceColor[5] = inpLipLine;
        }
        if (inpMole)
        {
            updateCMFaceColor[6] = inpMole;
        }
    }

    public void AddUpdateCMFaceLayoutFlags(bool inpPaint01, bool inpPaint02, bool inpMole)
    {
        if (inpPaint01)
        {
            updateCMFaceLayout[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMFaceLayout[3] = inpPaint02;
        }
        if (inpMole)
        {
            updateCMFaceLayout[6] = inpMole;
        }
    }

    public bool CreateFaceTexture()
    {
        ChaFileMakeup chaFileMakeup = !nowCoordinate.enableMakeup ? fileFace.baseMakeup : nowCoordinate.makeup;
        bool flag = false;
        bool flag2 = sex == 0 && exType == 1;
        ListInfoBase listInfoBase;
        if (updateCMFaceTex[0])
        {
            if ((bool)objHead)
            {
                ListInfoComponent component = objHead.GetComponent<ListInfoComponent>();
                if (null != component)
                {
                    listInfoBase = component.data;
                    string info = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTexAB);
                    string text = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex);
                    string info2 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
                    if (flag2)
                    {
                        listInfoBase = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_head, 0);
                        if (listInfoBase != null)
                        {
                            info = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTexAB);
                            text = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex);
                            info2 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
                        }
                    }
                    if ("0" != text && !hiPoly)
                    {
                        text += "_low";
                    }
                    Texture2D mainTexture = CommonLib.LoadAsset<Texture2D>(info, text, false, info2);
                    Singleton<Character>.Instance.AddLoadAssetBundle(info, string.Empty);
                    customTexCtrlFace.SetMainTexture(mainTexture);
                    flag = true;
                }
            }
            updateCMFaceTex[0] = false;
        }
        if (updateCMFaceColor[0])
        {
            customTexCtrlFace.SetColor(ChaShader._Color, fileBody.skinMainColor);
            flag = true;
            updateCMFaceColor[0] = false;
        }
        if (updateCMFaceTex[1])
        {
            if ((bool)objHead)
            {
                ListInfoComponent component2 = objHead.GetComponent<ListInfoComponent>();
                if (null != component2)
                {
                    listInfoBase = component2.data;
                    string info3 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskAB);
                    string text2 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskTex);
                    string info4 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
                    if (flag2)
                    {
                        listInfoBase = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_head, 0);
                        if (listInfoBase != null)
                        {
                            info3 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskAB);
                            text2 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskTex);
                            info4 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
                        }
                    }
                    if ("0" != text2 && !hiPoly)
                    {
                        text2 += "_low";
                    }
                    Texture2D tex = CommonLib.LoadAsset<Texture2D>(info3, text2, false, info4);
                    Singleton<Character>.Instance.AddLoadAssetBundle(info3, string.Empty);
                    customTexCtrlFace.SetTexture(ChaShader._ColorMask, tex);
                    flag = true;
                }
            }
            updateCMFaceTex[1] = false;
        }
        if (updateCMFaceColor[1])
        {
            customTexCtrlFace.SetColor(ChaShader._Color2, fileBody.skinSubColor);
            flag = true;
            updateCMFaceColor[1] = false;
        }
        if (!flag2)
        {
            if (updateCMFaceTex[2])
            {
                if (SetCreateTexture(customTexCtrlFace, false, ChaListDefine.CategoryNo.mt_face_paint, chaFileMakeup.paintId[0], ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.PaintTex, ChaShader._Texture3))
                {
                    flag = true;
                }
                updateCMFaceTex[2] = false;
            }
            if (updateCMFaceColor[2])
            {
                customTexCtrlFace.SetColor(ChaShader._Color3, chaFileMakeup.paintColor[0]);
                updateCMFaceColor[2] = false;
                flag = true;
            }
            if (updateCMFaceLayout[2])
            {
                Vector4 zero = Vector4.zero;
                zero.x = Mathf.Lerp(0.25f, -0.25f, chaFileMakeup.paintLayout[0].x);
                zero.y = Mathf.Lerp(0.3f, -0.3f, chaFileMakeup.paintLayout[0].y);
                zero.z = Mathf.Lerp(1f, -1f, chaFileMakeup.paintLayout[0].z);
                zero.w = Mathf.Lerp(-8f, 0.7f, chaFileMakeup.paintLayout[0].w);
                customTexCtrlFace.SetVector4(ChaShader._paint1, zero);
                updateCMFaceLayout[2] = false;
                flag = true;
            }
            if (updateCMFaceTex[3])
            {
                if (SetCreateTexture(customTexCtrlFace, false, ChaListDefine.CategoryNo.mt_face_paint, chaFileMakeup.paintId[1], ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.PaintTex, ChaShader._Texture7))
                {
                    flag = true;
                }
                updateCMFaceTex[3] = false;
            }
            if (updateCMFaceColor[3])
            {
                customTexCtrlFace.SetColor(ChaShader._Color7, chaFileMakeup.paintColor[1]);
                updateCMFaceColor[3] = false;
                flag = true;
            }
            if (updateCMFaceLayout[3])
            {
                Vector4 zero2 = Vector4.zero;
                zero2.x = Mathf.Lerp(0.25f, -0.25f, chaFileMakeup.paintLayout[1].x);
                zero2.y = Mathf.Lerp(0.3f, -0.3f, chaFileMakeup.paintLayout[1].y);
                zero2.z = Mathf.Lerp(1f, -1f, chaFileMakeup.paintLayout[1].z);
                zero2.w = Mathf.Lerp(-8f, 0.7f, chaFileMakeup.paintLayout[1].w);
                customTexCtrlFace.SetVector4(ChaShader._paint2, zero2);
                updateCMFaceLayout[3] = false;
                flag = true;
            }
            if (updateCMFaceTex[4])
            {
                if (SetCreateTexture(customTexCtrlFace, false, ChaListDefine.CategoryNo.mt_cheek, chaFileMakeup.cheekId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.CheekTex, ChaShader._Texture4))
                {
                    flag = true;
                }
                updateCMFaceTex[4] = false;
            }
            if (updateCMFaceColor[4])
            {
                customTexCtrlFace.SetColor(ChaShader._Color4, chaFileMakeup.cheekColor);
                updateCMFaceColor[4] = false;
                flag = true;
            }
            if (updateCMFaceTex[5])
            {
                if (SetCreateTexture(customTexCtrlFace, false, ChaListDefine.CategoryNo.mt_lipline, fileFace.lipLineId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.LiplineTex, ChaShader._Texture5))
                {
                    flag = true;
                }
                updateCMFaceTex[5] = false;
            }
            if (updateCMFaceColor[5])
            {
                customTexCtrlFace.SetColor(ChaShader._Color5, fileFace.lipLineColor);
                updateCMFaceColor[5] = false;
                flag = true;
            }
            if (updateCMFaceTex[6])
            {
                if (SetCreateTexture(customTexCtrlFace, false, ChaListDefine.CategoryNo.mt_mole, fileFace.moleId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.MoleTex, ChaShader._Texture6))
                {
                    flag = true;
                }
                updateCMFaceTex[6] = false;
            }
            if (updateCMFaceColor[6])
            {
                customTexCtrlFace.SetColor(ChaShader._Color6, fileFace.moleColor);
                updateCMFaceColor[6] = false;
                flag = true;
            }
            if (updateCMFaceLayout[6])
            {
                Vector4 zero3 = Vector4.zero;
                Vector4 moleLayout = fileFace.moleLayout;
                zero3.x = Mathf.Lerp(0.25f, -0.25f, moleLayout.x);
                Vector4 moleLayout2 = fileFace.moleLayout;
                zero3.y = Mathf.Lerp(0.3f, -0.3f, moleLayout2.y);
                Vector4 moleLayout3 = fileFace.moleLayout;
                zero3.w = Mathf.Lerp(0f, 0.7f, moleLayout3.w);
                customTexCtrlFace.SetVector4(ChaShader._hokuro, zero3);
                updateCMFaceLayout[6] = false;
                flag = true;
            }
        }
        if (flag)
        {
            customTexCtrlFace.SetNewCreateTexture();
        }
        if (releaseCustomInputTexture)
        {
            ReleaseFaceCustomTexture();
        }
        return true;
    }

    public bool ChangeSettingWhiteOfEye(bool updateTex, bool updateColor)
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (ctCreateEyeW == null)
        {
            return false;
        }
        if (!updateTex && !updateColor)
        {
            return false;
        }
        bool result = true;
        if (updateTex && !SetCreateTexture(ctCreateEyeW, true, ChaListDefine.CategoryNo.mt_eye_white, fileFace.whiteId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyeWhiteTex, ChaShader._MainTex))
        {
            result = false;
        }
        if (updateColor)
        {
            ctCreateEyeW.SetColor(ChaShader._Color, fileFace.whiteBaseColor);
            ctCreateEyeW.SetColor(ChaShader._Color2, fileFace.whiteSubColor);
        }
        Texture texture = ctCreateEyeW.RebuildTextureAndSetMaterial();
        if (null != texture && rendEyeW != null && rendEyeW.Length == 2)
        {
            if (null != rendEyeW[0] && null != rendEyeW[1])
            {
                rendEyeW[0].material.SetTexture(ChaShader._MainTex, texture);
                rendEyeW[1].material = rendEyeW[0].material;
            }
            else
            {
                result = false;
            }
        }
        else
        {
            result = false;
        }
        if (releaseCustomInputTexture)
        {
            ctCreateEyeW.SetMainTexture(null);
            ctCreateEyeW.SetTexture(ChaShader._MainTex, null);
        }
        return result;
    }

    public bool ChangeSettingEyeL(bool updateBaseTex, bool updateMaskTex, bool updateColorAndOffset)
    {
        return ChangeSettingEye(0, updateBaseTex, updateMaskTex, updateColorAndOffset);
    }

    public bool ChangeSettingEyeR(bool updateBaseTex, bool updateMaskTex, bool updateColorAndOffset)
    {
        return ChangeSettingEye(1, updateBaseTex, updateMaskTex, updateColorAndOffset);
    }

    public bool ChangeSettingEye(bool updateBaseTex, bool updateMaskTex, bool updateColorAndOffset)
    {
        return ChangeSettingEye(2, updateBaseTex, updateMaskTex, updateColorAndOffset);
    }

    public bool ChangeSettingEye(byte lr, bool updateBaseTex, bool updateMaskTex, bool updateColorAndOffset)
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        bool result = true;
        bool[] array = new bool[2]
        {
            lr != 1,
            lr != 0
        };
        CustomTextureCreate[] array2 = new CustomTextureCreate[2]
        {
            ctCreateEyeL,
            ctCreateEyeR
        };
        for (int i = 0; i < 2; i++)
        {
            if (array[i])
            {
                if (array2[i] == null)
                {
                    return false;
                }
                if (null == rendEye[i])
                {
                    return false;
                }
                if (!updateBaseTex && !updateMaskTex && !updateColorAndOffset)
                {
                    return false;
                }
                if (updateBaseTex && !SetCreateTexture(array2[i], true, ChaListDefine.CategoryNo.mt_eye, fileFace.pupil[i].id, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyeTex, ChaShader._MainTex))
                {
                    result = false;
                }
                if (updateMaskTex && !SetCreateTexture(array2[i], false, ChaListDefine.CategoryNo.mt_eye_gradation, fileFace.pupil[i].gradMaskId, ChaListDefine.KeyType.ColorMaskAB, ChaListDefine.KeyType.ColorMaskTex, ChaShader._ColorMask))
                {
                    result = false;
                }
                if (updateColorAndOffset)
                {
                    array2[i].SetColor(ChaShader._Color, fileFace.pupil[i].baseColor);
                    array2[i].SetColor(ChaShader._Color2, fileFace.pupil[i].subColor);
                    array2[i].SetFloat(ChaShader._Blend, fileFace.pupil[i].gradBlend);
                    float y = Mathf.Lerp(-0.5f, 0.5f, fileFace.pupil[i].gradOffsetY);
                    float w = Mathf.Lerp(-1f, 1f, fileFace.pupil[i].gradScale);
                    array2[i].SetVector4(ChaShader._grad, new Vector4(0f, y, 0f, w));
                }
                Texture texture = array2[i].RebuildTextureAndSetMaterial();
                if (null != texture)
                {
                    rendEye[i].material.SetTexture(ChaShader._MainTex, texture);
                }
                if (releaseCustomInputTexture)
                {
                    array2[i].SetMainTexture(null);
                    array2[i].SetTexture(ChaShader._MainTex, null);
                    array2[i].SetTexture(ChaShader._ColorMask, null);
                }
            }
        }
        return result;
    }

    public bool ChangeSettingEyeHLUpPosY()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexHLUpOffsetY = Mathf.Lerp(0.1f, -0.1f, fileFace.hlUpY);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexHLUpOffsetY(eyeTexHLUpOffsetY);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeHLDownPosY()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexHLDownOffsetY = Mathf.Lerp(0.1f, -0.1f, fileFace.hlDownY);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexHLDownOffsetY(eyeTexHLDownOffsetY);
            }
        }
        return true;
    }

    public bool ChangeSettingEyePosX()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexOffsetX = Mathf.Lerp(0.2f, -0.6f, fileFace.pupilX);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexOffsetX(eyeTexOffsetX);
            }
        }
        return true;
    }

    public bool ChangeSettingEyePosY()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexOffsetY = Mathf.Lerp(-0.5f, 0.5f, fileFace.pupilY);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexOffsetY(eyeTexOffsetY);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeScaleWidth()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexScaleX = Mathf.Lerp(1.8f, -0.2f, fileFace.pupilWidth);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexScaleX(eyeTexScaleX);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeScaleHeight()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        float eyeTexScaleY = Mathf.Lerp(1.8f, -0.2f, fileFace.pupilHeight);
        for (int i = 0; i < fileFace.pupil.Length; i++)
        {
            if (!(null == eyeLookMatCtrl[i]))
            {
                eyeLookMatCtrl[i].SetEyeTexScaleY(eyeTexScaleY);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeTilt()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (rendEye == null)
        {
            return false;
        }
        int num = 33;
        float[] array = new float[2]
        {
            Mathf.Lerp(0.02f, -0.02f, fileFace.shapeValueFace[num]),
            Mathf.Lerp(-0.02f, 0.02f, fileFace.shapeValueFace[num])
        };
        for (int i = 0; i < rendEye.Length; i++)
        {
            if (!(null == rendEye[i]))
            {
                rendEye[i].material.SetFloat(ChaShader._rotation, array[i]);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeHiUp()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (rendEye == null)
        {
            return false;
        }
        for (int i = 0; i < 2; i++)
        {
            if (!(null == rendEye[i]))
            {
                ChangeTexture(rendEye[i], ChaListDefine.CategoryNo.mt_eye_hi_up, fileFace.hlUpId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyeHiUpTex, ChaShader._overtex1, string.Empty);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeHiUpColor()
    {
        if (rendEye == null)
        {
            return false;
        }
        for (int i = 0; i < 2; i++)
        {
            if (!(null == rendEye[i]))
            {
                rendEye[i].material.SetColor(ChaShader._overcolor1, fileFace.hlUpColor);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeHiDown()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (rendEye == null)
        {
            return false;
        }
        for (int i = 0; i < 2; i++)
        {
            if (!(null == rendEye[i]))
            {
                ChangeTexture(rendEye[i], ChaListDefine.CategoryNo.mt_eye_hi_down, fileFace.hlDownId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyeHiDownTex, ChaShader._overtex2, string.Empty);
            }
        }
        return true;
    }

    public bool ChangeSettingEyeHiDownColor()
    {
        if (rendEye == null)
        {
            return false;
        }
        for (int i = 0; i < 2; i++)
        {
            if (!(null == rendEye[i]))
            {
                rendEye[i].material.SetColor(ChaShader._overcolor2, fileFace.hlDownColor);
            }
        }
        return true;
    }

    public bool ChangeSettingEyelineUp()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (null == rendEyelineUp)
        {
            return false;
        }
        if (2 > rendEyelineUp.materials.Length)
        {
            return false;
        }
        string addStr = string.Empty;
        if (fileFace.headId != 0)
        {
            addStr = string.Format("_{0}", fileFace.headId);
        }
        ChangeTexture(rendEyelineUp.materials[0], ChaListDefine.CategoryNo.mt_eyeline_up, fileFace.eyelineUpId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyelineUpTex, ChaShader._MainTex, addStr);
        ChangeTexture(rendEyelineUp.materials[1], ChaListDefine.CategoryNo.mt_eyeline_up, fileFace.eyelineUpId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyelineShadowTex, ChaShader._MainTex, addStr);
        return true;
    }

    public bool ChangeSettingEyelineColor()
    {
        if (null != rendEyelineUp)
        {
            rendEyelineUp.materials[0].SetColor(ChaShader._Color, fileFace.eyelineColor);
        }
        if (null != rendEyelineDown)
        {
            rendEyelineDown.material.SetColor(ChaShader._Color, fileFace.eyelineColor);
        }
        return true;
    }

    public bool UpdateEyelineShadowColor()
    {
        if (null == rendEyelineUp)
        {
            return false;
        }
        if (2 > rendEyelineUp.materials.Length)
        {
            return false;
        }
        rendEyelineUp.materials[1].SetColor(ChaShader._Color, fileBody.skinMainColor);
        return true;
    }

    public bool ChangeSettingEyelineDown()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (null == rendEyelineDown)
        {
            return false;
        }
        string addStr = string.Empty;
        if (fileFace.headId != 0)
        {
            addStr = string.Format("_{0}", fileFace.headId);
        }
        ChangeTexture(rendEyelineDown, ChaListDefine.CategoryNo.mt_eyeline_down, fileFace.eyelineDownId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyelineDownTex, ChaShader._MainTex, addStr);
        return true;
    }

    public bool ChangeSettingEyebrow()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChangeTexture(rendEyebrow, ChaListDefine.CategoryNo.mt_eyebrow, fileFace.eyebrowId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyebrowTex, ChaShader._MainTex, string.Empty);
        return true;
    }

    public bool ChangeSettingEyebrowColor()
    {
        if (null == rendEyebrow)
        {
            return false;
        }
        rendEyebrow.material.SetColor(ChaShader._Color, fileFace.eyebrowColor);
        return true;
    }

    public bool ChangeSettingNose()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChangeTexture(rendNose, ChaListDefine.CategoryNo.mt_nose, fileFace.noseId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.NoseTex, ChaShader._MainTex, string.Empty);
        return true;
    }

    public bool VisibleDoubleTooth()
    {
        GameObject referenceInfo = GetReferenceInfo(RefObjKey.ObjDoubleTooth);
        if ((bool)referenceInfo)
        {
            referenceInfo.SetActiveIfDifferent(fileFace.doubleTooth);
        }
        return true;
    }

    public bool ChangeSettingFaceDetail()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        ChangeTexture(customTexCtrlFace.matDraw, ChaListDefine.CategoryNo.mt_face_detail, fileFace.detailId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.NormallMapDetail, ChaShader._NormalMapDetail, string.Empty);
        ChangeTexture(customTexCtrlFace.matDraw, ChaListDefine.CategoryNo.mt_face_detail, fileFace.detailId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.LineMask, ChaShader._LineMask, string.Empty);
        return true;
    }

    public bool ChangeSettingFaceDetailPower()
    {
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        customTexCtrlFace.matDraw.SetFloat(ChaShader._DetailNormalMapScale, fileFace.detailPower);
        return true;
    }

    public bool ChangeSettingEyeShadow()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileMakeup chaFileMakeup = !nowCoordinate.enableMakeup ? fileFace.baseMakeup : nowCoordinate.makeup;
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        ChangeTexture(customTexCtrlFace.matDraw, ChaListDefine.CategoryNo.mt_eyeshadow, chaFileMakeup.eyeshadowId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.EyeshadowTex, ChaShader._overtex3, string.Empty);
        return true;
    }

    public bool ChangeSettingEyeShadowColor()
    {
        ChaFileMakeup chaFileMakeup = !nowCoordinate.enableMakeup ? fileFace.baseMakeup : nowCoordinate.makeup;
        if (customTexCtrlFace == null || null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        if (IsGagEyes())
        {
            Color eyeshadowColor = chaFileMakeup.eyeshadowColor;
            eyeshadowColor.a = 0f;
            customTexCtrlFace.matDraw.SetColor(ChaShader._overcolor3, eyeshadowColor);
        }
        else
        {
            customTexCtrlFace.matDraw.SetColor(ChaShader._overcolor3, chaFileMakeup.eyeshadowColor);
        }
        return true;
    }

    public bool ChangeSettingLip()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileMakeup chaFileMakeup = !nowCoordinate.enableMakeup ? fileFace.baseMakeup : nowCoordinate.makeup;
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        ChangeTexture(customTexCtrlFace.matDraw, ChaListDefine.CategoryNo.mt_lip, chaFileMakeup.lipId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.LipTex, ChaShader._overtex1, string.Empty);
        return true;
    }

    public bool ChangeSettingLipColor()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileMakeup chaFileMakeup = !nowCoordinate.enableMakeup ? fileFace.baseMakeup : nowCoordinate.makeup;
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        customTexCtrlFace.matDraw.SetColor(ChaShader._overcolor1, chaFileMakeup.lipColor);
        return true;
    }

    public bool ChangeSettingCheekGlossPower()
    {
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        customTexCtrlFace.matDraw.SetFloat(ChaShader._SpecularPower, fileFace.cheekGlossPower);
        return true;
    }

    public bool ChangeSettingLipGlossPower()
    {
        if (null == customTexCtrlFace.matDraw)
        {
            return false;
        }
        customTexCtrlFace.matDraw.SetFloat(ChaShader._SpecularPowerNail, fileFace.lipGlossPower);
        return true;
    }

    public bool SetFaceBaseMaterial()
    {
        if (null == customMatFace)
        {
            return false;
        }
        return SetBaseMaterial(rendFace, customMatFace);
    }

    public bool ReleaseFaceCustomTexture()
    {
        if (customTexCtrlFace == null)
        {
            return false;
        }
        customTexCtrlFace.SetTexture(ChaShader._MainTex, null);
        customTexCtrlFace.SetTexture(ChaShader._ColorMask, null);
        customTexCtrlFace.SetTexture(ChaShader._Texture3, null);
        customTexCtrlFace.SetTexture(ChaShader._Texture4, null);
        customTexCtrlFace.SetTexture(ChaShader._Texture5, null);
        customTexCtrlFace.SetTexture(ChaShader._Texture6, null);
        customTexCtrlFace.SetTexture(ChaShader._Texture7, null);
        Resources.UnloadUnusedAssets();
        return true;
    }

    public void ChangeCustomFaceWithoutCustomTexture()
    {
        ChangeSettingEyebrow();
        ChangeSettingEyebrowColor();
        ChangeSettingEyeHiUp();
        ChangeSettingEyeHiUpColor();
        ChangeSettingEyeHiDown();
        ChangeSettingEyeHiDownColor();
        ChangeSettingEyeHLUpPosY();
        ChangeSettingEyeHLDownPosY();
        ChangeSettingEyePosX();
        ChangeSettingEyePosY();
        ChangeSettingEyeScaleWidth();
        ChangeSettingEyeScaleHeight();
        ChangeSettingEyeTilt();
        ChangeSettingNose();
        ChangeSettingEyelineUp();
        ChangeSettingEyelineDown();
        ChangeSettingEyelineColor();
        UpdateEyelineShadowColor();
        VisibleDoubleTooth();
        ChangeSettingFaceDetail();
        ChangeSettingFaceDetailPower();
        ChangeSettingLip();
        ChangeSettingLipColor();
        ChangeSettingEyeShadow();
        ChangeSettingEyeShadowColor();
        ChangeSettingCheekGlossPower();
        ChangeSettingLipGlossPower();
    }

    public void ChangeSettingHairColor(int parts, bool c00, bool c01, bool c02)
    {
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (null == customHairComponent || customHairComponent.rendHair == null || customHairComponent.rendHair.Length == 0)
        {
            return;
        }
        ChaFileHair hair = chaFile.custom.hair;
        for (int i = 0; i < customHairComponent.rendHair.Length; i++)
        {
            if (c00)
            {
                Color baseColor = hair.parts[parts].baseColor;
                if (1f > baseColor.a)
                {
                    ChaFileHair.PartsInfo obj = hair.parts[parts];
                    Color baseColor2 = hair.parts[parts].baseColor;
                    float r = baseColor2.r;
                    Color baseColor3 = hair.parts[parts].baseColor;
                    float g = baseColor3.g;
                    Color baseColor4 = hair.parts[parts].baseColor;
                    obj.baseColor = new Color(r, g, baseColor4.b, 1f);
                }
                customHairComponent.rendHair[i].material.SetColor(ChaShader._Color, hair.parts[parts].baseColor);
            }
            if (c01)
            {
                Color startColor = hair.parts[parts].startColor;
                if (1f > startColor.a)
                {
                    ChaFileHair.PartsInfo obj2 = hair.parts[parts];
                    Color startColor2 = hair.parts[parts].startColor;
                    float r2 = startColor2.r;
                    Color startColor3 = hair.parts[parts].startColor;
                    float g2 = startColor3.g;
                    Color startColor4 = hair.parts[parts].startColor;
                    obj2.startColor = new Color(r2, g2, startColor4.b, 1f);
                }
                customHairComponent.rendHair[i].material.SetColor(ChaShader._Color2, hair.parts[parts].startColor);
            }
            if (c02)
            {
                Color endColor = hair.parts[parts].endColor;
                if (1f > endColor.a)
                {
                    ChaFileHair.PartsInfo obj3 = hair.parts[parts];
                    Color endColor2 = hair.parts[parts].endColor;
                    float r3 = endColor2.r;
                    Color endColor3 = hair.parts[parts].endColor;
                    float g3 = endColor3.g;
                    Color endColor4 = hair.parts[parts].endColor;
                    obj3.endColor = new Color(r3, g3, endColor4.b, 1f);
                }
                customHairComponent.rendHair[i].material.SetColor(ChaShader._Color3, hair.parts[parts].endColor);
            }
        }
    }

    public void ChangeSettingHairOutlineColor(int parts)
    {
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (!(null == customHairComponent) && customHairComponent.rendHair != null && customHairComponent.rendHair.Length != 0)
        {
            ChaFileHair hair = chaFile.custom.hair;
            for (int i = 0; i < customHairComponent.rendHair.Length; i++)
            {
                customHairComponent.rendHair[i].material.SetColor(ChaShader._LineColor, hair.parts[parts].outlineColor);
            }
        }
    }

    public int GetHairAcsColorNum(int parts)
    {
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (null == customHairComponent || customHairComponent.acsDefColor == null || customHairComponent.acsDefColor.Length == 0)
        {
            return 0;
        }
        return customHairComponent.acsDefColor.Length;
    }

    public void SetAcsDefaultColorParameterOnly(int parts)
    {
        ChaFileHair hair = chaFile.custom.hair;
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (!(null == customHairComponent))
        {
            int num = customHairComponent.acsDefColor.Length;
            for (int i = 0; i < num; i++)
            {
                hair.parts[parts].acsColor[i] = customHairComponent.acsDefColor[i];
            }
        }
    }

    public void ChangeSettingHairAcsColor(int parts)
    {
        int hairAcsColorNum = GetHairAcsColorNum(parts);
        if (hairAcsColorNum == 0)
        {
            return;
        }
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (null == customHairComponent)
        {
            return;
        }
        int[] array = new int[3]
        {
            ChaShader._Color,
            ChaShader._Color2,
            ChaShader._Color3
        };
        ChaFileHair hair = chaFile.custom.hair;
        for (int i = 0; i < customHairComponent.rendAccessory.Length; i++)
        {
            for (int j = 0; j < hairAcsColorNum; j++)
            {
                if (1f > hair.parts[parts].acsColor[j].a)
                {
                    hair.parts[parts].acsColor[j] = new Color(hair.parts[parts].acsColor[j].r, hair.parts[parts].acsColor[j].g, hair.parts[parts].acsColor[j].b, 1f);
                }
                customHairComponent.rendAccessory[i].material.SetColor(array[j], hair.parts[parts].acsColor[j]);
            }
        }
    }

    public void ChangeSettingHairLength(int parts)
    {
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (!(null == customHairComponent))
        {
            ChaFileHair hair = chaFile.custom.hair;
            customHairComponent.lengthRate = hair.parts[parts].length;
        }
    }

    public bool ChangeSettingHairFrontLength()
    {
        int num = 1;
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(num);
        if (null == customHairComponent)
        {
            return false;
        }
        ChaFileHair hair = chaFile.custom.hair;
        customHairComponent.lengthRate = hair.parts[num].length;
        return true;
    }

    public void LoadHairGlossMask()
    {
        ChaFileHair hair = chaFile.custom.hair;
        texHairGloss = GetTexture(ChaListDefine.CategoryNo.mt_hairgloss, hair.glossId, ChaListDefine.KeyType.MainTexAB, ChaListDefine.KeyType.MainTex, string.Empty);
    }

    public bool ChangeSettingHairGlossMaskAll()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        int[] array = (int[])Enum.GetValues(typeof(ChaFileDefine.HairKind));
        int[] array2 = array;
        foreach (int parts in array2)
        {
            ChangeSettingHairGlossMask(parts);
        }
        return true;
    }

    public void ChangeSettingHairGlossMask(int parts)
    {
        if (sex == 0 && exType == 1)
        {
            return;
        }
        ChaCustomHairComponent customHairComponent = GetCustomHairComponent(parts);
        if (null == customHairComponent || null == customHairComponent || customHairComponent.rendHair == null || customHairComponent.rendHair.Length == 0)
        {
            return;
        }
        for (int i = 0; i < customHairComponent.rendHair.Length; i++)
        {
            if (!(null == customHairComponent.rendHair[i]))
            {
                customHairComponent.rendHair[i].material.SetTexture(ChaShader._HairGloss, texHairGloss);
            }
        }
    }

    public void ChangeShakeHair(int parts)
    {
        if ((bool)objHair[parts])
        {
            DynamicBone[] componentsInChildren = objHair[parts].GetComponentsInChildren<DynamicBone>(true);
            DynamicBone[] array = componentsInChildren;
            foreach (DynamicBone dynamicBone in array)
            {
                dynamicBone.enabled = !fileHair.parts[parts].noShake;
            }
        }
    }

    public void AddUpdateCMBodyTexFlags(bool inpBase, bool inpSub, bool inpPaint01, bool inpPaint02, bool inpSunburn)
    {
        if (inpBase)
        {
            updateCMBodyTex[0] = inpBase;
        }
        if (inpSub)
        {
            updateCMBodyTex[1] = inpSub;
        }
        if (inpPaint01)
        {
            updateCMBodyTex[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMBodyTex[3] = inpPaint02;
        }
        if (inpSunburn)
        {
            updateCMBodyTex[4] = inpSunburn;
        }
    }

    public void AddUpdateCMBodyColorFlags(bool inpBase, bool inpSub, bool inpPaint01, bool inpPaint02, bool inpSunburn, bool inpNail)
    {
        if (inpBase)
        {
            updateCMBodyColor[0] = inpBase;
        }
        if (inpSub)
        {
            updateCMBodyColor[1] = inpSub;
        }
        if (inpPaint01)
        {
            updateCMBodyColor[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMBodyColor[3] = inpPaint02;
        }
        if (inpSunburn)
        {
            updateCMBodyColor[4] = inpSunburn;
        }
        if (inpNail)
        {
            updateCMBodyColor[5] = inpNail;
        }
    }

    public void AddUpdateCMBodyLayoutFlags(bool inpPaint01, bool inpPaint02)
    {
        if (inpPaint01)
        {
            updateCMBodyLayout[2] = inpPaint01;
        }
        if (inpPaint02)
        {
            updateCMBodyLayout[3] = inpPaint02;
        }
    }

    public bool CreateBodyTexture()
    {
        ChaFileBody body = chaFile.custom.body;
        bool flag = false;
        bool flag2 = sex == 0 && exType == 1;
        if (updateCMBodyTex[0])
        {
            string manifestName = "abdata";
            string assetBundleName = "chara/oo_base.unity3d";
            string text = "cf_body_00_t";
            if (flag2)
            {
                ListInfoBase listInfo = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_body, 0);
                if (listInfo != null)
                {
                    assetBundleName = listInfo.GetInfo(ChaListDefine.KeyType.MainTexAB);
                    text = listInfo.GetInfo(ChaListDefine.KeyType.MainTex);
                    manifestName = "add54";
                }
            }
            if (!hiPoly)
            {
                text += "_low";
            }
            Texture2D mainTexture = CommonLib.LoadAsset<Texture2D>(assetBundleName, text, false, manifestName);
            Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, manifestName);
            customTexCtrlBody.SetMainTexture(mainTexture);
            flag = true;
            updateCMBodyTex[0] = false;
        }
        if (updateCMBodyColor[0])
        {
            customTexCtrlBody.SetColor(ChaShader._Color, body.skinMainColor);
            flag = true;
            updateCMBodyColor[0] = false;
        }
        if (updateCMBodyTex[1])
        {
            string manifestName2 = "abdata";
            string assetBundleName2 = "chara/oo_base.unity3d";
            string text2 = sex != 0 ? "cf_body_00_mc" : "cm_body_00_mc";
            if (flag2)
            {
                ListInfoBase listInfo2 = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_body, 0);
                if (listInfo2 != null)
                {
                    assetBundleName2 = listInfo2.GetInfo(ChaListDefine.KeyType.ColorMaskAB);
                    text2 = listInfo2.GetInfo(ChaListDefine.KeyType.ColorMaskTex);
                    manifestName2 = "add54";
                }
            }
            if (!hiPoly)
            {
                text2 += "_low";
            }
            Texture2D tex = CommonLib.LoadAsset<Texture2D>(assetBundleName2, text2, false, manifestName2);
            Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName2, manifestName2);
            customTexCtrlBody.SetTexture(ChaShader._ColorMask, tex);
            flag = true;
            updateCMBodyTex[1] = false;
        }
        if (updateCMBodyColor[1])
        {
            customTexCtrlBody.SetColor(ChaShader._Color2, body.skinSubColor);
            flag = true;
            updateCMBodyColor[1] = false;
        }
        if (!flag2)
        {
            if (updateCMBodyTex[2])
            {
                if (SetCreateTexture(customTexCtrlBody, false, ChaListDefine.CategoryNo.mt_body_paint, body.paintId[0], ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.PaintTex, ChaShader._Texture3))
                {
                    flag = true;
                }
                updateCMBodyTex[2] = false;
            }
            if (updateCMBodyColor[2])
            {
                customTexCtrlBody.SetColor(ChaShader._Color3, body.paintColor[0]);
                updateCMBodyColor[2] = false;
                flag = true;
            }
            if (updateCMBodyLayout[2])
            {
                ListInfoBase listInfo3 = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.bodypaint_layout, body.paintLayoutId[0]);
                if (listInfo3 != null)
                {
                    float b = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterX) - listInfo3.GetInfoFloat(ChaListDefine.KeyType.MoveX);
                    float a = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterX) + listInfo3.GetInfoFloat(ChaListDefine.KeyType.MoveX);
                    float b2 = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterY) - listInfo3.GetInfoFloat(ChaListDefine.KeyType.MoveY);
                    float a2 = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterY) + listInfo3.GetInfoFloat(ChaListDefine.KeyType.MoveY);
                    float a3 = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterScale) - listInfo3.GetInfoFloat(ChaListDefine.KeyType.AddScale);
                    float b3 = listInfo3.GetInfoFloat(ChaListDefine.KeyType.CenterScale) + listInfo3.GetInfoFloat(ChaListDefine.KeyType.AddScale);
                    Vector4 value = default(Vector4);
                    value.x = Mathf.Lerp(a, b, body.paintLayout[0].x);
                    value.y = Mathf.Lerp(a2, b2, body.paintLayout[0].y);
                    value.z = Mathf.Lerp(-1f, 1f, body.paintLayout[0].z);
                    value.w = Mathf.Lerp(a3, b3, body.paintLayout[0].w);
                    customTexCtrlBody.SetVector4(ChaShader._paint1, value);
                    updateCMBodyLayout[2] = false;
                    flag = true;
                }
            }
            if (updateCMBodyTex[3])
            {
                if (SetCreateTexture(customTexCtrlBody, false, ChaListDefine.CategoryNo.mt_body_paint, body.paintId[1], ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.PaintTex, ChaShader._Texture6))
                {
                    flag = true;
                }
                updateCMBodyTex[3] = false;
            }
            if (updateCMBodyColor[3])
            {
                customTexCtrlBody.SetColor(ChaShader._Color6, body.paintColor[1]);
                updateCMBodyColor[3] = false;
                flag = true;
            }
            if (updateCMBodyLayout[3])
            {
                ListInfoBase listInfo4 = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.bodypaint_layout, body.paintLayoutId[1]);
                if (listInfo4 != null)
                {
                    float b4 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterX) - listInfo4.GetInfoFloat(ChaListDefine.KeyType.MoveX);
                    float a4 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterX) + listInfo4.GetInfoFloat(ChaListDefine.KeyType.MoveX);
                    float b5 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterY) - listInfo4.GetInfoFloat(ChaListDefine.KeyType.MoveY);
                    float a5 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterY) + listInfo4.GetInfoFloat(ChaListDefine.KeyType.MoveY);
                    float a6 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterScale) - listInfo4.GetInfoFloat(ChaListDefine.KeyType.AddScale);
                    float b6 = listInfo4.GetInfoFloat(ChaListDefine.KeyType.CenterScale) + listInfo4.GetInfoFloat(ChaListDefine.KeyType.AddScale);
                    Vector4 value2 = default(Vector4);
                    value2.x = Mathf.Lerp(a4, b4, body.paintLayout[1].x);
                    value2.y = Mathf.Lerp(a5, b5, body.paintLayout[1].y);
                    value2.z = Mathf.Lerp(-1f, 1f, body.paintLayout[1].z);
                    value2.w = Mathf.Lerp(a6, b6, body.paintLayout[1].w);
                    customTexCtrlBody.SetVector4(ChaShader._paint2, value2);
                    updateCMBodyLayout[3] = false;
                    flag = true;
                }
            }
            if (updateCMBodyTex[4])
            {
                if (SetCreateTexture(customTexCtrlBody, false, ChaListDefine.CategoryNo.mt_sunburn, body.sunburnId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.SunburnTex, ChaShader._Texture4))
                {
                    flag = true;
                }
                updateCMBodyTex[4] = false;
            }
            if (updateCMBodyColor[4])
            {
                customTexCtrlBody.SetColor(ChaShader._Color4, body.sunburnColor);
                updateCMBodyColor[4] = false;
                flag = true;
            }
            if (updateCMBodyColor[5])
            {
                customTexCtrlBody.SetColor(ChaShader._Color5, body.nailColor);
                updateCMBodyColor[5] = false;
                flag = true;
            }
        }
        if (flag)
        {
            customTexCtrlBody.SetNewCreateTexture();
        }
        if (releaseCustomInputTexture)
        {
            ReleaseBodyCustomTexture();
        }
        return true;
    }

    public bool ChangeSettingNip()
    {
        ChaFileBody body = chaFile.custom.body;
        ChangeTexture(customTexCtrlBody.matDraw, ChaListDefine.CategoryNo.mt_nip, body.nipId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.NipTex, ChaShader._overtex1, string.Empty);
        return true;
    }

    public bool ChangeSettingAreolaSize()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetFloat(ChaShader._nipsize, body.areolaSize);
        return true;
    }

    public bool ChangeSettingNipColor()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetColor(ChaShader._overcolor1, body.nipColor);
        return true;
    }

    public bool ChangeSettingNipGlossPower()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetFloat(ChaShader._nip_specular, body.nipGlossPower);
        return true;
    }

    public bool ChangeSettingUnderhair()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        ChangeTexture(customTexCtrlBody.matDraw, ChaListDefine.CategoryNo.mt_underhair, body.underhairId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.UnderhairTex, ChaShader._overtex2, string.Empty);
        return true;
    }

    public bool ChangeSettingUnderhairColor()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetColor(ChaShader._overcolor2, body.underhairColor);
        return true;
    }

    public bool VisibleAddBodyLine()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetFloat(ChaShader._linetexon, !body.drawAddLine ? 0f : 1f);
        return true;
    }

    public bool ChangeSettingBodyDetail()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        ChangeTexture(customTexCtrlBody.matDraw, ChaListDefine.CategoryNo.mt_body_detail, body.detailId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.NormallMapDetail, ChaShader._NormalMapDetail, string.Empty);
        ChangeTexture(customTexCtrlBody.matDraw, ChaListDefine.CategoryNo.mt_body_detail, body.detailId, ChaListDefine.KeyType.MainAB, ChaListDefine.KeyType.LineMask, ChaShader._LineMask, string.Empty);
        return true;
    }

    public bool ChangeSettingBodyDetailPower()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetFloat(ChaShader._DetailNormalMapScale, body.detailPower);
        return true;
    }

    public bool ChangeSettingSkinGlossPower()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        float value = Mathf.Lerp(body.skinGlossPower, 1f, chaFile.status.skinTuyaRate);
        customTexCtrlBody.matDraw.SetFloat(ChaShader._SpecularPower, value);
        return true;
    }

    public bool ChangeSettingNailGlossPower()
    {
        ChaFileBody body = chaFile.custom.body;
        if (null == customTexCtrlBody.matDraw)
        {
            return false;
        }
        customTexCtrlBody.matDraw.SetFloat(ChaShader._SpecularPowerNail, body.nailGlossPower);
        return true;
    }

    public Vector4 GetLayoutInfo(int id)
    {
        Vector4 zero = Vector4.zero;
        ListInfoBase listInfo = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.bodypaint_layout, id);
        if (listInfo != null)
        {
            float a = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterX) - listInfo.GetInfoFloat(ChaListDefine.KeyType.MoveX);
            float b = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterX) + listInfo.GetInfoFloat(ChaListDefine.KeyType.MoveX);
            float a2 = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterY) - listInfo.GetInfoFloat(ChaListDefine.KeyType.MoveY);
            float b2 = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterY) + listInfo.GetInfoFloat(ChaListDefine.KeyType.MoveY);
            float a3 = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterScale) - listInfo.GetInfoFloat(ChaListDefine.KeyType.AddScale);
            float b3 = listInfo.GetInfoFloat(ChaListDefine.KeyType.CenterScale) + listInfo.GetInfoFloat(ChaListDefine.KeyType.AddScale);
            float infoFloat = listInfo.GetInfoFloat(ChaListDefine.KeyType.PosX);
            float infoFloat2 = listInfo.GetInfoFloat(ChaListDefine.KeyType.PosY);
            float infoFloat3 = listInfo.GetInfoFloat(ChaListDefine.KeyType.Rot);
            float infoFloat4 = listInfo.GetInfoFloat(ChaListDefine.KeyType.Scale);
            zero.x = Mathf.InverseLerp(a, b, infoFloat);
            zero.y = Mathf.InverseLerp(a2, b2, infoFloat2);
            zero.z = Mathf.InverseLerp(-1f, 1f, infoFloat3);
            zero.w = Mathf.InverseLerp(a3, b3, infoFloat4);
        }
        return zero;
    }

    public bool SetBodyBaseMaterial()
    {
        if (null == customMatBody)
        {
            return false;
        }
        return SetBaseMaterial(rendBody, customMatBody);
    }

    public bool ReleaseBodyCustomTexture()
    {
        if (customTexCtrlBody == null)
        {
            return false;
        }
        customTexCtrlBody.SetTexture(ChaShader._MainTex, null);
        customTexCtrlBody.SetTexture(ChaShader._ColorMask, null);
        customTexCtrlBody.SetTexture(ChaShader._Texture3, null);
        customTexCtrlBody.SetTexture(ChaShader._Texture4, null);
        customTexCtrlBody.SetTexture(ChaShader._Texture6, null);
        Resources.UnloadUnusedAssets();
        return true;
    }

    public void ChangeCustomBodyWithoutCustomTexture()
    {
        ChangeSettingNip();
        ChangeSettingAreolaSize();
        ChangeSettingNipColor();
        ChangeSettingUnderhair();
        ChangeSettingUnderhairColor();
        VisibleAddBodyLine();
        ChangeSettingBodyDetail();
        ChangeSettingBodyDetailPower();
        ChangeSettingSkinGlossPower();
        ChangeSettingNailGlossPower();
        ChangeSettingNipGlossPower();
    }

    public bool InitShapeFace(Transform trfBone, string assetBundleAnmShapeFace, string assetAnmShapeFace, string manifest)
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (sibFace == null)
        {
            return false;
        }
        if (null == trfBone)
        {
            return false;
        }
        string cateInfoName = "cf_customhead";
        sibFace.InitShapeInfo(manifest, assetBundleAnmShapeFace, "list/customshape.unity3d", assetAnmShapeFace, cateInfoName, trfBone, sex);
        for (int i = 0; i < ShapeFaceNum; i++)
        {
            sibFace.ChangeValue(i, chaFile.custom.face.shapeValueFace[i]);
        }
        updateShapeFace = true;
        return true;
    }

    public void ReleaseShapeFace()
    {
        if ((sex != 0 || exType != 1) && sibFace != null)
        {
            sibFace.ReleaseShapeInfo();
        }
    }

    public bool SetShapeFaceValue(int index, float value)
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (index >= ShapeFaceNum)
        {
            return false;
        }
        chaFile.custom.face.shapeValueFace[index] = value;
        if (sibFace != null && sibFace.InitEnd)
        {
            sibFace.ChangeValue(index, value);
        }
        if (index == 33)
        {
            ChangeSettingEyeTilt();
        }
        updateShapeFace = true;
        return true;
    }

    public bool UpdateShapeFaceValueFromCustomInfo()
    {
        if (sex == 0 && exType == 1)
        {
            return true;
        }
        if (sibFace == null || !sibFace.InitEnd)
        {
            return false;
        }
        for (int i = 0; i < chaFile.custom.face.shapeValueFace.Length; i++)
        {
            sibFace.ChangeValue(i, chaFile.custom.face.shapeValueFace[i]);
        }
        updateShapeFace = true;
        return true;
    }

    public float GetShapeFaceValue(int index)
    {
        if (sex == 0 && exType == 1)
        {
            return 0f;
        }
        if (index >= ShapeFaceNum)
        {
            return 0f;
        }
        return chaFile.custom.face.shapeValueFace[index];
    }

    public void UpdateShapeFace()
    {
        if (sex == 0 && exType == 1 || sibFace == null || !sibFace.InitEnd)
        {
            return;
        }
        if (chaFile.status.disableMouthShapeMask)
        {
            int num = 0;
            if (fileFace.headId == 200)
            {
                num = 1;
            }
            else if (fileFace.headId == 201)
            {
                num = 2;
            }
            for (int i = 0; i < ChaFileDefine.cf_MouthShapeMaskID.Length; i++)
            {
                sibFace.ChangeValue(ChaFileDefine.cf_MouthShapeMaskID[i], ChaFileDefine.cf_MouthShapeDefault[num, i]);
            }
        }
        else
        {
            int[] cf_MouthShapeMaskID = ChaFileDefine.cf_MouthShapeMaskID;
#if FIX
            //Use a for loop to eliminate garbage
            for (var index = 0; index < cf_MouthShapeMaskID.Length; index++)
            {
                int num2 = cf_MouthShapeMaskID[index];
                sibFace.ChangeValue(num2, chaFile.custom.face.shapeValueFace[num2]);
            }
#else
            foreach (int num2 in cf_MouthShapeMaskID)
            {
                sibFace.ChangeValue(num2, chaFile.custom.face.shapeValueFace[num2]);
            }
#endif
        }
        sibFace.typeBone = fileCustom.body.typeBone;
        if (sibBody != null)
        {
            ShapeBodyInfoFemale shapeBodyInfoFemale = sibBody as ShapeBodyInfoFemale;
            ShapeHeadInfoFemale shapeHeadInfoFemale = sibFace as ShapeHeadInfoFemale;
            if (shapeBodyInfoFemale != null)
            {
                shapeHeadInfoFemale.headCorrectValue = 1f / (1f + shapeBodyInfoFemale.correctValue[2].vctScl.y);
                float num3 = sibFace.typeBone != 0 ? 1.08f : 1.2f;
                GameObject referenceInfo = GetReferenceInfo(RefObjKey.CORRECT_HEAD_DBCOL);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.localScale = new Vector3(num3, num3, num3);
                }
            }
        }
        sibFace.Update();
    }

    public void DisableShapeMouth(bool disable)
    {
        updateShapeFace = true;
        chaFile.status.disableMouthShapeMask = disable;
    }

    public bool InitShapeBody(Transform trfBone)
    {
        if (sibBody == null)
        {
            return false;
        }
        if (null == trfBone)
        {
            return false;
        }
        string anmKeyInfoName = "cf_anmShapeBody";
        string cateInfoName = "cf_custombody";
        sibBody.InitShapeInfo("abdata", "chara/oo_base.unity3d", "list/customshape.unity3d", anmKeyInfoName, cateInfoName, trfBone, sex);
        float[] array = new float[chaFile.custom.body.shapeValueBody.Length];
        chaFile.custom.body.shapeValueBody.CopyTo(array, 0);
        if (!hiPoly)
        {
            array = array.Select(delegate (float val)
            {
                return val < 0.5f ? Mathf.Lerp(0.2f, 0.5f, Mathf.InverseLerp(0f, 0.5f, val)) : Mathf.Lerp(0.5f, 0.8f, Mathf.InverseLerp(0.5f, 1f, val));
            }).ToArray();
            array[0] = chaFile.custom.body.shapeValueBody[0];
        }
        array[13] = Mathf.Lerp(array[13], 1f, chaFile.status.nipStandRate);
        for (int i = 0; i < ShapeBodyNum; i++)
        {
            sibBody.ChangeValue(i, array[i]);
        }
        updateShapeBody = true;
        updateBustSize = true;
        reSetupDynamicBoneBust = true;
        return true;
    }

    public void ReleaseShapeBody()
    {
        if (sibBody != null)
        {
            sibBody.ReleaseShapeInfo();
        }
    }

    public bool SetShapeBodyValue(int index, float value)
    {
        if (index >= ShapeBodyNum)
        {
            return false;
        }
        if (sex == 0 && index == 0)
        {
            value = exType != 0 ? 1f : 0.6f;
        }
        float num = chaFile.custom.body.shapeValueBody[index] = value;
        if (!hiPoly && sex != 0)
        {
            float num2 = !(num < 0.5f) ? Mathf.Lerp(0.5f, 0.8f, Mathf.InverseLerp(0.5f, 1f, num)) : Mathf.Lerp(0.2f, 0.5f, Mathf.InverseLerp(0f, 0.5f, num));
            num = num2;
            if (index == 0)
            {
                num = value;
            }
        }
        if (index == 13)
        {
            num = Mathf.Lerp(value, 1f, chaFile.status.nipStandRate);
        }
        if (sibBody != null && sibBody.InitEnd)
        {
            sibBody.ChangeValue(index, num);
        }
        updateShapeBody = true;
        updateBustSize = true;
        reSetupDynamicBoneBust = true;
        return true;
    }

    public bool UpdateShapeBodyValueFromCustomInfo()
    {
        if (sibBody == null || !sibBody.InitEnd)
        {
            return false;
        }
        float[] array = new float[chaFile.custom.body.shapeValueBody.Length];
        chaFile.custom.body.shapeValueBody.CopyTo(array, 0);
        if (!hiPoly)
        {
            array = array.Select(delegate (float val)
            {
                return val < 0.5f ? Mathf.Lerp(0.2f, 0.5f, Mathf.InverseLerp(0f, 0.5f, val)) : Mathf.Lerp(0.5f, 0.8f, Mathf.InverseLerp(0.5f, 1f, val));
            }).ToArray();
            array[0] = chaFile.custom.body.shapeValueBody[0];
        }
        array[13] = Mathf.Lerp(array[13], 1f, chaFile.status.nipStandRate);
        for (int i = 0; i < chaFile.custom.body.shapeValueBody.Length; i++)
        {
            sibBody.ChangeValue(i, array[i]);
        }
        if (sex == 0)
        {
            sibBody.ChangeValue(0, exType != 0 ? 1f : 0.6f);
        }
        updateShapeBody = true;
        updateBustSize = true;
        reSetupDynamicBoneBust = true;
        return true;
    }

    public float GetShapeBodyValue(int index)
    {
        if (index >= ShapeBodyNum)
        {
            return 0f;
        }
        return chaFile.custom.body.shapeValueBody[index];
    }

    public void UpdateShapeBody()
    {
        if (sibBody == null || !sibBody.InitEnd)
        {
            return;
        }
        ShapeBodyInfoFemale shapeBodyInfoFemale = sibBody as ShapeBodyInfoFemale;
        if (shapeBodyInfoFemale == null)
        {
            return;
        }
        shapeBodyInfoFemale.updateMask = 7;
        sibBody.typeBone = fileCustom.body.typeBone;
        sibBody.Update();
        if (!changeShapeBodyMask)
        {
            return;
        }
        float[] array = new float[ChaFileDefine.cf_BustShapeMaskID.Length];
        int[] array2 = new int[2]
        {
            1,
            2
        };
        float[] array3 = new float[9]
        {
            0.5f,
            0.5f,
            0.5f,
            0.5f,
            0.5f,
            0.5f,
            0f,
            0.5f,
            0.5f
        };
        for (int i = 0; i < 2; i++)
        {
            int num;
            for (int j = 0; j < ChaFileDefine.cf_BustShapeMaskID.Length; j++)
            {
                num = ChaFileDefine.cf_BustShapeMaskID[j];
                array[j] = !chaFile.status.disableBustShapeMask[i, j] ? chaFile.custom.body.shapeValueBody[num] : array3[j];
            }
            int cf_ShapeMaskNipStand = ChaFileDefine.cf_ShapeMaskNipStand;
            num = ChaFileDefine.cf_BustShapeMaskID[cf_ShapeMaskNipStand];
            array[cf_ShapeMaskNipStand] = !chaFile.status.disableBustShapeMask[i, cf_ShapeMaskNipStand] ? Mathf.Lerp(chaFile.custom.body.shapeValueBody[num], 1f, chaFile.status.nipStandRate) : 0.5f;
            for (int k = 0; k < ChaFileDefine.cf_BustShapeMaskID.Length; k++)
            {
                sibBody.ChangeValue(ChaFileDefine.cf_BustShapeMaskID[k], array[k]);
            }
            shapeBodyInfoFemale.updateMask = array2[i];
            sibBody.Update();
        }
        changeShapeBodyMask = false;
    }

    public void UpdateAlwaysShapeBody()
    {
        if (sibBody != null && sibBody.InitEnd)
        {
            sibBody.UpdateAlways();
        }
    }

    public void UpdateShapeBodyCalcForce()
    {
        if (sibBody != null && sibBody.InitEnd)
        {
            sibBody.typeBone = fileCustom.body.typeBone;
            sibBody.ForceUpdate();
        }
    }

    public void DisableShapeBodyID(int LR, int id, bool disable)
    {
        if (sibBody != null && sibBody.InitEnd && id < ChaFileDefine.cf_BustShapeMaskID.Length)
        {
            changeShapeBodyMask = true;
            updateShapeBody = true;
            switch (LR)
            {
                case 0:
                    chaFile.status.disableBustShapeMask[0, id] = disable;
                    break;
                case 1:
                    chaFile.status.disableBustShapeMask[1, id] = disable;
                    break;
                default:
                    chaFile.status.disableBustShapeMask[0, id] = disable;
                    chaFile.status.disableBustShapeMask[1, id] = disable;
                    break;
            }
            reSetupDynamicBoneBust = true;
        }
    }

    public void DisableShapeBust(int LR, bool disable)
    {
        if (sibBody == null || !sibBody.InitEnd)
        {
            return;
        }
        changeShapeBodyMask = true;
        updateShapeBody = true;
        switch (LR)
        {
            case 0:
                {
                    for (int k = 0; k < ChaFileDefine.cf_ShapeMaskBust.Length; k++)
                    {
                        chaFile.status.disableBustShapeMask[0, ChaFileDefine.cf_ShapeMaskBust[k]] = disable;
                    }
                    break;
                }
            case 1:
                {
                    for (int l = 0; l < ChaFileDefine.cf_ShapeMaskBust.Length; l++)
                    {
                        chaFile.status.disableBustShapeMask[1, ChaFileDefine.cf_ShapeMaskBust[l]] = disable;
                    }
                    break;
                }
            default:
                {
                    for (int i = 0; i < 2; i++)
                    {
                        for (int j = 0; j < ChaFileDefine.cf_ShapeMaskBust.Length; j++)
                        {
                            chaFile.status.disableBustShapeMask[i, ChaFileDefine.cf_ShapeMaskBust[j]] = disable;
                        }
                    }
                    break;
                }
        }
        reSetupDynamicBoneBust = true;
    }

    public void DisableShapeNip(int LR, bool disable)
    {
        if (sibBody == null || !sibBody.InitEnd)
        {
            return;
        }
        changeShapeBodyMask = true;
        updateShapeBody = true;
        switch (LR)
        {
            case 0:
                {
                    for (int k = 0; k < ChaFileDefine.cf_ShapeMaskNip.Length; k++)
                    {
                        chaFile.status.disableBustShapeMask[0, ChaFileDefine.cf_ShapeMaskNip[k]] = disable;
                    }
                    break;
                }
            case 1:
                {
                    for (int l = 0; l < ChaFileDefine.cf_ShapeMaskNip.Length; l++)
                    {
                        chaFile.status.disableBustShapeMask[1, ChaFileDefine.cf_ShapeMaskNip[l]] = disable;
                    }
                    break;
                }
            default:
                {
                    for (int i = 0; i < 2; i++)
                    {
                        for (int j = 0; j < ChaFileDefine.cf_ShapeMaskNip.Length; j++)
                        {
                            chaFile.status.disableBustShapeMask[i, ChaFileDefine.cf_ShapeMaskNip[j]] = disable;
                        }
                    }
                    break;
                }
        }
        reSetupDynamicBoneBust = true;
    }

    public void UpdateBustSoftnessAndGravity()
    {
        UpdateBustSoftness();
        UpdateBustGravity();
    }

    public void ChangeBustSoftness(float soft)
    {
        if (bustSoft != null)
        {
            bustSoft.Change(soft, default(int));
            reSetupDynamicBoneBust = true;
        }
    }

    public bool UpdateBustSoftness()
    {
        if (bustSoft != null)
        {
            bustSoft.ReCalc(default(int));
            reSetupDynamicBoneBust = true;
            return true;
        }
        return false;
    }

    public void ChangeBustGravity(float gravity)
    {
        if (bustGravity != null)
        {
            bustGravity.Change(gravity, default(int));
            reSetupDynamicBoneBust = true;
        }
    }

    public bool UpdateBustGravity()
    {
        if (bustGravity != null)
        {
            bustGravity.ReCalc(default(int));
            reSetupDynamicBoneBust = true;
            return true;
        }
        return false;
    }

    protected void InitializeControlFaceAll()
    {
        fbsaaVoice = new AudioAssist();
        InitializeControlFaceObject();
    }

    protected void InitializeControlFaceObject()
    {
        asVoice = null;
        texHairGloss = null;
    }

    protected void ReleaseControlFaceAll()
    {
        ReleaseControlFaceObject(false);
    }

    protected void ReleaseControlFaceObject(bool init = true)
    {
        if (init)
        {
            InitializeControlFaceObject();
        }
    }

    public void HideEyeHighlight(bool hide)
    {
        fileStatus.hideEyesHighlight = hide;
        float value = !hide ? 1f : 0f;
        if (base.rendEye == null)
        {
            return;
        }
        Renderer[] rendEye = base.rendEye;
        foreach (Renderer renderer in rendEye)
        {
            if (!(null == renderer))
            {
                Material material = renderer.material;
                if (null != material)
                {
                    material.SetFloat(ChaShader._isHighLight, value);
                }
            }
        }
    }

    public void ChangeEyesShaking(bool enable)
    {
        fileStatus.eyesYure = enable;
        if (eyeLookMatCtrl != null)
        {
            EyeLookMaterialControll[] eyeLookMatCtrl = base.eyeLookMatCtrl;
            foreach (EyeLookMaterialControll eyeLookMaterialControll in eyeLookMatCtrl)
            {
                eyeLookMaterialControll.ChangeShaking(enable);
            }
        }
    }

    public bool GetEyesShaking()
    {
        return fileStatus.eyesYure;
    }

    public int GetEyesPtnNum()
    {
        Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_eyeset);
        return categoryInfo != null ? categoryInfo.Keys.Count : 0;
    }

    public void ChangeEyesPtn(int ptn, bool blend = true)
    {
        if (eyesCtrl == null)
        {
            return;
        }
        fileStatus.eyesPtn = ptn;
        Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_eyeset);
        ListInfoBase value;
        if (!categoryInfo.TryGetValue(ptn, out value))
        {
            fileStatus.eyesPtn = ptn = 0;
            categoryInfo.TryGetValue(ptn, out value);
        }
        if (value == null)
        {
            return;
        }
        GameObject referenceInfo = GetReferenceInfo(RefObjKey.N_EyeBase);
        if ((bool)referenceInfo)
        {
            referenceInfo.SetActiveIfDifferent(1 == value.GetInfoInt(ChaListDefine.KeyType.EyeBase));
        }
        GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.N_Hitomi);
        if ((bool)referenceInfo2)
        {
            referenceInfo2.SetActiveIfDifferent(0 != value.GetInfoInt(ChaListDefine.KeyType.EyeHitomi));
        }
        GameObject[] array = new GameObject[3]
        {
            GetReferenceInfo(RefObjKey.N_Gag00),
            GetReferenceInfo(RefObjKey.N_Gag01),
            GetReferenceInfo(RefObjKey.N_Gag02)
        };
        int infoInt = value.GetInfoInt(ChaListDefine.KeyType.EyeObjNo);
        for (int i = 0; i < array.Length; i++)
        {
            if ((bool)array[i])
            {
                array[i].SetActiveIfDifferent(i + 1 == infoInt);
            }
        }
        if (rendEye != null && rendEye.Length == 2)
        {
            Texture value2 = null;
            string info = value.GetInfo(ChaListDefine.KeyType.EpsTexAB);
            string text = value.GetInfo(ChaListDefine.KeyType.EpsTex);
            float value3 = 0f;
            if ("0" != info && "0" != text)
            {
                if (!hiPoly)
                {
                    text += "_low";
                }
                value2 = CommonLib.LoadAsset<Texture2D>(info, text, false, string.Empty);
                value3 = 1f;
            }
            Renderer[] rendEye = base.rendEye;
            foreach (Renderer renderer in rendEye)
            {
                if (!(null == renderer))
                {
                    renderer.material.SetTexture(ChaShader._expression, value2);
                    renderer.material.SetFloat(ChaShader._exppower, value3);
                }
            }
        }
        if (array != null && array.Length != 0 && infoInt != 0)
        {
            Texture tex = null;
            string info2 = value.GetInfo(ChaListDefine.KeyType.MainTexAB);
            string text2 = value.GetInfo(ChaListDefine.KeyType.MainTex);
            if ("0" != info2 && "0" != text2)
            {
                if (!hiPoly)
                {
                    text2 += "_low";
                }
                tex = CommonLib.LoadAsset<Texture2D>(info2, text2, false, string.Empty);
            }
            string[] array2 = value.GetInfo(ChaListDefine.KeyType.TileAnim).Split('/');
            Vector4 zero = Vector4.zero;
            if (3 <= array2.Length)
            {
                float.TryParse(array2[0], out zero.x);
                float.TryParse(array2[1], out zero.y);
                float.TryParse(array2[2], out zero.z);
            }
            float infoFloat = value.GetInfoFloat(ChaListDefine.KeyType.SizeSpeed);
            float infoFloat2 = value.GetInfoFloat(ChaListDefine.KeyType.SizeWidth);
            float infoFloat3 = value.GetInfoFloat(ChaListDefine.KeyType.AngleSpeed);
            float infoFloat4 = value.GetInfoFloat(ChaListDefine.KeyType.Yurayura);
            ChangeGagEyesMaterial(infoInt - 1, tex, zero, infoFloat, infoFloat2, infoFloat3, infoFloat4);
        }
        SetForegroundEyesAndEyebrow();
        ChangeSettingEyeShadowColor();
        eyesCtrl.ChangePtn(value.GetInfoInt(ChaListDefine.KeyType.EyesPtn), blend);
    }

    public int GetEyesPtn()
    {
        return fileStatus.eyesPtn;
    }

    public void ChangeGagEyesMaterial(int no, Texture tex, Vector4 v4TileAnim, float sizeSpeed, float sizeWidth, float angleSpeed, float yurayura)
    {
        GameObject[] array = new GameObject[3]
        {
            GetReferenceInfo(RefObjKey.N_Gag00),
            GetReferenceInfo(RefObjKey.N_Gag01),
            GetReferenceInfo(RefObjKey.N_Gag02)
        };
        if (!(null == array[no]))
        {
            SkinnedMeshRenderer component = array[no].GetComponent<SkinnedMeshRenderer>();
            if (!(null == component))
            {
                matGag[no].SetTexture(ChaShader._MainTex, null);
                matGag[no].SetTexture(ChaShader._MainTex, tex);
                matGag[no].SetVector(ChaShader._TileAnimation, v4TileAnim);
                matGag[no].SetFloat(ChaShader._SizeSpeed, sizeSpeed);
                matGag[no].SetFloat(ChaShader._SizeWidth, sizeWidth);
                matGag[no].SetFloat(ChaShader._angleSpeed, angleSpeed);
                matGag[no].SetFloat(ChaShader._yurayura, yurayura);
                component.material = matGag[no];
                Resources.UnloadUnusedAssets();
            }
        }
    }

    public void ChangeEyesOpenMax(float maxValue)
    {
        if (eyesCtrl != null)
        {
            float num = Mathf.Clamp(maxValue, 0f, 1f);
            fileStatus.eyesOpenMax = num;
            eyesCtrl.OpenMax = Mathf.Clamp(maxValue, 0f, 0.92f);
            if (!fileStatus.eyesBlink)
            {
                eyesCtrl.SetOpenRateForce(num);
            }
        }
    }

    public float GetEyesOpenMax()
    {
        return fileStatus.eyesOpenMax;
    }

    public void ChangeEyebrowPtn(int ptn, bool blend = true)
    {
        if (eyebrowCtrl != null)
        {
            fileStatus.eyebrowPtn = ptn;
            eyebrowCtrl.ChangePtn(ptn, blend);
        }
    }

    public int GetEyebrowPtn()
    {
        return fileStatus.eyebrowPtn;
    }

    public void ChangeEyebrowOpenMax(float maxValue)
    {
        if (eyebrowCtrl != null)
        {
            float num = Mathf.Clamp(maxValue, 0f, 1f);
            fileStatus.eyebrowOpenMax = num;
            eyebrowCtrl.OpenMax = num;
            if (!fileStatus.eyesBlink)
            {
                eyebrowCtrl.SetOpenRateForce(1f);
            }
        }
    }

    public float GetEyebrowOpenMax()
    {
        return fileStatus.eyebrowOpenMax;
    }

    public void ChangeEyesBlinkFlag(bool blink)
    {
        if (!(null == fbsCtrl) && fbsCtrl.BlinkCtrl != null)
        {
            fileStatus.eyesBlink = blink;
            fbsCtrl.BlinkCtrl.SetFixedFlags((byte)(!blink ? 1 : 0));
            if (!blink)
            {
                eyesCtrl.SetOpenRateForce(1f);
                eyebrowCtrl.SetOpenRateForce(1f);
            }
        }
    }

    public bool GetEyesBlinkFlag()
    {
        return fileStatus.eyesBlink;
    }

    public void ChangeMouthPtn(int ptn, bool blend = true)
    {
        if (mouthCtrl != null)
        {
            fileStatus.mouthPtn = ptn;
            mouthCtrl.ChangePtn(ptn, blend);
            ChangeTongueState((byte)(ptn == 21 ? 1 : 0));
            mouthCtrl.UseAdjustWidthScale(ptn != 21 && ptn != 22);
        }
    }

    public int GetMouthPtn()
    {
        return fileStatus.mouthPtn;
    }

    public void ChangeMouthOpenMax(float maxValue)
    {
        if (mouthCtrl != null)
        {
            float num = Mathf.Clamp(maxValue, 0f, 1f);
            fileStatus.mouthOpenMax = num;
            mouthCtrl.OpenMax = num;
            if (fileStatus.mouthFixed)
            {
                mouthCtrl.FixedRate = num;
            }
        }
    }

    public float GetMouthOpenMax()
    {
        return fileStatus.mouthOpenMax;
    }

    public void ChangeMouthFixed(bool fix)
    {
        if (mouthCtrl != null)
        {
            fileStatus.mouthFixed = fix;
            if (fix)
            {
                mouthCtrl.FixedRate = fileStatus.mouthOpenMax;
            }
            else
            {
                mouthCtrl.FixedRate = -1f;
            }
        }
    }

    public bool GetMouthFixed()
    {
        return fileStatus.mouthFixed;
    }

    public void ChangeTongueState(byte state)
    {
        fileStatus.tongueState = state;
    }

    public byte GetTongueState()
    {
        return fileStatus.tongueState;
    }

    public bool SetVoiceTransform(Transform trfVoice)
    {
        if (null == trfVoice)
        {
            asVoice = null;
            return false;
        }
        asVoice = trfVoice.GetComponent<AudioSource>();
        if (null == asVoice)
        {
            return false;
        }
        return true;
    }

    private void UpdateBlendShapeVoice()
    {
        float voiceVaule = 0f;
        float correct = 3f;
        if (asVoice && asVoice.isPlaying)
        {
            voiceVaule = wavInfoData == null ? fbsaaVoice.GetAudioWaveValue(asVoice, correct) : wavInfoData.GetValue(asVoice.time);
        }
        if ((bool)fbsCtrl)
        {
            fbsCtrl.SetVoiceVaule(voiceVaule);
        }
        if (!fileStatus.mouthAdjustWidth || !objHeadBone)
        {
            return;
        }
        GameObject referenceInfo = GetReferenceInfo(RefObjKey.F_ADJUSTWIDTHSCALE);
        if (null != referenceInfo)
        {
            float x = 1f;
            if (mouthCtrl != null)
            {
                x = mouthCtrl.GetAdjustWidthScale();
            }
            referenceInfo.transform.SetLocalScaleX(x);
        }
    }

    public void ChangeLookEyesTarget(int targetType, Transform trfTarg = null, float rate = 0.5f, float rotDeg = 0f, float range = 1f, float dis = 2f)
    {
        if (null == eyeLookCtrl)
        {
            return;
        }
        if (targetType == -1)
        {
            targetType = fileStatus.eyesTargetType;
        }
        else
        {
            fileStatus.eyesTargetType = targetType;
        }
        eyeLookCtrl.target = null;
        if ((bool)trfTarg)
        {
            eyeLookCtrl.target = trfTarg;
            return;
        }
        if (targetType == 0)
        {
            if ((bool)Camera.main)
            {
                eyeLookCtrl.target = Camera.main.transform;
            }
        }
        else if (objEyesLookTarget && objEyesLookTargetP)
        {
            switch (targetType)
            {
                case 1:
                    rotDeg = 0f;
                    range = 1f;
                    break;
                case 2:
                    rotDeg = 45f;
                    range = 1f;
                    break;
                case 3:
                    rotDeg = 90f;
                    range = 1f;
                    break;
                case 4:
                    rotDeg = 135f;
                    range = 1f;
                    break;
                case 5:
                    rotDeg = 180f;
                    range = 1f;
                    break;
                case 6:
                    rotDeg = 225f;
                    range = 1f;
                    break;
                case 7:
                    rotDeg = 270f;
                    range = 1f;
                    break;
                case 8:
                    rotDeg = 315f;
                    range = 1f;
                    break;
            }
            objEyesLookTargetP.transform.SetLocalPosition(0f, 0.07f, 0f);
            eyeLookCtrl.target = objEyesLookTarget.transform;
            float y = Mathf.Lerp(0f, range, rate);
            eyeLookCtrl.target.SetLocalPosition(0f, y, dis);
            objEyesLookTargetP.transform.localEulerAngles = new Vector3(0f, 0f, 360f - rotDeg);
        }
        fileStatus.eyesTargetAngle = rotDeg;
        fileStatus.eyesTargetRange = range;
        fileStatus.eyesTargetRate = rate;
    }

    public void ChangeLookEyesPtn(int ptn)
    {
        if (!(null == eyeLookCtrl))
        {
            EyeLookController eyeLookCtrl = base.eyeLookCtrl;
            fileStatus.eyesLookPtn = ptn;
            eyeLookCtrl.ptnNo = ptn;
        }
    }

    public int GetLookEyesPtn()
    {
        return fileStatus.eyesLookPtn;
    }

    public void ChangeLookNeckTarget(int targetType, Transform trfTarg = null, float rate = 0.5f, float rotDeg = 0f, float range = 1f, float dis = 0.8f)
    {
        if (null == neckLookCtrl)
        {
            return;
        }
        if (targetType == -1)
        {
            targetType = fileStatus.neckTargetType;
        }
        else
        {
            fileStatus.neckTargetType = targetType;
        }
        neckLookCtrl.target = null;
        if ((bool)trfTarg)
        {
            neckLookCtrl.target = trfTarg;
            return;
        }
        if (targetType == 0)
        {
            if ((bool)Camera.main)
            {
                neckLookCtrl.target = Camera.main.transform;
            }
        }
        else if (objNeckLookTarget && objNeckLookTargetP)
        {
            switch (targetType)
            {
                case 1:
                    rotDeg = 0f;
                    range = 1f;
                    break;
                case 2:
                    rotDeg = 45f;
                    range = 1f;
                    break;
                case 3:
                    rotDeg = 90f;
                    range = 1f;
                    break;
                case 4:
                    rotDeg = 135f;
                    range = 1f;
                    break;
                case 5:
                    rotDeg = 180f;
                    range = 1f;
                    break;
                case 6:
                    rotDeg = 225f;
                    range = 1f;
                    break;
                case 7:
                    rotDeg = 270f;
                    range = 1f;
                    break;
                case 8:
                    rotDeg = 315f;
                    range = 1f;
                    break;
            }
            objNeckLookTargetP.transform.SetLocalPosition(0f, 0.27f, 0f);
            neckLookCtrl.target = objNeckLookTarget.transform;
            float y = Mathf.Lerp(0f, range, rate);
            neckLookCtrl.target.SetLocalPosition(0f, y, dis);
            objNeckLookTargetP.transform.localEulerAngles = new Vector3(0f, 0f, 360f - rotDeg);
        }
        fileStatus.neckTargetAngle = rotDeg;
        fileStatus.neckTargetRange = range;
        fileStatus.neckTargetRate = rate;
    }

    public void ChangeLookNeckPtn(int ptn, float rate = 1f)
    {
        if (!(null == neckLookCtrl))
        {
            NeckLookControllerVer2 neckLookCtrl = base.neckLookCtrl;
            fileStatus.neckLookPtn = ptn;
            neckLookCtrl.ptnNo = ptn;
            base.neckLookCtrl.rate = rate;
        }
    }

    public int GetLookNeckPtn()
    {
        return fileStatus.neckLookPtn;
    }

    public void SetForegroundEyesAndEyebrow()
    {
        bool eyes = true;
        if (Manager.Config.EtcData != null)
        {
            eyes = Manager.Config.EtcData.ForegroundEyes;
        }
        if (fileFace.foregroundEyes == 1)
        {
            eyes = false;
        }
        else if (fileFace.foregroundEyes == 2)
        {
            eyes = true;
        }
        bool eyebrow = true;
        if (Manager.Config.EtcData != null)
        {
            eyebrow = Manager.Config.EtcData.ForegroundEyebrow;
        }
        if (fileFace.foregroundEyebrow == 1)
        {
            eyebrow = false;
        }
        else if (fileFace.foregroundEyebrow == 2)
        {
            eyebrow = true;
        }
        if (null != foregroundCtrl)
        {
            foregroundCtrl.SetForeground(eyes, eyebrow);
        }
    }

    public void ChangeHohoAkaRate(float value)
    {
        fileStatus.hohoAkaRate = value;
        if ((bool)customMatFace)
        {
            Color color = customMatFace.GetColor(ChaShader._overcolor2);
            color.a = Mathf.Lerp(0f, 0.2f, fileStatus.hohoAkaRate);
            if (!Game.IsStudio && Manager.Config.EtcData != null && !Manager.Config.EtcData.hohoAka)
                color.a = 0f;
            customMatFace.SetColor(ChaShader._overcolor2, color);
        }
    }

    public bool IsGagEyes()
    {
        Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.cha_eyeset);
        int eyesPtn = GetEyesPtn();
        ListInfoBase value;
        if (!categoryInfo.TryGetValue(eyesPtn, out value))
        {
            return false;
        }
        if (value.GetInfoInt(ChaListDefine.KeyType.EyeObjNo) == 0)
        {
            return false;
        }
        return true;
    }

    protected void InitializeControlHandAll()
    {
        InitializeControlHandObject();
    }

    protected void InitializeControlHandObject()
    {
        sibHand = new ShapeHandInfo();
    }

    protected void ReleaseControlHandAll()
    {
        ReleaseControlHandObject();
    }

    protected void ReleaseControlHandObject(bool init = true)
    {
        if (sibHand != null)
        {
            sibHand.ReleaseShapeInfo();
        }
        if (init)
        {
            InitializeControlHandObject();
        }
    }

    public bool InitShapeHand(Transform trfBone)
    {
        if (sibHand == null)
        {
            return false;
        }
        if (null == trfBone)
        {
            return false;
        }
        string anmKeyInfoName = "cf_anmShapeHand";
        string cateInfoName = "cf_customhand";
        sibHand.InitShapeInfo("abdata", "chara/oo_hand.unity3d", "chara/oo_hand.unity3d", anmKeyInfoName, cateInfoName, trfBone, sex);
        for (int i = 0; i < 2; i++)
        {
            SetShapeHandValue(i, 0, 0, 0f);
        }
        return true;
    }

    public void ReleaseShapeHand()
    {
        if (sibHand != null)
        {
            sibHand.ReleaseShapeInfo();
        }
    }

    public bool GetEnableShapeHand(int lr)
    {
        return fileStatus.enableShapeHand[lr];
    }

    public void SetEnableShapeHand(int lr, bool enable)
    {
        fileStatus.enableShapeHand[lr] = enable;
    }

    public int GetShapeIndexCount()
    {
        return sibHand.GetKeyCount();
    }

    public int GetShapeHandIndex(int lr, int no)
    {
        if (2 <= no)
        {
            return 0;
        }
        return fileStatus.shapeHandPtn[lr, no];
    }

    public float GetShapeHandBlendValue(int lr)
    {
        return fileStatus.shapeHandBlendValue[lr];
    }

    public bool SetShapeHandValue(int lr, int idx01, int idx02, float blend)
    {
        fileStatus.shapeHandPtn[lr, 0] = idx01;
        fileStatus.shapeHandPtn[lr, 1] = idx02;
        fileStatus.shapeHandBlendValue[lr] = blend;
        if (sibHand != null && sibHand.InitEnd)
        {
            sibHand.ChangeValue(lr, idx01, idx02, blend);
        }
        return true;
    }

    public bool SetShapeHandIndex(int lr, int idx01 = -1, int idx02 = -1)
    {
        if (idx01 != -1)
        {
            fileStatus.shapeHandPtn[lr, 0] = idx01;
        }
        if (idx02 != -1)
        {
            fileStatus.shapeHandPtn[lr, 1] = idx02;
        }
        if (sibHand != null && sibHand.InitEnd)
        {
            sibHand.ChangeValue(lr, fileStatus.shapeHandPtn[lr, 0], fileStatus.shapeHandPtn[lr, 1], fileStatus.shapeHandBlendValue[lr]);
        }
        return true;
    }

    public bool SetShapeHandBlend(int lr, float blend)
    {
        fileStatus.shapeHandBlendValue[lr] = blend;
        if (sibHand != null && sibHand.InitEnd)
        {
            sibHand.ChangeValue(lr, fileStatus.shapeHandPtn[lr, 0], fileStatus.shapeHandPtn[lr, 1], fileStatus.shapeHandBlendValue[lr]);
        }
        return true;
    }

    public void UpdateAlwaysShapeHand()
    {
        if (sibHand != null && sibHand.InitEnd)
        {
            ShapeHandInfo shapeHandInfo = sibHand as ShapeHandInfo;
            shapeHandInfo.updateMask = 0;
            if (fileStatus.enableShapeHand[0])
            {
                shapeHandInfo.updateMask |= 1;
            }
            if (fileStatus.enableShapeHand[1])
            {
                shapeHandInfo.updateMask |= 2;
            }
            sibHand.UpdateAlways();
        }
    }

    protected void InitializeControlLoadAll()
    {
        InitializeControlLoadObject();
    }

    protected void InitializeControlLoadObject()
    {
        aaWeightsHead = new AssignedAnotherWeights();
        aaWeightsBody = new AssignedAnotherWeights();
        updateAlphaMask = true;
    }

    protected void ReleaseControlLoadAll()
    {
        ReleaseControlLoadObject(false);
    }

    protected void ReleaseControlLoadObject(bool init = true)
    {
        if (aaWeightsHead != null)
        {
            aaWeightsHead.Release();
        }
        if (aaWeightsBody != null)
        {
            aaWeightsBody.Release();
        }
        if (init)
        {
            InitializeControlLoadObject();
        }
    }

    public bool Load(bool reflectStatus = false)
    {
        StartCoroutine(LoadAsync(false, false));
        return true;
    }

    public IEnumerator LoadAsync(bool reflectStatus = false, bool asyncFlags = true)
    {
        bool sexFemale = sex != 0;
        string mainManifestName = Singleton<Character>.Instance.mainManifestName;
        byte[] status = null;
        if (reflectStatus)
        {
            status = chaFile.GetStatusBytes();
        }
        ReleaseObject();
        if (asyncFlags)
        {
            yield return null;
        }
        objTop = new GameObject("BodyTop");
        if ((bool)objRoot)
        {
            objTop.transform.SetParent(objRoot.transform, false);
        }
        if (asyncFlags)
        {
            SetActiveTop(false);
        }
        AddUpdateCMBodyTexFlags(true, true, true, true, true);
        AddUpdateCMBodyColorFlags(true, true, true, true, true, true);
        AddUpdateCMBodyLayoutFlags(true, true);
        CreateBodyTexture();
        LoadGagMaterial();
        string assetBundleName4;
        if (Game.IsStudio)
            assetBundleName4 = "studio/base/00.unity3d";
        else
            assetBundleName4 = "chara/oo_base.unity3d";
        string assetName5 = !hiPoly ? "p_cf_body_bone_low" : "p_cf_body_bone";
        if (asyncFlags)
        {
            yield return StartCoroutine(Load_Coroutine(assetBundleName4, assetName5, delegate (GameObject x)
            {
                objAnim = x;
            }, true, mainManifestName));
        }
        else
        {
            objAnim = CommonLib.LoadAsset<GameObject>(assetBundleName4, assetName5, true, mainManifestName);
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName4, mainManifestName);
        if (!objAnim)
        {
            yield break;
        }
        animBody = objAnim.GetComponent<Animator>();
        objAnim.transform.SetParent(objTop.transform, false);
        dictDynamicBoneBust = new Dictionary<DynamicBoneKind, DynamicBone_Ver02>();
        DynamicBone_Ver02[] componentsInChildren = objAnim.GetComponentsInChildren<DynamicBone_Ver02>(true);
        DynamicBone_Ver02[] array = componentsInChildren;
        foreach (DynamicBone_Ver02 dynamicBone_Ver in array)
        {
            if (dynamicBone_Ver.Comment == "左胸")
            {
                dictDynamicBoneBust[DynamicBoneKind.BreastL] = dynamicBone_Ver;
            }
            else if (dynamicBone_Ver.Comment == "右胸")
            {
                dictDynamicBoneBust[DynamicBoneKind.BreastR] = dynamicBone_Ver;
            }
            else if (dynamicBone_Ver.Comment == "左尻")
            {
                dictDynamicBoneBust[DynamicBoneKind.HipL] = dynamicBone_Ver;
                if (sex == 0)
                {
                    dynamicBone_Ver.enabled = false;
                }
            }
            else if (dynamicBone_Ver.Comment == "右尻")
            {
                dictDynamicBoneBust[DynamicBoneKind.HipR] = dynamicBone_Ver;
                if (sex == 0)
                {
                    dynamicBone_Ver.enabled = false;
                }
            }
        }
        objBodyBone = objAnim.transform.FindLoop("cf_j_root");
        if ((bool)objBodyBone)
        {
            aaWeightsBody.CreateBoneList(objBodyBone, string.Empty);
            CreateReferenceInfo(1uL, objBodyBone);
            if (!hiPoly)
            {
                GameObject referenceInfo = GetReferenceInfo(RefObjKey.a_n_back_L);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalRotationY(180f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_back_R);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalRotationY(180f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_bust);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPositionY(0.14f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_bust_f);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPositionZ(0.04f);
                    referenceInfo.transform.SetLocalRotationX(-35f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_ind_L);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(-0.0162f, -0.0024f, 0.0006f);
                    referenceInfo.transform.SetLocalRotation(0f, 0f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_ind_R);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(0.0162f, 0.0024f, -0.0006f);
                    referenceInfo.transform.SetLocalRotation(180f, 0f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_mid_L);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(-0.016f, -0.0028f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_mid_R);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(0.016f, 0.0028f, 0f);
                    referenceInfo.transform.SetLocalRotation(180f, 0f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_ring_L);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(-0.018f, -0.0018f, -0.0005f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_ring_R);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(0.018f, 0.0018f, 0.0005f);
                    referenceInfo.transform.SetLocalRotation(180f, 0f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_nip_L);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(0f, 0f, 0.05f);
                    referenceInfo.transform.SetLocalRotation(0f, 0f, 0f);
                }
                referenceInfo = GetReferenceInfo(RefObjKey.a_n_nip_R);
                if ((bool)referenceInfo)
                {
                    referenceInfo.transform.SetLocalPosition(0f, 0f, 0.05f);
                    referenceInfo.transform.SetLocalRotation(0f, 0f, 0f);
                }
            }
            NeckLookControllerVer2[] componentsInChildren2 = objBodyBone.GetComponentsInChildren<NeckLookControllerVer2>(true);
            if (componentsInChildren2.Length != 0)
            {
                neckLookCtrl = componentsInChildren2[0];
            }
            if ((bool)neckLookCtrl)
            {
                ChangeLookNeckTarget(fileStatus.neckTargetType);
                ChangeLookNeckPtn(0);
            }
            InitShapeBody(objBodyBone.transform);
            InitShapeHand(objBodyBone.transform);
            objNeckLookTargetP = new GameObject("N_NeckLookTargetP");
            GameObject referenceInfo2 = GetReferenceInfo(RefObjKey.NECK_LOOK_TARGET);
            if (objNeckLookTargetP && referenceInfo2)
            {
                objNeckLookTargetP.transform.SetParent(referenceInfo2.transform, false);
                objNeckLookTarget = new GameObject("N_NeckLookTarget");
                if ((bool)objNeckLookTarget)
                {
                    objNeckLookTarget.transform.SetParent(objNeckLookTargetP.transform, false);
                }
            }
            objEyesLookTargetP = new GameObject("N_EyesLookTargetP");
            referenceInfo2 = GetReferenceInfo(RefObjKey.HeadParent);
            if (objEyesLookTargetP && referenceInfo2)
            {
                objEyesLookTargetP.transform.SetParent(referenceInfo2.transform, false);
                objEyesLookTarget = new GameObject("N_EyesLookTarget");
                if ((bool)objEyesLookTarget)
                {
                    objEyesLookTarget.transform.SetParent(objEyesLookTargetP.transform, false);
                }
            }
        }
        string assetBundleName6;
        if (Game.IsStudio)
            assetBundleName6 = "studio/base/00.unity3d";
        else
            assetBundleName6 = "chara/oo_base.unity3d";
        string assetName6 = "p_cf_head_bone";
        if (asyncFlags)
        {
            yield return StartCoroutine(Load_Coroutine(assetBundleName6, assetName6, delegate (GameObject x)
            {
                objHeadBone = x;
            }, true, mainManifestName));
        }
        else
        {
            objHeadBone = CommonLib.LoadAsset<GameObject>(assetBundleName6, assetName6, true, mainManifestName);
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName6, mainManifestName);
        if (!objHeadBone)
        {
            yield break;
        }
        GameObject referenceInfo3 = GetReferenceInfo(RefObjKey.HeadParent);
        objHeadBone.transform.SetParent(referenceInfo3.transform, false);
        EyeLookController[] componentsInChildren3 = objHeadBone.GetComponentsInChildren<EyeLookController>(true);
        if (componentsInChildren3.Length != 0)
        {
            eyeLookCtrl = componentsInChildren3[0];
        }
        if ((bool)eyeLookCtrl)
        {
            EyeLookCalc component = eyeLookCtrl.GetComponent<EyeLookCalc>();
            if ((bool)component)
            {
                ChangeLookEyesTarget(fileStatus.eyesTargetType);
                ChangeLookEyesPtn(0);
            }
        }
        aaWeightsHead.CreateBoneList(objHeadBone, string.Empty);
        CreateReferenceInfo(2uL, objHeadBone);
        if (fileParam.exType == 0)
        {
            string assetBundleName5 = "chara/oo_base.unity3d";
            string assetName4 = hiPoly ? !sexFemale ? "p_cm_body_00" : "p_cf_body_00" : !sexFemale ? "p_cm_body_00_low" : "p_cf_body_00_low";
            if (asyncFlags)
            {
                yield return StartCoroutine(Load_Coroutine(assetBundleName5, assetName4, delegate (GameObject x)
                {
                    objBody = x;
                }, true, mainManifestName));
            }
            else
            {
                objBody = CommonLib.LoadAsset<GameObject>(assetBundleName5, assetName4, true, mainManifestName);
            }
            Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName5, mainManifestName);
            if ((bool)objBody)
            {
                CreateReferenceInfo(3uL, objBody);
                GameObject referenceInfo4 = GetReferenceInfo(RefObjKey.ObjBody);
                if ((bool)referenceInfo4)
                {
                    rendBody = referenceInfo4.GetComponent<Renderer>();
                }
                GameObject referenceInfo5 = GetReferenceInfo(RefObjKey.S_SimpleBody);
                if ((bool)referenceInfo5)
                {
                    rendSimpleBody = referenceInfo5.GetComponent<Renderer>();
                }
                GameObject referenceInfo6 = GetReferenceInfo(RefObjKey.S_SimpleTongue);
                if ((bool)referenceInfo6)
                {
                    rendSimpleTongue = referenceInfo6.GetComponent<Renderer>();
                }
                SetBodyBaseMaterial();
                objBody.transform.SetParent(objTop.transform, false);
                GameObject referenceInfo7 = GetReferenceInfo(RefObjKey.A_ROOTBONE);
                Transform rootBone = !referenceInfo7 ? null : referenceInfo7.transform;
                aaWeightsBody.AssignedWeightsAndSetBounds(objBody, "cf_j_root", bounds, rootBone);
                if (sex == 1 && hiPoly)
                {
                    assetBundleName5 = "chara/oo_base.unity3d";
                    assetName4 = "p_cf_body_00_Nml";
                    BustNormal value = null;
                    if (dictBustNormal.TryGetValue(BustNormalKind.NmlBody, out value))
                    {
                        value.Release();
                    }
                    value = new BustNormal();
                    value.Init(objBody, assetBundleName5, assetName4, string.Empty);
                    dictBustNormal[BustNormalKind.NmlBody] = value;
                }
                ChangeCustomBodyWithoutCustomTexture();
            }
        }
        else
        {
            ListInfoBase lib = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_body, 0);
            if (lib != null)
            {
                string manifestName = lib.GetInfo(ChaListDefine.KeyType.MainManifest);
                string assetBundleName2 = lib.GetInfo(ChaListDefine.KeyType.MainAB);
                string assetName2 = lib.GetInfo(ChaListDefine.KeyType.MainData);
                if (asyncFlags)
                {
                    yield return StartCoroutine(Load_Coroutine(assetBundleName2, assetName2, delegate (GameObject x)
                    {
                        objBody = x;
                    }, true, manifestName));
                }
                else
                {
                    objBody = CommonLib.LoadAsset<GameObject>(assetBundleName2, assetName2, true, manifestName);
                }
                Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName2, manifestName);
                if ((bool)objBody)
                {
                    CreateReferenceInfo(3uL, objBody);
                    GameObject referenceInfo8 = GetReferenceInfo(RefObjKey.ObjBody);
                    if ((bool)referenceInfo8)
                    {
                        rendBody = referenceInfo8.GetComponent<Renderer>();
                    }
                    GameObject referenceInfo9 = GetReferenceInfo(RefObjKey.S_SimpleBody);
                    if ((bool)referenceInfo9)
                    {
                        rendSimpleBody = referenceInfo9.GetComponent<Renderer>();
                    }
                    GameObject referenceInfo10 = GetReferenceInfo(RefObjKey.S_SimpleTongue);
                    if ((bool)referenceInfo10)
                    {
                        rendSimpleTongue = referenceInfo10.GetComponent<Renderer>();
                    }
                    SetBodyBaseMaterial();
                    objBody.transform.SetParent(objTop.transform, false);
                    GameObject referenceInfo11 = GetReferenceInfo(RefObjKey.A_ROOTBONE);
                    Transform rootBone2 = !referenceInfo11 ? null : referenceInfo11.transform;
                    aaWeightsBody.AssignedWeightsAndSetBounds(objBody, "cf_j_root", bounds, rootBone2);
                    ChangeCustomBodyWithoutCustomTexture();
                }
            }
        }
        if (hiPoly)
        {
            string assetBundleName = "chara/oo_base.unity3d";
            string assetName = "p_tang2";
            if (asyncFlags)
            {
                yield return StartCoroutine(Load_Coroutine(assetBundleName, assetName, delegate (GameObject x)
                {
                    objTongueEx = x;
                }, true, mainManifestName));
            }
            else
            {
                objTongueEx = CommonLib.LoadAsset<GameObject>(assetBundleName, assetName, true, mainManifestName);
            }
            Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, mainManifestName);
            if ((bool)objTongueEx)
            {
                GameObject referenceInfo12 = GetReferenceInfo(RefObjKey.HeadParent);
                objTongueEx.transform.SetParent(referenceInfo12.transform, false);
                animTongueEx = objTongueEx.GetComponent<Animator>();
            }
        }
        if (asyncFlags)
        {
            yield return null;
        }
        if (asyncFlags)
        {
            yield return StartCoroutine(ChangeHeadAsync(true));
        }
        else
        {
            ChangeHead(true);
        }
        LoadHairGlossMask();
        if (asyncFlags)
        {
            yield return StartCoroutine(ChangeHairAsync(true));
        }
        else
        {
            ChangeHair(true);
        }
        if (asyncFlags)
        {
            yield return StartCoroutine(ChangeClothesAsync(true));
        }
        else
        {
            ChangeClothes(true);
        }
        if (asyncFlags)
        {
            yield return StartCoroutine(ChangeAccessoryAsync(true));
        }
        else
        {
            ChangeAccessory(true);
        }
        updateBustSize = true;
        reSetupDynamicBoneBust = true;
        if (hiPoly)
        {
            UpdateSiru(true);
        }
        ChangeHohoAkaRate(fileStatus.hohoAkaRate);
        if (reflectStatus)
        {
            chaFile.SetStatusBytes(status);
        }
        if (Singleton<Character>.Instance.enableCharaLoadGCClear)
        {
            Resources.UnloadUnusedAssets();
            GC.Collect();
        }
        loadEnd = true;
    }

    public bool Reload(bool noChangeClothes = false, bool noChangeHead = false, bool noChangeHair = false, bool noChangeBody = false)
    {
        StartCoroutine(ReloadAsync(noChangeClothes, noChangeHead, noChangeHair, noChangeBody, false));
        return true;
    }

    public IEnumerator ReloadAsync(bool noChangeClothes = false, bool noChangeHead = false, bool noChangeHair = false, bool noChangeBody = false, bool asyncFlags = true)
    {
        if (asyncFlags)
        {
            SetActiveTop(false);
        }
        if (!noChangeBody)
        {
            AddUpdateCMBodyTexFlags(true, true, true, true, true);
            AddUpdateCMBodyColorFlags(true, true, true, true, true, true);
            AddUpdateCMBodyLayoutFlags(true, true);
            CreateBodyTexture();
            if (noChangeHead)
            {
                AddUpdateCMFaceTexFlags(true, true, true, true, true, true, true);
                AddUpdateCMFaceColorFlags(true, true, true, true, true, true, true);
                AddUpdateCMFaceLayoutFlags(true, true, true);
                CreateFaceTexture();
                ChangeCustomFaceWithoutCustomTexture();
            }
        }
        if (!noChangeHead)
        {
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeHeadAsync(true));
            }
            else
            {
                ChangeHead(true);
            }
        }
        if (!noChangeHair)
        {
            LoadHairGlossMask();
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeHairAsync(true));
            }
            else
            {
                ChangeHair(true);
            }
        }
        if (!noChangeClothes)
        {
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeClothesAsync(true));
            }
            else
            {
                ChangeClothes(true);
            }
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeAccessoryAsync(true));
            }
            else
            {
                ChangeAccessory(true);
            }
        }
        if (asyncFlags)
        {
            yield return null;
        }
        UpdateShapeBodyValueFromCustomInfo();
        UpdateShapeFaceValueFromCustomInfo();
        updateBustSize = true;
        reSetupDynamicBoneBust = true;
        if (!noChangeBody)
        {
            ChangeCustomBodyWithoutCustomTexture();
        }
        UpdateClothesStateAll();
        if (Singleton<Character>.Instance.enableCharaLoadGCClear)
        {
            Resources.UnloadUnusedAssets();
            GC.Collect();
        }
        if (asyncFlags)
        {
            yield return null;
        }
    }

    public void ChangeHead(bool forceChange = false)
    {
        StartCoroutine(ChangeHeadAsync(fileFace.headId, forceChange, false));
    }

    public void ChangeHead(int _headId, bool forceChange = false)
    {
        StartCoroutine(ChangeHeadAsync(_headId, forceChange, false));
    }

    public IEnumerator ChangeHeadAsync(bool forceChange = false)
    {
        yield return StartCoroutine(ChangeHeadAsync(fileFace.headId, forceChange));
    }

    public IEnumerator ChangeHeadAsync(int _headId, bool forceChange = false, bool asyncFlags = true)
    {
        if (_headId == -1 || !forceChange && null != objHead && _headId == fileFace.headId)
        {
            yield break;
        }
        string objName = "ct_head";
        if ((bool)objHead)
        {
            SafeDestroy(objHead);
            objHead = null;
            infoHead = null;
            ReleaseRefObject(4uL);
            ReleaseShapeFace();
            foregroundCtrl = null;
            rendFace = null;
            rendEye = new Renderer[2];
            rendEyeW = new Renderer[2];
            rendEyelineUp = null;
            rendEyelineDown = null;
            rendEyebrow = null;
            rendNose = null;
        }
        if (asyncFlags)
        {
            IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
            {
                objHead = o;
            }, true, sex != 0 || exType != 1 ? 100 : 501, _headId, objName, false, 0, null, 0);
            yield return StartCoroutine(cor);
        }
        else
        {
            objHead = LoadCharaFbxData(true, sex != 0 || exType != 1 ? 100 : 501, _headId, objName, false, 0, null, 0);
        }
        if ((bool)objHead)
        {
            CommonLib.CopySameNameTransform(objHeadBone.transform, objHead.transform);
            objHead.transform.SetParent(objHeadBone.transform, false);
            GameObject objRootBone = GetReferenceInfo(RefObjKey.A_ROOTBONE);
            Transform trfRootBone = !objRootBone ? null : objRootBone.transform;
            aaWeightsHead.AssignedWeightsAndSetBounds(objHead, "cf_J_N_FaceRoot", bounds, trfRootBone);
            if (asyncFlags)
            {
                yield return null;
            }
            ListInfoComponent libComponent = objHead.GetComponent<ListInfoComponent>();
            ListInfoBase listInfoBase = infoHead = libComponent.data;
            ListInfoBase lib = listInfoBase;
            foregroundCtrl = objHead.GetComponent<ChaForegroundComponent>();
            fileFace.headId = lib.Id;
            CreateReferenceInfo(4uL, objHead);
            GameObject objEyeL = GetReferenceInfo(RefObjKey.ObjEyeL);
            if ((bool)objEyeL)
            {
                eyeLookMatCtrl[0] = objEyeL.GetComponent<EyeLookMaterialControll>();
                if ((bool)eyeLookMatCtrl[0])
                {
                    eyeLookMatCtrl[0].script = eyeLookCtrl.eyeLookScript;
                    eyeLookMatCtrl[0].ReSetupMaterial();
                }
            }
            GameObject objEyeR = GetReferenceInfo(RefObjKey.ObjEyeR);
            if ((bool)objEyeR)
            {
                eyeLookMatCtrl[1] = objEyeR.GetComponent<EyeLookMaterialControll>();
                if ((bool)eyeLookMatCtrl[1])
                {
                    eyeLookMatCtrl[1].script = eyeLookCtrl.eyeLookScript;
                    eyeLookMatCtrl[1].ReSetupMaterial();
                }
            }
            GameObject refObj9 = GetReferenceInfo(RefObjKey.ObjFace);
            if (null != refObj9)
            {
                rendFace = refObj9.GetComponent<Renderer>();
            }
            rendEye = new Renderer[2];
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyeL);
            if (null != refObj9)
            {
                rendEye[0] = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyeR);
            if (null != refObj9)
            {
                rendEye[1] = refObj9.GetComponent<Renderer>();
            }
            rendEyeW = new Renderer[2];
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyeWL);
            if (null != refObj9)
            {
                rendEyeW[0] = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyeWR);
            if (null != refObj9)
            {
                rendEyeW[1] = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyeline);
            if (null != refObj9)
            {
                rendEyelineUp = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyelineLow);
            if (null != refObj9)
            {
                rendEyelineDown = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjEyebrow);
            if (null != refObj9)
            {
                rendEyebrow = refObj9.GetComponent<Renderer>();
            }
            refObj9 = GetReferenceInfo(RefObjKey.ObjNoseline);
            if (null != refObj9)
            {
                rendNose = refObj9.GetComponent<Renderer>();
            }
            string customManifest = lib.GetInfo(ChaListDefine.KeyType.MatManifest);
            string customAssetBundle = lib.GetInfo(ChaListDefine.KeyType.MatAB);
            string customAsset = lib.GetInfo(ChaListDefine.KeyType.MatData);
            InitBaseCustomTextureFace(customAssetBundle, customAsset, customManifest);
            AddUpdateCMFaceTexFlags(true, true, true, true, true, true, true);
            AddUpdateCMFaceColorFlags(true, true, true, true, true, true, true);
            AddUpdateCMFaceLayoutFlags(true, true, true);
            CreateFaceTexture();
            SetFaceBaseMaterial();
            ChangeSettingWhiteOfEye(true, true);
            ChangeSettingEye(2, true, true, true);
            if (hiPoly)
            {
                UpdateSiru(true);
            }
            ChangeHohoAkaRate(fileStatus.hohoAkaRate);
            InitShapeFace(objHeadBone.transform, lib.GetInfo(ChaListDefine.KeyType.MainAB), lib.GetInfo(ChaListDefine.KeyType.ShapeAnime), lib.GetInfo(ChaListDefine.KeyType.MainManifest));
            ChangeCustomFaceWithoutCustomTexture();
            HideEyeHighlight(false);
            fbsCtrl = objHead.GetComponent<FaceBlendShape>();
            if (null != fbsCtrl)
            {
                eyebrowCtrl = fbsCtrl.EyebrowCtrl;
                eyesCtrl = fbsCtrl.EyesCtrl;
                mouthCtrl = fbsCtrl.MouthCtrl;
                ChangeEyebrowPtn(fileStatus.eyebrowPtn);
                ChangeEyebrowOpenMax(fileStatus.eyebrowOpenMax);
                ChangeEyesPtn(fileStatus.eyesPtn);
                ChangeEyesOpenMax(fileStatus.eyesOpenMax);
                ChangeMouthPtn(fileStatus.mouthPtn);
                ChangeMouthOpenMax(fileStatus.mouthOpenMax);
                fbsCtrl.EyeLookController = eyeLookCtrl;
                ChangeEyesBlinkFlag(fileStatus.eyesBlink);
            }
        }
        if (asyncFlags)
        {
            yield return null;
        }
    }

    public bool ChangeHairFront(bool forceChange = false)
    {
        int num = 1;
        StartCoroutine(ChangeHairAsync(num, fileHair.parts[num].id, forceChange, false));
        return true;
    }

    public bool ChangeHairBack(bool forceChange = false)
    {
        int num = 0;
        StartCoroutine(ChangeHairAsync(num, fileHair.parts[num].id, forceChange, false));
        return true;
    }

    public bool ChangeHairSide(bool forceChange = false)
    {
        int num = 2;
        StartCoroutine(ChangeHairAsync(num, fileHair.parts[num].id, forceChange, false));
        return true;
    }

    public bool ChangeHairOption(bool forceChange = false)
    {
        int num = 3;
        StartCoroutine(ChangeHairAsync(num, fileHair.parts[num].id, forceChange, false));
        return true;
    }

    public void ChangeHair(bool forceChange = false)
    {
        int[] array = (int[])Enum.GetValues(typeof(ChaFileDefine.HairKind));
        int[] array2 = array;
        foreach (int num in array2)
        {
            StartCoroutine(ChangeHairAsync(num, fileHair.parts[num].id, forceChange, false));
        }
    }

    public void ChangeHair(int kind, int id, bool forceChange = false)
    {
        StartCoroutine(ChangeHairAsync(kind, id, forceChange, false));
    }

    public IEnumerator ChangeHairAsync(bool forceChange = false)
    {
        int[] hairKind = (int[])Enum.GetValues(typeof(ChaFileDefine.HairKind));
        int[] array = hairKind;
        foreach (int i in array)
        {
            yield return StartCoroutine(ChangeHairAsync(i, fileHair.parts[i].id, forceChange));
        }
    }

    public IEnumerator ChangeHairAsync(int kind, int id, bool forceChange = false, bool asyncFlags = true)
    {
        ChaListDefine.CategoryNo[] fbxType = new ChaListDefine.CategoryNo[5]
        {
            ChaListDefine.CategoryNo.bo_hair_b,
            ChaListDefine.CategoryNo.bo_hair_f,
            ChaListDefine.CategoryNo.bo_hair_s,
            ChaListDefine.CategoryNo.bo_hair_o,
            ChaListDefine.CategoryNo.bo_hair_o
        };
        string[] objName = new string[5]
        {
            "ct_hairB",
            "ct_hairF",
            "ct_hairS",
            "ct_hairO_01",
            "ct_hairO_02"
        };
        int[] defId = new int[5]
        {
            0,
            1,
            0,
            0,
            0
        };
        if (sex == 0 && exType == 1 && kind >= 1)
        {
            id = 0;
        }
        if (!forceChange && null != objHair[kind] && id == fileHair.parts[kind].id)
        {
            yield break;
        }
        if ((bool)objHair[kind])
        {
            SafeDestroy(objHair[kind]);
            objHair[kind] = null;
            infoHair[kind] = null;
            cusHairCmp[kind] = null;
        }
        GameObject objHairParent = GetReferenceInfo(RefObjKey.HairParent);
        if (asyncFlags)
        {
            IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
            {
                objHair[kind] = o;
            }, hiPoly, (int)(sex != 0 || exType != 1 || kind != 0 ? fbxType[kind] : ChaListDefine.CategoryNo.ex_bo_hair), id, objName[kind], false, 0, objHairParent.transform, defId[kind]);
            yield return StartCoroutine(cor);
        }
        else
        {
            objHair[kind] = LoadCharaFbxData(hiPoly, (int)(sex != 0 || exType != 1 || kind != 0 ? fbxType[kind] : ChaListDefine.CategoryNo.ex_bo_hair), id, objName[kind], false, 0, objHairParent.transform, defId[kind]);
        }
        if ((bool)objHair[kind])
        {
            ListInfoComponent component = objHair[kind].GetComponent<ListInfoComponent>();
            ListInfoBase listInfoBase = infoHair[kind] = component.data;
            fileHair.parts[kind].id = listInfoBase.Id;
            if (kind == 0)
            {
                fileHair.kind = listInfoBase.Kind;
            }
            cusHairCmp[kind] = objHair[kind].GetComponent<ChaCustomHairComponent>();
            if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusHairCmp[kind])
            {
            }
            ChangeSettingHairGlossMask(kind);
            ChangeSettingHairColor(kind, true, true, true);
            ChangeSettingHairOutlineColor(kind);
            ChangeSettingHairLength(kind);
            ChangeSettingHairAcsColor(kind);
            if (!hiPoly)
            {
                DynamicBone[] componentsInChildren = objHair[kind].GetComponentsInChildren<DynamicBone>(true);
                DynamicBone[] array = componentsInChildren;
                foreach (DynamicBone dynamicBone in array)
                {
                    dynamicBone.enabled = false;
                }
            }
            ChangeShakeHair(kind);
        }
        if (asyncFlags)
        {
            yield return null;
        }
    }

    public void ChangeClothes(bool forceChange = false)
    {
        int[] array = (int[])Enum.GetValues(typeof(ChaFileDefine.ClothesKind));
        int[] array2 = array;
        foreach (int num in array2)
        {
            StartCoroutine(ChangeClothesAsync(num, nowCoordinate.clothes.parts[num].id, nowCoordinate.clothes.subPartsId[0], nowCoordinate.clothes.subPartsId[1], nowCoordinate.clothes.subPartsId[2], forceChange, false));
        }
    }

    public void ChangeClothes(int kind, int id, int subId01, int subId02, int subId03, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesAsync(kind, id, subId01, subId02, subId03, forceChange, false));
    }

    public IEnumerator ChangeClothesAsync(bool forceChange = false)
    {
        int[] clothesKind = (int[])Enum.GetValues(typeof(ChaFileDefine.ClothesKind));
        int[] array = clothesKind;
        foreach (int i in array)
        {
            yield return StartCoroutine(ChangeClothesAsync(i, nowCoordinate.clothes.parts[i].id, nowCoordinate.clothes.subPartsId[0], nowCoordinate.clothes.subPartsId[1], nowCoordinate.clothes.subPartsId[2], forceChange));
        }
    }

    public IEnumerator ChangeClothesAsync(int kind, int id, int subId01, int subId02, int subId03, bool forceChange = false, bool asyncFlags = true)
    {
        if (asyncFlags)
        {
            switch (kind)
            {
                case 0:
                    yield return StartCoroutine(ChangeClothesTopAsync(id, subId01, subId02, subId03, forceChange));
                    updateAlphaMask = true;
                    break;
                case 1:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesBotAsync(id, forceChange));
                    }
                    break;
                case 2:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesBraAsync(id, forceChange));
                    }
                    break;
                case 3:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesShortsAsync(id, forceChange));
                    }
                    break;
                case 4:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesGlovesAsync(id, forceChange));
                    }
                    break;
                case 5:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesPanstAsync(id, forceChange));
                    }
                    break;
                case 6:
                    if (sex != 0 || exType != 1)
                    {
                        yield return StartCoroutine(ChangeClothesSocksAsync(id, forceChange));
                    }
                    break;
                case 7:
                    yield return StartCoroutine(ChangeClothesShoesAsync(0, id, forceChange));
                    break;
                case 8:
                    yield return StartCoroutine(ChangeClothesShoesAsync(1, id, forceChange));
                    break;
            }
            yield break;
        }
        switch (kind)
        {
            case 0:
                ChangeClothesTop(id, subId01, subId02, subId03, forceChange);
                updateAlphaMask = true;
                break;
            case 1:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesBot(id, forceChange);
                }
                break;
            case 2:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesBra(id, forceChange);
                }
                break;
            case 3:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesShorts(id, forceChange);
                }
                break;
            case 4:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesGloves(id, forceChange);
                }
                break;
            case 5:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesPanst(id, forceChange);
                }
                break;
            case 6:
                if (sex != 0 || exType != 1)
                {
                    ChangeClothesSocks(id, forceChange);
                }
                break;
            case 7:
                ChangeClothesShoes(0, id, forceChange);
                break;
            case 8:
                ChangeClothesShoes(1, id, forceChange);
                break;
        }
    }

    public void ChangeClothesTop(int id, int subId01, int subId02, int subId03, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesTopAsync(id, subId01, subId02, subId03, forceChange, false));
    }

    public IEnumerator ChangeClothesTopAsync(int id, int subId01, int subId02, int subId03, bool forceChange = false, bool asyncFlags = true)
    {
        updateAlphaMask = true;
        bool load = true;
        bool release = true;
        int kindNo = 0;
        string objName = "ct_clothesTop";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            BustNormal value = null;
            if (dictBustNormal.TryGetValue(BustNormalKind.NmlTop, out value))
            {
                value.Release();
            }
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            ReleaseRefObject(5uL);
            notBra = false;
            notBot = false;
            RemoveClothesStateKind(kindNo);
            if (null != texBodyAlphaMask && customMatBody)
            {
                customMatBody.SetTexture(ChaShader._AlphaMask, null);
            }
            texBodyAlphaMask = null;
            texBraAlphaMask = null;
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, sex != 0 || exType != 1 ? 105 : 503, id, objName, true, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, sex != 0 || exType != 1 ? 105 : 503, id, objName, true, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                CreateReferenceInfo(5uL, objClothes[kindNo]);
                notBra = listInfoBase.GetInfoInt(ChaListDefine.KeyType.NotBra) == 1;
                notBot = listInfoBase.GetInfoInt(ChaListDefine.KeyType.Coordinate) == 2;
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
                LoadAlphaMaskTexture(listInfoBase.GetInfo(ChaListDefine.KeyType.OverBodyMaskAB), listInfoBase.GetInfo(ChaListDefine.KeyType.OverBodyMask), 0);
                if ((bool)customMatBody)
                {
                    customMatBody.SetTexture(ChaShader._AlphaMask, texBodyAlphaMask);
                }
                LoadAlphaMaskTexture(listInfoBase.GetInfo(ChaListDefine.KeyType.OverBraMaskAB), listInfoBase.GetInfo(ChaListDefine.KeyType.OverBraMask), 1);
                if (rendBra != null)
                {
                    Vector2[] array = new Vector2[2]
                    {
                        new Vector2(0f, 0f),
                        new Vector2(0f, -1f)
                    };
                    Vector2[] array2 = new Vector2[2]
                    {
                        new Vector2(1f, 1f),
                        new Vector2(1f, 2f)
                    };
                    ListInfoBase listInfoBase2 = infoClothes[2];
                    if (listInfoBase2 != null)
                    {
                        int num = listInfoBase2.GetInfoInt(ChaListDefine.KeyType.Coordinate) == 2 ? 1 : 0;
                        string value2;
                        if (listInfoBase2.dictInfo.TryGetValue(105, out value2))
                        {
                            num = "1" == value2 ? 1 : 0;
                        }
                        if ((bool)rendBra[0])
                        {
                            rendBra[0].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                            rendBra[0].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                            rendBra[0].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                        }
                        if ((bool)rendBra[1])
                        {
                            rendBra[1].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                            rendBra[1].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                            rendBra[1].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                        }
                    }
                }
                string info = listInfoBase.GetInfo(ChaListDefine.KeyType.NormalData);
                if ("0" != info)
                {
                    BustNormal value3 = null;
                    if (!dictBustNormal.TryGetValue(BustNormalKind.NmlTop, out value3))
                    {
                        value3 = new BustNormal();
                    }
                    value3.Init(objClothes[kindNo], listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB), info, listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest));
                    dictBustNormal[BustNormalKind.NmlTop] = value3;
                }
            }
        }
        if ((bool)objClothes[kindNo])
        {
            InitBaseCustomTextureClothes(true, kindNo);
            if (loadWithDefaultColorAndPtn)
            {
                SetClothesDefaultColor(kindNo);
                for (int j = 0; j < 4; j++)
                {
                    nowCoordinate.clothes.parts[kindNo].colorInfo[j].pattern = 0;
                }
            }
            ChangeCustomClothes(true, kindNo, true, true, true, true, false);
            for (int k = 0; k < 2; k++)
            {
                ChangeCustomEmblem(kindNo, k);
            }
            if (releaseCustomInputTexture)
            {
                ReleaseBaseCustomTextureClothes(true, kindNo, false);
            }
            int[] partsKindNo = new int[3]
            {
                0,
                1,
                2
            };
            int[] partsID = new int[3]
            {
                subId01,
                subId02,
                subId03
            };
            int partsFig = Enum.GetNames(typeof(ChaFileDefine.ClothesSubKind)).Length;
            bool forcePartsChange = load || forceChange;
            if (sex != 0 || exType != 1)
            {
                if (asyncFlags)
                {
                    for (int i = 0; i < partsFig; i++)
                    {
                        yield return StartCoroutine(ChangeClothesTopPartsAsync(partsKindNo[i], partsID[i], forceChange));
                    }
                }
                else
                {
                    for (int l = 0; l < partsFig; l++)
                    {
                        ChangeClothesTopParts(partsKindNo[l], partsID[l], forcePartsChange);
                    }
                }
            }
        }
        if (hiPoly)
        {
            UpdateSiru(true);
        }
        if (!("0" == infoClothes[kindNo].GetInfo(ChaListDefine.KeyType.Coordinate)))
        {
            yield break;
        }
        int botKind = 1;
        if (null == objClothes[botKind])
        {
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeClothesBotAsync(nowCoordinate.clothes.parts[botKind].id));
            }
            else
            {
                ChangeClothesBot(nowCoordinate.clothes.parts[botKind].id);
            }
        }
    }

    public void ChangeClothesTopParts(int kind, int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesTopPartsAsync(kind, id, forceChange, false));
    }

    public IEnumerator ChangeClothesTopPartsAsync(int kind, int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int nowId = infoParts[kind] != null ? infoParts[kind].Id : -1;
        if (!forceChange && null != objParts[kind] && id == nowId)
        {
            load = false;
            release = false;
        }
        int topType = 0;
        ListInfoBase libTop = infoClothes[0];
        if (libTop != null)
        {
            topType = libTop.Kind;
        }
        bool noPartsItem = topType != 1 && 2 != topType;
        if (noPartsItem || null == objClothes[0])
        {
            load = false;
            release = true;
            nowCoordinate.clothes.subPartsId[kind] = 0;
        }
        ReleaseBaseCustomTextureClothes(false, kind);
        if (release && objParts[kind])
        {
            SafeDestroy(objParts[kind]);
            objParts[kind] = null;
            BustNormal value = null;
            if (dictBustNormal.TryGetValue(BustNormalKind.NmlTopPA, out value))
            {
                value.Release();
            }
            infoParts[kind] = null;
            cusClothesSubCmp[kind] = null;
            if (topType == 1 && kind == 0 || topType == 2 && kind != 2)
            {
                notBra = false;
            }
            ReleaseRefObject((ulong)(13L + kind));
            if (!noPartsItem)
            {
                if (null != texBodyAlphaMask && kind == 0)
                {
                    if ((bool)customMatBody)
                    {
                        customMatBody.SetTexture(ChaShader._AlphaMask, null);
                    }
                    texBodyAlphaMask = null;
                }
                if (null != texBraAlphaMask && kind == 0)
                {
                    if (rendBra != null)
                    {
                        Renderer[] rendBra = base.rendBra;
                        foreach (Renderer renderer in rendBra)
                        {
                            if (null != renderer)
                            {
                                renderer.material.SetTexture(ChaShader._AlphaMask, null);
                            }
                        }
                    }
                    texBraAlphaMask = null;
                }
            }
            if (kind == 0)
            {
                rendInner = new Renderer[2];
            }
            if (kind == 1 && null != texInnerAlphaMask)
            {
                if (rendInner != null)
                {
                    Renderer[] rendInner = base.rendInner;
                    foreach (Renderer renderer2 in rendInner)
                    {
                        if (null != renderer2)
                        {
                            renderer2.material.SetTexture(ChaShader._AlphaMask, null);
                        }
                    }
                }
                texInnerAlphaMask = null;
            }
        }
        if (load)
        {
            string[] goName = new string[3]
            {
                "ct_top_parts_A",
                "ct_top_parts_B",
                "ct_top_parts_C"
            };
            int cateNo = (topType != 1 ? 210 : 200) + kind;
            if (asyncFlags)
            {
                yield return null;
            }
            int[,] defaultId = new int[2, 3]
            {
                {
                    0,
                    0,
                    1
                },
                {
                    0,
                    1,
                    1
                }
            };
            int defFirstArrNo = topType != 1 ? 1 : 0;
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objParts[kind] = o;
                }, hiPoly, cateNo, id, goName[kind], true, 1, objClothes[0].transform, defaultId[defFirstArrNo, kind]);
                yield return StartCoroutine(cor);
            }
            else
            {
                objParts[kind] = LoadCharaFbxData(hiPoly, cateNo, id, goName[kind], true, 1, objClothes[0].transform, defaultId[defFirstArrNo, kind]);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objParts[kind])
            {
                ListInfoComponent component = objParts[kind].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoParts[kind] = component.data;
                cusClothesSubCmp[kind] = objParts[kind].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesSubCmp[kind])
                {
                }
                nowCoordinate.clothes.subPartsId[kind] = listInfoBase.Id;
                if (topType == 2)
                {
                    if (kind == 0)
                    {
                        if (listInfoBase.Id == 7)
                        {
                            cusClothesSubCmp[kind].useColorN02 = true;
                            cusClothesSubCmp[kind].useColorN03 = false;
                        }
                        else if (listInfoBase.Id == 9)
                        {
                            cusClothesSubCmp[kind].useColorN03 = true;
                        }
                    }
                    else if (kind == 1)
                    {
                        if (listInfoBase.Id == 11)
                        {
                            cusClothesSubCmp[kind].useColorN01 = false;
                            cusClothesSubCmp[kind].useColorN02 = true;
                            cusClothesSubCmp[kind].useColorN03 = false;
                        }
                        else if (listInfoBase.Id == 12)
                        {
                            cusClothesSubCmp[kind].useColorN01 = false;
                            cusClothesSubCmp[kind].useColorN02 = true;
                            cusClothesSubCmp[kind].useColorN03 = false;
                        }
                        else if (listInfoBase.Id == 9)
                        {
                            cusClothesSubCmp[kind].useColorN01 = false;
                            cusClothesSubCmp[kind].useColorN02 = true;
                            cusClothesSubCmp[kind].useColorN03 = false;
                        }
                    }
                }
                CreateReferenceInfo((ulong)(13L + kind), objParts[kind]);
                string info = listInfoBase.GetInfo(ChaListDefine.KeyType.NormalData);
                if (topType == 1 && kind == 0)
                {
                    notBra = listInfoBase.GetInfoInt(ChaListDefine.KeyType.NotBra) == 1;
                }
                if (topType == 2)
                {
                    ListInfoBase listInfoBase2 = infoParts[0];
                    ListInfoBase listInfoBase3 = infoParts[1];
                    notBra = false;
                    if (listInfoBase2 != null && listInfoBase2.GetInfoInt(ChaListDefine.KeyType.NotBra) == 1)
                    {
                        notBra = true;
                    }
                    if (listInfoBase3 != null && listInfoBase3.GetInfoInt(ChaListDefine.KeyType.NotBra) == 1)
                    {
                        notBra = true;
                    }
                }
                if ("0" != info)
                {
                    BustNormal value2 = null;
                    if (!dictBustNormal.TryGetValue(BustNormalKind.NmlTopPA, out value2))
                    {
                        value2 = new BustNormal();
                    }
                    value2.Init(objParts[kind], listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB), info, listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest));
                    dictBustNormal[BustNormalKind.NmlTopPA] = value2;
                }
                if (kind == 0)
                {
                    GameObject referenceInfo = GetReferenceInfo(RefObjKey.ObjInnerDef);
                    if (null != referenceInfo)
                    {
                        rendInner[0] = referenceInfo.GetComponent<Renderer>();
                    }
                    referenceInfo = GetReferenceInfo(RefObjKey.ObjInnerNuge);
                    if (null != referenceInfo)
                    {
                        rendInner[1] = referenceInfo.GetComponent<Renderer>();
                    }
                    if (rendInner != null)
                    {
                        if ((bool)rendInner[0])
                        {
                            rendInner[0].material.SetTexture(ChaShader._AlphaMask, texInnerAlphaMask);
                        }
                        if ((bool)rendInner[1])
                        {
                            rendInner[1].material.SetTexture(ChaShader._AlphaMask, texInnerAlphaMask);
                        }
                    }
                    LoadAlphaMaskTexture(listInfoBase.GetInfo(ChaListDefine.KeyType.OverBodyMaskAB), listInfoBase.GetInfo(ChaListDefine.KeyType.OverBodyMask), 0);
                    if ((bool)customMatBody)
                    {
                        customMatBody.SetTexture(ChaShader._AlphaMask, texBodyAlphaMask);
                    }
                    LoadAlphaMaskTexture(listInfoBase.GetInfo(ChaListDefine.KeyType.OverBraMaskAB), listInfoBase.GetInfo(ChaListDefine.KeyType.OverBraMask), 1);
                    if (rendBra != null)
                    {
                        Vector2[] array = new Vector2[2]
                        {
                            new Vector2(0f, 0f),
                            new Vector2(0f, -1f)
                        };
                        Vector2[] array2 = new Vector2[2]
                        {
                            new Vector2(1f, 1f),
                            new Vector2(1f, 2f)
                        };
                        ListInfoBase listInfoBase4 = infoClothes[2];
                        if (listInfoBase4 != null)
                        {
                            int num = listInfoBase4.GetInfoInt(ChaListDefine.KeyType.Coordinate) == 2 ? 1 : 0;
                            string value3;
                            if (listInfoBase4.dictInfo.TryGetValue(105, out value3))
                            {
                                num = "1" == value3 ? 1 : 0;
                            }
                            if ((bool)rendBra[0])
                            {
                                rendBra[0].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                                rendBra[0].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                                rendBra[0].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                            }
                            if ((bool)rendBra[1])
                            {
                                rendBra[1].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                                rendBra[1].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                                rendBra[1].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                            }
                        }
                    }
                }
                else if (kind == 1)
                {
                    LoadAlphaMaskTexture(listInfoBase.GetInfo(ChaListDefine.KeyType.OverInnerMaskAB), listInfoBase.GetInfo(ChaListDefine.KeyType.OverInnerMask), 2);
                    if (rendInner != null)
                    {
                        if ((bool)rendInner[0])
                        {
                            rendInner[0].material.SetTexture(ChaShader._AlphaMask, texInnerAlphaMask);
                        }
                        if ((bool)rendInner[1])
                        {
                            rendInner[1].material.SetTexture(ChaShader._AlphaMask, texInnerAlphaMask);
                        }
                    }
                    byte b = fileStatus.clothesState[0];
                    byte[,] array3 = new byte[4, 2]
                    {
                        {
                            1,
                            1
                        },
                        {
                            0,
                            1
                        },
                        {
                            0,
                            1
                        },
                        {
                            0,
                            0
                        }
                    };
                    ChangeAlphaMask(array3[b, 0], array3[b, 1]);
                }
            }
        }
        if (!objParts[kind])
        {
            yield break;
        }
        InitBaseCustomTextureClothes(false, kind);
        if (loadWithDefaultColorAndPtn)
        {
            SetClothesDefaultColor(0);
            for (int k = 0; k < 4; k++)
            {
                nowCoordinate.clothes.parts[0].colorInfo[k].pattern = 0;
            }
        }
        ChangeCustomClothes(false, kind, true, true, true, true, true);
        for (int l = 0; l < 2; l++)
        {
            ChangeCustomEmblem(0, l);
        }
        if (releaseCustomInputTexture)
        {
            ReleaseBaseCustomTextureClothes(true, kind, false);
        }
    }

    public void ChangeClothesBot(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesBotAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesBotAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 1;
        string objName = "ct_clothesBot";
        hideShortsBot = false;
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        Dictionary<int, ListInfoBase> dictFbx = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_top);
        if (dictFbx.Count != 0)
        {
            ListInfoBase value = null;
            if (dictFbx.TryGetValue(nowCoordinate.clothes.parts[0].id, out value) && "2" == value.GetInfo(ChaListDefine.KeyType.Coordinate))
            {
                load = false;
                release = true;
            }
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            ReleaseRefObject(6uL);
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 106, id, objName, true, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 106, id, objName, true, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                string value2;
                if (listInfoBase.dictInfo.TryGetValue(106, out value2))
                {
                    if (value2 == "1")
                    {
                        hideShortsBot = true;
                    }
                }
                else
                {
                    int[] source = new int[13]
                    {
                        2,
                        3,
                        9,
                        16,
                        22,
                        25,
                        28,
                        29,
                        30,
                        32,
                        34,
                        35,
                        45
                    };
                    if (source.Contains(id))
                    {
                        hideShortsBot = true;
                    }
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                CreateReferenceInfo(6uL, objClothes[kindNo]);
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if ((bool)objClothes[kindNo])
        {
            InitBaseCustomTextureClothes(true, kindNo);
            if (loadWithDefaultColorAndPtn)
            {
                SetClothesDefaultColor(kindNo);
                for (int i = 0; i < 4; i++)
                {
                    nowCoordinate.clothes.parts[kindNo].colorInfo[i].pattern = 0;
                }
            }
            ChangeCustomClothes(true, kindNo, true, true, true, true, false);
            for (int j = 0; j < 2; j++)
            {
                ChangeCustomEmblem(kindNo, j);
            }
            if (releaseCustomInputTexture)
            {
                ReleaseBaseCustomTextureClothes(true, kindNo, false);
            }
        }
        if (hiPoly)
        {
            UpdateSiru(true);
        }
    }

    public void ChangeClothesBra(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesBraAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesBraAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 2;
        string objName = "ct_bra";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            BustNormal value = null;
            if (dictBustNormal.TryGetValue(BustNormalKind.NmlBra, out value))
            {
                value.Release();
            }
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            ReleaseRefObject(7uL);
            notShorts = false;
            RemoveClothesStateKind(kindNo);
            rendBra = new Renderer[2];
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 107, id, objName, false, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 107, id, objName, false, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                notShorts = "2" == listInfoBase.GetInfo(ChaListDefine.KeyType.Coordinate);
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                CreateReferenceInfo(7uL, objClothes[kindNo]);
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
                GameObject referenceInfo = GetReferenceInfo(RefObjKey.ObjBraDef);
                if (null != referenceInfo)
                {
                    rendBra[0] = referenceInfo.GetComponent<Renderer>();
                }
                referenceInfo = GetReferenceInfo(RefObjKey.ObjBraNuge);
                if (null != referenceInfo)
                {
                    rendBra[1] = referenceInfo.GetComponent<Renderer>();
                }
                if (rendBra != null)
                {
                    Vector2[] array = new Vector2[2]
                    {
                        new Vector2(0f, 0f),
                        new Vector2(0f, -1f)
                    };
                    Vector2[] array2 = new Vector2[2]
                    {
                        new Vector2(1f, 1f),
                        new Vector2(1f, 2f)
                    };
                    int num = listInfoBase.GetInfoInt(ChaListDefine.KeyType.Coordinate) == 2 ? 1 : 0;
                    string value2;
                    if (listInfoBase.dictInfo.TryGetValue(105, out value2))
                    {
                        num = "1" == value2 ? 1 : 0;
                    }
                    if ((bool)rendBra[0])
                    {
                        rendBra[0].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                        rendBra[0].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                        rendBra[0].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                    }
                    if ((bool)rendBra[1])
                    {
                        rendBra[1].material.SetTexture(ChaShader._AlphaMask, texBraAlphaMask);
                        rendBra[1].material.SetTextureOffset(ChaShader._AlphaMask, array[num]);
                        rendBra[1].material.SetTextureScale(ChaShader._AlphaMask, array2[num]);
                    }
                }
                byte b = fileStatus.clothesState[0];
                byte[,] array3 = new byte[4, 2]
                {
                    {
                        1,
                        1
                    },
                    {
                        0,
                        1
                    },
                    {
                        0,
                        1
                    },
                    {
                        0,
                        0
                    }
                };
                ChangeAlphaMask(array3[b, 0], array3[b, 1]);
                string info = listInfoBase.GetInfo(ChaListDefine.KeyType.NormalData);
                if ("0" != info)
                {
                    BustNormal value3 = null;
                    if (!dictBustNormal.TryGetValue(BustNormalKind.NmlBra, out value3))
                    {
                        value3 = new BustNormal();
                    }
                    value3.Init(objClothes[kindNo], listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB), info, listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest));
                    dictBustNormal[BustNormalKind.NmlBra] = value3;
                }
            }
        }
        if ((bool)objClothes[kindNo])
        {
            InitBaseCustomTextureClothes(true, kindNo);
            if (loadWithDefaultColorAndPtn)
            {
                SetClothesDefaultColor(kindNo);
                for (int i = 0; i < 4; i++)
                {
                    nowCoordinate.clothes.parts[kindNo].colorInfo[i].pattern = 0;
                }
            }
            ChangeCustomClothes(true, kindNo, true, true, true, true, false);
            for (int j = 0; j < 2; j++)
            {
                ChangeCustomEmblem(kindNo, j);
            }
            if (releaseCustomInputTexture)
            {
                ReleaseBaseCustomTextureClothes(true, kindNo, false);
            }
        }
        if (hiPoly)
        {
            UpdateSiru(true);
        }
        if (!("0" == infoClothes[kindNo].GetInfo(ChaListDefine.KeyType.Coordinate)))
        {
            yield break;
        }
        int shortsKind = 3;
        if (null == objClothes[shortsKind])
        {
            if (asyncFlags)
            {
                yield return StartCoroutine(ChangeClothesShortsAsync(nowCoordinate.clothes.parts[shortsKind].id));
            }
            else
            {
                ChangeClothesShorts(nowCoordinate.clothes.parts[shortsKind].id);
            }
        }
    }

    public void ChangeClothesShorts(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesShortsAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesShortsAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 3;
        string objName = "ct_shorts";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        Dictionary<int, ListInfoBase> dictFbx = lstCtrl.GetCategoryInfo(ChaListDefine.CategoryNo.co_bra);
        if (dictFbx.Count != 0)
        {
            ListInfoBase value = null;
            if (dictFbx.TryGetValue(nowCoordinate.clothes.parts[2].id, out value) && "2" == value.GetInfo(ChaListDefine.KeyType.Coordinate))
            {
                load = false;
                release = true;
            }
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            ReleaseRefObject(8uL);
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 108, id, objName, false, 1, objTop.transform, sex != 0 ? 0 : 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 108, id, objName, false, 1, objTop.transform, sex != 0 ? 0 : 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                if (listInfoBase.Id == 17)
                {
                    for (int i = 0; i < cusClothesCmp[kindNo].rendNormal01.Length; i++)
                    {
                        cusClothesCmp[kindNo].rendNormal01[i].material.renderQueue = 3030;
                    }
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                CreateReferenceInfo(8uL, objClothes[kindNo]);
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if ((bool)objClothes[kindNo])
        {
            InitBaseCustomTextureClothes(true, kindNo);
            if (loadWithDefaultColorAndPtn)
            {
                SetClothesDefaultColor(kindNo);
                for (int j = 0; j < 4; j++)
                {
                    nowCoordinate.clothes.parts[kindNo].colorInfo[j].pattern = 0;
                }
            }
            ChangeCustomClothes(true, kindNo, true, true, true, true, false);
            for (int k = 0; k < 2; k++)
            {
                ChangeCustomEmblem(kindNo, k);
            }
            if (releaseCustomInputTexture)
            {
                ReleaseBaseCustomTextureClothes(true, kindNo, false);
            }
        }
        if (hiPoly)
        {
            UpdateSiru(true);
        }
    }

    public void ChangeClothesGloves(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesGlovesAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesGlovesAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 4;
        string objName = "ct_gloves";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 109, id, objName, false, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 109, id, objName, false, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if (!objClothes[kindNo])
        {
            yield break;
        }
        InitBaseCustomTextureClothes(true, kindNo);
        if (loadWithDefaultColorAndPtn)
        {
            SetClothesDefaultColor(kindNo);
            for (int i = 0; i < 4; i++)
            {
                nowCoordinate.clothes.parts[kindNo].colorInfo[i].pattern = 0;
            }
        }
        ChangeCustomClothes(true, kindNo, true, true, true, true, false);
        for (int j = 0; j < 2; j++)
        {
            ChangeCustomEmblem(kindNo, j);
        }
        if (releaseCustomInputTexture)
        {
            ReleaseBaseCustomTextureClothes(true, kindNo, false);
        }
    }

    public void ChangeClothesPanst(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesPanstAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesPanstAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 5;
        string objName = "ct_panst";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            ReleaseRefObject(10uL);
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 110, id, objName, false, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 110, id, objName, false, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                if (listInfoBase.Id == 11)
                {
                    for (int i = 0; i < cusClothesCmp[kindNo].rendNormal01.Length; i++)
                    {
                        cusClothesCmp[kindNo].rendNormal01[i].material.renderQueue = 3040;
                    }
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                CreateReferenceInfo(10uL, objClothes[kindNo]);
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if ((bool)objClothes[kindNo])
        {
            InitBaseCustomTextureClothes(true, kindNo);
            if (loadWithDefaultColorAndPtn)
            {
                SetClothesDefaultColor(kindNo);
                for (int j = 0; j < 4; j++)
                {
                    nowCoordinate.clothes.parts[kindNo].colorInfo[j].pattern = 0;
                }
            }
            ChangeCustomClothes(true, kindNo, true, true, true, true, false);
            for (int k = 0; k < 2; k++)
            {
                ChangeCustomEmblem(kindNo, k);
            }
            if (releaseCustomInputTexture)
            {
                ReleaseBaseCustomTextureClothes(true, kindNo, false);
            }
        }
        if (hiPoly)
        {
            UpdateSiru(true);
        }
    }

    public void ChangeClothesSocks(int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesSocksAsync(id, forceChange, false));
    }

    public IEnumerator ChangeClothesSocksAsync(int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 6;
        string objName = "ct_socks";
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, 111, id, objName, false, 1, objTop.transform, 0);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, 111, id, objName, false, 1, objTop.transform, 0);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                if (listInfoBase.Id == 21)
                {
                    for (int i = 0; i < cusClothesCmp[kindNo].rendNormal01.Length; i++)
                    {
                        cusClothesCmp[kindNo].rendNormal01[i].material.renderQueue = 3050;
                    }
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if (!objClothes[kindNo])
        {
            yield break;
        }
        InitBaseCustomTextureClothes(true, kindNo);
        if (loadWithDefaultColorAndPtn)
        {
            SetClothesDefaultColor(kindNo);
            for (int j = 0; j < 4; j++)
            {
                nowCoordinate.clothes.parts[kindNo].colorInfo[j].pattern = 0;
            }
        }
        ChangeCustomClothes(true, kindNo, true, true, true, true, false);
        for (int k = 0; k < 2; k++)
        {
            ChangeCustomEmblem(kindNo, k);
        }
        if (releaseCustomInputTexture)
        {
            ReleaseBaseCustomTextureClothes(true, kindNo, false);
        }
    }

    public void ChangeClothesShoes(int type, int id, bool forceChange = false)
    {
        StartCoroutine(ChangeClothesShoesAsync(type, id, forceChange, false));
    }

    public IEnumerator ChangeClothesShoesAsync(int type, int id, bool forceChange = false, bool asyncFlags = true)
    {
        bool load = true;
        bool release = true;
        int kindNo = 0;
        string objName = string.Empty;
        int defID = 0;
        if (type == 0)
        {
            kindNo = 7;
            objName = "ct_shoes_inner";
            defID = 0;
        }
        else
        {
            kindNo = 8;
            objName = "ct_shoes_outer";
            defID = 0;
        }
        int nowId = infoClothes[kindNo] != null ? infoClothes[kindNo].Id : -1;
        if (!forceChange && null != objClothes[kindNo] && id == nowId)
        {
            load = false;
            release = false;
        }
        ReleaseBaseCustomTextureClothes(true, kindNo);
        if (release && objClothes[kindNo])
        {
            SafeDestroy(objClothes[kindNo]);
            objClothes[kindNo] = null;
            infoClothes[kindNo] = null;
            cusClothesCmp[kindNo] = null;
            RemoveClothesStateKind(kindNo);
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objClothes[kindNo] = o;
                }, hiPoly, sex != 0 || exType != 1 ? 112 : 504, id, objName, false, 1, objTop.transform, defID);
                yield return StartCoroutine(cor);
            }
            else
            {
                objClothes[kindNo] = LoadCharaFbxData(hiPoly, sex != 0 || exType != 1 ? 112 : 504, id, objName, false, 1, objTop.transform, defID);
            }
            if (asyncFlags)
            {
                yield return null;
            }
            if ((bool)objClothes[kindNo])
            {
                ListInfoComponent component = objClothes[kindNo].GetComponent<ListInfoComponent>();
                ListInfoBase listInfoBase = infoClothes[kindNo] = component.data;
                cusClothesCmp[kindNo] = objClothes[kindNo].GetComponent<ChaClothesComponent>();
                if (!(listInfoBase.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusClothesCmp[kindNo])
                {
                }
                nowCoordinate.clothes.parts[kindNo].id = listInfoBase.Id;
                AddClothesStateKind(kindNo, listInfoBase.GetInfo(ChaListDefine.KeyType.StateType));
            }
        }
        if (!objClothes[kindNo])
        {
            yield break;
        }
        InitBaseCustomTextureClothes(true, kindNo);
        if (loadWithDefaultColorAndPtn)
        {
            SetClothesDefaultColor(kindNo);
            for (int i = 0; i < 4; i++)
            {
                nowCoordinate.clothes.parts[kindNo].colorInfo[i].pattern = 0;
            }
        }
        ChangeCustomClothes(true, kindNo, true, true, true, true, false);
        for (int j = 0; j < 2; j++)
        {
            ChangeCustomEmblem(kindNo, j);
        }
        if (releaseCustomInputTexture)
        {
            ReleaseBaseCustomTextureClothes(true, kindNo, false);
        }
    }

    public void ChangeAccessory(bool forceChange = false)
    {
        for (int i = 0; i < 20; i++)
        {
            StartCoroutine(ChangeAccessoryAsync(i, nowCoordinate.accessory.parts[i].type, nowCoordinate.accessory.parts[i].id, nowCoordinate.accessory.parts[i].parentKey, forceChange, false));
        }
    }

    public void ChangeAccessory(int slotNo, int type, int id, string parentKey, bool forceChange = false)
    {
        StartCoroutine(ChangeAccessoryAsync(slotNo, type, id, parentKey, forceChange, false));
    }

    public IEnumerator ChangeAccessoryAsync(bool forceChange = false)
    {
        for (int i = 0; i < 20; i++)
        {
            yield return StartCoroutine(ChangeAccessoryAsync(i, nowCoordinate.accessory.parts[i].type, nowCoordinate.accessory.parts[i].id, nowCoordinate.accessory.parts[i].parentKey, forceChange));
        }
    }

    public IEnumerator ChangeAccessoryAsync(int slotNo, int type, int id, string parentKey, bool forceChange = false, bool asyncFlags = true)
    {
        if (!MathfEx.RangeEqualOn(0, slotNo, 19))
        {
            yield break;
        }
        ListInfoBase lib = null;
        bool load = true;
        bool release = true;
        if (type == 120 || !MathfEx.RangeEqualOn(121, type, 130))
        {
            release = true;
            load = false;
        }
        else
        {
            if (id == -1)
            {
                release = false;
                load = false;
            }
            int num = infoAccessory[slotNo] != null ? infoAccessory[slotNo].Category : -1;
            int num2 = infoAccessory[slotNo] != null ? infoAccessory[slotNo].Id : -1;
            if (!forceChange && null != objAccessory[slotNo] && type == num && id == num2)
            {
                load = false;
                release = false;
            }
            if (id != -1)
            {
                Dictionary<int, ListInfoBase> categoryInfo = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)type);
                if (categoryInfo == null)
                {
                    release = true;
                    load = false;
                }
                else if (!categoryInfo.TryGetValue(id, out lib))
                {
                    release = true;
                    load = false;
                }
                else if (!hiPoly)
                {
                    bool flag = true;
                    if (type == 123 && lib.Kind == 1)
                    {
                        flag = false;
                    }
                    if (type == 122 && lib.GetInfoInt(ChaListDefine.KeyType.HideHair) == 1)
                    {
                        flag = false;
                    }
                    if (Manager.Config.EtcData.loadHeadAccessory && type == 122 && lib.Kind == 1)
                    {
                        flag = false;
                    }
                    if (Manager.Config.EtcData.loadAllAccessory)
                    {
                        flag = false;
                    }
                    if (flag)
                    {
                        release = true;
                        load = false;
                    }
                }
            }
        }
        if (release)
        {
            if (!load)
            {
                nowCoordinate.accessory.parts[slotNo].MemberInit();
                nowCoordinate.accessory.parts[slotNo].type = 120;
            }
            if ((bool)objAccessory[slotNo])
            {
                SafeDestroy(objAccessory[slotNo]);
                objAccessory[slotNo] = null;
                infoAccessory[slotNo] = null;
                cusAcsCmp[slotNo] = null;
                for (int i = 0; i < 2; i++)
                {
                    objAcsMove[slotNo, i] = null;
                }
            }
        }
        if (load)
        {
            if (asyncFlags)
            {
                yield return null;
            }
            byte weight = 0;
            Transform trfParent = null;
            if ("null" == lib.GetInfo(ChaListDefine.KeyType.Parent))
            {
                weight = 2;
                trfParent = objTop.transform;
            }
            if (asyncFlags)
            {
                IEnumerator cor = LoadCharaFbxDataAsync(delegate (GameObject o)
                {
                    objAccessory[slotNo] = o;
                }, true, type, id, "ca_slot" + slotNo.ToString("00"), false, weight, trfParent, -1);
                yield return StartCoroutine(cor);
            }
            else
            {
                objAccessory[slotNo] = LoadCharaFbxData(true, type, id, "ca_slot" + slotNo.ToString("00"), false, weight, trfParent, -1);
            }
            if ((bool)objAccessory[slotNo])
            {
                ListInfoComponent component = objAccessory[slotNo].GetComponent<ListInfoComponent>();
                lib = infoAccessory[slotNo] = component.data;
                cusAcsCmp[slotNo] = objAccessory[slotNo].GetComponent<ChaAccessoryComponent>();
                if (!(lib.GetInfo(ChaListDefine.KeyType.MainData) != "p_dummy") || null == cusAcsCmp[slotNo])
                {
                }
                nowCoordinate.accessory.parts[slotNo].type = type;
                nowCoordinate.accessory.parts[slotNo].id = lib.Id;
                objAcsMove[slotNo, 0] = objAccessory[slotNo].transform.FindLoop("N_move");
                objAcsMove[slotNo, 1] = objAccessory[slotNo].transform.FindLoop("N_move2");
            }
        }
        if ((bool)objAccessory[slotNo])
        {
            if (loadWithDefaultColorAndPtn)
            {
                SetAccessoryDefaultColor(slotNo);
            }
            ChangeAccessoryColor(slotNo);
            if (string.Empty == parentKey)
            {
                parentKey = lib.GetInfo(ChaListDefine.KeyType.Parent);
            }
            ChangeAccessoryParent(slotNo, parentKey);
            UpdateAccessoryMoveFromInfo(slotNo);
            nowCoordinate.accessory.parts[slotNo].partsOfHead = ChaAccessoryDefine.CheckPartsOfHead(parentKey);
            if (!hiPoly && !Manager.Config.EtcData.loadAllAccessory)
            {
                DynamicBone[] componentsInChildren = objAccessory[slotNo].GetComponentsInChildren<DynamicBone>(true);
                DynamicBone[] array = componentsInChildren;
                foreach (DynamicBone dynamicBone in array)
                {
                    dynamicBone.enabled = false;
                }
            }
            ChangeShakeAccessory(slotNo);
        }
        SetHideHairAccessory();
    }

    private GameObject LoadCharaFbxData(bool _hiPoly, int category, int id, string createName, bool copyDynamicBone, byte copyWeights, Transform trfParent, int defaultId, bool worldPositionStays = false)
    {
        GameObject actObj = null;
        StartCoroutine(LoadCharaFbxDataAsync(delegate (GameObject o)
        {
            actObj = o;
        }, _hiPoly, category, id, createName, copyDynamicBone, copyWeights, trfParent, defaultId, false, worldPositionStays));
        return actObj;
    }

    private IEnumerator LoadCharaFbxDataAsync(Action<GameObject> actObj, bool _hiPoly, int category, int id, string createName, bool copyDynamicBone, byte copyWeights, Transform trfParent, int defaultId, bool AsyncFlags = true, bool worldPositionStays = false)
    {
        Dictionary<int, ListInfoBase> work2 = null;
        work2 = lstCtrl.GetCategoryInfo((ChaListDefine.CategoryNo)category);
        if (work2.Count == 0)
        {
            actObj(null);
            yield break;
        }
        ListInfoBase lib = null;
        if (!work2.TryGetValue(id, out lib))
        {
            if (defaultId == -1)
            {
                actObj(null);
                yield break;
            }
            if (id != defaultId)
            {
                work2.TryGetValue(defaultId, out lib);
            }
            if (lib == null && !work2.TryGetValue(0, out lib))
            {
                actObj(null);
                yield break;
            }
        }
        else if (category == 105 || category == 107)
        {
            int infoInt = lib.GetInfoInt(ChaListDefine.KeyType.Sex);
            bool flag = false;
            if (sex == 0 && infoInt == 3)
            {
                flag = true;
            }
            else if (sex == 1 && infoInt == 2)
            {
                flag = true;
            }
            if (flag)
            {
                if (id != defaultId)
                {
                    work2.TryGetValue(defaultId, out lib);
                }
                if (lib == null && !work2.TryGetValue(0, out lib))
                {
                    actObj(null);
                    yield break;
                }
            }
        }
        if (category == 501)
        {
            lib = lstCtrl.GetListInfo(ChaListDefine.CategoryNo.ex_bo_head, 0);
            if (lib == null)
            {
                actObj(null);
                yield break;
            }
        }
        string assetName = lib.GetInfo(ChaListDefine.KeyType.MainData);
        if (string.Empty == assetName)
        {
            yield break;
        }
        if (!_hiPoly)
        {
            assetName += "_low";
        }
        string manifestName = lib.GetInfo(ChaListDefine.KeyType.MainManifest);
        string assetBundleName = lib.GetInfo(ChaListDefine.KeyType.MainAB);
        GameObject newObj = null;
        if (AsyncFlags)
        {
            yield return StartCoroutine(Load_Coroutine(assetBundleName, assetName, delegate (GameObject x)
            {
                newObj = x;
            }, true, manifestName));
        }
        else
        {
            newObj = CommonLib.LoadAsset<GameObject>(assetBundleName, assetName, true, manifestName);
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, manifestName);
        if (null == newObj)
        {
            actObj(null);
            yield break;
        }
        newObj.name = createName;
        if ((bool)trfParent)
        {
            newObj.transform.SetParent(trfParent, worldPositionStays);
        }
        DynamicBoneCollider[] dbc = objBodyBone.GetComponentsInChildren<DynamicBoneCollider>(true);
        Dictionary<string, GameObject> dictBone = aaWeightsBody.dictBone;
        DynamicBone[] db = newObj.GetComponentsInChildren<DynamicBone>(true);
        DynamicBone[] array = db;
        foreach (DynamicBone dynamicBone in array)
        {
            if (copyDynamicBone)
            {
                if ((bool)dynamicBone.m_Root)
                {
                    foreach (KeyValuePair<string, GameObject> item in dictBone)
                    {
                        if (!(item.Key != dynamicBone.m_Root.name))
                        {
                            dynamicBone.m_Root = item.Value.transform;
                            break;
                        }
                    }
                }
                if (dynamicBone.m_Exclusions != null && dynamicBone.m_Exclusions.Count != 0)
                {
                    for (int j = 0; j < dynamicBone.m_Exclusions.Count; j++)
                    {
                        if (!(null == dynamicBone.m_Exclusions[j]))
                        {
                            foreach (KeyValuePair<string, GameObject> item2 in dictBone)
                            {
                                if (!(item2.Key != dynamicBone.m_Exclusions[j].name))
                                {
                                    dynamicBone.m_Exclusions[j] = item2.Value.transform;
                                    break;
                                }
                            }
                        }
                    }
                }
                if (dynamicBone.m_notRolls != null && dynamicBone.m_notRolls.Count != 0)
                {
                    for (int k = 0; k < dynamicBone.m_notRolls.Count; k++)
                    {
                        if (!(null == dynamicBone.m_notRolls[k]))
                        {
                            foreach (KeyValuePair<string, GameObject> item3 in dictBone)
                            {
                                if (!(item3.Key != dynamicBone.m_notRolls[k].name))
                                {
                                    dynamicBone.m_notRolls[k] = item3.Value.transform;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            if (dynamicBone.m_Colliders != null)
            {
                dynamicBone.m_Colliders.Clear();
                for (int l = 0; l < dbc.Length; l++)
                {
                    dynamicBone.m_Colliders.Add(dbc[l]);
                }
            }
        }
        GameObject objRootBone = GetReferenceInfo(RefObjKey.A_ROOTBONE);
        Transform trfRootBone = !objRootBone ? null : objRootBone.transform;
        switch (copyWeights)
        {
            case 1:
                aaWeightsBody.AssignedWeightsAndSetBounds(newObj, "cf_j_root", bounds, trfRootBone);
                break;
            case 2:
                aaWeightsHead.AssignedWeightsAndSetBounds(newObj, "cf_J_N_FaceRoot", bounds, trfRootBone);
                break;
        }
        ListInfoComponent libComponent = newObj.AddComponent<ListInfoComponent>();
        libComponent.data = lib.Clone();
        actObj(newObj);
    }

    protected bool InitBaseCustomTextureBody()
    {
        if (customTexCtrlBody != null)
        {
            customTexCtrlBody.Release();
            customTexCtrlBody = null;
        }
        string text = "abdata";
        string text2 = "chara/mm_base.unity3d";
        string drawMatName = sex != 0 ? "cf_m_body" : "cm_m_body";
        string createMatName = "cf_m_body_create";
        if (sex == 0 && exType == 1)
        {
            text = "add54";
            text2 = "chara/54/chara_mat_00.unity3d";
            drawMatName = "cm_m_ex_body_00";
            createMatName = "cm_m_ex_body_00_create";
        }
        int num = !hiPoly ? 512 : 2048;
        customTexCtrlBody = new CustomTextureControl(objRoot.transform);
        customTexCtrlBody.Initialize(text2, drawMatName, text, text2, createMatName, text, num, num);
        return true;
    }

    protected bool InitBaseCustomTextureFace(string drawAssetBundleName, string drawAssetName, string drawManifest)
    {
        if (customTexCtrlFace != null)
        {
            customTexCtrlFace.Release();
            customTexCtrlFace = null;
        }
        string createMatManifest = "abdata";
        string createMatABName = "chara/mm_base.unity3d";
        string createMatName = "cf_m_face_create";
        if (sex == 0 && exType == 1)
        {
            drawManifest = "add54";
            createMatManifest = "add54";
            createMatABName = "chara/54/chara_mat_00.unity3d";
            drawAssetName = "cm_m_ex_face_00";
            createMatName = "cm_m_ex_face_00_create";
        }
        int num = !hiPoly ? 512 : 1024;
        customTexCtrlFace = new CustomTextureControl(objRoot.transform);
        customTexCtrlFace.Initialize(drawAssetBundleName, drawAssetName, drawManifest, createMatABName, createMatName, createMatManifest, num, num);
        return true;
    }

    protected bool InitBaseCustomTextureEtc()
    {
        int num = !hiPoly ? 256 : 512;
        if (ctCreateEyeW != null)
        {
            ctCreateEyeW.Release();
            ctCreateEyeW = null;
        }
        string empty = "chara/mm_base.unity3d";
        string empty2 = "cf_m_eyewhite_create";
        ctCreateEyeW = new CustomTextureCreate(objRoot.transform);
        ctCreateEyeW.Initialize(empty, empty2, string.Empty, num, num);
        if (ctCreateEyeL != null)
        {
            ctCreateEyeL.Release();
            ctCreateEyeL = null;
        }
        empty = "chara/mm_base.unity3d";
        empty2 = "cf_m_eye_create";
        ctCreateEyeL = new CustomTextureCreate(objRoot.transform);
        ctCreateEyeL.Initialize(empty, empty2, string.Empty, num, num);
        if (ctCreateEyeR != null)
        {
            ctCreateEyeR.Release();
            ctCreateEyeR = null;
        }
        empty = "chara/mm_base.unity3d";
        empty2 = "cf_m_eye_create";
        ctCreateEyeR = new CustomTextureCreate(objRoot.transform);
        ctCreateEyeR.Initialize(empty, empty2, string.Empty, num, num);
        return true;
    }

    protected void ReleaseBaseCustomTextureClothes(bool main, int parts, bool createTex = true)
    {
        if (main)
        {
            for (int i = 0; i < 3; i++)
            {
                if (ctCreateClothes[parts, i] != null)
                {
                    if (createTex)
                    {
                        ctCreateClothes[parts, i].Release();
                    }
                    else
                    {
                        ctCreateClothes[parts, i].ReleaseCreateMaterial();
                    }
                    ctCreateClothes[parts, i] = null;
                }
            }
            return;
        }
        for (int j = 0; j < 3; j++)
        {
            if (ctCreateClothesSub[parts, j] != null)
            {
                if (createTex)
                {
                    ctCreateClothesSub[parts, j].Release();
                }
                else
                {
                    ctCreateClothesSub[parts, j].ReleaseCreateMaterial();
                }
                ctCreateClothesSub[parts, j] = null;
            }
        }
    }

    protected bool InitBaseCustomTextureClothes(bool main, int parts)
    {
        if (main && infoClothes == null)
        {
            return false;
        }
        if (!main && infoParts == null)
        {
            return false;
        }
        ListInfoBase listInfoBase = !main ? infoParts[parts] : infoClothes[parts];
        string empty = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty2 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTexAB);
        if ("0" == empty2)
        {
            empty2 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty3 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex);
        string empty4 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty5 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex02AB);
        if ("0" == empty5)
        {
            empty5 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty6 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex02);
        string empty7 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty8 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex03AB);
        if ("0" == empty8)
        {
            empty8 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty9 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainTex03);
        string empty10 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty11 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskAB);
        if ("0" == empty11)
        {
            empty11 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty12 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMaskTex);
        string empty13 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty14 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMask02AB);
        if ("0" == empty14)
        {
            empty14 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty15 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMask02Tex);
        string empty16 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainManifest);
        string empty17 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMask03AB);
        if ("0" == empty17)
        {
            empty17 = listInfoBase.GetInfo(ChaListDefine.KeyType.MainAB);
        }

        string empty18 = listInfoBase.GetInfo(ChaListDefine.KeyType.ColorMask03Tex);
        if ("0" == empty2 || "0" == empty3)
        {
            return false;
        }
        if (!hiPoly)
        {
            empty3 += "_low";
        }
        Texture2D texture2D = CommonLib.LoadAsset<Texture2D>(empty2, empty3, false, empty);
        if (null == texture2D)
        {
            return false;
        }

        if ("0" == empty11 || "0" == empty12)
        {
            Resources.UnloadAsset(texture2D);
            return false;
        }
        if (!hiPoly)
        {
            empty12 += "_low";
        }
        Texture2D texture2D2 = CommonLib.LoadAsset<Texture2D>(empty11, empty12, false, empty10);
        if (null == texture2D2)
        {
            Resources.UnloadAsset(texture2D);
            return false;
        }
        Texture2D texture2D3 = null;
        if ("0" != empty5 && "0" != empty6)
        {
            if (!hiPoly)
            {
                empty6 += "_low";
            }
            texture2D3 = CommonLib.LoadAsset<Texture2D>(empty5, empty6, false, empty4);
        }
        Texture2D texture2D4 = null;
        if ("0" != empty14 && "0" != empty15)
        {
            if (!hiPoly)
            {
                empty15 += "_low";
            }
            texture2D4 = CommonLib.LoadAsset<Texture2D>(empty14, empty15, false, empty13);
        }
        Texture2D texture2D5 = null;
        if ("0" != empty8 && "0" != empty9)
        {
            texture2D5 = CommonLib.LoadAsset<Texture2D>(empty8, empty9, false, empty7);
        }
        Texture2D texture2D6 = null;
        if ("0" != empty17 && "0" != empty18)
        {
            texture2D6 = CommonLib.LoadAsset<Texture2D>(empty17, empty18, false, empty16);
        }
        string createMatABName = "chara/mm_base.unity3d";
        string createMatName = "cf_m_clothesN_create";
        for (int i = 0; i < 3; i++)
        {
            CustomTextureCreate customTextureCreate = null;
            Texture2D texture2D7;
            Texture2D texture2D8;
            switch (i)
            {
                case 0:
                    texture2D7 = texture2D;
                    texture2D8 = texture2D2;
                    break;
                case 1:
                    texture2D7 = texture2D3;
                    texture2D8 = texture2D4;
                    break;
                default:
                    texture2D7 = texture2D5;
                    texture2D8 = texture2D6;
                    break;
            }
            if (null != texture2D7)
            {
                customTextureCreate = new CustomTextureCreate(objRoot.transform);
                int width = texture2D7.width;
                int height = texture2D7.height;
                customTextureCreate.Initialize(createMatABName, createMatName, string.Empty, width, height);
                customTextureCreate.SetMainTexture(texture2D7);
                customTextureCreate.SetTexture(ChaShader._ColorMask, texture2D8);
            }
            if (main)
            {
                ctCreateClothes[parts, i] = customTextureCreate;
            }
            else
            {
                ctCreateClothesSub[parts, i] = customTextureCreate;
            }
        }
        return true;
    }

    public bool LoadGagMaterial()
    {
        string assetBundleName = "chara/mm_base.unity3d";
        string[] array = new string[3]
        {
            "cf_m_gageye_00",
            "cf_m_gageye_01",
            "cf_m_gageye_02"
        };
        for (int i = 0; i < 3; i++)
        {
            matGag[i] = CommonLib.LoadAsset<Material>(assetBundleName, array[i], true, string.Empty);
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, string.Empty);
        return true;
    }

    public bool LoadAlphaMaskTexture(string assetBundleName, string assetName, byte type)
    {
        if ("0" == assetBundleName || "0" == assetName)
        {
            return false;
        }
        if (!hiPoly)
        {
            assetName += "_low";
        }
        string manifestName = "abdata";
        if (exType == 1)
        {
            manifestName = "add54";
        }
        Texture texture = CommonLib.LoadAsset<Texture>(assetBundleName, assetName, false, manifestName);
        if (null == texture)
        {
            return false;
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, manifestName);
        switch (type)
        {
            case 0:
                texBodyAlphaMask = texture;
                break;
            case 1:
                texBraAlphaMask = texture;
                break;
            case 2:
                texInnerAlphaMask = texture;
                break;
        }
        return true;
    }

    private void MozInit(params GameObject[] objMoz)
    {
        Texture[] array = new Texture[4];
        string assetBundleName = "chara/etc.unity3d";
        string mainManifestName = Singleton<Character>.Instance.mainManifestName;
        for (int i = 0; i < array.Length; i++)
        {
            array[i] = CommonLib.LoadAsset<Texture>(assetBundleName, "cf_t_mnpb_00_" + i.ToString("00"), false, mainManifestName);
        }
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, mainManifestName);
        foreach (GameObject gameObject in objMoz)
        {
            if (!(null == gameObject))
            {
                ChangePtnAnime changePtnAnime = gameObject.AddComponent<ChangePtnAnime>();
                changePtnAnime.Init(array);
            }
        }
    }

    public bool InitializeExpression(bool _enable = true)
    {
        expression = objRoot.AddComponent<ExpressionBone>();
        expression.trfBoneRoot = objRoot.transform;
        expression.Initialize();
        expression.enable = _enable;
        return true;
    }

    public void LoadHitObject()
    {
        ReleaseHitObject();
        string assetBundleName = "chara/oo_base.unity3d";
        string assetName = "p_cf_body_00_hit";
        objHitBody = CommonLib.LoadAsset<GameObject>(assetBundleName, assetName, true, string.Empty);
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, string.Empty);
        if ((bool)objHitBody)
        {
            objHitBody.transform.SetParent(objTop.transform, false);
            aaWeightsBody.AssignedWeights(objHitBody, "cf_j_root");
            SkinnedCollisionHelper[] componentsInChildren = objHitBody.GetComponentsInChildren<SkinnedCollisionHelper>(true);
            SkinnedCollisionHelper[] array = componentsInChildren;
            foreach (SkinnedCollisionHelper skinnedCollisionHelper in array)
            {
                skinnedCollisionHelper.Init();
            }
            hitBodyCtrlCmp = objHitBody.GetComponent<ChaHitControlComponent>();
            if (sex == 0 && null != hitBodyCtrlCmp)
            {
                hitBodyCtrlCmp.ChangeBustBlendShapeValue(90f);
            }
        }
        if (sex == 0 || !(null != objHead))
        {
            return;
        }
        ListInfoComponent component = objHead.GetComponent<ListInfoComponent>();
        string manifestName = component.data.dictInfo[32];
        assetBundleName = component.data.dictInfo[30];
        assetName = component.data.dictInfo[31] + "_hit";
        objHitHead = CommonLib.LoadAsset<GameObject>(assetBundleName, assetName, true, manifestName);
        Singleton<Character>.Instance.AddLoadAssetBundle(assetBundleName, manifestName);
        if ((bool)objHitHead)
        {
            objHitHead.transform.SetParent(objTop.transform, false);
            aaWeightsHead.AssignedWeights(objHitHead, "cf_J_N_FaceRoot");
            SkinnedCollisionHelper[] componentsInChildren2 = objHitHead.GetComponentsInChildren<SkinnedCollisionHelper>(true);
            SkinnedCollisionHelper[] array2 = componentsInChildren2;
            foreach (SkinnedCollisionHelper skinnedCollisionHelper2 in array2)
            {
                skinnedCollisionHelper2.Init();
            }
        }
    }

    public void ReleaseHitObject()
    {
        if ((bool)objHitBody)
        {
            SkinnedCollisionHelper[] componentsInChildren = objHitBody.GetComponentsInChildren<SkinnedCollisionHelper>(true);
            SkinnedCollisionHelper[] array = componentsInChildren;
            foreach (SkinnedCollisionHelper skinnedCollisionHelper in array)
            {
                skinnedCollisionHelper.Release();
            }
            SafeDestroy(objHitBody);
            objHitBody = null;
        }
        if ((bool)objHitHead)
        {
            SkinnedCollisionHelper[] componentsInChildren2 = objHitHead.GetComponentsInChildren<SkinnedCollisionHelper>(true);
            SkinnedCollisionHelper[] array2 = componentsInChildren2;
            foreach (SkinnedCollisionHelper skinnedCollisionHelper2 in array2)
            {
                skinnedCollisionHelper2.Release();
            }
            SafeDestroy(objHitHead);
            objHitHead = null;
        }
        hitBodyCtrlCmp = null;
    }
}
